@@ -47,7 +47,7 @@ namespace wb {
     virtual std::string get_name() { return WBComponentBasic::name(); }
     virtual std::string get_diagram_class_name() { return model_Diagram::static_class_name(); }
 
-    void delete_selection();
+    //void delete_selection();
     
   protected:
     virtual void load_app_options(bool update);
@@ -77,9 +77,13 @@ namespace wb {
                                       const base::Rect &rect);
 
     void delete_object(ModelDiagramForm *view, const base::Point &pos);
-    virtual model_ObjectRef clone_object(const model_ObjectRef &object, const model_LayerRef &destlayer, grt::CopyContext &copy_context, bool copy_only);
 
-    virtual bool delete_model_object(const model_ObjectRef &object, std::map<std::string,bool> &options);
+    virtual bool delete_model_object(const model_ObjectRef &object, bool figure_only);
+    
+    virtual bool can_paste_object(const grt::ObjectRef &object);
+    virtual model_ObjectRef paste_object(ModelDiagramForm *view, const grt::ObjectRef &object, grt::CopyContext &copy_context);
+    virtual void copy_object_to_clipboard(const grt::ObjectRef &object,
+                                     grt::CopyContext &copy_context);
 
     void reset_tool(ModelDiagramForm *view, void*);
     bool handle_motion_event(ModelDiagramForm*, base::Point, mdc::EventState, void*);
