@@ -94,8 +94,10 @@ namespace wb {
     virtual void add_new_stored_script(const workbench_physical_ModelRef &model, const std::string &path="");
     virtual void add_new_stored_note(const workbench_physical_ModelRef &model, const std::string &path= "");
 
-    void interactive_place_db_objects(ModelDiagramForm *vform, int x, int y, 
+    std::list<model_FigureRef> interactive_place_db_objects(ModelDiagramForm *vform, int x, int y, 
       const std::list<db_DatabaseObjectRef> &objects);
+    std::list<model_FigureRef> interactive_place_db_objects(ModelDiagramForm *vform, int x, int y, 
+      const std::list<db_DatabaseObjectRef> &objects, grt::CopyContext& copy_context);
     
     model_FigureRef place_db_object(ModelDiagramForm *view, const base::Point &pos, const db_DatabaseObjectRef &object,
                          bool select_figure= true);
@@ -209,13 +211,12 @@ namespace wb {
     void add_schema_listeners(const db_SchemaRef &schema);
     void add_schema_object_listeners(const grt::ObjectRef &object);
 
-    virtual bool delete_model_object(const model_ObjectRef &object, std::map<std::string,bool> &options);
+    virtual bool delete_model_object(const model_ObjectRef &object, bool figure_only);
 
     virtual bool handles_figure(const model_ObjectRef &figure);
-    virtual model_ObjectRef clone_object(const model_ObjectRef &object, const model_LayerRef &destlayer,
-      grt::CopyContext &copy_context, bool copy_only);
-    virtual bool can_paste_non_model_object(const grt::ObjectRef &object);
-    virtual void paste_non_model_object(ModelDiagramForm *view, const grt::ObjectRef &object);
+    virtual bool can_paste_object(const grt::ObjectRef &object);
+    virtual model_ObjectRef paste_object(ModelDiagramForm *view, const grt::ObjectRef &object, grt::CopyContext &copy_context);
+    virtual void copy_object_to_clipboard(const grt::ObjectRef &object, grt::CopyContext &copy_context);
 
     virtual GrtObjectRef get_object_for_figure(const model_ObjectRef &object);
 
