@@ -35,6 +35,7 @@
 #include "wbcanvas/workbench_physical_model_impl.h"
 #include "workbench/wb_model_file.h"
 
+#include "grtdb/db_object_helpers.h"
 #include "grt/clipboard.h"
 #include "grt/common.h"
 #include "base/file_utilities.h"
@@ -79,6 +80,10 @@ WBContextModel::WBContextModel(WBContextUI *wbui)
   cmdui->add_builtin_command("addModelRoutine",
                              boost::bind(&WBContextModel::add_model_rgroup, this),
                              validate);
+
+  cmdui->add_builtin_command("removeFigure",
+                             boost::bind(&WBContextModel::remove_figure, this),
+                             boost::bind(&WBContextModel::has_selected_figures, this));
   
   base::NotificationCenter::get()->add_observer(this, "GNMainFormChanged");
   
@@ -107,7 +112,7 @@ WBContextModel::~WBContextModel()
   cmdui->remove_builtin_command("addModelTable");
   cmdui->remove_builtin_command("addModelView");
   cmdui->remove_builtin_command("addModelRoutine");
-
+  cmdui->remove_builtin_command("removeFigure");
   _file= 0;
   delete _overview;
 }
@@ -272,8 +277,12 @@ void WBContextModel::model_created(ModelFile *file, workbench_DocumentRef doc)
   _file= file;
   _doc= doc;
 
+  std::string target_version = _wbui->get_wb()->get_grt_manager()->get_app_option_string("DefaultTargetMySQLVersion");
+  if (target_version.empty())
+    target_version = "5.1.22";
+  
   _wbui->get_wb()->get_component<WBComponentLogical>()->setup_logical_model(doc.get_grt(), _doc);
-  _wbui->get_wb()->get_component<WBComponentPhysical>()->setup_physical_model(doc.get_grt(), _doc, "Mysql", "5.1.22");
+  _wbui->get_wb()->get_component<WBComponentPhysical>()->setup_physical_model(doc.get_grt(), _doc, "Mysql", target_version);
   
   _wbui->get_wb()->foreach_component(boost::bind(&WBComponent::reset_document, _1));
   
@@ -515,7 +524,7 @@ ModelDiagramForm *WBContextModel::get_diagram_form(mdc::CanvasView *view)
 
 void WBContextModel::notify_diagram_created(ModelDiagramForm *view)
 {
-    scoped_connect(view->get_model_diagram()->signal_changed(),
+   view->scoped_connect(view->get_model_diagram()->signal_changed(),
         boost::bind(&WBContextModel::diagram_object_changed, this, _1, _2, view));
     
   // now called from wb_component_physical.cpp:model_list_changed
@@ -553,7 +562,7 @@ void WBContextModel::update_current_diagram(bec::UIForm *form)
     if (diagram.is_valid() && diagram->owner().is_valid())
       diagram->owner()->currentDiagram(diagram);    
     
-    (*_wbui->get_command_ui()->signal_validate_edit_menu_items())();
+    _wbui->get_command_ui()->revalidate_edit_menu_items();
   }
 }
 
@@ -597,6 +606,22 @@ bool WBContextModel::has_selected_schema()
   return false;
 }
 
+bool WBContextModel::has_selected_figures()
+{
+  ModelDiagramForm *view;
+  model_DiagramRef diagram(get_active_model_diagram(false));
+  if (!diagram.is_valid()) // in case an editor in a diagram tab is active
+  {
+    diagram = get_active_model_diagram(true);
+    view = dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  }
+  else
+    view= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_form());
+  if (view && view->has_selection())
+    return true;
+  return false;
+}
+
 bool WBContextModel::has_selected_model()
 {
   if (_wbui->get_active_main_form() == _overview)
@@ -631,6 +656,97 @@ void WBContextModel::add_model_rgroup()
   _overview->get_active_schema_node()->add_new_db_routine(_wbui->get_wb());
 }
 
+void WBContextModel::remove_figure()
+{
+  ModelDiagramForm *view;
+  model_DiagramRef diagram(get_active_model_diagram(false));
+  if (!diagram.is_valid()) // in case an editor in a diagram tab is active
+  {
+    diagram = get_active_model_diagram(true);
+    view = dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  }
+  else
+    view= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_form());
+  if (view)
+    view->remove_selection();
+}
+
+
+GrtObjectRef WBContextModel::duplicate_object(const db_DatabaseObjectRef &object, grt::CopyContext &copy_context)
+{
+  std::set<std::string> skip;
+  skip.insert("oldName");
+  
+  if (object.is_instance(db_Table::static_class_name()))
+  {
+    db_TableRef table(db_TableRef::cast_from(object));
+    
+    // copy table
+    db_TableRef dbtable(db_TableRef::cast_from(copy_context.copy(table, skip)));
+    
+    copy_context.update_references();
+    
+    // post-processing
+    // - uniquefy foreign key names
+    int max_fk_len = workbench_physical_ModelRef::cast_from(dbtable->owner()->owner()->owner())->rdbms()->maximumIdentifierLength();
+    grt::ListRef<db_ForeignKey> fks(dbtable->foreignKeys());
+    std::set<std::string> used_fk_names = bec::SchemaHelper::get_foreign_key_names(db_SchemaRef::cast_from(dbtable->owner()));
+    for (size_t c= fks.count(), i = 0; i < c; i++)
+    {
+      db_ForeignKeyRef fk(fks[i]);
+      
+      fk->name(bec::SchemaHelper::get_unique_foreign_key_name(used_fk_names, fk->name(), max_fk_len));
+    }
+    
+    if (grt::find_named_object_in_list(db_SchemaRef::cast_from(dbtable->owner())->tables(), dbtable->name()).is_valid())
+      dbtable->name(grt::get_name_suggestion_for_list_object(db_SchemaRef::cast_from(dbtable->owner())->tables(),
+                                                             *dbtable->name()+"_copy"));
+    grt::AutoUndo undo(get_grt());
+    
+    db_SchemaRef::cast_from(dbtable->owner())->tables().insert(dbtable);
+    
+    undo.end(strfmt(_("Duplicate Table '%s'"), dbtable->name().c_str()));
+    
+    return dbtable;
+  }
+  else if (object.is_instance(db_View::static_class_name()))
+  {
+    db_ViewRef view(db_ViewRef::cast_from(object));
+    
+    // copy view
+    db_ViewRef dbview(db_ViewRef::cast_from(copy_context.copy(view)));
+    
+    if (grt::find_named_object_in_list(db_SchemaRef::cast_from(dbview->owner())->views(), dbview->name()).is_valid())
+      dbview->name(grt::get_name_suggestion_for_list_object(db_SchemaRef::cast_from(dbview->owner())->views(),
+                                                            *dbview->name()+"_copy"));
+    grt::AutoUndo undo(get_grt());
+    
+    db_SchemaRef::cast_from(dbview->owner())->views().insert(dbview);
+    
+    undo.end(strfmt(_("Duplicate View '%s'"), dbview->name().c_str()));
+
+    return dbview;
+  }
+  else if (object.is_instance(db_RoutineGroup::static_class_name()))
+  {
+    db_RoutineGroupRef routineGroup(db_RoutineGroupRef::cast_from(object));
+    
+    // copy routineGroup
+    db_RoutineGroupRef dbroutineGroup(db_RoutineGroupRef::cast_from(copy_context.copy(routineGroup)));
+    
+    if (grt::find_named_object_in_list(db_SchemaRef::cast_from(dbroutineGroup->owner())->routineGroups(), dbroutineGroup->name()).is_valid())
+      dbroutineGroup->name(grt::get_name_suggestion_for_list_object(db_SchemaRef::cast_from(dbroutineGroup->owner())->routineGroups(),
+                                                                    *dbroutineGroup->name()+"_copy"));
+    grt::AutoUndo undo(get_grt());
+    
+    db_SchemaRef::cast_from(dbroutineGroup->owner())->routineGroups().insert(dbroutineGroup);
+    
+    undo.end(strfmt(_("Duplicate Routine Group '%s'"), dbroutineGroup->name().c_str()));
+    return dbroutineGroup;
+  }
+  return GrtObjectRef();
+}
+
 
 void WBContextModel::update_plugin_arguments_pool(ArgumentPool &args)
 {
@@ -726,6 +842,7 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   bec::MenuItem item;
   bool can_delete= objects->count() > 0;
   bool can_copy= can_delete;
+  bool can_remove= true;
   if (model != NULL)
     for (std::vector<bec::NodeId>::const_iterator iterator= nodes.begin(); iterator != nodes.end(); iterator++)
     {
@@ -753,7 +870,14 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   item.name= "builtin:paste";
   item.type= MenuAction;
   items.push_back(item);
-  
+
+// this is already implemented, but not yet exposed.. uncomment once the whole thing is working
+//  item.enabled= form->can_paste();
+//  item.caption= strfmt(_("Duplicate %s"), wb->get_clipboard()->get_content_description().c_str());
+//  item.name= "builtin:duplicate";
+//  item.type= MenuAction;
+//  items.push_back(item);
+
   item.type= MenuSeparator;
   items.push_back(item);
   
@@ -764,11 +888,16 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   {
     add_object_plugins_to_popup_menu(objects, groups, items);
     
+    bool has_objects = false;
+    bool connections_only = true;
     grt::ListRef<GrtObject> model_objects(wb->get_grt());
     for (size_t c= objects.count(), i= 0; i < c; i++)
     {
+      if (!objects[i].is_instance(model_Connection::static_class_name()))
+        connections_only = false;
       if (objects[i].is_instance(model_Object::static_class_name()))
       {
+        has_objects = true;
         FOREACH_COMPONENT(wb->_components, compo)
         {
           GrtObjectRef model_object((*compo)->get_object_for_figure(model_ObjectRef::cast_from(objects[i])));
@@ -781,9 +910,13 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
         }
       }
     }
-    
+    if (connections_only || !has_objects)
+      can_remove = false;
+
     if (model_objects.count() > 0)
       add_object_plugins_to_popup_menu(model_objects, groups, items);
+    else
+      can_remove = false;
   }
   
   // At the end of the list there is the delete command.
@@ -801,6 +934,11 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   item.type= MenuAction;
   items.push_back(item);
   
+  item.caption= label.empty() ? _("Remove Figure") : strfmt(_("Remove Figure %s"), label.c_str());
+  item.name= "builtin:removeFigure";
+  item.type= MenuAction;
+  item.enabled = can_remove;
+  items.push_back(item);
   return int(items.size() - initial_count);
 }
 
@@ -992,7 +1130,7 @@ void WBContextModel::selection_changed()
       }
     }
   }
-  (*_wbui->get_command_ui()->signal_validate_edit_menu_items())();
+  _wbui->get_command_ui()->revalidate_edit_menu_items();
 }
 
 
@@ -1120,9 +1258,9 @@ void WBContextModel::add_new_diagram(const model_ModelRef &model)
 {
   _wbui->get_wb()->show_status_text(_("Creating Diagram..."));
   
-  model_DiagramRef view(model_DiagramRef::cast_from(_wbui->get_wb()->execute_in_grt_thread("Create new diagram",
-                                                                          boost::bind(&model_Model::addNewDiagram, &model.content(), true))));
-  
+ // model_DiagramRef view(model_DiagramRef::cast_from(_wbui->get_wb()->execute_in_grt_thread("Create new diagram",
+   //                                                                       boost::bind(&model_Model::addNewDiagram, &model.content(), true))));
+  model_DiagramRef view = model->addNewDiagram(true);
   if (view.is_valid())
   {
     view->get_data()->realize();
@@ -1152,51 +1290,29 @@ void WBContextModel::switch_diagram(const model_DiagramRef &view)
  * @param figure to be removed
  ****************************************************************************
  */
-bool WBContextModel::delete_object(model_ObjectRef object, std::map<std::string, bool>& options)
+bool WBContextModel::delete_object(model_ObjectRef object)
 {
   model_DiagramRef view(model_DiagramRef::cast_from(object->owner()));
   
   FOREACH_COMPONENT(_wbui->get_wb()->_components, iter)
   {
     if ((*iter)->handles_figure(object))
-      return (*iter)->delete_model_object(object, options);
+      return (*iter)->delete_model_object(object, false);
   }
   return false;
 }
 
 
-void WBContextModel::delete_canvas_item(mdc::CanvasItem *item)
+bool WBContextModel::remove_figure(model_ObjectRef object)
 {
-  model_DiagramRef view(get_view_with_id(item->get_layer()->get_view()->get_tag()));
+  model_DiagramRef view(model_DiagramRef::cast_from(object->owner()));
   
-  model_FigureRef figure;
-  figure= grt::find_object_in_list(view->figures(), item->get_tag());
-  if (figure.is_valid())
+  FOREACH_COMPONENT(_wbui->get_wb()->_components, iter)
   {
-    std::map<std::string, bool> map;
-    delete_object(figure, map);
-    return;
+    if ((*iter)->handles_figure(object))
+      return (*iter)->delete_model_object(object, true);
   }
-  
-  model_ConnectionRef conn;
-  conn= grt::find_object_in_list(view->connections(), item->get_tag());
-  if (conn.is_valid())
-  {
-    std::map<std::string, bool> map;
-    delete_object(conn, map);
-    return;
-  }
-  
-  model_LayerRef layer;
-  layer= grt::find_object_in_list(view->layers(), item->get_tag());
-  if (layer.is_valid())
-  {
-    std::map<std::string, bool> map;
-    delete_object(layer, map);
-    return;
-  }
-  
-  throw std::logic_error("Could not find GRT object for canvas item");
+  return false;
 }
 
 #endif // Canvas_Objects____
