@@ -65,7 +65,7 @@ namespace wb
     
     void register_diagram_form(ModelDiagramForm *view);
     
-    ModelDiagramForm *get_diagram_form_for_diagram_id(const std::string &id) { return _model_forms[id]; }
+    ModelDiagramForm *get_diagram_form_for_diagram_id(const std::string &id) { return _model_forms.find(id) == _model_forms.end() ? 0 : _model_forms[id]; }
     ModelDiagramForm *get_diagram_form(mdc::CanvasView *view);
     
     void notify_diagram_created(ModelDiagramForm *view);
@@ -91,7 +91,10 @@ namespace wb
     void switch_diagram(const model_DiagramRef &view);
         
     bool delete_diagram(const model_DiagramRef &view);
-    bool delete_object(model_ObjectRef object, std::map<std::string, bool>& options);
+    bool delete_object(model_ObjectRef object);
+    bool remove_figure(model_ObjectRef object);
+    
+    GrtObjectRef duplicate_object(const db_DatabaseObjectRef &object, grt::CopyContext &copy_context);
 
   public:
     void update_plugin_arguments_pool(bec::ArgumentPool &args);
@@ -112,9 +115,7 @@ namespace wb
     virtual void free_canvas_view(mdc::CanvasView *view);
     
     mdc::CanvasView* create_diagram_main(const model_DiagramRef &mview);
-    
-    void delete_canvas_item(mdc::CanvasItem *item);
-    
+        
     void update_current_diagram(bec::UIForm *form);
     
     void diagram_object_changed(const std::string &member, const grt::ValueRef &ovalue, ModelDiagramForm *view);
@@ -123,11 +124,13 @@ namespace wb
     
     bool has_selected_model();
     bool has_selected_schema();
+    bool has_selected_figures();
     void add_model_schema();
     void add_model_table();
     void add_model_view();
     void add_model_rgroup();
     void add_model_diagram();
+    void remove_figure();
 
     void page_settings_changed(const std::string &field, const grt::ValueRef &value);
     
