@@ -44,12 +44,16 @@
 
 #include "wb_physical_model_diagram_features.h"
 #include "base/string_utilities.h"
+#include "base/log.h"
 #include <boost/lambda/bind.hpp>
 
 #include "mforms/menubar.h"
 
 #define DEFAULT_TOOL "basic/select"
 
+
+DEFAULT_LOG_DOMAIN("ModelDiagram");
+
 using namespace wb;
 using namespace bec;
 using namespace base;
@@ -829,8 +833,6 @@ void ModelDiagramForm::handle_mouse_double_click(mdc::MouseButton button, int x,
 {
   stop_editing();
   
-  Point pos(_view->window_to_canvas(x, y));
-  
   if (button != mdc::ButtonLeft && button != mdc::ButtonMiddle)
     return;
 
@@ -866,17 +868,14 @@ bool ModelDiagramForm::handle_key(const mdc::KeyInfo &key, bool press, mdc::Even
       {
         if (iter->command.find("tool:") == 0)
           set_tool(iter->command.substr(strlen("tool:")));
+        else if (iter->command == "zoomin")
+          zoom_in();
+        else if (iter->command == "zoomout")
+          zoom_out();
+        else if (iter->command == "zoomdefault")
+          set_zoom(1);
         else
-        {
-          if (iter->command == "zoomin")
-            zoom_in();
-          else
-            if (iter->command == "zoomout")
-              zoom_out();
-            else
-              if (iter->command == "zoomdefault")
-                set_zoom(1);
-        }
+          get_wb()->get_ui()->get_command_ui()->activate_command(iter->command);
         return true;
       }
     }
@@ -953,7 +952,9 @@ void ModelDiagramForm::set_tool(std::string tool)
         (*item)->set_checked(false);
     }
   }
-  _owner->get_wb()->tool_changed(_view);
+
+  if (_owner->get_wb()->tool_changed)
+    _owner->get_wb()->tool_changed(_view);
 }
 
 
@@ -982,7 +983,7 @@ void ModelDiagramForm::reset_tool(bool notify)
   _handle_motion = (boost::bind(f));
   _reset_tool = (boost::bind(f));
 
-  if (notify)
+  if (notify && _owner->get_wb()->tool_changed)
     _owner->get_wb()->tool_changed(_view);
 }
 
@@ -1051,7 +1052,7 @@ bool ModelDiagramForm::can_select_all()
 
 static void check_if_can_paste(WBComponent *compo, const grt::ObjectRef &object, bool *result)
 {
-  if (compo->can_paste_non_model_object(object))
+  if (compo->can_paste_object(object))
     *result= true;
 }
 
@@ -1069,18 +1070,10 @@ bool ModelDiagramForm::can_paste()
       g_message("copy buffer has null value");
       return false;
     }
-    if (model_ObjectRef::can_wrap(*iter))
-    {
-      if (!wb->get_component_handling(model_ObjectRef::cast_from(*iter)))
-        return false;
-    }
-    else
-    {
-      bool result= false;
-      wb->foreach_component(boost::bind(check_if_can_paste, _1, *iter, &result));
-      if (!result)
-        return false;
-    }
+    bool result= false;
+    wb->foreach_component(boost::bind(check_if_can_paste, _1, *iter, &result));
+    if (!result)
+      return false;
   }
   return !get_clipboard()->empty();
 }
@@ -1132,38 +1125,40 @@ void ModelDiagramForm::cut()
 
   um->begin_undo_group();
   copy();
-  int count= delete_selection(true);
+  
+  int count= get_copiable_selection().count();
+  remove_selection();
   um->end_undo_group();
   um->set_action_description(strfmt(_("Cut %s"), edit_target_name.c_str()));
 
-  _owner->get_wb()->show_status_text(strfmt(_("%i object(s) cut."), count));
+  _owner->get_wb()->show_status_text(strfmt(_("%i figure(s) cut."), count));
 }
 
 
 void ModelDiagramForm::copy()
 {
+  // object must be duplicated on copy because if the object is edited after copy
+  // whatever is pasted should still be in the same state as it was on copy
   grt::ListRef<model_Object> selection(get_copiable_selection());
   bec::Clipboard *clip= get_clipboard();
-  model_LayerRef target_layer(get_model_diagram()->rootLayer());
   int count= 0;
   grt::CopyContext copy_context(_owner->get_grt());
-
+  
   clip->clear();
   for (size_t c= selection.count(), i= 0; i < c; i++)
   {
     WBComponent *compo= _owner->get_wb()->get_component_handling(selection.get(i));
     if (compo)
     {
-      model_ObjectRef copy(compo->clone_object(selection.get(i), target_layer, copy_context, true));
-      clip->append_data(copy);
+      compo->copy_object_to_clipboard(selection.get(i), copy_context);
       count++;
     }
   }
   clip->set_content_description(get_edit_target_name());
-
+  
   copy_context.finish();
   clip->changed();
-
+  
   _owner->get_wb()->show_status_text(strfmt(_("%i object(s) copied."), count));
 }
 
@@ -1178,7 +1173,7 @@ static void get_component_that_can_paste(WBComponent *compo,
                                          const grt::ObjectRef &object, 
                                          WBComponent **result)
 {
-  if (compo->can_paste_non_model_object(object))
+  if (compo->can_paste_object(object))
     *result= compo;
 }
 
@@ -1190,11 +1185,12 @@ void ModelDiagramForm::paste()
 
   WBContext *wb= _owner->get_wb();
   int count= 0;
+  int duplicated = 0;
 
   _paste_offset+= 20;
-
-  _owner->get_wb()->show_status_text(_("Pasting objects..."));
-
+  
+  _owner->get_wb()->show_status_text(_("Pasting figures..."));
+  
   grt::CopyContext context(wb->get_grt());
 
   grt::AutoUndo undo(wb->get_grt());
@@ -1202,55 +1198,47 @@ void ModelDiagramForm::paste()
   _model_diagram->unselectAll();
 
   std::list<grt::ObjectRef> data(get_clipboard()->get_data());
+
+  // find what objects are in layers so that if they are also selected individually, they don't
+  // get copied again
+  std::set<std::string> skip_objects;
   for (std::list<grt::ObjectRef>::iterator iter= data.begin();
-    iter != data.end(); ++iter)
+       iter != data.end(); ++iter)
   {
-    model_LayerRef layer= _model_diagram->rootLayer();
-    if (model_ObjectRef::can_wrap(*iter))
+    if (model_LayerRef::can_wrap(*iter))
     {
-      WBComponent *compo= wb->get_component_handling(model_ObjectRef::cast_from(*iter));
-
-      if (!compo) continue;
-
-      if (model_ObjectRef::can_wrap(*iter))
+      grt::ListRef<model_Figure> figures(model_LayerRef::cast_from(*iter)->figures());
+      GRTLIST_FOREACH(model_Figure, figures, fig)
       {
-        model_ObjectRef object(compo->clone_object(model_ObjectRef::cast_from(*iter), layer, context, false));
-        if (object.is_valid())
-        {
-          if (object.is_instance(model_Figure::static_class_name()))
-          {
-            model_FigureRef figure(model_FigureRef::cast_from(object));
-            figure->top(*figure->top() + _paste_offset);
-            figure->left(*figure->left() + _paste_offset);
-          }
-          else if (object.is_instance(model_Layer::static_class_name()))
-          {
-            model_LayerRef layer(model_LayerRef::cast_from(object));
-            layer->top(*layer->top() + _paste_offset);
-            layer->left(*layer->left() + _paste_offset);
-          }
-          count++;
-
-          _model_diagram->selectObject(object);
-        }
+        skip_objects.insert((*fig)->id());
       }
     }
-    else
+  }
+
+  for (std::list<grt::ObjectRef>::iterator iter= data.begin();
+    iter != data.end(); ++iter)
+  {    
+    // paste the object
+    WBComponent *compo= 0;
+    wb->foreach_component(boost::bind(get_component_that_can_paste, _1, *iter, &compo));
+    if (compo)
     {
-      WBComponent *compo= 0;
-      wb->foreach_component(
-        boost::bind(get_component_that_can_paste, _1, *iter, &compo)); 
-      if (compo)
+      if (skip_objects.find((*iter)->id()) == skip_objects.end())
       {
-        compo->paste_non_model_object(this, *iter);
+        compo->paste_object(this, *iter, context);
         count++;
       }
     }
+    else
+      log_warning("Don't know how to paste %s\n", iter->class_name().c_str());
   }
   context.finish();
   undo.end(strfmt(_("Paste %s"), get_clipboard()->get_content_description().c_str()));
 
-  _owner->get_wb()->show_status_text(strfmt(_("%i object(s) pasted."), count));
+  if (duplicated == 0)
+    _owner->get_wb()->show_status_text(strfmt(_("%i figure(s) pasted."), count));
+  else
+    _owner->get_wb()->show_status_text(strfmt(_("%i figure(s) pasted, %i duplicated."), count, duplicated));
 }
 
 
@@ -1264,14 +1252,34 @@ void ModelDiagramForm::select_all()
 }
 
 
-void ModelDiagramForm::delete_selection()
+void ModelDiagramForm::remove_selection()
 {
-  int count= delete_selection(false);
-  _owner->get_wb()->show_status_text(strfmt(_("%i object(s) deleted."), count));
+  grt::UndoManager *um= _owner->get_grt()->get_undo_manager();
+  grt::ListRef<model_Object> selection= get_selection();
+  
+  std::vector<model_ObjectRef> objects;
+  std::string edit_target_name= get_edit_target_name();
+  
+  um->begin_undo_group();
+  
+  for (size_t c= selection.count(), i= 0; i < c; i++)
+  {
+    if (selection.get(i).is_instance(model_Object::static_class_name()))
+      objects.push_back(model_ObjectRef::cast_from(selection.get(i)));
+  }
+  
+  for (size_t c= objects.size(), i= 0; i < c; i++)
+    _owner->get_wb()->get_model_context()->remove_figure(objects[i]);
+  
+  um->end_undo_group();
+  um->set_action_description(strfmt(_("Remove %s"), edit_target_name.c_str()));
+
+  _owner->get_wb()->show_status_text(strfmt(_("%i figure(s) removed. The corresponding DB objects were kept."), (int)objects.size()));
 }
 
 
-int ModelDiagramForm::delete_selection(bool noquestions)
+//XXX unused? but it's virtual, need to check if this is used anywhere...
+void ModelDiagramForm::delete_selection()
 {
   grt::UndoManager *um= _owner->get_grt()->get_undo_manager();
   grt::ListRef<model_Object> selection= get_selection();
@@ -1287,27 +1295,13 @@ int ModelDiagramForm::delete_selection(bool noquestions)
       objects.push_back(model_ObjectRef::cast_from(selection.get(i)));
   }
 
-  std::map<std::string, bool> options;
-
-  if (noquestions)
-    options["deletedbobjects"]= true;
-  else
-  {
-    std::string option;
-
-    option= _owner->get_wb()->get_wb_options().get_string("workbench.physical:DeleteObjectConfirmation", "ask");
-    if (option == "delete")
-      options["deletedbobjects"]= true;
-    else if (option == "keep")
-      options["deletedbobjects"]= false;
-  }
   for (size_t c= objects.size(), i= 0; i < c; i++)
-    _owner->get_wb()->get_model_context()->delete_object(objects[i], options);
+    _owner->get_wb()->get_model_context()->delete_object(objects[i]);
 
   um->end_undo_group();
   um->set_action_description(strfmt(_("Delete %s"), edit_target_name.c_str()));
 
-  return objects.size();
+  _owner->get_wb()->show_status_text(strfmt(_("%i object(s) deleted."), (int)objects.size()));
 }
 
 
