@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -42,6 +42,7 @@
 #include "base/string_utilities.h"
 
 #include "mforms/toolbar.h"
+#include "mforms/menubar.h"
 
 #include "base/trackable.h"
 
@@ -507,9 +508,9 @@ bool PhysicalOverviewBE::can_close()
   return _wb->can_close_document();
 }
 
-bool PhysicalOverviewBE::close()
+void PhysicalOverviewBE::close()
 {
-  return _wb->close_document();
+  _wb->close_document();
 }
 
 std::string PhysicalOverviewBE::identifier() const
