@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -69,7 +69,7 @@ bool SchemaObjectNode::is_renameable()
 
 void SchemaObjectNode::copy_object(WBContext *wb, bec::Clipboard *clip)
 {
-  clip->append_data(grt::copy_object(wb->get_grt_manager()->get_grt(), object));
+  clip->append_data(grt::copy_object(object));
   clip->set_content_description(label);
 }
 
