@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,14 +48,14 @@ using namespace base;
 
 class PrivilegeObjectNode : public OverviewBE::ObjectNode
 {  
-  sigc::connection _changed_conn;
+  boost::signals2::connection _changed_conn;
 public:
-  PrivilegeObjectNode(GrtObjectRef o, const sigc::slot<void,const std::string&,const grt::ValueRef&> &refresh_slot) 
+  PrivilegeObjectNode(GrtObjectRef o, const boost::function<void (const std::string&,const grt::ValueRef&)> &refresh_slot) 
   {
     object= o; 
     
     if (refresh_slot)
-      _changed_conn = o->signal_changed().connect(refresh_slot);
+      _changed_conn = o->signal_changed()->connect(refresh_slot);
   }
   
   virtual ~PrivilegeObjectNode()
@@ -63,7 +63,7 @@ public:
     _changed_conn.disconnect();
   }
 
-  sigc::slot<void, WBComponentPhysical*> remove;
+  boost::function<void (WBComponentPhysical*)> remove;
 
   virtual void delete_object(WBContext *wb) 
   {
@@ -76,7 +76,7 @@ public:
   virtual void copy_object(WBContext *wb, bec::Clipboard *clip)
   {
     clip->clear();
-    clip->append_data(grt::copy_object(wb->get_grt(), object));
+    clip->append_data(grt::copy_object(object));
     clip->set_content_description(label);
   }
 
@@ -113,11 +113,11 @@ class UserListNode : public OverviewBE::ContainerNode
 private:
   std::string id;
   grt::ListRef<GrtNamedObject> _list;
-  sigc::slot<void, WBComponentPhysical*, db_UserRef> _remove;
+  boost::function<void (WBComponentPhysical*, db_UserRef)> _remove;
   PhysicalOverviewBE *_owner;
 public:
   UserListNode(const std::string &name, const db_CatalogRef &catalog, grt::ListRef<GrtNamedObject> list,
-    const sigc::slot<void, WBComponentPhysical*, db_UserRef> &remove, PhysicalOverviewBE *owner)
+    const boost::function<void (WBComponentPhysical*, db_UserRef)> &remove, PhysicalOverviewBE *owner)
     : ContainerNode(OverviewBE::OItem), _list(list), _remove(remove), _owner(owner)
   {
     id= catalog->id() + "/" + name;
@@ -151,13 +151,13 @@ public:
     
     for (size_t c= _list.count(), i= 0; i < c; i++)
     {
-      PrivilegeObjectNode *node= new PrivilegeObjectNode(_list[i], sigc::mem_fun(this, &UserListNode::refresh));
+      PrivilegeObjectNode *node= new PrivilegeObjectNode(_list[i], boost::bind(&UserListNode::refresh, this, _1, _2));
       
       node->type= OverviewBE::OItem;
       node->label= _list[i]->name();
       node->small_icon= IconManager::get_instance()->get_icon_id(_list[i]->get_metaclass(), Icon16);
       node->large_icon= IconManager::get_instance()->get_icon_id(_list[i]->get_metaclass(), Icon48);
-      node->remove= sigc::bind<db_UserRef>(_remove, db_UserRef::cast_from(_list[i]));
+      node->remove= boost::bind(_remove, _1, db_UserRef::cast_from(_list[i]));
 
       children.push_back(node);
     }
@@ -174,11 +174,11 @@ class RoleListNode : public OverviewBE::ContainerNode
 private:
   std::string id;
   grt::ListRef<GrtNamedObject> _list;
-  sigc::slot<void, WBComponentPhysical*, db_RoleRef> _remove;
+  boost::function<void (WBComponentPhysical*, db_RoleRef)> _remove;
   PhysicalOverviewBE *_owner;
 public:
   RoleListNode(const std::string &name, const db_CatalogRef &catalog, grt::ListRef<GrtNamedObject> list,
-    const sigc::slot<void, WBComponentPhysical*, db_RoleRef> &remove, PhysicalOverviewBE *owner)
+    const boost::function<void (WBComponentPhysical*, db_RoleRef)> &remove, PhysicalOverviewBE *owner)
     : ContainerNode(OverviewBE::OItem), _list(list), _remove(remove), _owner(owner)
   {
     id= catalog->id() + "/" + name;
@@ -213,13 +213,13 @@ public:
     for (size_t c= _list.count(), i= 0; i < c; i++)
     {
       PrivilegeObjectNode *node= new PrivilegeObjectNode(_list[i], 
-                                                         sigc::mem_fun(this, &RoleListNode::refresh));
+                                                         boost::bind(&RoleListNode::refresh, this, _1, _2));
       
       node->type= OverviewBE::OItem;
       node->label= _list[i]->name();
       node->small_icon= IconManager::get_instance()->get_icon_id(_list[i]->get_metaclass(), Icon16);
       node->large_icon= IconManager::get_instance()->get_icon_id(_list[i]->get_metaclass(), Icon48);
-      node->remove= sigc::bind<db_RoleRef>(_remove, db_RoleRef::cast_from(_list[i]));
+      node->remove= boost::bind(_remove, _1, db_RoleRef::cast_from(_list[i]));
 
       children.push_back(node);
     }
@@ -245,7 +245,7 @@ void wb::internal::PrivilegeInfoNode::paste_object(WBContext *wb, bec::Clipboard
   {
     if ((*iter).is_instance(db_User::static_class_name()))
     {
-      db_UserRef dbuser(db_UserRef::cast_from(grt::copy_object(wb->get_grt(), *iter)));
+      db_UserRef dbuser(db_UserRef::cast_from(grt::copy_object(*iter)));
 
       grt::AutoUndo undo(wb->get_grt());
       
@@ -258,7 +258,7 @@ void wb::internal::PrivilegeInfoNode::paste_object(WBContext *wb, bec::Clipboard
     }
     else if ((*iter).is_instance(db_Role::static_class_name()))
     {
-      db_RoleRef dbrole(db_RoleRef::cast_from(grt::copy_object(wb->get_grt(), *iter)));
+      db_RoleRef dbrole(db_RoleRef::cast_from(grt::copy_object(*iter)));
 
       grt::AutoUndo undo(wb->get_grt());
 
@@ -311,7 +311,7 @@ wb::internal::PrivilegeInfoNode::PrivilegeInfoNode(const db_CatalogRef &catalog,
   description= "Privileges";
   display_mode= OverviewBE::MSmallIcon;
 
-  OverviewBE::AddObjectNode *add_node= new OverviewBE::AddObjectNode(sigc::mem_fun(this, &PrivilegeInfoNode::add_new_user));
+  OverviewBE::AddObjectNode *add_node= new OverviewBE::AddObjectNode(boost::bind(&PrivilegeInfoNode::add_new_user, this, _1));
   add_node->label= _("Add User");
   add_node->type= OverviewBE::OItem;
   add_node->small_icon= IconManager::get_instance()->get_icon_id("db.User.$.png", Icon16, "add");
@@ -320,13 +320,13 @@ wb::internal::PrivilegeInfoNode::PrivilegeInfoNode(const db_CatalogRef &catalog,
   {
     UserListNode *node;
     node= new UserListNode("Users", catalog, grt::ListRef<GrtNamedObject>::cast_from(catalog->users()),
-      sigc::mem_fun(&WBComponentPhysical::remove_user), owner);
+      boost::bind(&WBComponentPhysical::remove_user, _1, _2), owner);
     children.push_back(node);
     node->children.insert(node->children.begin(), add_node);
   }
 
 
-  add_node= new OverviewBE::AddObjectNode(sigc::mem_fun(this, &PrivilegeInfoNode::add_new_role));
+  add_node= new OverviewBE::AddObjectNode(boost::bind(&PrivilegeInfoNode::add_new_role, this, _1));
   add_node->label= _("Add Role");
   add_node->type= OverviewBE::OItem;
   add_node->small_icon= IconManager::get_instance()->get_icon_id("db.Role.$.png", Icon16, "add");
@@ -334,7 +334,7 @@ wb::internal::PrivilegeInfoNode::PrivilegeInfoNode(const db_CatalogRef &catalog,
 
   {
     RoleListNode *node= new RoleListNode("Roles", catalog, grt::ListRef<GrtNamedObject>::cast_from(catalog->roles()),
-      sigc::mem_fun(&WBComponentPhysical::remove_role), owner);
+      boost::bind(&WBComponentPhysical::remove_role, _1, _2), owner);
     children.push_back(node);
 
     node->children.insert(node->children.begin(), add_node);
