@@ -37,6 +37,7 @@
 #include "grts/structs.db.query.h"
 #include "base/trackable.h"
 
+#include "grtdb/db_object_helpers.h"
 #include "base/log.h"
 
 #include "mforms/code_editor.h"
@@ -701,7 +702,7 @@ static void *connect_editor(SqlEditorForm::Ref editor)
   catch (sql::AuthenticationError &exc)
   {
     log_error("Got an authentication error during connection: %s\n", exc.what());
-    return new sql::AuthenticationError(exc);
+    return new std::string(exc.what());
   }
   catch (std::exception &exc)
   {
@@ -740,6 +741,17 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
   {
     std::string tmp(*result);
     delete result;
+    
+    if (tmp == ":PASSWORD_EXPIRED")
+    {
+      grt::BaseListRef args(conn->get_grt(), grt::AnyType);
+      args.ginsert(conn);
+      int result = *grt::IntegerRef::cast_from(conn->get_grt()->call_module_function("WbAdmin", "handleExpiredPassword", args));
+      if (result)
+        return create_connected_editor(conn);
+      throw grt::user_cancelled("password reset cancelled by user");
+    }
+    
     throw std::runtime_error(tmp);
   }
   
@@ -755,14 +767,9 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
 
     {
       std::map<std::string, std::string> details(editor->connection_details());
-      GrtVersionRef version(object.get_grt());
+      GrtVersionRef version = CatalogHelper::parse_version(object.get_grt(), details["dbmsProductVersion"]);
       version->owner(object);
       version->name(grt::StringRef(details["dbmsProductName"]));
-      int major = 0, minor = 0, release = 0;
-      sscanf(details["dbmsProductVersion"].c_str(), "%i.%i.%i", &major, &minor, &release);
-      version->majorNumber(grt::IntegerRef(major));
-      version->minorNumber(grt::IntegerRef(minor));
-      version->releaseNumber(grt::IntegerRef(release));
       object->serverVersion(version);
     }
 
