@@ -68,7 +68,6 @@ const int LiveSchemaTree::TABLE_INDEXES_NODE_INDEX = 1;
 const int LiveSchemaTree::TABLE_FOREIGN_KEYS_NODE_INDEX = 2;
 const int LiveSchemaTree::TABLE_TRIGGERS_NODE_INDEX = 3;
 
-
 const char* LiveSchemaTree::_schema_tokens[15] = {"",                                                           // Empty item to use 0 as not found
                                                   "CASCADE", "SET NULL", "SET DEFAULT", "RESTRICT", "NO ACTION", // The update/delete rules on foreign keys
                                                   "BTREE", "FULLTEXT", "HASH", "RTREE",                          // The index types
@@ -335,7 +334,75 @@ LiveSchemaTree::LiveSchemaTree(grt::GRT* grt)
 : _grt(grt), _model_view(NULL), _case_sensitive_identifiers(false), 
 _is_schema_contents_enabled(true), _enabled_events(false), _base(0), _filter_type(Any), _schema_pattern(0), _object_pattern(0)
 {
+  fill_node_icons();
+  
   compare_func = boost::bind(&LiveSchemaTree::compare_identifiers, this, _1, _2);
+  
+  // Setup the schema node collection skeleton
+  mforms::TreeNodeCollectionSkeleton schema_nodes(_icon_paths[Schema]);
+  
+  mforms::TreeNodeSkeleton tables(TABLES_CAPTION, _icon_paths[TableCollection], TABLES_TAG);
+  schema_nodes.children.push_back(tables);
+
+  mforms::TreeNodeSkeleton views(VIEWS_CAPTION, _icon_paths[ViewCollection], VIEWS_TAG);
+  schema_nodes.children.push_back(views);
+
+  mforms::TreeNodeSkeleton routines(ROUTINES_CAPTION, _icon_paths[RoutineCollection], ROUTINES_TAG);
+  schema_nodes.children.push_back(routines);
+  
+  _node_collections[Schema] = schema_nodes;
+  
+  // Setup the table node collection skeleton
+  mforms::TreeNodeCollectionSkeleton table_nodes(_icon_paths[Table]);
+
+  mforms::TreeNodeSkeleton columns(COLUMNS_CAPTION, _icon_paths[ColumnCollection], COLUMNS_TAG);
+  table_nodes.children.push_back(columns);
+
+  mforms::TreeNodeSkeleton indexes(INDEXES_CAPTION, _icon_paths[IndexCollection], INDEXES_TAG);
+  table_nodes.children.push_back(indexes);
+
+  mforms::TreeNodeSkeleton foreign_keys(FOREIGN_KEYS_CAPTION, _icon_paths[ForeignKeyCollection], FOREIGN_KEYS_TAG);
+  mforms::TreeNodeSkeleton fetching_fk(FETCHING_CAPTION, _icon_paths[ForeignKey], "");
+  foreign_keys.children.push_back(fetching_fk);
+  table_nodes.children.push_back(foreign_keys);
+
+  mforms::TreeNodeSkeleton triggers(TRIGGERS_CAPTION, _icon_paths[TriggerCollection], TRIGGERS_TAG);
+  mforms::TreeNodeSkeleton fetching_trigger(FETCHING_CAPTION, _icon_paths[Trigger], "");
+  triggers.children.push_back(fetching_trigger);
+  table_nodes.children.push_back(triggers);
+  
+  _node_collections[Table] = table_nodes;
+
+  // Setup the view node collection skeleton
+  mforms::TreeNodeCollectionSkeleton view_nodes(_icon_paths[View]);
+  mforms::TreeNodeSkeleton fetching_view(FETCHING_CAPTION, _icon_paths[View], "");
+  view_nodes.children.push_back(fetching_view);
+  _node_collections[View] = view_nodes;
+  
+  mforms::TreeNodeCollectionSkeleton procedure_nodes(_icon_paths[Procedure]);
+  _node_collections[Procedure] = procedure_nodes;
+  
+  mforms::TreeNodeCollectionSkeleton function_nodes(_icon_paths[Function]);
+  _node_collections[Function] = function_nodes;
+  
+
+  mforms::TreeNodeCollectionSkeleton table_column_nodes(_icon_paths[TableColumn]);
+  _node_collections[TableColumn] = table_column_nodes;
+
+  mforms::TreeNodeCollectionSkeleton view_column_nodes(_icon_paths[ViewColumn]);
+  _node_collections[ViewColumn] = view_column_nodes;
+
+
+  mforms::TreeNodeCollectionSkeleton index_nodes(_icon_paths[Index]);
+  _node_collections[Index] = index_nodes;
+
+
+  mforms::TreeNodeCollectionSkeleton trigger_nodes(_icon_paths[Trigger]);
+  _node_collections[Trigger] = trigger_nodes;
+
+
+  mforms::TreeNodeCollectionSkeleton fk_nodes(_icon_paths[ForeignKey]);
+  _node_collections[ForeignKey] = fk_nodes;
 }
 
 LiveSchemaTree::~LiveSchemaTree()
@@ -382,18 +449,18 @@ bool LiveSchemaTree::identifiers_equal(const std::string &a, const std::string &
   return g_strcasecmp(a.c_str(), b.c_str()) == 0;
 }
 
-void LiveSchemaTree::setup_node(mforms::TreeNodeRef node, ObjectType type, mforms::TreeNodeData* pdata)
+void LiveSchemaTree::setup_node(mforms::TreeNodeRef node, ObjectType type, mforms::TreeNodeData* pdata, bool ignore_null_data)
 {
   switch(type)
   {
   case Schema:
-    setup_schema_node(node, pdata);
+    node->set_data(pdata ? pdata : new SchemaData());
     break;
   case Table:
-    setup_table_node(node, pdata);
+    node->set_data(pdata ? pdata : new TableData());
     break;
   case View:
-    setup_view_node(node, pdata);
+    node->set_data(pdata ? pdata : new ViewData());
     break;
   case Procedure:
     node->set_data(pdata ? pdata : new ProcedureData());
@@ -402,6 +469,9 @@ void LiveSchemaTree::setup_node(mforms::TreeNodeRef node, ObjectType type, mform
     node->set_data(pdata ? pdata : new FunctionData());
     break;
   case ViewColumn:
+    if (pdata || !ignore_null_data)
+      node->set_data(pdata ? pdata : new ColumnData(type));
+    break;
   case TableColumn:
     node->set_data(pdata ? pdata : new ColumnData(type));
     break;
@@ -409,21 +479,46 @@ void LiveSchemaTree::setup_node(mforms::TreeNodeRef node, ObjectType type, mform
     node->set_data(pdata ? pdata : new IndexData());
     break;
   case Trigger:
-    node->set_data(pdata ? pdata : new TriggerData());
+    if (pdata || !ignore_null_data)
+      node->set_data(pdata ? pdata : new TriggerData());
     break;
   case ForeignKey:
-    node->set_data(pdata ? pdata : new FKData());
+    if (pdata || !ignore_null_data)
+      node->set_data(pdata ? pdata : new FKData());
     break;
   default:
     break;
   }
-
-  // Configures the icon on the node
-  set_node_icon(node, type);
 }
 
+void LiveSchemaTree::fill_node_icons()
+{
+  _icon_paths[Schema] = get_node_icon_path(Schema);
+  _icon_paths[TableCollection] = get_node_icon_path(TableCollection);
+  _icon_paths[ViewCollection] = get_node_icon_path(ViewCollection);
+  _icon_paths[RoutineCollection] = get_node_icon_path(RoutineCollection);
+  _icon_paths[Table] = get_node_icon_path(Table);
+  _icon_paths[View] = get_node_icon_path(View);
+  _icon_paths[Procedure] = get_node_icon_path(Procedure);
+  _icon_paths[Function] = get_node_icon_path(Function);
+  _icon_paths[ColumnCollection] = get_node_icon_path(ColumnCollection);
+  _icon_paths[IndexCollection] = get_node_icon_path(IndexCollection);
+  _icon_paths[ForeignKeyCollection] = get_node_icon_path(ForeignKeyCollection);
+  _icon_paths[TriggerCollection] = get_node_icon_path(TriggerCollection);
+  _icon_paths[ViewColumn] = get_node_icon_path(ViewColumn);
+  _icon_paths[TableColumn] = get_node_icon_path(TableColumn);
+  _icon_paths[Index] = get_node_icon_path(Index);
+  _icon_paths[ForeignKey] = get_node_icon_path(ForeignKey);
+  _icon_paths[Trigger] = get_node_icon_path(Trigger);
+}
 
-void LiveSchemaTree::set_node_icon(mforms::TreeNodeRef node, ObjectType type)
+std::string LiveSchemaTree::get_node_icon_path(ObjectType type)
+{
+  bec::IconId icon = get_node_icon(type);
+  return bec::IconManager::get_instance()->get_icon_file(icon);
+}
+
+bec::IconId LiveSchemaTree::get_node_icon(ObjectType type)
 {
   std::string suffix;
   bec::IconId icon;
@@ -437,15 +532,7 @@ void LiveSchemaTree::set_node_icon(mforms::TreeNodeRef node, ObjectType type)
   switch(type)
   {
   case Schema:
-    {
-      SchemaData *pdata = dynamic_cast<SchemaData*> (node->get_data());
-      if (pdata->fetched)
-        icon = bec::IconManager::get_instance()->get_icon_id("db.Schema.side." + suffix, bec::Icon16);
-      else if (pdata->fetching)
-        icon = bec::IconManager::get_instance()->get_icon_id("db.Schema.loading.side." + suffix, bec::Icon16);
-      else
-        icon = bec::IconManager::get_instance()->get_icon_id("db.Schema.unloaded.side." + suffix, bec::Icon16);
-    }
+    icon = bec::IconManager::get_instance()->get_icon_id("db.Schema.unloaded.side." + suffix, bec::Icon16);
     break;
   case TableCollection:
     icon = bec::IconManager::get_instance()->get_icon_id("db.Table.many.side." + suffix, bec::Icon16);
@@ -456,21 +543,12 @@ void LiveSchemaTree::set_node_icon(mforms::TreeNodeRef node, ObjectType type)
   case RoutineCollection:
     icon = bec::IconManager::get_instance()->get_icon_id("db.Routine.many.side." + suffix, bec::Icon16);
     break;
-
   case Table:
     icon = bec::IconManager::get_instance()->get_icon_id("db.Table.side." + suffix, bec::Icon16);
     break;
   case View:
-    {
-      ViewData *pdata = dynamic_cast<ViewData*> (node->get_data());
-
-      if (pdata->columns_load_error)
-        icon = bec::IconManager::get_instance()->get_icon_id("db.View.broken.side." + suffix, bec::Icon16);
-      else
-        icon = bec::IconManager::get_instance()->get_icon_id("db.View.side." + suffix, bec::Icon16);
-    }
+    icon = bec::IconManager::get_instance()->get_icon_id("db.View.side." + suffix, bec::Icon16);
     break;
-  // TODO: Update to use the proper icon names on Procedures and Functions
   case Procedure:
     icon = bec::IconManager::get_instance()->get_icon_id("db.Routine.side." + suffix, bec::Icon16);
     break;
@@ -489,21 +567,11 @@ void LiveSchemaTree::set_node_icon(mforms::TreeNodeRef node, ObjectType type)
   case TriggerCollection:
     icon = bec::IconManager::get_instance()->get_icon_id("db.Trigger.many.side." + suffix, bec::Icon16);
     break;
-
   case ViewColumn:
     icon = bec::IconManager::get_instance()->get_icon_id("db.Column.side." + suffix, bec::Icon16);
     break;
   case TableColumn:
-    {
-      ColumnData *pdata = dynamic_cast<ColumnData*> (node->get_data());
-
-      if (pdata->is_pk)
-        icon = bec::IconManager::get_instance()->get_icon_id("db.Column.pk.side." + suffix, bec::Icon16);
-      else if (pdata->is_fk)
-        icon = bec::IconManager::get_instance()->get_icon_id("db.Column.fk.side." + suffix, bec::Icon16);
-      else
-        icon = bec::IconManager::get_instance()->get_icon_id("db.Column.side." + suffix, bec::Icon16);
-    }
+    icon = bec::IconManager::get_instance()->get_icon_id("db.Column.side." + suffix, bec::Icon16);
     break;
   case Index:
     icon = bec::IconManager::get_instance()->get_icon_id("db.Index.side." + suffix, bec::Icon16);
@@ -518,85 +586,103 @@ void LiveSchemaTree::set_node_icon(mforms::TreeNodeRef node, ObjectType type)
     break;
   }
   
-  node->set_icon_path(0, bec::IconManager::get_instance()->get_icon_file(icon));
+  return icon;
 }
 
-void LiveSchemaTree::setup_schema_node(mforms::TreeNodeRef node, mforms::TreeNodeData* pdata)
+void LiveSchemaTree::update_node_icon(mforms::TreeNodeRef node)
 {
-  node->set_data(pdata ? pdata : new SchemaData());
+  std::string suffix;
+  bec::IconId icon = 0;
 
-  mforms::TreeNodeRef child_node;
+#ifdef _WIN32
+  suffix = "win.$.png";
+#else
+  suffix = "$.png";
+#endif
 
-  // Creates the tables node...
-  child_node = node->add_child();
-  child_node->set_string(0, TABLES_CAPTION);
-  child_node->set_tag(TABLES_TAG);
-  set_node_icon(child_node, TableCollection);
+  LSTData* pnode_data = dynamic_cast<LSTData*> (node->get_data());
+  if (pnode_data)
+  {
+    switch(pnode_data->get_type())
+    {
+      case Schema:
+        {
+          SchemaData *pdata = dynamic_cast<SchemaData*> (node->get_data());
+          if (pdata->fetched)
+            icon = bec::IconManager::get_instance()->get_icon_id("db.Schema.side." + suffix, bec::Icon16);
+          else if (pdata->fetching)
+            icon = bec::IconManager::get_instance()->get_icon_id("db.Schema.loading.side." + suffix, bec::Icon16);
+        }
+        break;
 
-  // Creates the views node...
-  child_node = node->add_child();
-  child_node->set_string(0, VIEWS_CAPTION);
-  child_node->set_tag(VIEWS_TAG);
-  set_node_icon(child_node, ViewCollection);
-  
-  // Creates the routines node...
-  child_node = node->add_child();
-  child_node->set_string(0, ROUTINES_CAPTION);
-  child_node->set_tag(ROUTINES_TAG);
-  set_node_icon(child_node, RoutineCollection);
+      case View:
+        {
+          ViewData *pdata = dynamic_cast<ViewData*> (node->get_data());
+
+          if (pdata->columns_load_error)
+            icon = bec::IconManager::get_instance()->get_icon_id("db.View.broken.side." + suffix, bec::Icon16);
+        }
+        break;
+      case TableColumn:
+        {
+          ColumnData *pdata = dynamic_cast<ColumnData*> (node->get_data());
+
+          if (pdata->is_pk)
+            icon = bec::IconManager::get_instance()->get_icon_id("db.Column.pk.side." + suffix, bec::Icon16);
+          else if (pdata->is_fk)
+            icon = bec::IconManager::get_instance()->get_icon_id("db.Column.fk.side." + suffix, bec::Icon16);
+        }
+        break;
+      // No other item changes icon
+      default:
+        break;
+    }
+    
+    if (icon)
+      node->set_icon_path(0, bec::IconManager::get_instance()->get_icon_file(icon));
+  }
 }
 
-void LiveSchemaTree::setup_view_node(mforms::TreeNodeRef node, mforms::TreeNodeData* pdata)
+/*
+ * Function: update_change_data
+ * Description: Uses the received information to
+ *              - Generate the list of nodes to be removed
+ *              - Purge children to let there only the items to be added
+ * Parameters:
+ *   parent: the tree node for which the children list will be updated
+ *   children: the list of names that will be used on the process
+ *   type: the type of children that will be affected (some nodes may have children of different types)
+ *   to_remove: a vector containing the nodes to be removed from the parent node
+ */
+void LiveSchemaTree::update_change_data(mforms::TreeNodeRef parent, std::list<std::string>& children, ObjectType type, std::vector<mforms::TreeNodeRef>& to_remove)
 {
-  node->set_data(pdata ? pdata : new ViewData());
+  mforms::TreeNodeRef node;
 
-  mforms::TreeNodeRef child_node;
+    int total_nodes = parent->count();
+    if (total_nodes == 1 && parent->get_child(0)->get_string(0) == FETCHING_CAPTION)
+      to_remove.push_back(parent->get_child(0));
+    else
+    {
+      for(int index = 0; index < total_nodes; index++)
+      {
+        node = parent->get_child(index);
+        LSTData *pchild_data = dynamic_cast<LSTData*>(node->get_data());
+      
+        // Ensure only items of the same type are analyzed
+        if ( pchild_data && pchild_data->get_type() == type)
+        {
+          std::list<std::string>::iterator found_child = std::find(children.begin(), children.end(), node->get_string(0));
 
-  // Creates the tables node...
-  child_node = node->add_child();
-  child_node->set_string(0, FETCHING_CAPTION);
-  setup_node(child_node, ViewColumn);
-}
-
-void LiveSchemaTree::setup_table_node(mforms::TreeNodeRef node, mforms::TreeNodeData* pdata)
-{
-  node->set_data(pdata ? pdata : new TableData());
-
-  mforms::TreeNodeRef child_node;
-
-  // Creates the columns node...
-  child_node = node->add_child();
-  child_node->set_string(0, COLUMNS_CAPTION);
-  child_node->set_tag(COLUMNS_TAG);
-  set_node_icon(child_node, ColumnCollection);
-
-  // Creates the indexes node...
-  child_node = node->add_child();
-  child_node->set_string(0, INDEXES_CAPTION);
-  child_node->set_tag(INDEXES_TAG);
-  set_node_icon(child_node, IndexCollection);
-
-  // Creates the foreign keys node...
-  child_node = node->add_child();
-  child_node->set_string(0, FOREIGN_KEYS_CAPTION);
-  child_node->set_tag(FOREIGN_KEYS_TAG);
-  set_node_icon(child_node, ForeignKeyCollection);
-
-  // Adds the loading... node
-  child_node = child_node->add_child();
-  child_node->set_string(0, FETCHING_CAPTION);
-  setup_node(child_node, ForeignKey);
-
-  // Creates the triggers node...
-  child_node = node->add_child();
-  child_node->set_string(0, TRIGGERS_CAPTION);
-  child_node->set_tag(TRIGGERS_TAG);
-  set_node_icon(child_node, TriggerCollection);
-
-  // Adds the loading... node
-  child_node = child_node->add_child();
-  child_node->set_string(0, FETCHING_CAPTION);
-  setup_node(child_node, Trigger);
+          // If not found, the item will be removed
+          if ( found_child == children.end())
+            to_remove.push_back(node);
+          // If found, the item is removed from the incoming list
+          // to let only nodes to be added
+          else
+            children.erase(found_child);
+        }
+      }
+    }
 }
 
 /*
@@ -613,6 +699,8 @@ void LiveSchemaTree::setup_table_node(mforms::TreeNodeRef node, mforms::TreeNode
  *   just_append: suppresses the logic of removing nodes (i.e. when searching nodes with different filters
  *                they are only appended on the base tree)
  * Return Value: a boolean value indicating whether the children list changed (nodes were added or removed)
+ * 
+ * NOTE : That children may change, so if the original list is needed, the caller needs to have a copy
  */
 bool LiveSchemaTree::update_node_children(mforms::TreeNodeRef parent, std::list<std::string>& children, ObjectType type, bool sorted, bool just_append)
 {
@@ -638,32 +726,13 @@ bool LiveSchemaTree::update_node_children(mforms::TreeNodeRef parent, std::list<
     mforms::TreeNodeRef node;
     bool added = false;
     bool removed = false;
-    std::set<std::string> existing_children;
     std::vector<mforms::TreeNodeRef> childs_to_remove;
-    std::list<std::string>::const_iterator index, end = children.end();
     
     _model_view->freeze_refresh();
     
-    // Sorts the children if needed
-    if (sorted)
-      children.sort(compare_func);
+    // Calculates the nodes to be removed and the new nodes to be created
+    update_change_data(parent, children, type, childs_to_remove);
     
-    // Gets from the parent the list of childs matching the specified type
-    int total_nodes = parent->count();
-    for(int index = 0; index < total_nodes; index++)
-    {
-      node = parent->get_child(index);
-      LSTData *pchild_data = dynamic_cast<LSTData*>(node->get_data());
-      
-      if ( pchild_data && pchild_data->get_type() == type)
-      {
-        if (std::find(children.begin(), children.end(), node->get_string(0)) == children.end())
-          childs_to_remove.push_back(node);
-        else
-          existing_children.insert(node->get_string(0));
-      }
-    }
-
     // Removes deleted children if needed...
     if (!just_append && !childs_to_remove.empty())
     {
@@ -673,39 +742,95 @@ bool LiveSchemaTree::update_node_children(mforms::TreeNodeRef parent, std::list<
       removed = true;
     }
 
-    // Adds new children...
-    for(index = children.begin(); index != end; index++)
+    // If at this point there are nodes on the parent, then the new nodes
+    // need to be inserted into the right position.
+    // If not, as they are sorted, they just get added.
+    std::vector<mforms::TreeNodeRef> added_nodes;
+    std::vector<mforms::TreeNodeRef> group_added_nodes;
+    std::string icon_path = get_node_icon_path(type);
+
+    if (sorted)
+      children.sort(compare_func);
+    
+    if (!children.empty())
     {
-      mforms::TreeNodeRef new_child;
+      // it and it_end are used on the iteration process
+      // start and end are used to determine a group to be inserted
+      // which by default is the whole list
+      std::list<std::string>::const_iterator   it = children.begin(),
+                                               start = it,
+                                               it_end = children.end(),
+                                               end = it_end;
 
-      // If it is appending and there were nodes already, they need to be inserted in 
-      // the correct positions to keep the sorting
-      if (sorted && existing_children.size())
+      // final_group will be used to prevent searching for the position
+      // of the childs once one has been found to be at the end.
+      bool final_group = false;
+      int last_position = -1;
+      _node_collections[type].captions.clear();
+
+      if (parent->count() > 0 && sorted)
       {
-        int target_position = 0;
-        if (!find_child_position(parent, *index, type, target_position))
-          new_child = parent->insert_child(target_position);
+        // Initializes last_position with the position of the first
+        // Item to be inserted
+        find_child_position(parent, *it, type, last_position);
+
+        end = start;
+        
+        do
+        {
+          it++;
+          if (it != it_end)
+          {
+            int position = 0;
+            if (!find_child_position(parent, *it, type, position))
+            {
+              if (position != last_position)
+              {
+                // Assigns the captions for the group to be inserted
+                _node_collections[type].captions.assign(start, it);
+
+                group_added_nodes = parent->add_node_collection(_node_collections[type], last_position);
+                added_nodes.insert(added_nodes.end(), group_added_nodes.begin(), group_added_nodes.end());
+
+                // Start is reset to the iterator position
+                start = it;
+                last_position = position + _node_collections[type].captions.size();
+
+                // Shortcut to quit once an item has been found to be located
+                // At the end of the list
+                final_group = (last_position == -1);
+              }
+            }
+          }
+
+          // All the time end will advance with the iterator
+          end = it;
+        }while (it != it_end && !final_group);
       }
-      else if (existing_children.find(*index) == existing_children.end())
-        new_child = parent->add_child();
 
-      
-      if (new_child)
+      // Inserts the last group of nodes...
+      end = children.end();
+      _node_collections[type].captions.assign(start, end);
+      if (!_node_collections[type].captions.empty())
       {
-        // Adds any new item into the children list so it is considered
-        // while adding the rest
-        existing_children.insert(*index);
+        // positions the index on the previous position (last in group)
+        group_added_nodes = parent->add_node_collection(_node_collections[type], last_position);
+        added_nodes.insert(added_nodes.end(), group_added_nodes.begin(), group_added_nodes.end());
+      }
 
-        new_child->set_string(0, *index);
-        setup_node(new_child, type);
+      for(size_t index = 0; index < added_nodes.size(); index++)
+      {
+        setup_node(added_nodes[index], type);
         added = true;
       }
     }
-
-    ret_val = (added || removed);
     
+    ret_val = (added || removed);
+
+    std::string icon = get_node_icon_path(type);
+
     _model_view->thaw_refresh();
-  }
+    }
 
   return ret_val;
 }
@@ -782,15 +907,7 @@ void LiveSchemaTree::update_live_object_state(ObjectType type, const std::string
     if (type == Schema)
     {
       if (created)
-      {
-        int target_position = 0;
-        if (!find_child_position(_model_view->root_node(), new_obj_name, type, target_position))
-        {
-          schema_node = _model_view->root_node()->insert_child(target_position);
-          schema_node->set_string(0, new_obj_name);
-          setup_node(schema_node, Schema);
-        }
-      }
+        insert_node(_model_view->root_node(), new_obj_name, type);
       else if (deleted)
       {
         schema_node = get_child_node(_model_view->root_node(), old_obj_name);
@@ -838,36 +955,29 @@ void LiveSchemaTree::update_live_object_state(ObjectType type, const std::string
         {
           if (created)
           {
+            mforms::TreeNodeRef parent_node;
             int target_position = 0;
             switch(type)
             {
             case Table:
               if (!find_child_position(schema_node->get_child(TABLES_NODE_INDEX), new_obj_name, type, target_position))
-              {
-                object_node = schema_node->get_child(TABLES_NODE_INDEX)->insert_child(target_position);
-                setup_node(object_node, Table);
-              }
+                parent_node = schema_node->get_child(TABLES_NODE_INDEX);
               break;
             case View:
               if (!find_child_position(schema_node->get_child(VIEWS_NODE_INDEX), new_obj_name, type, target_position))
-              {
-                object_node = schema_node->get_child(VIEWS_NODE_INDEX)->insert_child(target_position);
-                setup_node(object_node, View);
-              }
+                parent_node = schema_node->get_child(VIEWS_NODE_INDEX);
               break;
             case Procedure:
             case Function:
               if (!find_child_position(schema_node->get_child(ROUTINES_NODE_INDEX), new_obj_name, type, target_position))
-              {
-                object_node = schema_node->get_child(ROUTINES_NODE_INDEX)->insert_child(target_position);
-                setup_node(object_node, type);
-              }
+                parent_node = schema_node->get_child(ROUTINES_NODE_INDEX);
               break;
             default:
               break;
             }
-
-            object_node->set_string(0, new_obj_name);
+            
+            if (parent_node)
+              insert_node(parent_node, new_obj_name, type);
           }
           else
           {
@@ -881,8 +991,8 @@ void LiveSchemaTree::update_live_object_state(ObjectType type, const std::string
 }
 
 void LiveSchemaTree::schema_contents_arrived(const std::string &schema_name,
-  std::list<std::string>& tables, std::list<std::string>& views,
-  std::list<std::string>& procedures, std::list<std::string>& functions, bool just_append)
+  std::list<std::string>* tables, std::list<std::string>* views,
+  std::list<std::string>* procedures, std::list<std::string>* functions, bool just_append)
 {
   if (_base)
   {
@@ -903,18 +1013,18 @@ void LiveSchemaTree::schema_contents_arrived(const std::string &schema_name,
 
         int old_table_count = tables_node->count();
         int old_view_count = tables_node->count();
-
-        update_node_children(tables_node,   tables,     Table,    true, just_append);
-        update_node_children(views_node,    views,      View,     true, just_append);
-        update_node_children(routines_node, procedures, Procedure,true, just_append);
-        update_node_children(routines_node, functions,  Function, true, just_append);
+        
+        update_node_children(tables_node,   *tables,     Table,    true, just_append);
+        update_node_children(views_node,    *views,      View,     true, just_append);
+        update_node_children(routines_node, *procedures, Procedure,true, just_append);
+        update_node_children(routines_node, *functions,  Function, true, just_append);
         
 
         // If there were nodes that means this is a refresh, in such case loaded tables
         // must be reloaded so the changes are displayed
         if (old_table_count)
         {
-            for(size_t index = 0; index < tables_node->count(); index++)
+            for (int index = 0; index < tables_node->count(); index++)
             {
                 mforms::TreeNodeRef pnode = tables_node->get_child(index);
                 if (pnode->is_expanded())
@@ -926,7 +1036,7 @@ void LiveSchemaTree::schema_contents_arrived(const std::string &schema_name,
         // must be reloaded so the changes are displayed
         if (old_view_count)
         {
-            for(size_t index = 0; index < views_node->count(); index++)
+            for (int index = 0; index < views_node->count(); index++)
             {
                 mforms::TreeNodeRef pnode = views_node->get_child(index);
                 if (pnode->is_expanded())
@@ -940,11 +1050,16 @@ void LiveSchemaTree::schema_contents_arrived(const std::string &schema_name,
         if (!just_append)
           pdata->fetched = true;
 
-        set_node_icon(schema_node, Schema);
+        update_node_icon(schema_node);
 
         tables_node->set_string(0, TABLES_CAPTION);
         views_node->set_string(0, VIEWS_CAPTION);
         routines_node->set_string(0, ROUTINES_CAPTION);
+
+        delete tables;
+        delete views;
+        delete procedures;
+        delete functions;
       }
     }
   }
@@ -1038,7 +1153,7 @@ void LiveSchemaTree::load_schema_content(mforms::TreeNodeRef& schema_node)
     schema_node->get_child(VIEWS_NODE_INDEX)->set_string(0, VIEWS_CAPTION + " " + FETCHING_CAPTION);
     schema_node->get_child(ROUTINES_NODE_INDEX)->set_string(0, ROUTINES_CAPTION + " " + FETCHING_CAPTION);
 
-    set_node_icon(schema_node, Schema);
+    update_node_icon(schema_node);
 
     if (boost::shared_ptr<FetchDelegate> delegate = _fetch_delegate.lock())
     {
@@ -1089,6 +1204,7 @@ mforms::TreeNodeRef LiveSchemaTree::create_node_for_object(const std::string &sc
 {
   bool created_schema = false;
   mforms::TreeNodeRef object_node = mforms::TreeNodeRef();
+  mforms::TreeNodeRef parent_node = mforms::TreeNodeRef();
 
   if (_model_view)
   {
@@ -1097,33 +1213,28 @@ mforms::TreeNodeRef LiveSchemaTree::create_node_for_object(const std::string &sc
     // Creates the schema if doesnt exist...
     if (!schema_node)
     {
-      schema_node = _model_view->root_node()->add_child();
-      schema_node->set_string(0, schema_name);
-      setup_node(schema_node, Schema);
+      schema_node = insert_node(_model_view->root_node(), schema_name, Schema);
       created_schema = true;
     }
 
     switch(type)
     {
     case Table:
-      object_node = schema_node->get_child(TABLES_NODE_INDEX)->add_child();
+      parent_node = schema_node->get_child(TABLES_NODE_INDEX);
       break;
     case View:
-      object_node = schema_node->get_child(VIEWS_NODE_INDEX)->add_child();
+      parent_node = schema_node->get_child(VIEWS_NODE_INDEX);
       break;
     case Procedure:
     case Function:
-      object_node = schema_node->get_child(ROUTINES_NODE_INDEX)->add_child();
+      parent_node = schema_node->get_child(ROUTINES_NODE_INDEX);
       break;
     default:
       break;
     }
 
-    if (object_node)
-    {
-      object_node->set_string(0, name);
-      setup_node(object_node, type);
-    }
+    if (parent_node)
+      object_node = insert_node(parent_node, name, type);
     else if (created_schema)
       schema_node->remove_from_parent();
   }
@@ -1713,8 +1824,10 @@ bool LiveSchemaTree::activate_popup_item_for_nodes(const std::string &name, cons
       return true;
     }
     
+    bool clipboard_or_editor = (0 == name.find("clipboard_")) || (0 == name.find("editor_"));
+
     if ((0 == name.find("select_data")) || (0 == name.find("edit_data")) ||
-        (0 == name.find("clipboard_")) || (0 == name.find("editor_")))
+        clipboard_or_editor)
     {
       for(index = unsorted_nodes.begin(); index != end; index++)
       {
@@ -1729,7 +1842,9 @@ bool LiveSchemaTree::activate_popup_item_for_nodes(const std::string &name, cons
           type = pdata->get_type();
           schema_name = get_schema_name(pnode);
           
-          if (is_object_type(DatabaseObject, type))
+          // Clipboard or editor options should be available for Schemas too, otherwise
+          // only tables, views and routines
+          if (is_object_type(clipboard_or_editor ? DatabaseObject : SchemaObject, type))
             object_name = pnode->get_string(0);
           else
           {
@@ -2053,20 +2168,22 @@ bool LiveSchemaTree::filter_children(ObjectType type, mforms::TreeNodeRef& sourc
         type = pdata->get_type();
       }
 
-      mforms::TreeNodeRef target_node = target->add_child();
-      target_node->set_string(0, source_node->get_string(0));
-      setup_node(target_node, type, source_node->get_data());
+      std::vector<mforms::TreeNodeRef> group_added_nodes;
+      _node_collections[type].captions.clear();
+      _node_collections[type].captions.push_back(source_node->get_string(0));
+      group_added_nodes = target->add_node_collection(_node_collections[type]);
+      setup_node(group_added_nodes[0], type, source_node->get_data(), true);
 
       // For each found node, continues with their children...
       if (type == Schema || type == Table || type == View)
-        filter_children_collection(source_node, target_node);
+        filter_children_collection(source_node, group_added_nodes[0]);
 
-      if (source_node->is_expanded() != target_node->is_expanded())
+      if (source_node->is_expanded() != group_added_nodes[0]->is_expanded())
       {
-        if (source_node->is_expanded())
-          target_node->expand();
+        if (group_added_nodes[0]->is_expanded())
+          group_added_nodes[0]->expand();
         else
-          target_node->collapse();
+          group_added_nodes[0]->collapse();
       }
     }
   }
@@ -2410,7 +2527,7 @@ void LiveSchemaTree::expand_toggled(mforms::TreeNodeRef node, bool value)
             if (pdata->columns_load_error)
             {
               node->remove_children();
-              set_node_icon(node, View);
+              update_node_icon(node);
             }
           }
           break;
@@ -2528,6 +2645,9 @@ std::vector<std::string> LiveSchemaTree::get_node_path(const mforms::TreeNodeRef
 
 /*
 * get_node_from_path: Finds a node in a tree following a name path
+* TODO: This will not work in the case of the routines, as there's no way to know if the path is
+*       for a procedure or for a function, on this case a sequential search will be done so
+*       procedures will be searched first all the time
 */
 mforms::TreeNodeRef LiveSchemaTree::get_node_from_path(std::vector<std::string> path)
 {
@@ -2548,8 +2668,7 @@ mforms::TreeNodeRef LiveSchemaTree::get_node_from_path(std::vector<std::string>
       std::string tag = temp_node->get_tag();
       
       use_binary_search = (tag == TABLES_TAG ||
-                           tag == VIEWS_TAG ||
-                           tag == ROUTINES_TAG);
+                           tag == VIEWS_TAG );
     }
     else
       error = true;
@@ -2579,6 +2698,26 @@ bool LiveSchemaTree::is_object_type(ObjectTypeValidation validation, ObjectType
   return false;
 }
 
+mforms::TreeNodeRef LiveSchemaTree::insert_node(mforms::TreeNodeRef parent, const std::string& name, ObjectType type)
+{
+  mforms::TreeNodeRef node;
+
+  int target_position = 0;
+  if (!find_child_position(parent, name, type, target_position))
+  {
+    std::vector<mforms::TreeNodeRef> group_added_nodes;
+
+    _node_collections[type].captions.clear();
+    _node_collections[type].captions.push_back(name);
+    group_added_nodes = parent->add_node_collection(_node_collections[type], target_position);
+    node = group_added_nodes[0];
+              
+    setup_node(node, type);
+  }
+
+  return node;
+}
+
 void LiveSchemaTree::reload_object_data(mforms::TreeNodeRef& node)
 {
     ViewData* pdata = dynamic_cast<ViewData*>(node->get_data());
@@ -2662,4 +2801,4 @@ void LiveSchemaTree::discard_object_data(mforms::TreeNodeRef& node, int data_mas
         parent_node = node->get_child(TABLE_FOREIGN_KEYS_NODE_INDEX);
         parent_node->remove_children();
     }
-}
+}
\ No newline at end of file
