@@ -33,9 +33,7 @@ namespace wb
   class MYSQLWBBACKEND_PUBLIC_FUNC LiveSchemaTree: base::trackable
   {
   public:
-#if defined(_DEBUG)
   friend class LiveSchemaTreeTester;
-#endif
 
     static const short COLUMN_DATA = 0x01;
     static const short TRIGGER_DATA = 0x02;
@@ -272,7 +270,7 @@ namespace wb
       virtual std::string get_object_name() { return _("Schema"); }
     };
 
-    typedef boost::function<void (const std::string& schema_name, std::list<std::string>& tables, std::list<std::string>& views, std::list<std::string>& procedures, std::list<std::string>& functions, bool just_append)> NewSchemaContentArrivedSlot;
+    typedef boost::function<void (const std::string& schema_name, std::list<std::string>* tables, std::list<std::string>* views, std::list<std::string>* procedures, std::list<std::string>* functions, bool just_append)> NewSchemaContentArrivedSlot;
     typedef boost::function<void (const std::string& schema_name, const std::string& object_name, ObjectType obj_type, ObjectType child_type, const std::map<std::string, LSTData*> &children)> NewObjectDetailsArrivedSlot;
     typedef boost::function<bool (mforms::TreeNodeRef, std::list<std::string>&, ObjectType, bool sorted, bool just_append)>NodeChildrenUpdaterSlot;
 
@@ -311,8 +309,8 @@ namespace wb
 
     bool _case_sensitive_identifiers;
 
-     boost::function<bool (const std::string& first, const std::string& second)> compare_func;
-    void schema_contents_arrived(const std::string &schema_name, std::list<std::string>& tables, std::list<std::string>& views, std::list<std::string>& procedures, std::list<std::string>& functions, bool just_append);
+    boost::function<bool (const std::string& first, const std::string& second)> compare_func;
+    void schema_contents_arrived(const std::string &schema_name, std::list<std::string> *tables, std::list<std::string> *views, std::list<std::string> *procedures, std::list<std::string> *functions, bool just_append);
     void load_table_details(mforms::TreeNodeRef& node, int fetch_mask);
     void fetch_table_details(ObjectType object_type, const std::string schema_name, const std::string object_name, int fetch_mask);
     void load_schema_content(mforms::TreeNodeRef& schema_node);
@@ -329,7 +327,7 @@ namespace wb
     bool is_object_type(ObjectTypeValidation validation, ObjectType type);
   public:
     LiveSchemaTree(grt::GRT* grtm);
-    ~LiveSchemaTree();
+    virtual ~LiveSchemaTree();
 
     void set_model_view(mforms::TreeNodeView* target);
     void set_delegate(boost::shared_ptr<Delegate> delegate);
@@ -362,11 +360,10 @@ namespace wb
     void set_base(LiveSchemaTree *base) { _base = base; }
 
     bool update_node_children(mforms::TreeNodeRef parent, std::list<std::string>& children, ObjectType type, bool sorted = false, bool just_append = false);
-    void setup_node(mforms::TreeNodeRef node, ObjectType type, mforms::TreeNodeData* data = NULL);
-    void setup_schema_node(mforms::TreeNodeRef node, mforms::TreeNodeData* data = NULL);
-    void setup_view_node(mforms::TreeNodeRef node, mforms::TreeNodeData* data = NULL);
-    void setup_table_node(mforms::TreeNodeRef node, mforms::TreeNodeData* data = NULL);
-    void set_node_icon(mforms::TreeNodeRef node, ObjectType type);
+    void update_change_data(mforms::TreeNodeRef parent, std::list<std::string>& children, ObjectType type, std::vector<mforms::TreeNodeRef>& to_remove);
+    mforms::TreeNodeRef insert_node(mforms::TreeNodeRef parent, const std::string& name, ObjectType type);
+    void setup_node(mforms::TreeNodeRef node, ObjectType type, mforms::TreeNodeData* data = NULL, bool ignore_null_data = false);
+    void update_node_icon(mforms::TreeNodeRef node);
 
     void update_schemata(std::list<std::string> &schema_list);
     bool compare_identifiers(const std::string &first, const std::string &second);
@@ -395,6 +392,14 @@ namespace wb
     GPatternSpec* _object_pattern;
 
     static const char* _schema_tokens[15];
+    
+    std::map<ObjectType, std::string> _icon_paths;
+    
+    std::map<ObjectType, mforms::TreeNodeCollectionSkeleton> _node_collections;
+    
+    void fill_node_icons();
+    std::string get_node_icon_path(ObjectType type);
+    bec::IconId get_node_icon(ObjectType type);
   };
 };
 
