@@ -229,7 +229,7 @@ void SqlEditorForm::finish_startup()
   if (_usr_dbc_conn && !_usr_dbc_conn->active_schema.empty())
     _live_tree->on_active_schema_change(_usr_dbc_conn->active_schema);
 
-  _grtm->run_once_when_idle(boost::bind(&SqlEditorForm::update_menu_and_toolbar, this));
+  _grtm->run_once_when_idle(this, boost::bind(&SqlEditorForm::update_menu_and_toolbar, this));
 }
 
 //-------------------------------------------------------------------------------------------------- 
@@ -440,6 +440,11 @@ void SqlEditorForm::close()
   exec_sql_task->disconnect_callbacks();
   reset_keep_alive_thread();
   _grtm->replace_status_text("SQL Editor closed");
+
+  delete _menu;
+  _menu = 0;
+  delete _toolbar;
+  _toolbar = 0;
 }
 
 
@@ -676,14 +681,8 @@ void SqlEditorForm::create_connection(sql::Dbc_connection_handler::Ref &dbc_conn
   int read_timeout = _grtm->get_app_option_int("DbSqlEditor:ReadTimeOut");
   if (read_timeout > 0)
     temp_connection->parameterValues().set("OPT_READ_TIMEOUT", grt::IntegerRef(read_timeout));
+  temp_connection->parameterValues().set("CLIENT_INTERACTIVE", grt::IntegerRef(1));
 
-  // we could either always pick last used schema or only pick last used schema if the connection doesn't have a default schema
-  // since customer from bug #11758776 wants the 1st, we do that for now...
-  //if (temp_connection->parameterValues().get_string("schema") == "")
-  // don't override the default schema used to connect, because the schema may have been deleted and the connection will fail
-  // further, the schema can't be changed from the connection editor. Default schema is set once connected
-//  temp_connection->parameterValues().gset("schema", temp_connection->parameterValues().get_string("DbSqlEditor:LastDefaultSchema"));
-  
   dbc_conn->ref= dbc_drv_man->getConnection(temp_connection, tunnel, auth,
                                             boost::bind(&SqlEditorForm::init_connection, this, _1, _2, dbc_conn));
   //! dbms-specific code
@@ -692,30 +691,45 @@ void SqlEditorForm::create_connection(sql::Dbc_connection_handler::Ref &dbc_conn
     throw std::runtime_error("MySQL Server version is older than 5.0, which is not supported");
   }
 
-  if (dbc_conn->active_schema.empty())
+  // Activate the schema that was last active when this connection was open.
+  // If there's no active schema use the default schema.
+  std::string last_default = temp_connection->parameterValues().get_string("DbSqlEditor:LastDefaultSchema");
+  if (last_default.empty())
+    last_default = temp_connection->parameterValues().get_string("schema");
+  if (!last_default.empty())
   {
-    std::string last_default = temp_connection->parameterValues().get_string("DbSqlEditor:LastDefaultSchema");
     try
     {
-      if (!last_default.empty())
-        dbc_conn->ref->setSchema(last_default);
+      dbc_conn->ref->setSchema(last_default);
+      dbc_conn->active_schema = last_default;
+      
+      _grtm->run_once_when_idle(this, boost::bind(&set_active_schema, shared_from_this(), last_default));
     }
     catch (std::exception &exc)
     {
       log_error("Can't restore LastDefaultSchema (%s): %s", last_default.c_str(), exc.what());
       temp_connection->parameterValues().gset("DbSqlEditor:LastDefaultSchema", "");
     }
- 
-    _grtm->run_once_when_idle(boost::bind(&set_active_schema, shared_from_this(), last_default));
-    //dbc_conn->active_schema= dbc_conn->ref->getSchema();
   }
-  else
-    dbc_conn->ref->setSchema(dbc_conn->active_schema);
+
   dbc_conn->ref->setAutoCommit(autocommit_mode);
   dbc_conn->autocommit_mode= dbc_conn->ref->getAutoCommit();
 }
 
 
+struct ConnectionErrorInfo
+{
+  sql::AuthenticationError *auth_error;
+  bool password_expired;
+
+  ConnectionErrorInfo() : auth_error(0), password_expired(0) {}
+  ~ConnectionErrorInfo()
+  {
+    delete auth_error;
+  }
+};
+
+
 void SqlEditorForm::connect()
 {
   sql::Authentication::Ref auth = _dbc_auth;//sql::Authentication::create(_connection, "");
@@ -736,17 +750,20 @@ void SqlEditorForm::connect()
   // connect
   for (;;)
   {
-    // if an auth error happens in the worker thread, this ptr will be set
-    sql::AuthenticationError *auth_error_ptr = 0;
+    // if an error happens in the worker thread, this ptr will be set
+    ConnectionErrorInfo error_ptr;
     
     // connection must happen in the worker thread
     try
     {
-      exec_sql_task->exec(true, boost::bind(&SqlEditorForm::do_connect, this, _1, tunnel, auth, boost::ref(auth_error_ptr)));
+      exec_sql_task->exec(true, boost::bind(&SqlEditorForm::do_connect, this, _1, tunnel, auth, &error_ptr));
     }
     catch (grt::grt_runtime_error)
     {
-      if (!auth_error_ptr)
+      if (error_ptr.password_expired)
+        throw std::runtime_error(":PASSWORD_EXPIRED");
+
+      if (!error_ptr.auth_error)
         throw;
 
       if (current_method == NoPassword)
@@ -793,7 +810,7 @@ std::string create_html_line(const std::string& name, const std::string& value)
 
 //--------------------------------------------------------------------------------------------------
 
-grt::StringRef SqlEditorForm::do_connect(grt::GRT *grt, boost::shared_ptr<sql::TunnelConnection> tunnel, sql::Authentication::Ref &auth, sql::AuthenticationError *&autherr_ptr)
+grt::StringRef SqlEditorForm::do_connect(grt::GRT *grt, boost::shared_ptr<sql::TunnelConnection> tunnel, sql::Authentication::Ref &auth, ConnectionErrorInfo *err_ptr)
 {
   try
   {
@@ -863,45 +880,15 @@ grt::StringRef SqlEditorForm::do_connect(grt::GRT *grt, boost::shared_ptr<sql::T
   }
   catch (sql::AuthenticationError &exc)
   {
-    autherr_ptr = new sql::AuthenticationError(exc);
+    err_ptr->auth_error = new sql::AuthenticationError(exc);
     throw;
   }
-
-  db_mgmt_RdbmsRef rdbms= this->rdbms();
-  std::string database_package= *rdbms->databaseObjectPackage();
-
-  workbench_physical_ModelRef model(_grtm->get_grt());
-  rdbms->owner(model);
-  model->rdbms(rdbms);
-  db_CatalogRef catalog= _grtm->get_grt()->create_object<db_Catalog>(database_package + ".Catalog");
+  catch (sql::SQLException &exc)
   {
-    catalog->owner(model);
-    catalog->name("default");
-    catalog->oldName("default");
-    catalog->version(rdbms->version());
-    grt::replace_contents(catalog->simpleDatatypes(), rdbms->simpleDatatypes());
-
-    // setup list of synonyms
-    grt::Module *module= _grtm->get_grt()->get_module("DbMySQL");
-    if (module)
-    {
-      grt::BaseListRef args(_grtm->get_grt());
-      args.ginsert(rdbms);
-      grt::ListRef<db_UserDatatype> user_types(grt::ListRef<db_UserDatatype>::cast_from(module->call_function("getDefaultUserDatatypes", args)));
-      
-      if (user_types.is_valid())
-      {
-        GRTLIST_FOREACH(db_UserDatatype, user_types, ut)
-        {
-          (*ut)->owner(catalog);
-          catalog->userDatatypes().insert(*ut);
-        }
-      }
-    }    
-    
-    model->catalog(catalog);
+    if (exc.getErrorCode() == 1820)
+      err_ptr->password_expired = true;
+    throw;
   }
-   
   if (_grtm->get_app_option_int("DbSqlEditor:CodeCompletionEnabled") == 1)
   {
     std::string cache_dir = _grtm->get_user_datadir() + "/cache/";
@@ -1173,7 +1160,7 @@ void SqlEditorForm::exec_sql_retaining_editor_contents(const std::string &sql_sc
   exec_sql_task->exec(sync,
     boost::bind(&SqlEditorForm::do_exec_sql, this, _1,
                weak_ptr_from(this), boost::shared_ptr<std::string>(new std::string(sql_script)),
-               Sql_editor::Ref(),(ExecFlags)(Retaining | (dont_add_limit_clause?DontAddLimitClause:0)),
+               editor,(ExecFlags)(Retaining | (dont_add_limit_clause?DontAddLimitClause:0)),
                recordsets));
 }
 
@@ -1605,13 +1592,16 @@ grt::StringRef SqlEditorForm::do_exec_sql(grt::GRT *grt, Ptr self_ptr, boost::sh
                     rs->generator_query(statement);
                     rs->action_list().register_action("recall_query",
                       boost::bind(&SqlEditorForm::recall_recordset_query, this, Recordset::Ptr(rs)));
+                    rs->action_list().register_action("record_export",
+                                                      boost::bind(&SqlEditorForm::show_export_recordset, this, sql_editor_index(editor), Recordset::Ptr(rs)));
+                    rs->action_list().register_action("record_import",
+                                                      boost::bind(&SqlEditorForm::show_import_recordset, this, sql_editor_index(editor), Recordset::Ptr(rs)));
 
                     // XXX: refresh recordset title on status bar change? Huh?
                     scoped_connect(&rs->refresh_ui_status_bar_signal,
                       boost::bind(&bec::RefreshUI::do_partial_ui_refresh, this, (int)RefreshRecordsetTitle));
 
                     rs->data_storage(data_storage);
-                    rs->export_wizard= boost::bind(&SqlEditorForm::show_export_recordset, this, sql_editor_index(editor), Recordset::Ptr(rs));
                     rs->popup_menu_items_slot= boost::bind(&SqlEditorForm::recordset_popup_menu_items, this, _1, _2, Recordset::Ptr(rs));
                     rs->call_popup_menu_item_slot= boost::bind(&SqlEditorForm::call_recordset_popup_menu_item, this, _1, _2, _3, Recordset::Ptr(rs));
                     rs->reset(true);
@@ -2121,10 +2111,13 @@ void SqlEditorForm::active_schema(const std::string &value)
     update_editor_title_schema(value);
     
     grt_manager()->replace_status_text(strfmt(_("Active schema changed to %s"), value.c_str()));
+    
+    _grtm->get_grt()->call_module_function("Workbench", "saveConnections", grt::BaseListRef());
   }
   CATCH_ANY_EXCEPTION_AND_DISPATCH(_("Set active schema"))
 }
 
+
 db_mgmt_RdbmsRef SqlEditorForm::rdbms()
 {
   if (_connection.is_valid())
