@@ -22,6 +22,7 @@
 #include "wb_sql_editor_form_ui.h"
 #include "query_side_palette.h"
 #include "wb_context_names.h"
+#include "objimpl/db.query/db_query_Resultset.h"
 #include "grtui/inserts_export_form.h"
 
 #include "sqlide/wb_sql_editor_tree_controller.h"
@@ -296,8 +297,8 @@ boost::shared_ptr<mforms::ToolBar> SqlEditorForm::setup_editor_toolbar()
   bool has_json_explain = false;
   {
     int major, minor;
-    if (sscanf(_connection_details["dbmsProductVersion"].c_str(), "%i.%i", &major, &minor) == 2
-        && major > 5 || (major == 5 && minor >= 6))
+    if ((sscanf(_connection_details["dbmsProductVersion"].c_str(), "%i.%i", &major, &minor) == 2
+        && major > 5) || (major == 5 && minor >= 6))
       has_json_explain = true;
   }
   if (has_json_explain)
@@ -454,6 +455,39 @@ void SqlEditorForm::show_export_recordset(int editor_index, Recordset::Ptr rs_pt
 }
 
 
+void SqlEditorForm::show_import_recordset(int editor_index, Recordset::Ptr rs_ptr)
+{
+  try
+  {
+    RETURN_IF_FAIL_TO_RETAIN_WEAK_PTR (Recordset, rs_ptr, rs)
+    {
+      grt::BaseListRef args(_grtm->get_grt());
+      
+      db_query_ResultsetRef rset;
+      db_query_QueryEditorRef qeditor(db_query_QueryEditorRef::cast_from(_sql_editors[editor_index]->editor->grtobj()));
+      for (size_t c = qeditor->resultsets().count(), i = 0; i < c; i++)
+      {
+        if (qeditor->resultsets()[i]->get_data()->recordset.get() == rs)
+        {
+          rset = qeditor->resultsets()[i];
+          break;
+        }
+      }
+      
+      if (rset.is_valid())
+      {
+        args.ginsert(rset);
+        grt::Module *module = _grtm->get_grt()->get_module("SQLIDEUtils");
+        if (module)
+          module->call_function("importRecordsetDataFromFile", args);
+      }
+    }
+  }
+  catch (const std::exception &exc)
+  {
+    mforms::Utilities::show_error("Error importing recordset", exc.what(), "OK");
+  }
+}
 
 
 bool SqlEditorForm::run_live_object_alteration_wizard(const std::string &alter_script, bec::DBObjectEditorBE* obj_editor, RowId log_id,
