@@ -45,8 +45,8 @@
 
 #define APP_MAJOR_NUMBER 5
 #define APP_MINOR_NUMBER 2
-#define APP_RELEASE_NUMBER 44
-#define APP_REVISION_NUMBER 9933
+#define APP_RELEASE_NUMBER 45
+#define APP_REVISION_NUMBER 10251
 #define APP_RELEASE_TYPE ""
 
 
@@ -180,7 +180,7 @@ namespace wb {
     boost::function<void (bool)> lock_gui;
 
     // Closes the application
-    boost::function<void ()> quit_application;
+    boost::function<bool ()> quit_application;
   };
 
 
@@ -202,6 +202,8 @@ namespace wb {
     bool verbose;
     bool quit_when_done;
     bool no_logo;
+    bool testing;     // True if we are currently running unit tests.
+    bool init_python; // True by default. Can be switched off for testing.
 
     WBOptions();
     bool parse_args(char **argv, int argc, int *retval = NULL);
@@ -218,7 +220,7 @@ namespace wb {
 
    public:
     WBContext(WBContextUI *ui, bool verbose = false);
-    ~WBContext();
+    virtual ~WBContext();
 
     bool software_rendering_enforced();
     bool opengl_rendering_enforced();
@@ -263,6 +265,9 @@ namespace wb {
     // Admin
     void add_new_admin_window(const db_mgmt_ServerInstanceRef &target);
 
+    // Migration
+    void add_new_migration_window();
+
     // GUI Plugin
     void register_builtin_plugins(grt::ListRef<app_Plugin> plugins);
 
@@ -319,6 +324,7 @@ namespace wb {
       return grt::Ref<C>::cast_from(obj);
     }
 
+    bool cancel_idle_tasks();
     void flush_idle_tasks();
 
  // utilities for error reporting
@@ -514,7 +520,7 @@ namespace wb {
 
     boost::function<void (std::string)> perform_command;
     
-    boost::function<void ()> quit_application;
+    boost::function<bool ()> quit_application;
   };
 
   struct GUILock
