@@ -50,6 +50,8 @@
 #include "base/log.h"
 #include "base/notifications.h"
 
+#include "model/wb_user_datatypes.h"
+
 #include "plugin_install_window.h"
 
 using namespace wb;
@@ -59,7 +61,7 @@ using namespace base;
 DEFAULT_LOG_DOMAIN(DOMAIN_WB_CONTEXT_UI)
 
 WBContextUI::WBContextUI(bool verbose)
-  : _wb(new WBContext(this, verbose)), _usertypes_tree(_wb), _command_ui(_wb)
+  : _wb(new WBContext(this, verbose)), _usertypes_tree(new UserDatatypeListBE(_wb)), _command_ui(_wb)
 {
   NodeId n; // workaround a bug causing crash when output tab is shown with GLib-ERROR **: The thread system is not yet initialized.
 
@@ -105,7 +107,8 @@ WBContextUI::~WBContextUI()
   
   _wb->do_close_document(true);
 
-  // TODO: is it really necessary to set all member variables to NULL in a dtor?
+  // is it really necessary to set all member variables to NULL in a dtor?. 
+  // -- Yes, some of these d-tors may trigger things that try to access the destroyed members
   delete _addon_download_window;
   _addon_download_window = 0;
   delete _plugin_install_window;
@@ -114,6 +117,11 @@ WBContextUI::~WBContextUI()
   _find_dialog = 0;
   delete _history_tree;
   _history_tree = 0;
+  //XXX workaround for crash in linux
+#if defined(_WIN32) || defined(__APPLE__)
+  delete _usertypes_tree;
+#endif
+  _usertypes_tree = 0;
   if (_home_screen != NULL)
   {
     _home_screen->release();
@@ -162,15 +170,16 @@ bool WBContextUI::init(WBFrontendCallbacks *callbacks, WBOptions *options)
     //_shell_window = new GRTShellWindow(_wb->get_grt_manager());
     flag= _wb->init_(callbacks, options);
 
-    // has to be called after WBContext is initialized
-    add_backend_builtin_commands();
+    if (!options->testing)
+    {
+      // has to be called after WBContext is initialized
+      add_backend_builtin_commands();
 
-    // look for auto-save files (must be done at startup to not confuse with autosaves created by ourself)
-    WBContextModel::detect_auto_save_files(_wb->get_auto_save_dir());
-    WBContextSQLIDE::detect_auto_save_files(_wb->get_auto_save_dir());
+      // look for auto-save files (must be done at startup to not confuse with autosaves created by ourself)
+      WBContextModel::detect_auto_save_files(_wb->get_auto_save_dir());
+      WBContextSQLIDE::detect_auto_save_files(_wb->get_auto_save_dir());
+    }
 
-    // Finally show initial home screen.
-    show_home_screen();
     _initializing = false;
   }
   catch (const std::exception& e)
@@ -200,6 +209,7 @@ GRTShellWindow* WBContextUI::get_shell_window()
 
 void WBContextUI::init_finish(WBOptions *options)
 {  
+  show_home_screen();
   _wb->init_finish_(options);
 }
 
@@ -257,8 +267,8 @@ void WBContextUI::reset()
   
   _wb->get_component<WBComponentPhysical>()->update_catalog_tree();
 
-  _usertypes_tree.set_catalog(_wb->get_document()->physicalModels()[0]->catalog());
-  _usertypes_tree.set_rdbms(_wb->get_document()->physicalModels()[0]->rdbms());
+  _usertypes_tree->set_catalog(_wb->get_document()->physicalModels()[0]->catalog());
+  _usertypes_tree->set_rdbms(_wb->get_document()->physicalModels()[0]->rdbms());
 }
 
 
@@ -267,7 +277,7 @@ void WBContextUI::history_changed()
   if (_wb->has_unsaved_changes() != _last_unsaved_changes_state)
     _wb->request_refresh(RefreshDocument, "", (NativeHandle)0);
 
-  get_command_ui()->revalidate_edit_menu_items();
+  _wb->get_grt_manager()->run_once_when_idle(boost::bind(&CommandUI::revalidate_edit_menu_items, get_command_ui()));
   _last_unsaved_changes_state= _wb->has_unsaved_changes();
 }
 
@@ -413,7 +423,7 @@ bec::ValueTreeBE *WBContextUI::get_catalog_tree()
 
 UserDatatypeListBE *WBContextUI::get_usertypes_tree()
 {
-  return &_usertypes_tree;
+  return _usertypes_tree;
 }
 
 
@@ -518,7 +528,9 @@ void WBContextUI::form_changed()
 
   bec::UIForm *form = get_active_main_form();
   if (form && form->get_menubar())
-    form->get_menubar()->validate();
+  {
+    get_command_ui()->revalidate_menu_bar(form->get_menubar());
+  }
 }
 
 HistoryTreeBE *WBContextUI::get_history_tree()
