@@ -34,8 +34,6 @@
 #include "base/trackable.h"
 #include "base/notifications.h"
 
-#include "model/wb_user_datatypes.h"
-
 namespace bec 
 {
   class ValueTreeBE;
@@ -55,7 +53,7 @@ namespace wb {
   class PhysicalOverviewBE;
   class HistoryTreeBE;
   class DiagramOptionsBE;
-
+  class UserDatatypeListBE;
   class ModelDiagramForm;
 
 
@@ -208,7 +206,7 @@ namespace wb {
     HistoryTreeBE *_history_tree;
     bec::ValueTreeBE *_catalog_tree;
     FindDialogBE *_find_dialog;
-    UserDatatypeListBE _usertypes_tree;
+    UserDatatypeListBE *_usertypes_tree;
     OutputView *_output_view;
     AddOnDownloadWindow *_addon_download_window;
     PluginInstallWindow *_plugin_install_window;
