@@ -291,7 +291,7 @@ void WBContextUI::show_home_screen()
     _home_screen->set_on_close(boost::bind(&WBContextUI::home_screen_closing, this));
     _home_screen->set_callback((home_screen_action_callback)&WBContextUI::home_action_callback, this);
     _home_screen->handle_context_menu = boost::bind(&WBContextUI::handle_home_context_menu, this, _1, _2, _3);
-    
+
     // setup context menus
     bec::MenuItemList items, pitems;
     bec::MenuItem item, sep;
@@ -1012,6 +1012,10 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
                                     "db.Catalog");
         
         _wb->execute_plugin("db.plugin.database.rev_eng", args);
+
+        // if the model is still empty, just close it
+        if (_wb->get_document()->physicalModels()[0]->catalog()->schemata().count() == 0)
+          _wb->close_document();
       }
       else
         _wb->show_status_text("Error creating document");
@@ -1033,6 +1037,10 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
         args.add_entries_for_object("activeCatalog", _wb->get_document()->physicalModels()[0]->catalog(),
                                     "db.Catalog");
         _wb->execute_plugin("db.mysql.plugin.import.sql", args);
+
+        // if the model is still empty, just close it
+        if (_wb->get_document()->physicalModels()[0]->catalog()->schemata().count() == 0)
+            _wb->close_document();
       }
       else
         _wb->show_status_text("Error creating document");
@@ -1383,29 +1391,42 @@ void WBContextUI::refresh_home_models(int selected)
       model_count++;
   }
 
-  // If there is no entry in the MRU list then scan the examples folder for the Sakila model
-  // file and add this as initial entry, so that a new user has an easy start.
-  if (model_count == 0)
+  
+  int sample_models_already_shown = _wb->read_state("WBSampleModelFilesShown", "home", 0);
+  if (sample_models_already_shown == 0)
   {
-    std::list<std::string> examples_paths;
-#ifdef _WIN32
-    examples_paths.push_back(mforms::Utilities::get_special_folder(mforms::WinProgramFilesX86) +
-      "\\MySQL\\Samples and Examples 5.5");
-    examples_paths.push_back(mforms::Utilities::get_special_folder(mforms::WinProgramFiles) +
-      "\\MySQL\\Samples and Examples 5.5");
-#endif
-    for (std::list<std::string>::const_iterator path_iterator = examples_paths.begin();
-      path_iterator != examples_paths.end(); path_iterator++)
+    // If there is no entry in the MRU list then scan the examples folder for the Sakila model
+    // file and add this as initial entry, so that a new user has an easy start.
+    if (model_count == 0)
     {
-      if (g_file_test(path_iterator->c_str(), G_FILE_TEST_IS_DIR))
+      std::list<std::string> examples_paths;
+#ifdef _WIN32
+      examples_paths.push_back(mforms::Utilities::get_special_folder(mforms::WinProgramFilesX86) +
+        "\\MySQL\\Samples and Examples 5.5");
+      examples_paths.push_back(mforms::Utilities::get_special_folder(mforms::WinProgramFiles) +
+        "\\MySQL\\Samples and Examples 5.5");
+      examples_paths.push_back(_wb->get_grt_manager()->get_basedir()+"/extras");
+#elif defined(__APPLE__)
+      examples_paths.push_back(_wb->get_grt_manager()->get_basedir()+"/../SharedSupport");
+#else
+      examples_paths.push_back(_wb->get_grt_manager()->get_basedir()+"/extras");
+#endif
+
+      for (std::list<std::string>::const_iterator path_iterator = examples_paths.begin();
+        path_iterator != examples_paths.end(); path_iterator++)
       {
-        std::string pattern = make_path(*path_iterator, "*.mwb");
-        std::list<std::string> sample_model_files = base::scan_for_files_matching(pattern, true);
-        for (std::list<std::string>::const_iterator iterator = sample_model_files.begin();
-          iterator != sample_model_files.end(); iterator++)
-          recentFiles->insert_checked(grt::StringRef(*iterator));
+        g_message("%s", path_iterator->c_str());
+        if (g_file_test(path_iterator->c_str(), G_FILE_TEST_IS_DIR))
+        {
+          std::string pattern = make_path(*path_iterator, "*.mwb");
+          std::list<std::string> sample_model_files = base::scan_for_files_matching(pattern, true);
+          for (std::list<std::string>::const_iterator iterator = sample_model_files.begin();
+            iterator != sample_model_files.end(); iterator++)
+            recentFiles->insert_checked(grt::StringRef(*iterator));
+        }
       }
     }
+    _wb->save_state("WBSampleModelFilesShown", "home", 1);
   }
 
   for (grt::StringListRef::const_iterator end = recentFiles.end(), f = recentFiles.begin();
