@@ -1306,7 +1306,7 @@ int WorkbenchImpl::showDocumentProperties()
 
 int WorkbenchImpl::showModelOptions(const workbench_physical_ModelRef &model)
 {
-  PreferencesForm prefs(_wb->get_ui(), model.id());
+  PreferencesForm prefs(_wb->get_ui(), model);
   prefs.show();
   
   return 0;
@@ -1752,6 +1752,7 @@ int WorkbenchImpl::createConnectionsFromLocalServers()
 }
 
 extern std::string get_admin_script_for_os(const std::string &os);
+
 //--------------------------------------------------------------------------------------------------
 
 /**
@@ -1946,15 +1947,25 @@ std::string WorkbenchImpl::getFullVideoAdapterInfo(bool indent)
     grt::DictListRef entries = wmiQuery(session, "select * from Win32_VideoController");
     wmiCloseSession(session);
 
+    // Try finding the first video controller that is marked as being online. This doesn't work
+    // reliably all the time, though. So if we don't find such a controller we explicitly ask for
+    // the first one in the system, which should give most of the time the proper result.
     size_t count = entries->count();
+    grt::DictRef entry;
+    
     for (size_t i = 0; i < count; i++)
     {
-      grt::DictRef entry(entries[i]);
+      entry = entries[i];
+      if (entry.get_string("Availability", "0") == "3")
+        break;
+    }
 
-      if (entry.get_string("Availability", "0") != "3")
-        continue;
+    if (!entry.is_valid() && count > 0)
+      entry = entries[0];
 
-      result << tab << "Active video adapter (" << i << "): " << entry.get_string("Caption", "") << '\n';
+    if (entry.is_valid())
+    {
+      result << tab << "Active video adapter " << entry.get_string("Caption", "unknown") << '\n';
       std::string value = entry.get_string("AdapterRAM", "");
       if (value.size() > 0)
       {
@@ -1975,13 +1986,19 @@ std::string WorkbenchImpl::getFullVideoAdapterInfo(bool indent)
       result << tab << "Driver version: " << entry.get_string("DriverVersion", "unknown") << '\n';
       result << tab << "Installed display drivers: " << entry.get_string("InstalledDisplayDrivers", "unknown") << '\n';
     }
+    else
+    {
+      result << "No video adapter info available\n";
+    }
 #else
     // TODO: other OSes go here.
+    result << "No video adapter info available\n";
 #endif
   }
   catch (...)
   {
     // If for some reason (e.g. insufficient rights) the retrieval fails then we return an empty list.
+    result << "No video adapter info available\n";
   }
   return result.str();
 }
