@@ -71,8 +71,9 @@ GRTManager::GRTManager(bool threaded, bool verbose)
 : _has_unsaved_changes(false), _threaded(threaded), _verbose(verbose)
 {
   _globals_tree_soft_lock_count= 0;
-  _current_idle_serial = 0;
-  
+
+  _current_idle_signal = 0;
+
   init_all();
 
   _idle_mutex= g_mutex_new();
@@ -379,15 +380,15 @@ void GRTManager::show_message(const std::string &title, const std::string &messa
 }
 
 
-void GRTManager::initialize(const std::string &loader_module_path)
+void GRTManager::initialize(bool init_python, const std::string &loader_module_path)
 {
   IntegerRef result;
 
   _dispatcher->start(_dispatcher);
 
   load_structs();
-    
-  init_module_loaders(loader_module_path);
+
+  init_module_loaders(loader_module_path, init_python);
 
 #ifdef _WIN32
   add_python_module_dir(_grt, _basedir + "\\python");
@@ -414,6 +415,33 @@ bool GRTManager::initialize_shell(const std::string &shell_type)
   return true;
 }
 
+/**
+ * Cancels all pending idle tasks. Useful if their execution is no longer necessary and can even cause
+ * a crash because the used objects are going soon.
+ * Returns true if the task could be completed, false if the manager is currently in in idle execution.
+ * Warning: canceling idle tasks unconditionally might lead to other problems, so use with extreme care.
+ */
+bool GRTManager::cancel_idle_tasks()
+{
+//   { TODO
+//     GMutexLock disp_map_mutex(_disp_map_mutex);
+//     for (DispMap::iterator i = _disp_map.begin(), i_end = _disp_map.end(); i != i_end; ++i)
+//       i->first->cancel_all_tasks();
+//   }
+
+  if (_idle_blocked)
+    return false;
+
+  block_idle_tasks(); // TODO: use idle mutex.
+  GMutexLock lock(_idle_mutex);
+  _current_idle_signal = 0;
+  _idle_signals[0].disconnect_all_slots();
+  _idle_signals[1].disconnect_all_slots();
+
+  unblock_idle_tasks();
+
+  return true;
+}
 
 void GRTManager::perform_idle_tasks()
 {
@@ -438,16 +466,23 @@ void GRTManager::perform_idle_tasks()
 
   if (!_idle_blocked)
   {
-    block_idle_tasks();
+    if (!_idle_signals[_current_idle_signal].empty())
+    {
+      block_idle_tasks(); // TODO: that's not thread safe, why isn't the idle mutex used.
 
-    int emited_group = _current_idle_serial;
-    ++_current_idle_serial;
-    // emit everything
-    _idle_signal();
+      int signal_to_emit = 0;
+      {
+        GMutexLock lock(_idle_mutex);
+        signal_to_emit = _current_idle_signal;
+        _current_idle_signal = _current_idle_signal ? 0 : 1;
+      }
 
-    _idle_signal.disconnect(emited_group);
+      _idle_signals[signal_to_emit]();
 
-    unblock_idle_tasks();
+      _idle_signals[signal_to_emit].disconnect_all_slots();
+
+      unblock_idle_tasks();
+    }
   }
 }
 
@@ -456,20 +491,32 @@ boost::signals2::connection GRTManager::run_once_when_idle(const boost::function
 {
   if (!slot)
     throw std::invalid_argument("Adding null slot for idle");
+
   GMutexLock lock(_idle_mutex);
-  return _idle_signal.connect(_current_idle_serial, slot);
+  return _idle_signals[_current_idle_signal].connect(slot);
 }
 
 
+void GRTManager::run_once_when_idle(base::trackable *owner, const boost::function<void ()> &slot)
+{
+  if (!slot)
+    throw std::invalid_argument("Adding null slot for idle");
+
+  GMutexLock lock(_idle_mutex);
+  owner->track_connection(_idle_signals[_current_idle_signal].connect(slot));
+}
+
+
+
 void GRTManager::block_idle_tasks()
 {
-  _idle_blocked= true;
+  _idle_blocked++;
 }
 
 
 void GRTManager::unblock_idle_tasks()
 {
-  _idle_blocked= false;
+  _idle_blocked--;
 }
 
 
@@ -689,11 +736,11 @@ bool GRTManager::load_structs()
 
 
 
-bool GRTManager::init_module_loaders(const std::string &loader_module_path)
+bool GRTManager::init_module_loaders(const std::string &loader_module_path, bool init_python)
 {
   if (_verbose)
     _shell->write_line(_("Initializing Loaders..."));
-  if (!init_loaders(loader_module_path))
+  if (!init_loaders(loader_module_path, init_python))
     _shell->write_line(_("Failed initializing Loaders."));
   
   return true;
@@ -758,7 +805,7 @@ void GRTManager::rescan_modules()
 }
 
 
-bool GRTManager::init_loaders(const std::string &loader_module_path)
+bool GRTManager::init_loaders(const std::string &loader_module_path, bool init_python)
 {
   try
   {
@@ -770,16 +817,19 @@ bool GRTManager::init_loaders(const std::string &loader_module_path)
     _shell->write_line(strfmt("Error initializing Lua loader: %s", exc.what()));
   }
   
-  try
+  if (init_python)
   {
-    if (grt::init_python_support(_grt, loader_module_path))
+    try
     {
-      if (_verbose) _shell->write_line(_("Python loader initialized."));
+      if (grt::init_python_support(_grt, loader_module_path))
+      {
+        if (_verbose) _shell->write_line(_("Python loader initialized."));
+      }
+    }
+    catch (std::exception &exc)
+    {
+      _shell->write_line(strfmt("Error initializing Python loader: %s", exc.what()));
     }
-  }
-  catch (std::exception &exc)
-  {
-    _shell->write_line(strfmt("Error initializing Python loader: %s", exc.what()));
   }
 
   return true;
