@@ -59,8 +59,8 @@ public:
   void write(const std::string &text);
   void writef(const char *fmt, ...);
 
-  void set_output_handler(const sigc::slot<void,const std::string&> &slot);
-  void set_ready_handler(const sigc::slot<void,const std::string&> &slot);
+  void set_output_handler(const boost::function<void (const std::string&)> &slot);
+  void set_ready_handler(const boost::function<void (const std::string&)> &slot);
 
   void flush_shell_output();
 
@@ -95,9 +95,9 @@ protected:
   std::list<std::string> _history; // most recent first
   std::list<std::string>::iterator _history_ptr;
 
-  sigc::slot<void,const std::string&> _ready_slot;
+  boost::function<void (const std::string&)> _ready_slot;
 
-  sigc::slot<void, const std::string&> _output_slot;
+  boost::function<void (const std::string&)> _output_slot;
 
   GStaticMutex _text_queue_mutex;
   std::list<std::string> _text_queue;
