@@ -24,6 +24,7 @@
 #include <algorithm>
 #endif
 
+#include "base/file_utilities.h"
 #include "icon_manager.h"
 #include "common.h"
 
@@ -124,6 +125,15 @@ std::string IconManager::get_icon_path(const std::string &file)
   {
     std::string path= _basedir + G_DIR_SEPARATOR + *i + G_DIR_SEPARATOR + file;
 
+#ifdef __APPLE__
+    std::string mac_path = base::strip_extension(path) + "_mac.png";
+    if (g_file_test(mac_path.c_str(), G_FILE_TEST_EXISTS))
+    {
+      _icon_paths[file]= mac_path;
+      return mac_path;
+    } 
+#endif
+    
     if (g_file_test(path.c_str(), G_FILE_TEST_EXISTS))
     {
       _icon_paths[file]= path;
