@@ -56,7 +56,7 @@ public:
   operator T* () { return _ptr; }
 };
 
-
+// Important: this functions works only for model objects, not live objects!
 db_mgmt_RdbmsRef get_rdbms_for_db_object(const ::grt::ValueRef &object)
 {
   GrtObjectRef parent= GrtObjectRef::cast_from(object);
@@ -87,6 +87,164 @@ db_SimpleDatatypeRef CatalogHelper::get_datatype(grt::ListRef<db_SimpleDatatype>
   return db_SimpleDatatypeRef();
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Parses the given version string into its components and returns a GRT version class.
+ * Unspecified components are set to -1 to allow for fuzzy comparisons.
+ */
+GrtVersionRef CatalogHelper::parse_version(grt::GRT *grt, const std::string &target_version)
+{
+  int major = 0, minor = -1, release = -1, build = -1;
+  
+  sscanf(target_version.c_str(), "%i.%i.%i.%i", &major, &minor, &release, &build);
+  
+  GrtVersionRef version(grt);
+  version->name("Version");
+  version->majorNumber(major);
+  version->minorNumber(minor);
+  version->releaseNumber(release);
+  version->buildNumber(build);
+  
+  return version;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Compares the given version numbers to see if a is equal to b.
+ * In order to support wildcards, missing values are interpreted as matching, e.g. 5 is equal to 5.1
+ * and 5.1 is equal to 5.1.1.
+ */
+static bool version_equal(GrtVersionRef a, GrtVersionRef b)
+{
+  // Major version number is always there.
+  if (a->majorNumber() != b->majorNumber())
+    return false;
+  
+  if (a->minorNumber() == -1 || b->minorNumber() == -1)
+    return true;
+  
+  if (a->minorNumber() != b->minorNumber())
+    return false;
+
+  if (a->releaseNumber() == -1 || b->releaseNumber() == -1)
+    return true;
+  
+  if (a->releaseNumber() != b->releaseNumber())
+    return false;
+
+  if (a->buildNumber() == -1 || b->buildNumber() == -1)
+    return true;
+
+  if (a->buildNumber() != b->buildNumber())
+    return false;
+
+  return true;
+}
+
+/**
+ * This test is similar to the one above (except that it tests for greater-than relations).
+ * For this however we have to treat unset values differently to handle cases like
+ * 5 > 5.1 (false) or 5.1 > 5 (true) correctly.
+ */
+static bool version_greater(GrtVersionRef a, GrtVersionRef b)
+{
+  if (a->majorNumber() > b->majorNumber()) // Major number should always be set.
+    return true;
+
+  if (a->majorNumber() == b->majorNumber())
+  {
+    if (a->minorNumber() == -1) // An unset value can never be larger than anything else.
+      return false;
+    
+    if (b->minorNumber() == -1) // An unset value is always smaller than any other set value.
+        return true;
+    
+    if (a->minorNumber() > b->minorNumber())
+      return true;
+    
+    // Same approach for release + build numbers.
+    if (a->minorNumber() == b->minorNumber())
+    {
+      if (a->releaseNumber() == -1)
+        return false;
+
+      if (b->releaseNumber() == -1)
+        return true;
+
+      if (a->releaseNumber() > b->releaseNumber())
+        return true;
+
+      if (a->releaseNumber() == b->releaseNumber())
+      {
+        if (a->buildNumber() == -1)
+          return false;
+
+        if (b->buildNumber() == -1)
+          return true;
+
+        if (a->buildNumber() > b->buildNumber())
+          return true;
+      }
+      return false;
+    }
+    return false;
+  }
+
+  return false;
+}
+
+
+bool CatalogHelper::is_type_valid_for_version(const db_SimpleDatatypeRef &type, const GrtVersionRef &target_version)
+{
+  std::string validity = type->validity();
+  GrtVersionRef valid_version;
+  if (!validity.empty())
+  {
+    bool match = false;
+    switch (validity[0])
+    {
+      case '<':
+        if (validity[1] == '=')
+        {
+          valid_version = CatalogHelper::parse_version(type.get_grt(), validity.substr(2));
+          if (version_equal(target_version, valid_version) || version_greater(valid_version, target_version))
+            match = true;
+        }
+        else
+        {
+          valid_version = CatalogHelper::parse_version(type.get_grt(), validity.substr(1));
+          if (version_greater(valid_version, target_version))
+            match = true;
+        }
+        break;
+      case '>':
+        if (validity[1] == '=')
+        {
+          valid_version = CatalogHelper::parse_version(type.get_grt(), validity.substr(2));
+          if (version_equal(target_version, valid_version) || version_greater(target_version, valid_version))
+            match = true;
+        }
+        else
+        {
+          valid_version = CatalogHelper::parse_version(type.get_grt(), validity.substr(1));
+          if (version_greater(target_version, valid_version))
+            match = true;
+        }
+        break;
+      case '=':
+        valid_version = CatalogHelper::parse_version(type.get_grt(), validity.substr(1));
+        if (version_equal(target_version, valid_version))
+          match = true;
+        break;
+    }
+    return match;
+  }
+  return true;
+}
+
+
 
 std::string CatalogHelper::dbobject_to_dragdata(const db_DatabaseObjectRef &object)
 {
@@ -321,7 +479,7 @@ db_IndexRef TableHelper::create_index_for_fk(grt::GRT *grt, const db_ForeignKeyR
 }
 
 
-db_IndexRef TableHelper::find_index_usable_by_fk(const db_ForeignKeyRef &fk)
+db_IndexRef TableHelper::find_index_usable_by_fk(const db_ForeignKeyRef &fk, const db_IndexRef &other_than)
 {
   size_t column_count = fk->columns().count();
   db_TableRef table(db_TableRef::cast_from(fk->owner()));
@@ -329,6 +487,9 @@ db_IndexRef TableHelper::find_index_usable_by_fk(const db_ForeignKeyRef &fk)
   for (size_t c= table->indices().count(), i= 0; i < c; i++)
   {
     db_IndexRef index(table->indices()[i]);
+
+    if (index == other_than)
+      continue;
     
     if (index->columns().count() == column_count)
     {
@@ -418,41 +579,80 @@ db_ForeignKeyRef TableHelper::create_empty_foreign_key(grt::GRT *grt, const db_T
 
   table->foreignKeys().insert(fk);
 
+  /* don't always create an index for the FK, it should only be created if there are no other indexes that match it
+   the check for indexes should be done every time the FK columns are edited
   db_IndexRef index(create_index_for_fk(grt, fk));
   fk->index(index);
   table->indices().insert(index);
-
+  */
   undo.end(_("Create Foreign Key"));
 
   return fk;
 }
 
 
-void TableHelper::update_foreign_key_index(grt::GRT *grt, const db_ForeignKeyRef &fk)
+void TableHelper::update_foreign_key_index(const db_ForeignKeyRef &fk)
 {
   //! todo: add undo group
 
+  db_TableRef table(fk->owner());
   db_IndexRef index(fk->index());
-  
-  if (!index.is_valid())
-  {
-    g_warning("ForeignKey %s has no index", fk->name().c_str());
-    return;
-  }
 
-  // recreate index columns
-  while (index->columns().count() > 0)
-    index->columns().remove(0);
-
-  ListRef<db_Column> fk_columns(fk->columns());
-  for (size_t n= 0, count= fk_columns.count(); n < count; ++n)
+  if (index.is_valid())
   {
-    db_ColumnRef column(fk_columns.get(n));
-    db_IndexColumnRef index_column(grt->create_object<db_IndexColumn>(index.get_metaclass()->get_member_type("columns").content.object_class));
-    index_column->owner(index);
+    if (find_index_usable_by_fk(fk, index).is_valid())
+    {
+      // the index from this FK is redundant, so remove it
+      fk->index(db_IndexRef());
+      table->indices().remove_value(index);
+      return;
+    }
+
+    // make sure that the columns in the index match the ones in the FK
+
+    // remove columns that are gone from the FK
+    for (int i = index->columns().count()-1; i >= 0; --i)
+    {
+      if (fk->columns().get_index(index->columns()[i]) == grt::BaseListRef::npos)
+        index->columns().remove(i);
+    }
+
+    // recreate index columns
+    while (index->columns().count() > 0)
+      index->columns().remove(0);
+
+    ListRef<db_Column> fk_columns(fk->columns());
+    for (size_t n= 0, count= fk_columns.count(); n < count; ++n)
+    {
+      db_ColumnRef column(fk_columns.get(n));
+      db_IndexColumnRef index_column(fk->get_grt()->create_object<db_IndexColumn>(index.get_metaclass()->get_member_type("columns").content.object_class));
+      index_column->owner(index);
       //"name", column->name().valueptr(),
-    index_column->referencedColumn(column);
-    index->columns().insert(index_column);
+      index_column->referencedColumn(column);
+      index->columns().insert(index_column);
+    }
+
+    if (index->columns().count() == 0)
+    {
+      // the index is empty, remove it
+      fk->index(db_IndexRef());
+      table->indices().remove_value(index);
+      return;
+    }
+  }
+  else
+  {
+    // check if there's a suitable index
+    if (find_index_usable_by_fk(fk).is_valid())
+      return; // suitable index exists, no action needed
+
+    // create a new index for the FK if it's valid at all
+    if (fk->columns().count() > 0)
+    {
+      index = create_index_for_fk(table->get_grt(), fk);
+      fk->index(index);
+      table->indices().insert(index);
+    }
   }
 }
 
@@ -969,8 +1169,8 @@ void CatalogHelper::apply_defaults(db_mysql_ColumnRef column)
 
 void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_engine)
 {
-  cat->defaultCharacterSetName("latin1");
-  cat->defaultCollationName("latin1_swedish_ci");
+  cat->defaultCharacterSetName("utf8");
+  cat->defaultCollationName("utf8_general_ci");
 
   for(size_t i= 0, schemata_count= cat->schemata().count(); i < schemata_count; i++)
   {
@@ -978,7 +1178,6 @@ void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_
 
     if(strlen(schema->defaultCharacterSetName().c_str()) == 0)
       schema->defaultCharacterSetName(cat->defaultCharacterSetName());
-
     if(strlen(schema->defaultCollationName().c_str()) == 0)
       schema->defaultCollationName(get_cs_def_collation(schema->defaultCharacterSetName().c_str()));
 
@@ -989,8 +1188,11 @@ void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_
       if(strlen(table->defaultCharacterSetName().c_str()) == 0)
         table->defaultCharacterSetName(schema->defaultCharacterSetName());
 
+      if(table->defaultCharacterSetName() == schema->defaultCharacterSetName())
+      {
       if(strlen(table->defaultCollationName().c_str()) == 0)
         table->defaultCollationName(schema->defaultCollationName());
+      }
       
       if(strlen(table->tableEngine().c_str()) == 0)
         table->tableEngine(default_engine.empty()?"InnoDB":default_engine);
@@ -1015,3 +1217,288 @@ void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_
     }
   }
 }
+
+static int pcre_compile_exec(const char *pattern, const char *str, int *patres, int patresnum)
+{
+  const char *errptr;
+  int erroffs;
+  int c;
+  pcre *patre= pcre_compile(pattern, 0, &errptr, &erroffs, NULL);
+  if (!patre)
+    throw std::logic_error("error compiling regex "+std::string(errptr));
+
+  c= pcre_exec(patre, NULL, str, strlen(str), 0, 0, patres, patresnum);
+  pcre_free(patre);
+
+  return c;
+}
+
+static db_SimpleDatatypeRef findType(const grt::ListRef<db_SimpleDatatype> &types, const GrtVersionRef &target_version, const std::string &name)
+{
+  for (size_t c= types.count(), i= 0; i < c; i++)
+  {
+    if (g_strcasecmp(types[i]->name().c_str(), name.c_str())==0)
+    {
+      if (!target_version.is_valid() || CatalogHelper::is_type_valid_for_version(types[i], target_version))
+        return types[i];
+    }
+  }
+  return db_SimpleDatatypeRef();
+}
+
+static bool parseTypeDefinition(const std::string &type,
+                                const GrtVersionRef &targetVersion,
+                                const grt::ListRef<db_SimpleDatatype> &typeList,
+                                db_SimpleDatatypeRef &simpleType,
+                                int &precision,
+                                int &scale,
+                                int &length,
+                                std::string &explicitParams)
+{
+  auto_array_ptr<char> buffer(new char[type.size()+1]);
+  int patres[21];
+  int c;
+    
+  /* split the datatype name from the parameters
+   * (\w+)\s*(\(.*\))?
+   *
+   * 1.. TYPE
+   * 2.. params (optional)
+   */
+  
+  c= pcre_compile_exec("^(\\w+)\\s*(\\(.*\\))?\\s*$", type.c_str(), patres, sizeof(patres)/sizeof(int));
+  
+  if (c > 0 &&
+      pcre_copy_substring(type.c_str(), patres, c, 1, buffer, static_cast<int>(type.size()+1)) > 0)
+  {
+    // find the simple datatype with this type_name
+    simpleType= findType(typeList, targetVersion, std::string(buffer));
+    
+    if (simpleType.is_valid())
+    {
+      std::string params;
+      std::string param1;
+      std::string param2;
+      
+      // split the parameter into its components
+      
+      if (pcre_copy_substring(type.c_str(), patres, c, 2, buffer, static_cast<int>(type.size()+1)) > 0)
+        params= buffer;
+      
+      switch (*simpleType->parameterFormatType())
+      {
+        case 0: // no params
+          if (!params.empty())
+            return false;
+          break;
+          
+        case 1: // (n)
+          c= pcre_compile_exec("^\\((\\d+)\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+          }
+          else
+            return false;
+          break;
+          
+        case 2: // [(n)]
+          c= pcre_compile_exec("^(\\((\\d+)\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+          }
+          break;
+          
+        case 3: // (m, n)
+          c= pcre_compile_exec("^\\((\\d+),\\s*(\\d+)\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+            
+            if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
+              param2= buffer;
+            else
+              return false;
+          }
+          else
+            return false;
+          break;
+          
+        case 4: // (m[,n])
+          c= pcre_compile_exec("^\\((\\d+)(,\\s*(\\d+))?\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+            if (pcre_copy_substring(params.c_str(), patres, c, 3, buffer, static_cast<int>(params.size()+1)) > 0)
+              param2= buffer;
+          }
+          else
+            return false;
+          break;
+          
+        case 5: // [(m,n)]
+          c= pcre_compile_exec("^(\\((\\d+)(,\\s*(\\d+))\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+            if (pcre_copy_substring(params.c_str(), patres, c, 4, buffer, static_cast<int>(params.size()+1)) > 0)
+              param2= buffer;
+            else
+              return false;
+          }
+          break;
+          
+        case 6: // [(m[,n])]
+          c= pcre_compile_exec("^(\\((\\d+)(,\\s*(\\d+))?\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+            if (pcre_copy_substring(params.c_str(), patres, c, 4, buffer, static_cast<int>(params.size()+1)) > 0)
+              param2= buffer;
+          }
+          break;
+          
+        case 10: // ('a','b','c')
+          if (params.empty())
+            return false;
+                
+          c= pcre_compile_exec("^\\((.*?)\\)?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          // For now we just keep the parameter list as a whole. If necessary this can be split later for other uses.
+          param1= buffer;
+          // validate format
+          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            std::list<std::string> tokens;
+            if (!bec::tokenize_string_list(std::string(buffer), '\'', true, tokens))
+              return false;
+          }
+          else
+            return false;
+          break;
+      }
+      
+      // get precision
+      if (*simpleType->numericPrecision() != bec::EMPTY_TYPE_PRECISION)
+      {
+        if (!param1.empty())
+          precision= atoi(param1.c_str());
+        
+        // get scale
+        if (*simpleType->numericScale() != bec::EMPTY_TYPE_SCALE)
+        {
+          if (!param2.empty())
+            scale= atoi(param2.c_str());
+        }
+      }
+      // get length
+      else if (simpleType->characterMaximumLength() != bec::EMPTY_TYPE_MAXIMUM_LENGTH 
+               || simpleType->characterOctetLength()   != bec::EMPTY_TYPE_OCTET_LENGTH)
+      {
+        if (!param1.empty())
+          length= atoi(param1.c_str());
+      }
+      else if (*simpleType->parameterFormatType() == 10)
+        explicitParams= param1;
+    }
+    else
+      return false; // unknown type
+  }
+  else
+    return false; // bad format
+  
+  return true;
+}
+
+bool bec::parseType(const std::string &type,
+    const GrtVersionRef &targetVersion,
+    const grt::ListRef<db_SimpleDatatype> &typeList,
+    const grt::ListRef<db_UserDatatype>& user_types,
+    const grt::ListRef<db_SimpleDatatype>& default_type_list,
+    db_SimpleDatatypeRef &simpleType,
+    db_UserDatatypeRef& userType,
+    int &precision,
+    int &scale,
+    int &length,
+    std::string &datatypeExplicitParams)
+{
+    if (user_types.is_valid())
+    {
+        std::string::size_type argp= type.find('(');
+        std::string typeName= type;
+
+        if (argp != std::string::npos)
+            typeName= type.substr(0, argp);
+
+        // 1st check if this is a user defined type
+        for (size_t c= user_types.count(), i= 0; i < c; i++)
+        {
+            db_UserDatatypeRef utype(user_types[i]);
+
+            if (g_strcasecmp(utype->name().c_str(), typeName.c_str()) == 0)
+            {
+                userType= utype;
+                break;
+            }
+        }
+    }
+
+    if (userType.is_valid())
+    {
+        // if the type spec has an argument, we replace the arguments from the type definition
+        // with the one provided by the user
+        std::string finalType= userType->sqlDefinition();
+        std::string::size_type tp;
+        bool overriden_args= false;
+
+        if ((tp= type.find('(')) != std::string::npos) // has user specified args
+        {
+            std::string::size_type p= finalType.find('('); 
+            if (p != std::string::npos) // strip the original args
+                finalType= finalType.substr(0, p);
+
+            // extract the user spcified args and append to the specification
+            finalType.append(type.substr(tp));
+
+            overriden_args= true;
+        }
+
+        // parse usertype definition    
+        if (!parseTypeDefinition(finalType, targetVersion, typeList.is_valid() ? typeList : default_type_list, simpleType,
+            precision, scale, length, datatypeExplicitParams))
+            return false;
+
+        simpleType = db_SimpleDatatypeRef();
+        userType = userType;
+        if (!overriden_args)
+        {
+            precision = bec::EMPTY_COLUMN_PRECISION;
+            scale = bec::EMPTY_COLUMN_SCALE;
+            length = bec::EMPTY_COLUMN_LENGTH;
+            datatypeExplicitParams = "";
+        }
+    }
+    else
+    {
+        if (!parseTypeDefinition(type, targetVersion, typeList.is_valid() ? typeList : default_type_list, simpleType,
+            precision, scale, length, datatypeExplicitParams))
+            return false;
+
+        userType = db_UserDatatypeRef();
+    }
+    return true;
+}
