@@ -57,14 +57,16 @@ namespace bec {
   {
     static void apply_defaults(db_mysql_CatalogRef catalog, std::string default_engine);
     static db_SimpleDatatypeRef get_datatype(grt::ListRef<db_SimpleDatatype> types, const std::string &name);
+    static GrtVersionRef parse_version(grt::GRT *grt, const std::string &version);
+    static bool is_type_valid_for_version(const db_SimpleDatatypeRef &type, const GrtVersionRef &target_version);
 
     static std::string dbobject_list_to_dragdata(const std::list<db_DatabaseObjectRef> &object);
     static std::list<db_DatabaseObjectRef> dragdata_to_dbobject_list(const db_CatalogRef &catalog, const std::string &object);
 
     static std::string dbobject_to_dragdata(const db_DatabaseObjectRef &object);
     static db_DatabaseObjectRef dragdata_to_dbobject(const db_CatalogRef &catalog, const std::string &object);
-    private:
-      static void apply_defaults(db_mysql_ColumnRef column);
+  private:
+    static void apply_defaults(db_mysql_ColumnRef column);
 
   };
 
@@ -98,10 +100,10 @@ namespace bec {
                                                      const grt::DictRef &options);
 
     static db_IndexRef create_index_for_fk(grt::GRT *grt, const db_ForeignKeyRef &fk, const size_t max_len = 64);
-    static db_IndexRef find_index_usable_by_fk(const db_ForeignKeyRef &fk);
+    static db_IndexRef find_index_usable_by_fk(const db_ForeignKeyRef &fk, const db_IndexRef &other_than = db_IndexRef());
     static std::string generate_foreign_key_name();
     static db_ForeignKeyRef create_empty_foreign_key(grt::GRT *grt, const db_TableRef &table, const std::string &name);
-    static void update_foreign_key_index(grt::GRT *grt, const db_ForeignKeyRef &fk);
+    static void update_foreign_key_index(const db_ForeignKeyRef &fk);
 
     static bool rename_foreign_key(const db_TableRef &table, db_ForeignKeyRef &fk, const std::string &new_name);
     
@@ -195,6 +197,17 @@ namespace bec {
     Schema_action sa(cat, rdbms);
     ct::for_each<ct::Schemata>(cat, sa);
   }
+  bool parseType(const std::string &type,
+    const GrtVersionRef &target_version,
+    const grt::ListRef<db_SimpleDatatype> &typeList,
+    const grt::ListRef<db_UserDatatype>& user_types,
+    const grt::ListRef<db_SimpleDatatype>& default_type_list,
+    db_SimpleDatatypeRef &simpleType,
+    db_UserDatatypeRef& userType,
+    int &precision,
+    int &scale,
+    int &length,
+    std::string &datatypeExplicitParams);
 
 };
 
