@@ -31,6 +31,7 @@
 #include "mforms/code_editor.h"
 
 #include "sqlide/sql_editor_be.h"
+#include "db_object_helpers.h"
 
 #define DEFAULT_COLLATION_CAPTION "default collation"
 
@@ -94,14 +95,14 @@ bool DBObjectEditorBE::can_close()
 {
   bool res = BaseEditor::can_close();
 
-  if (res && is_editing_live_object() && !on_apply_changes_to_live_object.empty())
+  if (!res || (is_editing_live_object() && !on_apply_changes_to_live_object.empty()))
   {
-    bool is_object_modified= on_apply_changes_to_live_object(this, true);
+    bool is_object_modified= !res || on_apply_changes_to_live_object(this, true);
     if (is_object_modified)
     {
       int user_choice= mforms::Utilities::show_warning(
-        _("Object was changed"),
-        _("Do you want to apply changes made to this object?"),
+        base::strfmt(_("Object %s was changed"), get_name().c_str()),
+        base::strfmt(_("Do you want to apply changes made to %s?"), get_name().c_str()),
         _("Apply"),
         _("Cancel"),
         _("Ignore")
@@ -109,15 +110,16 @@ bool DBObjectEditorBE::can_close()
 
       if (mforms::ResultOk == user_choice)
       {
-        on_apply_changes_to_live_object(this, false);
+        res = on_apply_changes_to_live_object(this, false);
       }
       else if (mforms::ResultCancel == user_choice)
       {
         res= false;
       }
+      else
+        res= true;
     }
   }
-
   return res;
 }
 
@@ -147,6 +149,26 @@ std::string DBObjectEditorBE::get_object_type()
 }
 
 
+GrtVersionRef DBObjectEditorBE::get_rdbms_target_version()
+{
+  if (is_editing_live_object())
+    return get_rdbms()->version();
+  else
+  {
+    if (get_catalog()->version().is_valid())
+      return get_catalog()->version();
+    
+    std::string target_version = get_grt_manager()->get_app_option_string("DefaultTargetMySQLVersion");
+    if (target_version.empty())
+      target_version = "5.5";
+    
+    GrtVersionRef version = CatalogHelper::parse_version(get_grt_manager()->get_grt(), target_version);
+
+    return version;
+  }
+}
+
+
 std::string DBObjectEditorBE::get_name()
 {
   return get_dbobject()->name();
@@ -483,18 +505,6 @@ Sql_editor::Ref DBObjectEditorBE::get_sql_editor()
   return _sql_editor;
 }
 
-
-bool DBObjectEditorBE::is_editor_dirty()
-{
-  if (_sql_editor)
-  {
-    mforms::CodeEditor* code_editor = _sql_editor->get_editor_control();
-    return code_editor->is_dirty();
-  }
-  return false;
-}
-
-
 void DBObjectEditorBE::set_sql(const std::string &sql, bool sync, const db_DatabaseObjectRef &template_obj, const std::string &comment)
 {
   _sql_parser_log.clear();
