@@ -35,6 +35,8 @@ namespace bec {
   public:
     RoutineGroupEditorBE(GRTManager *grtm, const db_RoutineGroupRef &group, const db_mgmt_RdbmsRef &rdbms);
 
+    virtual std::string get_title();
+
     virtual db_DatabaseObjectRef get_dbobject() { return get_routine_group(); }
     virtual db_RoutineGroupRef get_routine_group() { return _group; }
 
@@ -53,6 +55,9 @@ namespace bec {
     virtual void delete_routine_with_name(const std::string& str);
     virtual void remove_routine_by_index(size_t index);
     virtual void append_routine_with_id(const std::string& id);
+  
+  private:
+    std::string set_routine_newlines(const std::string &routine);
 
   protected:
     virtual std::string get_object_type();
