@@ -25,14 +25,14 @@
 
 #include "db_object_helpers.h"
 #include "grtpp_util.h"
+#include "grt/clipboard.h"
 #include "grt/validation_manager.h"
 
 #include "base/string_utilities.h"
 #include "base/file_utilities.h"
+#include "objimpl/db.query/db_query_EditableResultset.h"
 
-#ifdef __APPLE__
 #include "base/log.h"
-#endif
 
 #include "mforms/utilities.h"
 
@@ -40,7 +40,7 @@ using namespace bec;
 using namespace grt;
 using namespace base;
 
-DEFAULT_LOG_DOMAIN("Table Editor backend")
+DEFAULT_LOG_DOMAIN("TableEditorBE")
 
 /*
  The big mess of user datatypes, synonyms and column flags.
@@ -98,11 +98,17 @@ std::vector<std::string> TableColumnsListBE::get_datatype_names()
   types.push_back("DATETIME");       
   types.push_back("BLOB");
   
+  GrtVersionRef target_version(_owner->get_rdbms_target_version());
   std::vector<db_SimpleDatatypeRef> sorted_types;
   grt::ListRef<db_SimpleDatatype> stypes(_owner->get_catalog()->simpleDatatypes());
   for (grt::ListRef<db_SimpleDatatype>::const_iterator iter= stypes.begin(); iter != stypes.end(); ++iter)
+  {
+    if (target_version.is_valid() && !bec::CatalogHelper::is_type_valid_for_version(*iter, target_version))
+      continue;
+    
     sorted_types.push_back(*iter);
-  
+  }
+
   // sort by group and name
   std::sort(sorted_types.begin(), sorted_types.end(), sort_simple_type);
   
@@ -1072,6 +1078,7 @@ MenuItemList TableColumnsListBE::get_popup_items_for_nodes(const std::vector<Nod
   sep.type = MenuSeparator;
 
   MenuItem item;
+
   item.caption = "Move Up";
   item.name    = "moveUpToolStripMenuItem";
   item.enabled = nodes.size() == 1;
@@ -1082,6 +1089,34 @@ MenuItemList TableColumnsListBE::get_popup_items_for_nodes(const std::vector<Nod
   item.enabled = nodes.size() == 1;
   items.push_back(item);
   items.push_back(sep);
+  
+  item.caption = "Copy";
+  item.name    = "copyColumnToolStripMenuItem";
+  item.enabled = nodes.size() > 0;
+  items.push_back(item);
+  
+  item.caption = "Cut";
+  item.name    = "cutColumnToolStripMenuItem";
+  item.enabled = nodes.size() > 0;
+  items.push_back(item);
+  
+  item.caption = "Paste";
+  item.name    = "pasteColumnToolStripMenuItem";
+  item.enabled = false;
+  bec::Clipboard *clip = _owner->get_grt_manager()->get_clipboard();
+  if (clip->is_data())
+  {
+    std::list<grt::ObjectRef> data = clip->get_data();
+    bool ok = true;
+    for (std::list<grt::ObjectRef>::iterator i = data.begin(); i != data.end(); ++i)
+    {
+      if (!db_ColumnRef::can_wrap(*i))
+        ok = false;
+    }
+    item.enabled = ok;
+  }
+  
+  items.push_back(item);
 
   item.caption = "Delete Selected Columns";
   item.name    = "deleteSelectedColumnsToolStripMenuItem";
@@ -1130,6 +1165,63 @@ bool TableColumnsListBE::activate_popup_item_for_nodes(const std::string &name,
       reorder(nodes.front(), row+1);
     }    
   }
+  else if (name == "copyColumnToolStripMenuItem")
+  {
+    bec::Clipboard *clip = _owner->get_grt_manager()->get_clipboard();
+    clip->clear();
+    for (std::vector<NodeId>::reverse_iterator iter= nodes.rbegin();
+         iter != nodes.rend(); ++iter)
+    {
+      if ((*iter)[0] < real_count())
+      {
+        db_ColumnRef col(_owner->get_table()->columns().get((*iter)[0]));
+        clip->append_data(col);
+      }
+    }
+    clip->set_content_description("Table Column");
+  }
+  else if (name == "cutColumnToolStripMenuItem")
+  {
+    bec::Clipboard *clip = _owner->get_grt_manager()->get_clipboard();
+    clip->clear();
+    for (std::vector<NodeId>::reverse_iterator iter= nodes.rbegin();
+         iter != nodes.rend(); ++iter)
+    {
+      if ((*iter)[0] < real_count())
+      {
+        db_ColumnRef col(_owner->get_table()->columns().get((*iter)[0]));
+        clip->append_data(col);
+      }
+    }
+    clip->set_content_description("Table Column");
+    for (std::vector<NodeId>::reverse_iterator iter= nodes.rbegin();
+         iter != nodes.rend(); ++iter)
+      delete_node(*iter);
+  }
+  else if (name == "pasteColumnToolStripMenuItem")
+  {
+    bec::Clipboard *clip = _owner->get_grt_manager()->get_clipboard();
+    if (clip->is_data())
+    {
+      AutoUndoEdit undo(_owner);
+      std::list<grt::ObjectRef> data = clip->get_data();
+      std::list<NodeId> added_cols;
+      for (std::list<grt::ObjectRef>::reverse_iterator i = data.rbegin(); i != data.rend(); ++i)
+      {
+        db_ColumnRef column(db_ColumnRef::cast_from(*i));
+        added_cols.push_back(_owner->duplicate_column(column));
+      }
+      
+      if (!nodes.empty())
+      {
+        for (std::list<NodeId>::const_iterator i = added_cols.begin(); i != added_cols.end(); ++i)
+          reorder(*i, nodes[0][0]+1);
+      }
+      refresh();
+      _owner->update_change_date();
+      undo.end(base::strfmt("Paste columns to table %s", _owner->get_name().c_str()));
+    }
+  }
   else if (name == "deleteSelectedColumnsToolStripMenuItem")
   {
     for (std::vector<NodeId>::reverse_iterator iter= nodes.rbegin();
@@ -2165,7 +2257,9 @@ NodeId FKConstraintListBE::add_column(const db_ColumnRef &column, const db_Colum
     fk->columns().insert(column);
     fk->referencedColumns().insert(refcolumn);
 
-    TableHelper::update_foreign_key_index(fk.get_grt(), fk); //!
+    // make sure that there's an index that matches the columns in the FK or
+    // that the one created for it is up to date
+    TableHelper::update_foreign_key_index(fk);
 
     _owner->update_change_date();
 
@@ -2206,7 +2300,7 @@ void FKConstraintListBE::remove_column(const NodeId &node)
   {
     fk->referencedColumns().remove(i);
   }
-  TableHelper::update_foreign_key_index(fk.get_grt(), fk);//!
+  TableHelper::update_foreign_key_index(fk);
 
   _owner->update_change_date();
   undo.end(
@@ -2407,7 +2501,7 @@ bool FKConstraintListBE::set_field(const NodeId &node, int column, const std::st
           
           get_columns()->refresh();
 
-          TableHelper::update_foreign_key_index(fk.get_grt(), fk); //!
+          TableHelper::update_foreign_key_index(fk);
 
           _owner->update_change_date();
           bec::ValidationManager::validate_instance(_owner->get_table(), "chk_fk_lgc");
@@ -2632,6 +2726,29 @@ NodeId TableEditorBE::add_column(const std::string &name)
 }
 
 
+NodeId TableEditorBE::duplicate_column(const db_ColumnRef &column)
+{
+  db_ColumnRef new_column = shallow_copy_object(column);
+
+  std::string name = new_column->name();
+  for (int i = 1;; i++)
+  {
+    if (!find_named_object_in_list(get_table()->columns(), name).is_valid())
+      break;
+    name = strfmt("%s_copy%i", name.c_str(), i);
+  }
+  if (name != *new_column->name())
+    new_column->name(name);
+  new_column->owner(get_table());
+  get_table()->addColumn(new_column);
+
+  bec::ValidationManager::validate_instance(new_column, CHECK_NAME);
+  bec::ValidationManager::validate_instance(_table, "columns-count");
+  
+  return NodeId(get_table()->columns().count()-1);
+}
+
+
 db_ColumnRef TableEditorBE::get_column_with_name(const std::string &name)
 {
   return grt::find_named_object_in_list<db_Column>(get_table()->columns(), name, "name");
@@ -2915,8 +3032,13 @@ Recordset::Ref TableEditorBE::get_inserts_model()
     _inserts_model= Recordset::create(_grtm);
     _inserts_model->show_apply_buttons(true);
     _inserts_model->data_storage(_inserts_storage);
-    _inserts_model->export_wizard = boost::bind(&TableEditorBE::show_export_wizard, this, (mforms::Form*)0);
     _inserts_model->refresh();
+    
+    _inserts_model->action_list().register_action("record_export",
+                                                  boost::bind(&TableEditorBE::show_export_wizard, this, (mforms::Form*)0));
+    _inserts_model->action_list().register_action("record_import",
+                                                  boost::bind(&TableEditorBE::show_import_wizard, this));
+
   }
   return _inserts_model;
 }
@@ -2953,6 +3075,25 @@ void TableEditorBE::show_export_wizard(mforms::Form *owner)
 
 //--------------------------------------------------------------------------------------------------
 
+void TableEditorBE::show_import_wizard()
+{
+  grt::BaseListRef args(_grtm->get_grt());
+  
+  db_TableRef table(get_table());
+  if (table.is_valid())
+  {
+    args.ginsert(grtwrap_editablerecordset(table, _inserts_model));
+
+    grt::Module *module = _grtm->get_grt()->get_module("SQLIDEUtils");
+    if (module)
+      module->call_function("importRecordsetDataFromFile", args);
+    else
+      log_error("Can't find module SQLIDEUtils for record importer");
+  }
+}
+                                                  
+//--------------------------------------------------------------------------------------------------
+
 Sql_editor::Ref TableEditorBE::get_sql_editor()
 {
   Sql_editor::Ref sql_editor= DBObjectEditorBE::get_sql_editor();
