@@ -47,18 +47,40 @@ void RolePrivilegeListBE::refresh()
 {
   _role_privilege= _owner->get_object_list()->get_selected_object_info();
 
-  _privileges.clear();
+  _privileges = grt::StringListRef();
   if (_role_privilege.is_valid())
   {
     grt::ListRef<db_mgmt_PrivilegeMapping> mappings(_owner->get_rdbms()->privilegeNames());
 
     for (size_t c= mappings.count(), i= 0; i < c; i++)
     {
-      if (_role_privilege->databaseObject().is_valid() &&
-        _role_privilege->databaseObject().is_instance(mappings[i]->structName()))
+      if (_role_privilege->databaseObject().is_valid())
       {
-        _privileges= mappings[i]->privileges();
-        break;
+        if (_role_privilege->databaseObject().is_instance(mappings[i]->structName()))
+        {
+          _privileges= mappings[i]->privileges();
+          break;
+        }
+      }
+      else if (!_role_privilege->databaseObjectType().empty())
+      {
+        std::string objectType;
+        if (_role_privilege->databaseObjectType() == "SCHEMA")
+          objectType = "db.mysql.Schema";
+        else if (_role_privilege->databaseObjectType() == "TABLE")
+          objectType = "db.mysql.Table";
+        else if (_role_privilege->databaseObjectType() == "ROUTINE")
+          objectType = "db.mysql.Routine";
+        else if (_role_privilege->databaseObjectType() == "FUNCTION")
+          objectType = "db.mysql.Routine";
+        else if (_role_privilege->databaseObjectType() == "PROCEDURE")
+          objectType = "db.mysql.Routine";
+        
+        if (objectType == *mappings[i]->structName())
+        {
+          _privileges= mappings[i]->privileges();
+          break;
+        }
       }
     }
   }
@@ -183,6 +205,31 @@ MenuItemList RoleObjectListBE::get_popup_items_for_nodes(const std::vector<NodeI
   MenuItemList items;
   
   MenuItem item;
+
+  item.enabled = true;
+  item.type    = MenuAction;
+  
+  db_RoleRef role(_owner->get_role());
+  if (role.is_valid() && role->owner().is_valid())
+  {
+    db_CatalogRef catalog(db_CatalogRef::cast_from(role->owner()));
+    
+    GRTLIST_FOREACH(db_Schema, catalog->schemata(), schema)
+    {
+      item.caption = base::strfmt(_("Add Schema '%s'"), (*schema)->name().c_str());
+      item.name    = base::strfmt("adds:%s", (*schema)->name().c_str());
+      items.push_back(item);
+      item.caption = base::strfmt(_("Add Tables '%s.*'"), (*schema)->name().c_str());
+      item.name    = base::strfmt("addt:%s", (*schema)->name().c_str());
+      items.push_back(item);
+    }
+  }
+  item.caption = "";
+  item.name    = "sep";
+  item.type    = MenuSeparator;
+  items.push_back(item);
+
+  item.type    = MenuAction;
   item.caption = _("Delete Selected");
   item.name    = "deleteObject";
   item.enabled = nodes.size() > 0;
@@ -199,6 +246,10 @@ bool RoleObjectListBE::activate_popup_item_for_nodes(const std::string &name, co
     for (std::vector<NodeId>::const_reverse_iterator node= nodes.rbegin(); node != nodes.rend(); ++node)
       _owner->remove_object(*node);
   }
+  else if (name.substr(0, 5) == "adds:")
+    _owner->add_object("SCHEMA", name.substr(5)+".*");
+  else if (name.substr(0, 5) == "addt:")
+    _owner->add_object("TABLE", name.substr(5)+".*");
   else
     return false;
   
@@ -218,7 +269,7 @@ bool RoleObjectListBE::get_field_grt(const NodeId &node, int column, grt::ValueR
     if (priv.is_valid() && priv->databaseObject().is_valid())
       value= priv->databaseObject()->name();
     else
-      value= grt::StringRef("");
+      value= grt::StringRef(base::strfmt("%s: %s", priv->databaseObjectType().c_str(), priv->databaseObjectName().c_str()));
     return true;
   }
   return false;
@@ -354,6 +405,22 @@ bool RoleEditorBE::add_dropped_objectdata(const std::string &data)
 }
 
 
+bool RoleEditorBE::add_object(const std::string &type, const std::string &name)
+{
+  db_RolePrivilegeRef priv(_grtm->get_grt());
+  priv->databaseObjectType(type);
+  priv->databaseObjectName(name);
+  priv->owner(get_role());
+  
+  AutoUndoEdit undo(this);
+  
+  get_role()->privileges().insert(priv);
+  
+  undo.end(strfmt(_("Add Object %s '%s' to Role '%s'"), type.c_str(), name.c_str(), get_name().c_str()));
+  return true;
+}
+
+
 bool RoleEditorBE::add_object(db_DatabaseObjectRef object)
 {
   grt::ListRef<db_mgmt_PrivilegeMapping> mappings(get_rdbms()->privilegeNames());
