@@ -113,6 +113,8 @@ void AutoCompleteCache::shutdown()
 
 AutoCompleteCache::~AutoCompleteCache()
 {
+  g_assert(_shutdown);
+
   delete _sqconn;
   g_mutex_free(_sqconn_mutex);
   g_mutex_free(_pending_mutex);
@@ -318,27 +320,26 @@ std::vector<std::string> AutoCompleteCache::get_matching_function_names(const st
   return std::vector<std::string>();
 }
 
-
+/**
+ *  Queries the database to check if the schema cache has been already loaded.
+ */
 bool AutoCompleteCache::refresh_schema_cache_if_needed(const std::string &schema)
 {
-  // Queries the database to check if teh schema cache has been already loaded
+  if (!_shutdown)
   {
-    if (!_shutdown)
+    // Ensures shutdown is not done while processing
+    base::GMutexLock sd_lock(_shutdown_mutex);
+    base::GMutexLock lock(_sqconn_mutex);
+    sqlite::query q(*_sqconn, "SELECT last_refresh FROM schemas WHERE name LIKE ? ESCAPE '\\' ");
+    q.bind(1, base::escape_sql_string(schema, true) + "%");
+    if (q.emit())
     {
-      // Ensures shutdown is not done while processing
-      base::GMutexLock sd_lock(_shutdown_mutex);
-      base::GMutexLock lock(_sqconn_mutex);
-      sqlite::query q(*_sqconn, "SELECT last_refresh FROM schemas WHERE name LIKE ? ESCAPE '\\' ");
-      q.bind(1, base::escape_sql_string(schema, true) + "%");
-      if (q.emit())
+      boost::shared_ptr<sqlite::result> matches(q.get_result());
+      // schema info is already loaded in cache
+      if (matches->get_int(0))
       {
-        boost::shared_ptr<sqlite::result> matches(q.get_result());
-        // schema info is already loaded in cache
-        if (matches->get_int(0))
-        {
-          log_debug3("schema %s is already cached\n", schema.c_str());
-          return false;
-        }
+        log_debug3("schema %s is already cached\n", schema.c_str());
+        return false;
       }
     }
   }
@@ -876,24 +877,21 @@ void AutoCompleteCache::update_table_columns(const std::string &schema, const st
 
 void AutoCompleteCache::add_pending_refresh(const std::string& task)
 {
+  if (!_shutdown)
   {
-    if (!_shutdown)
+    // Ensures shutdown is not done while processing
+    base::GMutexLock sd_lock(_shutdown_mutex);
+    base::GMutexLock lock(_pending_mutex);
+    if ((!task.empty() || !_schema_list_fetched) && std::find(_pending_refresh_schema.begin(), _pending_refresh_schema.end(), task)
+        == _pending_refresh_schema.end())
     {
-      // Ensures shutdown is not done while processing
-      base::GMutexLock sd_lock(_shutdown_mutex);
-      base::GMutexLock lock(_pending_mutex);
-      if (!_schema_list_fetched && std::find(_pending_refresh_schema.begin(), _pending_refresh_schema.end(), task)
-          == _pending_refresh_schema.end())
-      {
-        _pending_refresh_schema.push_back(task);
-      }
+      _pending_refresh_schema.push_back(task);
     }
   }
 
-  // Calls the creation of the worker thread, at this point the queue
-  // can't be empty so the thread is needed, and will be created ONLY 
-  // if it doesn't already exist
-  create_worker_thread();
+  // Create the worker thread if there's work to do. Does nothing if there's already a thread.
+  if (_pending_refresh_schema.size() > 0)
+    create_worker_thread();
 }
 
 bool AutoCompleteCache::get_pending_refresh(std::string &task)
@@ -919,7 +917,7 @@ bool AutoCompleteCache::get_pending_refresh(std::string &task)
 
 void AutoCompleteCache::create_worker_thread()
 {
-  // fire up thread to start cacheing
+  // Fire up thread to start caching.
   if (!_refresh_thread)
   {
     if (!_shutdown)
