@@ -61,9 +61,76 @@ class AutoCompleteCache;
 /** Base class for SQL Editor Backend classes.
  Subclassed by specific RDBMS support modules.
  */
+
 class WBPUBLICBACKEND_PUBLIC_FUNC Sql_editor : public boost::enable_shared_from_this<Sql_editor>, public base::trackable
 {
 public:
+  struct TableReference
+  {
+    std::string schema;
+    std::string table;
+    std::string alias;
+  };
+
+  enum AutoCompletionWantedParts
+  {
+    CompletionWantNothing      = 0,
+    CompletionWantOperators    = 1 << 0,
+    CompletionWantMajorKeywords = 1 << 1, // Keywords that can start a query.
+    CompletionWantKeywords     = 1 << 2,
+    CompletionWantFunctions    = 1 << 3,
+    CompletionWantRoutines     = 1 << 4,
+    CompletionWantSchemas      = 1 << 5,
+    CompletionWantTables       = 1 << 6,
+    CompletionWantColumns      = 1 << 7,
+  };
+
+  // Context structure for code completion results and token info. Located here as we need it also
+  // for unit testing. Otherwise it is completely internal.
+  struct AutoCompletionContext
+  {
+    AutoCompletionWantedParts wanted_parts;
+
+    bool check_identifier; // Sometimes we should not try to parse identifiers.
+
+    // There can be at most 3 id parts in a qualified identifier (schema.table.column). But due to
+    // the ambiguity of id.id, we can have different schemas for table and column lookup.
+    std::string table_schema;
+    std::string column_schema;
+    std::string table;
+    std::string column;
+
+    // Position + other info of the token at the caret.
+    unsigned int token_line;
+    unsigned int token_start;
+    unsigned int token_length;
+    unsigned int token_type;
+    std::string token;
+
+    // Current caret position, statement to parse and input typed by the user.
+    unsigned int line;
+    unsigned int offset;
+    std::string statement;
+    std::string typed_part;
+
+    std::vector<TableReference> references; // As in FROM, UPDATE etc.
+
+    AutoCompletionContext()
+    {
+      wanted_parts = CompletionWantMajorKeywords;
+      check_identifier = true;
+
+      token_line = 0;
+      token_start = 0;
+      token_length = 0;
+      token_type = 0;
+
+      line = 0;
+      offset = 0;
+    };
+
+  };
+
   typedef boost::shared_ptr<Sql_editor> Ref;
   typedef boost::weak_ptr<Sql_editor> Ptr;
 
@@ -107,10 +174,12 @@ public:
   void set_sql_check_enabled(bool val);
   
   void show_auto_completion(bool auto_choose_single);
-  void update_auto_completion(const std::string &typed_part);
+  std::vector<std::pair<int, std::string> >  update_auto_completion(const std::string &typed_part);
   void cancel_auto_completion();
   void set_auto_completion_cache(AutoCompleteCache *cache);
 
+  void create_auto_completion_list(AutoCompletionContext &context);
+
   std::string selected_text();
   void set_selected_text(const std::string &new_text);
   void insert_text(const std::string &new_text);
@@ -139,7 +208,7 @@ private:
   void setup_auto_completion();
   std::string get_written_part(int position);
   virtual bool fill_auto_completion_keywords(std::vector<std::pair<int, std::string> > &entries,
-    bool keywords, bool functions);
+    bool major_keywords, bool keywords, bool functions);
 
   void text_changed(int position, int length, int lines_changed, bool added);
   void char_added(int char_code);
@@ -175,6 +244,9 @@ private:
   std::string _current_schema;
   std::string _last_ac_statement; // The last statement we used for auto completion.
 
+  long _server_version;
+  std::string _sql_mode;
+  std::set<std::string> _charsets; // For lookup needed by parsers to determine repertoires.
 };
 
 #endif /* _SQL_EDITOR_BE_H_ */
