@@ -313,7 +313,7 @@ public:
   // ----- Edit menu handling
   void validate_edit_menu()
   {
-    (*inner->get_command_ui()->signal_validate_edit_menu_items())();
+    inner->get_command_ui()->revalidate_edit_menu_items();
   }
 
   void edit_undo()
@@ -350,8 +350,8 @@ public:
 
   bool edit_can_copy()
   {
-    if (inner->get_active_form() && inner->get_active_form()->can_copy())
-      return true;
+    if (inner->get_active_form())
+      return inner->get_active_form()->can_copy();
     return false;
   }
 
