@@ -1075,7 +1075,7 @@ namespace Aga.Controls.Tree
 					for (int n = list.Count - 1; n >= 0; n--)
 					{
 						int index = list[n];
-						if (index >= 0 && index <= parent.Nodes.Count)
+						if (index >= 0 && index < parent.Nodes.Count) // ml: bug fix
 							parent.Nodes.RemoveAt(index);
 						else
 							throw new ArgumentOutOfRangeException("Index out of range");
