@@ -1,14 +1,29 @@
 /*
-   Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+
+
 #include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
 #include <stdlib.h>
@@ -199,6 +214,7 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 	sql::SQLString schema;
 	sql::SQLString defaultCharset("utf8");
 	sql::SQLString characterSetResults("utf8");
+        sql::SQLString preInitScript;
 
 	sql::SQLString sslKey, sslCert, sslCA, sslCAPath, sslCipher;
 	bool ssl_used = false, schema_used = false;
@@ -263,6 +279,12 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 			} else {
 				throw sql::InvalidArgumentException("No string value passed for characterSetResults");
 			}
+		} else if (!it->first.compare("preInitScript")) {
+                        if ((p_s = boost::get< sql::SQLString >(&it->second))) {
+                                  preInitScript = *p_s;
+                        } else {
+                                  throw sql::InvalidArgumentException("No string value passed for preInitScript");
+                        }
 		} else if (!it->first.compare("sslKey")) {
 			if ((p_s = boost::get< sql::SQLString >(&it->second))) {
 				sslKey = *p_s;
@@ -499,7 +521,7 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 
 	if (ssl_used) {
 		/* According to the docs, always returns 0 */
-		proxy->ssl_set(sslKey.c_str(), sslCert.c_str(), sslCA.c_str(), sslCAPath.c_str(), sslCipher.c_str());
+		proxy->ssl_set(sslKey, sslCert, sslCA, sslCAPath, sslCipher);
 	}
 	CPP_INFO_FMT("hostName=%s", hostName.c_str());
 	CPP_INFO_FMT("user=%s", userName.c_str());
@@ -525,7 +547,12 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 	if (opt_reconnect) {
 		proxy->options(MYSQL_OPT_RECONNECT, (const char *) &opt_reconnect_value);
 	}
-
+  
+        if (preInitScript.length() > 0)
+        {
+                boost::scoped_ptr< sql::Statement > stmt(createStatement());
+                stmt->execute(preInitScript);
+        }
 	setAutoCommit(true);
 	setTransactionIsolation(sql::TRANSACTION_REPEATABLE_READ);
 	// Different Values means we have to set different result set encoding
