@@ -31,7 +31,7 @@ import string
 import subprocess
 import sys
 import time
-from mysql.utilities.exception import MUTLibError
+from mysql.utilities.exception import UtilError, UtilDBError, MUTLibError
 
 # Constants
 MAX_SERVER_POOL = 10
@@ -222,7 +222,7 @@ class Server_list(object):
         # Connect to the new instance
         try:
             self.new_server.connect()
-        except MUTLibError, e:
+        except UtilError, e:
             raise MUTLibError("Cannot connect to spawned server: %s" % \
                                e.errmsg)
             
@@ -266,19 +266,44 @@ class Server_list(object):
         # Build the shutdown command
         cmd = ""
         res = server.show_server_variable("basedir")
-        cmd += os.path.normpath(os.path.join(res[0][1], "bin", "mysqladmin"))
+        mysqladmin_client = "mysqladmin"
+        if not os.name == "posix":
+            mysqladmin_client = "mysqladmin.exe"
+        mysqladmin_path= os.path.normpath(os.path.join(res[0][1], "bin",
+                                                   mysqladmin_client))
+        if not os.path.exists(mysqladmin_path):
+            mysqladmin_path= os.path.normpath(os.path.join(res[0][1], "client",
+                                                       mysqladmin_client))
+        if not os.path.exists(mysqladmin_path) and not os.name == 'posix':
+            mysqladmin_path= os.path.normpath(os.path.join(res[0][1],
+                                                       "client/debug",
+                                                       mysqladmin_client))
+        if not os.path.exists(mysqladmin_path) and not os.name == 'posix':
+            mysqladmin_path= os.path.normpath(os.path.join(res[0][1],
+                                                       "client/release",
+                                                       mysqladmin_client))
+        cmd += mysqladmin_path
         cmd += " shutdown "
         cmd += self.get_connection_parameters(server)
         res = server.show_server_variable("datadir")
         datadir = res[0][1]
 
+        # Kill all connections so shutdown will work correctly
+        res = server.exec_query("SHOW PROCESSLIST")
+        for row in res:
+            if not row[7] or not row[7].upper().startswith("SHOW PROCESS"):
+                try:
+                    server.exec_query("KILL CONNECTION %s" % row[0])
+                except UtilDBError: # Ok to ignore KILL failures
+                    pass
+
         # disconnect user
         server.disconnect()
         
         # Stop the server
         file = os.devnull
         f_out = open(file, 'w')
-        proc = subprocess.Popen(cmd, shell=True,
+        proc = subprocess.Popen(cmd, shell=True, 
                                 stdout = f_out, stderr = f_out)
         ret_val = proc.wait()
         f_out.close()
