@@ -34,6 +34,18 @@ _DATABASE, _TABLE, _VIEW, _TRIG, _PROC, _FUNC, _EVENT, _GRANT = "DATABASE", \
 _IMPORT_LIST = [_TABLE, _VIEW, _TRIG, _PROC, _FUNC, _EVENT,
                 _GRANT, _DATA_DECORATE]
 _DEFINITION_LIST = [_TABLE, _VIEW, _TRIG, _PROC, _FUNC, _EVENT, _GRANT]
+_BASIC_COMMANDS = ["CREATE", "USE", "GRANT", "DROP"]
+_DATA_COMMANDS = ["INSERT", "UPDATE"]
+_RPL_COMMANDS = ["START", "STOP", "CHANGE"]
+_RPL_PREFIX = "-- "
+_RPL = len(_RPL_PREFIX)
+_GTID_COMMANDS = ["SET @MYSQLUTILS_TEMP_L", "SET @@SESSION.SQL_LOG_",
+                          "SET @@GLOBAL.GTID_PURG"]
+_GTID_PREFIX = 22
+_GTID_SKIP_WARNING = "# WARNING: GTID commands are present in the import " + \
+    "file but the server does not support GTIDs. Commands are ignored."
+_GTID_MISSING_WARNING = "# WARNING: GTIDs are enabled on this server but " + \
+    "the import file did not contain any GTID commands."
 
 def _read_row(file, format, skip_comments=False):
     """Read a row of from the file.
@@ -48,12 +60,16 @@ def _read_row(file, format, skip_comments=False):
     Returns (tuple) - one row of data
     """
 
-    if format == "SQL":
+    warnings_found = []
+    if format == "sql":
         # Easiest - just read a row and return it.
         for row in file.readlines():
-            if row[0] != '#':
+            if row.startswith("# WARNING"):
+                warnings_found.append(row)
+                continue
+            if not (row.startswith('#') or row.startswith('--')):
                 yield row.strip('\n')
-    elif format == "VERTICAL":
+    elif format == "vertical":
         # This format is a bit trickier. We need to read a set of rows that
         # encompass the data row. They will appear in this format:
         #   ****** <header> ******
@@ -69,6 +85,23 @@ def _read_row(file, format, skip_comments=False):
         header = []
         data_row = []
         for row in file.readlines():
+            # Show warnings from file
+            if row.startswith("# WARNING"):
+                warnings_found.append(row)
+                continue
+            # Process replication commands
+            if row[0:_RPL] == _RPL_PREFIX:
+                # find first word
+                first_word = row[_RPL:row.find(' ', _RPL)].upper()
+                if first_word in _RPL_COMMANDS:
+                    yield [row.strip('\n')]
+                    continue
+                # Check for GTID commands
+                elif len(row) > _GTID_PREFIX + _RPL and \
+                   row[_RPL:_GTID_PREFIX + _RPL] \
+                      in _GTID_COMMANDS:
+                    yield [row.strip('\n')]
+                    continue
             # Skip comment rows
             if row[0] == '#':
                 if len(header) > 0:
@@ -121,19 +154,37 @@ def _read_row(file, format, skip_comments=False):
     else:
         separator = ","
         # Use CSV reader to read the row
-        if format == "CSV":
+        if format == "csv":
             separator = ","
-        elif format == "TAB":
+        elif format == "tab":
             separator = "\t"
-        elif format == "GRID":
+        elif format == "grid":
             separator = "|"
         csv_reader = csv.reader(file, delimiter=separator)
         for row in csv_reader:
-            if format == "GRID":
+            if row[0].startswith("# WARNING"):
+                warnings_found.append(row[0])
+                continue
+            # find first word
+            if row[0][0:_RPL] == _RPL_PREFIX:
+                first_word = row[0][_RPL:_RPL+row[0][_RPL:].find(' ')].upper()
+            else:
+                first_word = ""
+            # Process replication commands
+            if row[0][0:_RPL] == _RPL_PREFIX:
+                if first_word in _RPL_COMMANDS:
+                    yield row
+                # Check for GTID commands
+                elif len(row[0]) > _GTID_PREFIX + _RPL and \
+                   row[0][_RPL:_GTID_PREFIX + _RPL] in _GTID_COMMANDS:
+                    yield row
+                    
+            elif format == "grid":
                 if len(row[0]) > 0:
                     if row[0][0] == '+':
                         continue
-                    elif row[0][0] == '#' and not skip_comments:
+                    elif (row[0][0] == '#' or row[0][0:2] == "--") and \
+                         not skip_comments:
                         yield row
                 else:
                     new_row = []
@@ -141,9 +192,15 @@ def _read_row(file, format, skip_comments=False):
                         new_row.append(col.strip())
                     yield new_row
             else:
-                if len(row[0]) == 0 or row[0][0] != '#' or \
-                   row[0][0] == '#' and not skip_comments:
+                if (len(row[0]) == 0 or row[0][0] != '#' or \
+                    row[0][0:2] != "--") or ((row[0][0] == '#' or \
+                    row[0][0:2] == "--") and not skip_comments):
                     yield row
+    if warnings_found:
+        print "CAUTION: The following %s warning " % len(warnings_found) + \
+              "messages were included in the import file:"
+        for row in warnings_found:
+            print row.strip('\n')
 
 
 def _check_for_object_list(row, obj_type):
@@ -185,9 +242,10 @@ def read_next(file, format, no_headers=False):
     """
     cmd_type = ""
     multiline = False
-    if format == "SQL":
+    if format == "sql":
         sql_cmd = ""
-        for row in _read_row(file, "SQL", True):
+        for row in _read_row(file, "sql", True):
+            first_word = row[0:row.find(' ')].upper() # find first word
             # Skip these nonsense rows
             if len(row) == 0 or row[0] == "#"or row[0:2] == "||":
                 continue
@@ -203,23 +261,31 @@ def read_next(file, format, no_headers=False):
                     #yield goes here
                     yield (cmd_type, sql_cmd)
                     sql_cmd = ""
-                cmd_type = "SQL"
+                cmd_type = "sql"
                 multiline = True
-            elif row[0:len("CREATE")].upper() == "CREATE" or \
-                 row[0:len("USE")].upper() == "USE" or \
-                 row[0:len("GRANT")].upper() == "GRANT" or \
-                 row[0:len("DROP")].upper() == "DROP":
+            elif first_word in _BASIC_COMMANDS:
                 if len(sql_cmd) > 0:
                     #yield goes here
                     yield (cmd_type, sql_cmd)
-                cmd_type = "SQL"
+                cmd_type = "sql"
                 sql_cmd = row
-            elif row[0:len("INSERT")].upper() == "INSERT" or \
-                 row[0:len("UPDATE")].upper() == "UPDATE":
+            elif first_word in _RPL_COMMANDS:
                 if len(sql_cmd) > 0:
                     #yield goes here
                     yield (cmd_type, sql_cmd)
+                cmd_type = "RPL_COMMAND"
+                sql_cmd = row
+            elif len(row) > _GTID_PREFIX and \
+                  row[0:_GTID_PREFIX] in _GTID_COMMANDS:
+                #yield goes here
+                yield (cmd_type, sql_cmd)
+                cmd_type = "GTID_COMMAND"
+                sql_cmd = row
+            elif first_word in _DATA_COMMANDS:
                 cmd_type = "DATA"
+                if len(sql_cmd) > 0:
+                    #yield goes here
+                    yield (cmd_type, sql_cmd)
                 sql_cmd = row
             else:
                 sql_cmd += row
@@ -228,6 +294,25 @@ def read_next(file, format, no_headers=False):
         table_rows = []
         found_obj = ""
         for row in _read_row(file, format, False):
+            # find first word
+            if row[0][0:_RPL] == _RPL_PREFIX:
+                first_word = row[0][_RPL:_RPL+row[0][_RPL:].find(' ',
+                                                                 _RPL)].upper()
+            else:
+                first_word = ""
+            if row[0][0:_RPL] == _RPL_PREFIX and \
+                first_word in _RPL_COMMANDS:
+                # join the parts if CSV or TAB
+                if format in ['csv', 'tab']:
+                    yield("RPL_COMMAND", ", ".join(row).strip("--"))
+                else:
+                    yield("RPL_COMMAND", row[0][_RPL:])
+                continue
+            if row[0][0:_RPL] == _RPL_PREFIX and \
+               len(row[0]) > _GTID_PREFIX + _RPL and \
+               row[0][_RPL:_GTID_PREFIX + _RPL] in _GTID_COMMANDS:
+                yield("GTID_COMMAND", row[0][_RPL:])
+                continue
             # Check to see if we have a marker for rows of objects or data
             for obj in _IMPORT_LIST:
                 if _check_for_object_list(row[0], obj):
@@ -250,11 +335,11 @@ def read_next(file, format, no_headers=False):
                 continue
             else:
                 # We're reading rows here
-                if len(row[0]) > 0 and row[0][0] == "#":
+                if len(row[0]) > 0 and (row[0][0] == "#" or row[0][0:2] == "--"):
                     continue
                 else:
                     yield (cmd_type, row)
-        if row[0][0] != "#":
+        if row[0][0] != "#" and row[0][0:2] != "--":
             yield (cmd_type, row)
 
 
@@ -266,8 +351,8 @@ def _get_db(row):
     Returns (string) database name or None if not found
     """
     db_name = None
-    if (row[0] in _DEFINITION_LIST or row[0] == "SQL"):
-        if row[0] == "SQL":
+    if (row[0] in _DEFINITION_LIST or row[0] == "sql"):
+        if row[0] == "sql":
             # Need crude parse here for database statement.
             parts = ()
             parts = row[1].split(" ")
@@ -457,7 +542,7 @@ def _build_create_objects(obj_type, db, definitions):
                           (defn[col_ref.get("ACTION_ORIENTATION",5)],
                            defn[col_ref.get("ACTION_STATEMENT",7)])
             create_strings.append(create_str)
-        elif obj_type == "PROCEDURE" or obj_type == "FUNCTION":
+        elif obj_type in ("PROCEDURE", "FUNCTION"):
             create_str = "CREATE DEFINER=%s" % defn[col_ref.get("DEFINER",5)]
             create_str += " %s `%s`.`%s`(%s)" % \
                           (obj_type, db,
@@ -496,8 +581,10 @@ def _build_create_objects(obj_type, db, definitions):
                 tbl = "*"
             create_str = "GRANT %s ON %s.%s TO %s" % (priv, db, tbl, user)
             create_strings.append(create_str)
+        elif obj_type in ["RPL_COMMAND", "GTID_COMMAND"]:
+            create_strings.append([defn])
         else:
-            raise UtilError("Unknown object type discovered: %s", obj_type)
+            raise UtilError("Unknown object type discovered: %s" % obj_type)
     return create_strings
 
 
@@ -544,17 +631,6 @@ def _build_col_metadata(obj_type, definitions):
     return table_col_list
 
 
-def _to_sql(obj):
-    """Convert a value to a suitable SQL value placing quotes where needed.
-
-    obj[in]           object (value) to convert
-
-    Returns (string) converted value
-    """
-    from mysql.connector.conversion import MySQLConverter
-    return MySQLConverter().quote(obj)
-
-
 def _build_insert_data(col_names, tbl_name, data):
     """Build simple INSERT statements for data.
 
@@ -564,8 +640,10 @@ def _build_insert_data(col_names, tbl_name, data):
 
     Returns (string) the INSERT statement.
     """
+    from mysql.utilities.common.sql_transform import to_sql
+    
     return "INSERT INTO %s (" % tbl_name + ",".join(col_names) + \
-           ") VALUES (" + ','.join(imap(_to_sql, data))  + ");"
+           ") VALUES (" + ','.join(imap(to_sql, data))  + ");"
 
 
 def _skip_sql(sql, options):
@@ -675,13 +753,14 @@ def _exec_statements(statements, destination, format, options, dryrun=False):
         try:
             if dryrun:
                 print statement
-            elif format != "SQL" or not _skip_sql(statement, options):
-                destination.exec_query(statement)
+            elif format != "sql" or not _skip_sql(statement, options):
+                res = destination.exec_query(statement)
         # Here we capture any exception and raise UtilError to communicate to
         # the script/user. Since all util errors (exceptions) derive from
         # Exception, this is safe.
         except Exception, e:
-            raise UtilError("Invalid statement:\n%s" % statement)
+            raise UtilError("Invalid statement:\n%s" % statement +
+                            "\nERROR: %s" % e.errmsg)
     return True
 
 
@@ -694,8 +773,9 @@ def _get_column_metadata(tbl_class, table_col_list):
 
     for tbl_col_def in table_col_list:
         if tbl_col_def[0] == tbl_class.tbl_name:
-            return tbl_class.get_column_metadata(tbl_col_def[1])
-    return None
+            tbl_class.get_column_metadata(tbl_col_def[1])
+            return True
+    return False
 
 
 def import_file(dest_val, file_name, options):
@@ -749,32 +829,34 @@ def import_file(dest_val, file_name, options):
         tbl = Table(destination, tbl_name)
         # Need to check to see if table exists!
         if tbl.exists():
-            col_meta = tbl.get_column_metadata()
+            tbl.get_column_metadata()
+            col_meta = True
         elif len(table_col_list) > 0:
             col_meta = _get_column_metadata(tbl, table_col_list)
         else:
             fix_cols = []
             fix_cols.append((tbl.tbl_name, columns))
             col_meta = _get_column_metadata(tbl, fix_cols)
-        if col_meta is None:
+        if not col_meta:
             raise UtilError("Cannot build bulk insert statements without "
                                  "the table definition.")
-        ins_strs = tbl.make_bulk_insert(table_rows, tbl.db_name, col_meta)
+        ins_strs = tbl.make_bulk_insert(table_rows, tbl.db_name)
         if len(ins_strs[0]) > 0:
             statements.extend(ins_strs[0])
         if len(ins_strs[1]) > 0 and not skip_blobs:
             for update in ins_strs[1]:
-                statements.append(update[0] % "'%s'" % update[1])
+                statements.append(update)
 
     # Gather options
-    format = options.get("format", "SQL").upper()
+    format = options.get("format", "sql")
     no_headers = options.get("no_headers", False)
     quiet = options.get("quiet", False)
-    import_type = options.get("import_type", "DEFINITIONS").upper()
+    import_type = options.get("import_type", "definitions")
     single = options.get("single", True)
     dryrun = options.get("dryrun", False)
     do_drop = options.get("do_drop", False)
     skip_blobs = options.get("skip_blobs", False)
+    skip_gtid = options.get("skip_gtid", False)
 
     # Attempt to connect to the destination server
     conn_options = {
@@ -792,15 +874,15 @@ def import_file(dest_val, file_name, options):
                          False, options.get("quiet", False))
 
     if not quiet:
-        if import_type == "BOTH":
+        if import_type == "both":
             str = "definitions and data"
         else:
-            str = import_type.lower()
+            str = import_type
         print "# Importing %s from %s." % (str, file_name)
 
     # Setup variables we will need
     skip_header = not no_headers
-    if format == "SQL":
+    if format == "sql":
         skip_header = False
     get_db = True
     check_privileges = False
@@ -814,9 +896,28 @@ def import_file(dest_val, file_name, options):
     statements = []
     table_col_list = []
     tbl_name = ""
+    skip_rpl = options.get("skip_rpl", False)
+    gtid_command_found = False
+    supports_gtid = servers[0].supports_gtid() == 'ON'
+    skip_gtid_warning_printed = False
 
     # Read the file one object/definition group at a time
-    for row in read_next(file, format.upper()):
+    for row in read_next(file, format):
+        # Check for replication command
+        if row[0] == "RPL_COMMAND":
+            if not skip_rpl:
+                statements.append(row[1])
+            continue
+        if row[0] == "GTID_COMMAND":
+            gtid_command_found = True
+            if not supports_gtid:
+                # only display warning once
+                if not skip_gtid_warning_printed:
+                    print _GTID_SKIP_WARNING
+                    skip_gtid_warning_printed = True
+            elif not skip_gtid:
+                statements.append(row[1])
+            continue
         # If this is the first pass, get the database name from the file
         if get_db:
             if skip_header:
@@ -824,11 +925,12 @@ def import_file(dest_val, file_name, options):
             else:
                 db_name = _get_db(row)
                 get_db = False
-                if do_drop and import_type != "DATA":
+                if do_drop and import_type != "data":
                     statements.append("DROP DATABASE IF EXISTS `%s`;" % \
                                       db_name)
-                if import_type != "DATA":
-                    if not _skip_object("CREATE_DB", options):
+                if import_type != "data":
+                    if not _skip_object("CREATE_DB", options) and \
+                       not format == 'sql':
                         statements.append("CREATE DATABASE `%s`;" % db_name)
 
         # This is the first time through the loop so we must
@@ -841,15 +943,15 @@ def import_file(dest_val, file_name, options):
 
             dest_db.check_write_access(dest_val['user'], dest_val['host'],
                                        access_options)
-
+            
         # Now check to see if we want definitions, data, or both:
-        if row[0] == "SQL" or row[0] in _DEFINITION_LIST:
-            if format != "SQL" and len(row[1]) == 1:
+        if row[0] == "sql" or row[0] in _DEFINITION_LIST:
+            if format != "sql" and len(row[1]) == 1:
                 raise UtilError("Cannot read an import file generated with "
                                 "--display=NAMES")
 
-            if import_type == "DEFINITIONS" or import_type == "BOTH":
-                if format == "SQL":
+            if import_type in ("definitions", "both"):
+                if format == "sql":
                     statements.append(row[1])
                 else:
                     if obj_type == "":
@@ -868,10 +970,10 @@ def import_file(dest_val, file_name, options):
                 _process_definitions(statements, table_col_list, db_name)
                 definitions = []
 
-            if import_type == "DATA" or import_type == "BOTH":
+            if import_type in ("data", "both"):
                 if _skip_object("DATA", options):
                     continue  # skip data
-                elif format == "SQL":
+                elif format == "sql":
                     statements.append(row[1])
                 else:
                     if row[0] == "BEGIN_DATA":
@@ -910,6 +1012,10 @@ def import_file(dest_val, file_name, options):
     _exec_statements(statements, destination, format, options, dryrun)
 
     file.close()
+    
+    # Check gtid process
+    if supports_gtid and not gtid_command_found:
+        print _GTID_MISSING_WARNING
 
     if not quiet:
         print "#...done."
