@@ -131,13 +131,13 @@ def _get_db_dir_size(folder):
         return None
     for item in os.listdir(folder):
         name, ext = os.path.splitext(item)
-        if ext.upper() != ".MYD" and ext.upper() != ".MYI" and \
+        if ext.upper() not in (".MYD", ".MYI") and \
            name.upper() not in ('SLOW_LOG', 'GENERAL_LOG'):
             itemfolder = os.path.join(folder, item)
             if os.path.isfile(itemfolder):
                 total_size += os.path.getsize(itemfolder)
             elif os.path.isdir(itemfolder):
-                total_size += getfolderSize(itemfolder)
+                total_size += _get_db_dir_size(itemfolder)
     return total_size
 
 
@@ -150,23 +150,29 @@ def _find_tablespace_files(folder, verbosity=0):
     """
     total = 0
     tablespaces = []
-    for item in os.listdir(folder):
-        itempath = os.path.join(folder, item)
-        if os.path.isfile(itempath):
+    # skip inaccessible files.
+    try:
+        for item in os.listdir(folder):
+            itempath = os.path.join(folder, item)
             name, ext = os.path.splitext(item)
-            if ext.upper() == "IBD":
-                size = os.path.getsize(itempath)
-                total += size
-                if verbosity > 0:
-                    row = (item, size,'file tablespace', '')
-                else:
-                    row = (item, size)
-
-                tablespaces.append(row)
-        else:
-            subdir, tot = _find_tablespace_files(itempath, verbosity)
-            total += tot
-            tablespaces.extend(subdir)
+            if os.path.isfile(itempath):
+                name, ext = os.path.splitext(item)
+                if ext.upper() == "IBD":
+                    size = os.path.getsize(itempath)
+                    total += size
+                    if verbosity > 0:
+                        row = (item, size,'file tablespace', '')
+                    else:
+                        row = (item, size)
+    
+                    tablespaces.append(row)
+            else:
+                subdir, tot = _find_tablespace_files(itempath, verbosity)
+                if subdir is not None:
+                    total += tot
+                    tablespaces.extend(subdir)
+    except:
+        return (None, None)
 
     return tablespaces, total
 
@@ -427,15 +433,14 @@ def _build_db_list(server, rows, include_list, datadir, format=False,
             # Put in commas and justify strings
             for i in range(0,num_cols):
                 fmt_data[i] = locale.format("%d", row[i+1], grouping=True)
-                fmt_data[i] = "{0:>{1}}".format(dbdir_size, max_col[i])
-                if num_cols == 4: # get all columns
-                    fmt_rows.append((row[0], fmt_data[0], fmt_data[1],
-                                     fmt_data[2], fmt_data[3]))
-                elif num_cols == 3:
-                    fmt_rows.append((row[0], fmt_data[0], fmt_data[1],
-                                     fmt_data[2]))
-                else:
-                    fmt_rows.append((row[0], fmt_data[0]))
+            if num_cols == 4: # get all columns
+                fmt_rows.append((row[0], fmt_data[0], fmt_data[1],
+                                 fmt_data[2], fmt_data[3]))
+            elif num_cols == 3:
+                fmt_rows.append((row[0], fmt_data[0], fmt_data[1],
+                                 fmt_data[2]))
+            else:
+                fmt_rows.append((row[0], fmt_data[0]))
     else:
         fmt_rows = results
 
@@ -454,19 +459,22 @@ def _build_db_list(server, rows, include_list, datadir, format=False,
                     if format:
                         fmt_data = ['','','','','']
                         for i in range(0,num_cols):
-                            fmt_data[i] = locale.format("%d",
-                                                        row[i+1], grouping=True)
-                            fmt_data[i] = "{0:>{1}}".format(dbdir_size,
-                                                            max_col[i])
-                            if num_cols == 4: # get all columns
-                                fmt_rows.insert(0, (db[0], fmt_data[0],
-                                                    fmt_data[1], fmt_data[2],
-                                                    fmt_data[3]))
-                            elif num_cols == 3:
-                                fmt_rows.insert(0, (db[0], fmt_data[0],
-                                                 fmt_data[1], fmt_data[2]))
+                            if type(row[i+1]) == type(int):
+                                fmt_data[i] = locale.format("%s",
+                                                            int(row[i+1]),
+                                                            grouping=True)
                             else:
-                                fmt_rows.insert(0, (db[0], fmt_data[0]))
+                                fmt_data[i] = locale.format("%s", row[i+1],
+                                                            grouping=True)
+                        if num_cols == 4: # get all columns
+                            fmt_rows.insert(0, (db[0], fmt_data[0],
+                                                fmt_data[1], fmt_data[2],
+                                                fmt_data[3]))
+                        elif num_cols == 3:
+                            fmt_rows.insert(0, (db[0], fmt_data[0],
+                                             fmt_data[1], fmt_data[2]))
+                        else:
+                            fmt_rows.insert(0, (db[0], fmt_data[0]))
                     else:
                         if num_cols == 4:
                             fmt_rows.insert(0, (db[0], 0, 0, 0, 0))
@@ -495,7 +503,7 @@ def show_database_usage(server, datadir, dblist, options):
     
     from mysql.utilities.common.format import print_list
 
-    format = options.get("format", "GRID")
+    format = options.get("format", "grid")
     no_headers = options.get("no_headers", False)
     verbosity = options.get("verbosity", 0)
     have_read = options.get("have_read", False)
@@ -526,7 +534,7 @@ def show_database_usage(server, datadir, dblist, options):
 
     # Get list of databases with sizes and formatted when necessary
     columns, rows, db_total = _build_db_list(server, res, dblist, datadir,
-                                             format == "GRID",
+                                             format == "grid",
                                              have_read, verbosity,
                                              include_empty or do_all)
 
@@ -553,7 +561,7 @@ def show_logfile_usage(server, options):
     """
     from mysql.utilities.common.format import print_list
 
-    format = options.get("format", "GRID")
+    format = options.get("format", "grid")
     no_headers = options.get("no_headers", False)
     verbosity = options.get("verbosity", 0)
     have_read = options.get("have_read", False)
@@ -577,7 +585,7 @@ def show_logfile_usage(server, options):
     fmt_logs = []
     columns = ['log_name', 'size']
     if len(logs) > 0:
-        if format == 'GRID':
+        if format == 'grid':
             max_col = _get_formatted_max_width(logs, columns, 1)
             if max_col < len('size'):
                 max_col = len('size')
@@ -617,7 +625,7 @@ def show_log_usage(server, datadir, options):
     """
     from mysql.utilities.common.format import print_list
 
-    format = options.get("format", "GRID")
+    format = options.get("format", "grid")
     no_headers = options.get("no_headers", False)
     verbosity = options.get("verbosity", 0)
     have_read = options.get("have_read", False)
@@ -713,7 +721,7 @@ def show_innodb_usage(server, datadir, options):
     """
     from mysql.utilities.common.format import print_list
 
-    format = options.get("format", "GRID")
+    format = options.get("format", "grid")
     no_headers = options.get("no_headers", False)
     verbosity = options.get("verbosity", 0)
     have_read = options.get("have_read", False)
@@ -721,8 +729,7 @@ def show_innodb_usage(server, datadir, options):
 
     # Check to see if we have innodb
     res = server.show_server_variable('have_innodb')
-    if res != [] and \
-       (res[0][1].upper() == "NO" or res[0][1] == "DISABLED"):
+    if res != [] and res[0][1].upper() in ("NO", "DISABLED"):
         print "# InnoDB is disabled on this server."
         return True
 
