@@ -22,16 +22,16 @@ This module contains abstractions of MySQL replication functionality.
 import os
 import re
 import time
-from mysql.utilities.common.lock import Lock
 from mysql.utilities.common.server import Server
-from mysql.utilities.exception import UtilError, UtilDBError
-from mysql.utilities.exception import UtilRplError, UtilBinlogError
+from mysql.utilities.exception import UtilError, UtilRplWarn, UtilRplError
 
 _MASTER_INFO_COL = [
     'Master_Log_File', 'Read_Master_Log_Pos', 'Master_Host', 'Master_User',
     'Master_Password', 'Master_Port', 'Connect_Retry', 'Master_SSL_Allowed',
     'Master_SSL_CA_File', 'Master_SSL_CA_Path', 'Master_SSL_Cert',
-    'Master_SSL_Cipher', 'Master_SSL_Key', 'Master_SSL_Verify_Server_Cert'
+    'Master_SSL_Cipher', 'Master_SSL_Key', 'Master_SSL_Verify_Server_Cert',
+    'Heartbeat', 'Bind', 'Ignored_server_ids', 'Uuid', 'Retry_count',
+    'SSL_CRL', 'SSL_CRL_Path', 'Enabled_auto_position',
 ]
 
 _SLAVE_IO_STATE, _SLAVE_MASTER_HOST, _SLAVE_MASTER_USER, _SLAVE_MASTER_PORT, \
@@ -58,8 +58,8 @@ _NO_RPL_USER = "No --rpl-user specified and multiple users found with " + \
 _RPL_USER_PASS = "No --rpl-user specified and the user found with " + \
                  "replication privileges requires a password."
 
-_GTID_DONE = "SELECT @@GLOBAL.GTID_DONE"
-_GTID_WAIT = "SELECT SQL_THREAD_WAIT_AFTER_GTIDS('%s', %s)"
+_GTID_EXECUTED = "SELECT @@GLOBAL.GTID_EXECUTED"
+_GTID_WAIT = "SELECT WAIT_UNTIL_SQL_THREAD_AFTER_GTIDS('%s', %s)"
 
 def _get_list(rows, cols):
     """Return a list of information in GRID format to stdout.
@@ -272,6 +272,34 @@ class Replication(object):
                                  "master.")
         
         return []
+        
+        
+    def check_server_uuids(self):
+        """Check UUIDs on master and slave
+        
+        This method will check the UUIDs on the master and slave. It will
+        raise exceptions for error conditions. 
+        
+        Returns [] if compatible or no UUIDs used, list of errors if not
+        """
+        master_uuid = self.master.get_uuid()
+        slave_uuid = self.slave.get_uuid()
+        
+        # Check for both not supporting UUIDs.
+        if master_uuid is None and slave_uuid is None:
+            return []
+        
+        # Check for unbalanced servers - one with UUID, one without
+        if master_uuid is None or slave_uuid is None:
+            raise UtilRplError("%s does not support UUIDs." %
+                               "Master" if master_uuid is None else "Slave")
+
+        # Check for uuid uniqueness
+        if master_uuid == slave_uuid:
+            raise UtilRplError("The slave's UUID is the same as the "
+                                 "master.")
+        
+        return []
 
 
     def check_innodb_compatibility(self, options):
@@ -422,12 +450,7 @@ class Replication(object):
         state = self.slave.get_io_running()
         if not state:
             raise UtilRplError("Slave is stopped.")
-        host, port = self.slave.get_master_host_port()
-        if self.master.host == '127.0.0.1':
-            m_host = 'localhost'
-        else:
-            m_host = self.master.host
-        if host != m_host or int(port) != int(self.master.port) or \
+        if not self.slave.is_configured_for_master(self.master) or \
            state.upper() != "YES":
             return False
         return True
@@ -486,16 +509,19 @@ class Replication(object):
         return errors
     
     
-    def create_rpl_user(self, rpl_user):
+    def create_rpl_user(self, r_user, r_pass=None):
         """Create the replication user and grant privileges
         
         If the user exists, check privileges and add privileges as needed.
         Calls Master class method to execute.
         
+        r_user[in]     user to create
+        r_pass[in]     password for user to create (optional)
+
         Returns bool - True = success, False = errors
         """
-        return self.master.create_rpl_user(rpl_user, self.slave.host,
-                                           self.slave.port, self.verbosity)
+        return self.master.create_rpl_user(self.slave.host, self.slave.port,
+                                           r_user, r_pass, self.verbosity)
 
     
     def setup(self, rpl_user, num_tries):
@@ -519,7 +545,7 @@ class Replication(object):
         r_user, r_pass = re.match("(\w+)(?:\:(\w+))?", rpl_user).groups()
         
         # Check to see if rpl_user is present, else create her
-        if not self.create_rpl_user(rpl_user):
+        if not self.create_rpl_user(r_user, r_pass):
             return False
 
         # Read master log file information
@@ -592,7 +618,10 @@ class Replication(object):
             else:
                 print "# Starting slave from the beginning..."
         res = self.slave.start(self.query_options)
-        
+
+        # Add commit because C/Py are auto_commit=0 by default        
+        self.slave.exec_query("COMMIT")
+    
         # Check slave status
         i = 0
         while i < num_tries:
@@ -688,6 +717,7 @@ class Master(Server):
         
         assert not options.get("conn_info") == None
         
+        self.options = options
         Server.__init__(self, options)
         
 
@@ -728,14 +758,15 @@ class Master(Server):
         return self.exec_query(_RPL_USER_QUERY, options)
 
 
-    def create_rpl_user(self, user, host, port, verbosity=0):
+    def create_rpl_user(self, host, port, r_user, r_pass=None, verbosity=0):
         """Create the replication user and grant privileges
         
         If the user exists, check privileges and add privileges as needed.
         
-        user[in]       user to create in form 'user:passwd' (:passwd optional)
         host[in]       host of the slave
         port[in]       port of the slave
+        r_user[in]     user to create
+        r_pass[in]     password for user to create (optional)
         verbosity[in]  verbosity of output
                        Default = 0
         
@@ -744,8 +775,6 @@ class Master(Server):
         
         from mysql.utilities.common.user import User
 
-        r_user, r_pass = re.match("(\w+)(?:\:(\w+))?", user).groups()
-
         # Create user class instance
         user = User(self, "%s@%s:%s" % (r_user, host, port), verbosity)
 
@@ -807,12 +836,44 @@ class Master(Server):
         return (rpl_ok, errors)
 
 
-    def get_slaves(self):
+    def _check_discovered_slave(self, conn_dict):
+        """ Check discovered slave is configured to this master
+        
+        This method attempts to determine if the slave specified is
+        configured to connect to this master.
+        
+        conn_dict[in]  dictionary of connection information
+        
+        Returns bool - True - is configured, False - not configured
+        """
+        if conn_dict['conn_info']['host'] == '127.0.0.1':
+            conn_dict['conn_info']['host'] = 'localhost'
+        # Now we must attempt to connect to the slave.
+        slave_conn = Slave(conn_dict)
+        is_configured = False
+        try:
+            slave_conn.connect()
+            # Skip discovered slaves that are not configured
+            # to connect to the master
+            if slave_conn.is_configured_for_master(self):
+                is_configured = True
+        except:
+            pass # if connect errors, ignore slave
+        finally:
+            slave_conn.disconnect()
+
+        return is_configured
+
+
+    def get_slaves(self, user, password):
         """Return the slaves registered for this master.
         
         This method returns a list of slaves (host, port) if this server is
         a master in a replication topology and has slaves registered.
 
+        user[in]       user login
+        password[in]   user password
+        
         Returns list - [host:port, ...]
         """
         def _get_slave_info(host, port):
@@ -824,26 +885,284 @@ class Master(Server):
             return slave_info
         
         slaves = []
-        
+        no_host_slaves = []
         res = self.exec_query("SHOW SLAVE HOSTS")
         if not res == []:
             res.sort()  # Sort for conformity
             for row in res:
-                slaves.append(_get_slave_info(row[1], row[2]))
-                    
+                info = _get_slave_info(row[1], row[2])
+                conn_dict = {
+                    'conn_info' : { 'user' : user, 'passwd' : password,
+                                    'host' : row[1], 'port' : row[2],
+                                    'socket' : None },
+                    'role'      : 'slave',
+                    'verbose'   : self.options.get("verbosity", 0) > 0,
+                }
+                if not row[1]:
+                    no_host_slaves.append(info)
+                elif self._check_discovered_slave(conn_dict):
+                    slaves.append(info)
+                else:
+                    # Ignore any slaves that appear in the SHOW SLAVE HOSTS
+                    # but are not configured to attach to this master.
+                    pass
+        
+        if no_host_slaves:
+            print "WARNING: There are slaves that have not been registered" + \
+                  " with --report-host or --report-port."
+            if self.options.get("verbosity", 0) > 0:
+              for row in no_host_slaves:
+                  print "\t", row
+            
         return slaves
-
-
-    def block_writes(block_timeout=3):
-        """Block writes for a specific period of time
+    
+    
+    def get_gtid_purged_statement(self):
+        """General the SET @@GTID_PURGED statement for backup
+        
+        Returns string - statement for slave if GTID=ON, else None
         """
+        if self.supports_gtid == "ON":
+            gtid_executed = self.exec_query("SELECT @@GLOBAL.GTID_EXECUTED")[0]
+            return 'SET @@GLOBAL.GTID_PURGED = "%s"' % gtid_executed
+        else:
+            return None
+
+
+class MasterInfo(object):
+    """The MasterInfo is an abstraction of the mechanism for storing the
+    master information for slave servers. It is designed to return an
+    implementation neutral representation of the information for use in
+    newer servers that use a table to store the information as well as
+    older servers that use a file to store the information.
+    """
+    
+    def __init__(self, slave, options):
+        """Constructor
+
+        The method accepts one of the following types for options['conn_info']:
+        
+            - dictionary containing connection information including:
+              (user, passwd, host, port, socket)
+            - connection string in the form: user:pass@host:port:socket
+            - an instance of the Server class
+             
+        options[in]        options for controlling behavior:
+          filename         filename for master info file - valid only for
+                           servers with master-info-repository=FILE or
+                           versions prior to 5.6.5.
+          verbosity        determines level of output. Default = 0.
+          quiet            turns off all messages except errors.
+                           Default is False.
+        """
+        
+        assert slave is not None, "MasterInfo requires an instance of Slave."
+        self.slave = slave
+        self.filename = options.get("master_info", "master.info")
+        self.quiet = options.get("quiet", False)
+        self.verbosity = options.get("verbosity", 0)
+        self.values = {}      # internal dictionary of the values
+        self.repo = "FILE"
+        if self.slave is not None:
+            res = self.slave.show_server_variable("master_info_repository")
+            if res is not None and res != [] and \
+               res[0][1].upper() == "TABLE":
+                self.repo = "TABLE"
 
     
-    def unblock_writes():
-        """Stop blocking writes
+    def read(self):
+        """Read the master information
+        
+        This method reads the master information either from a file or a
+        table depending on the availability of and setting for
+        master-info-repository. If missing (server version < 5.6.5), it
+        defaults to reading from a file.
+        
+        Returns bool - True = success
         """
+        if self.verbosity > 2:
+            print "# Reading master information from a %s." % self.repo.lower()
+        if self.repo == "FILE":
+            import socket
+            
+            # Check host name of this host. If not the same, issue error.
+            if self.slave.is_alias(socket.gethostname()):
+                return self._read_master_info_file()
+            else:
+                raise UtilRplWarn("Cannot read master information file "
+                                  "from a remote machine.")
+        else:
+            return self._read_master_info_table()
 
 
+    def _check_read(self, refresh=False):
+        """Check if master information has been read
+        
+        refresh[in]    if True, re-read the master information.
+                       Default is False.
+        
+        If the master information has not been read, read it and populate
+        the dictionary.
+        """
+        # Read the values if not already read or user says to refresh them.
+        if self.values is None or self.values == {} or refresh:
+            self.read()
+
+
+    def _build_dictionary(self, rows):
+        """Build the internal dictionary of values.
+        
+        rows[in]       Rows as read from the file or table
+        """
+        for i in range(0, len(rows)):
+            self.values[_MASTER_INFO_COL[i]] = rows[i]
+            
+    
+    def _read_master_info_file(self):
+        """Read the contents of the master.info file.
+        
+        This method will raise an error if the file is missing or cannot be
+        read by the user.
+        
+        Returns bool - success = True
+        """
+        contents = []
+        res = self.slave.show_server_variable('datadir')
+        if res is None or res == []:
+            raise UtilRplError("Cannot get datadir.")
+        datadir = res[0][1]
+        if self.filename == 'master.info':
+            self.filename = os.path.join(datadir, self.filename)
+
+        if not os.path.exists(self.filename):
+            raise UtilRplError("Cannot find master information file: "
+                               "%s." % self.filename)
+        try:
+            mfile = open(self.filename, 'r')
+            num = int(mfile.readline())
+            # Protect overrun of array if master_info file length is
+            # changed (more values added).
+            if num > len(_MASTER_INFO_COL):
+                num = len(_MASTER_INFO_COL)
+        except:
+            raise UtilRplError("Cannot read master information file: "
+                               "%s.\nUser needs to have read access to "
+                               "the file." % self.filename)
+        # Build the dictionary 
+        for i in range(1, num):
+            contents.append(mfile.readline().strip('\n'))
+        self._build_dictionary(contents)
+        mfile.close()
+
+        return True
+        
+        
+    def _read_master_info_table(self):
+        """Read the contents of the slave_master_info table.
+        
+        This method will raise an error if the file is missing or cannot be
+        read by the user.
+        
+        Returns bool - success = True
+        """
+        res = None        
+        try:
+            res = self.slave.exec_query("SELECT * FROM "
+                                        "mysql.slave_master_info")
+        except UtilError, e:
+            raise UtilRplError("Unable to read the slave_master_info table. "
+                               "Error: %s" % e.errmsg)
+        if res is None or res == []:
+            return False
+
+        # Build dictionary for the information with column information
+        rows = []
+        for i in range(0, len(res[0][1:])):
+            rows.append(res[0][i+1])
+        self._build_dictionary(rows)
+
+        return True
+        
+        
+    def show_master_info(self, refresh=False):
+        """Display the contents of the master information.
+
+        refresh[in]    if True, re-read the master information.
+                       Default is False.
+        """
+        # Check to see if we need to read the information
+        self._check_read(refresh)
+        stop = len(self.values)
+        for i in range(0, stop):
+            print "{0:>30} : {1}".format(_MASTER_INFO_COL[i],
+                                         self.values[_MASTER_INFO_COL[i]])
+        
+    
+    def check_master_info(self, refresh=False):
+        """Check to see if master info file matches slave status
+        
+        This method will return a list of discrepancies if the master.info
+        file does not match slave status. It will also raise errors if there
+        are problem accessing the master.info file.
+        
+        refresh[in]    if True, re-read the master information.
+                       Default is False.
+        
+        Returns [] - no exceptions, list if exceptions found
+        """
+        # Check to see if we need to read the information
+        self._check_read(refresh)
+        errors = []
+        res = self.slave.get_status()
+        if res != []:
+            state = res[0][_SLAVE_IO_STATE]
+            if not state:
+                raise UtilRplError("Slave is stopped.")
+            m_host = res[0][_SLAVE_MASTER_HOST]
+            m_port = res[0][_SLAVE_MASTER_PORT]
+            rpl_user = res[0][_SLAVE_MASTER_USER]
+            if m_host != self.values['Master_Host'] or \
+               int(m_port) != int(self.values['Master_Port']) or \
+               rpl_user != self.values['Master_User']:
+                errors.append("Slave is connected to master differently "
+                              "than what is recorded in the master "
+                              "information file. Master information file "
+                              "= user=%s, host=%s, port=%s." %
+                              (self.values['Master_User'],
+                               self.values['Master_Host'],
+                               self.values['Master_Port']))
+
+        return errors
+   
+
+    def get_value(self, key, refresh=False):
+        """Returns the value found for the key or None if key not found.
+
+        refresh[in]    if True, re-read the master information.
+                       Default is False.
+        
+        Returns value - Value found for the key or None if key missing
+        """
+        # Check to see if we need to read the information        
+        self._check_read(refresh)
+        try:
+            return self.values[key]
+        except:
+            return None
+    
+    def get_master_info(self, refresh=False):
+        """Returns the master information dictionary.
+
+        refresh[in]    if True, re-read the master information.
+                       Default is False.
+        
+        Returns dict - master information
+        """
+        # Check to see if we need to read the information
+        self._check_read(refresh)
+        return self.values
+        
+
 class Slave(Server):
     """The Slave class is a subclass of the Server class. It represents a
     MySQL server performing the role of a slave in a replication topology.
@@ -884,6 +1203,7 @@ class Slave(Server):
         assert not options.get("conn_info") == None
         self.options = options
         Server.__init__(self, options)
+        self.master_info = None
         
 
     def get_status(self, col_options={}):
@@ -1046,103 +1366,12 @@ class Slave(Server):
             stop = len(res[0])
             cols = res[0]
             rows = res[1]
-            for i in range(0,stop):
+            for i in range(0, stop):
                 print "{0:>30} : {1}".format(cols[i], rows[0][i])
         else:
             raise UtilRplError("Cannot get slave status or slave is "
                                  "not configured as a slave or not "
                                  "started.")
-        
-        
-    def get_master_info(self, filename, silent=False):
-        """Return the contents of the master.info file.
-        
-        This method will raise an error if the file is missing or cannot be
-        read by the user.
-        
-        filename[in]  path to master information file
-        silent[in]    if True, do not print or raise errors
-                      (default = False)
-
-        Returns dictionary - values in master.info 
-        """
-        contents = {}
-        
-        res = self.show_server_variable('datadir')
-        if res is None or res == []:
-            raise UtilRplError("Cannot get datadir.")
-        datadir = res[0][1]
-        if filename == 'master.info':
-            filename = os.path.join(datadir, filename)
-
-        if os.path.exists(filename):
-            mfile = open(filename, 'r')
-            num = int(mfile.readline())
-            # Protect overrun of array if master_info file length is
-            # changed (more values added).
-            if num > len(_MASTER_INFO_COL):
-                num = len(_MASTER_INFO_COL)
-            for i in range(1,num):
-                contents[_MASTER_INFO_COL[i-1]] = mfile.readline().strip('\n')
-            mfile.close()
-        else:
-            if silent:
-                return None
-            else:
-                raise UtilRplError("Cannot read master information file: "
-                                     "%s." % filename)
-
-        return contents
-        
-        
-    def show_master_info(self, options):
-        """Display the contents of the master information file.
-        
-        options[in]   dictionary of options (verbose, pedantic)
-        """
-        
-        filename = options.get("master_info", 'master.info')
-        contents = self.get_master_info(filename)
-        stop = len(contents)
-        for i in range(0,stop):
-            print "{0:>30} : {1}".format(_MASTER_INFO_COL[i],
-                                         contents[_MASTER_INFO_COL[i]])
-        
-    
-    def check_master_info(self, options):
-        """Check to see if master info file matches slave status
-        
-        This method will return a list of discrepancies if the master.info
-        file does not match slave status. It will also raise errors if there
-        are problem accessing the master.info file.
-        
-        options[in]   dictionary of options (verbose, pedantic)
-
-        Returns [] - no exceptions, list if exceptions found
-        """
-        errors = []
-        filename = options.get("master_info", "master.info")
-        master_info = self.get_master_info(filename)
-        res = self.get_status()
-        if res != []:
-            state = res[0][_SLAVE_IO_STATE]
-            if not state:
-                raise UtilRplError("Slave is stopped.")
-            m_host = res[0][_SLAVE_MASTER_HOST]
-            m_port = res[0][_SLAVE_MASTER_PORT]
-            rpl_user = res[0][_SLAVE_MASTER_USER]
-            if m_host != master_info['Master_Host'] or \
-               int(m_port) != int(master_info['Master_Port']) or \
-               rpl_user != master_info['Master_User']:
-                errors.append("Slave is connected to master differently "
-                              "than what is recorded in the master "
-                              "information file. Master information file "
-                              "= user=%s, host=%s, port=%s." %
-                              (master_info['Master_User'],
-                               master_info['Master_Host'],
-                               master_info['Master_Port']))
-
-        return errors
     
 
     def get_rpl_user(self):
@@ -1150,10 +1379,11 @@ class Slave(Server):
         
         Returns - tuple = (user, password) or (None, None) if errors
         """
-        filename = self.options.get("master_info", "master.info")
-        master_info = self.get_master_info(filename)
-        if master_info is not None:
-            return (master_info['Master_User'], master_info['Master_Password'])
+        self.master_info = MasterInfo(self, self.options)
+        m_host = self.master_info.get_value("Master_User")
+        m_passwd = self.master_info.get_value("Master_Password")
+        if m_host is not None:
+            return (m_host, m_passwd)
         return (None, None)
 
 
@@ -1184,11 +1414,11 @@ class Slave(Server):
     def num_gtid_behind(self, master_gtids):
         """Get the number of transactions the slave is behind the master.
         
-        master_gtids[in]  the master's GTID_DONE list
+        master_gtids[in]  the master's GTID_EXECUTED list
         
         Returns int - number of trans behind master
         """
-        slave_gtids = self.exec_query(_GTID_DONE)[0][0]
+        slave_gtids = self.exec_query(_GTID_EXECUTED)[0][0]
         gtids = self.exec_query("SELECT GTID_SUBTRACT('%s','%s')" %
                                (master_gtids[0][0], slave_gtids))[0]
         if len(gtids) == 1 and len(gtids[0]) == 0:
@@ -1224,7 +1454,7 @@ class Slave(Server):
         This method requires that the server supports GTIDs.
         
         master_gtid[in]  the list of gtids from the master
-                         obtained via SELECT @@GLOBAL.GTID_DONE on master
+                         obtained via SELECT @@GLOBAL.GTID_EXECUTED on master
         timeout[in]      timeout for waiting for slave to catch up
                          Note: per GTID call. Default is 3.
         verbose[in]      if True, print query used.
@@ -1275,8 +1505,8 @@ class Slave(Server):
                                "slave is not connected to a master and no "
                                "master information was provided.")
         elif self.is_connected():
-            filename = self.options.get("master_info", "master.info")
-            master_info = self.get_master_info(filename, True)
+            m_info = MasterInfo(self, self.options)
+            master_info = m_info.get_master_info()
             if master_info is None and master_values == {}:
                 raise UtilRplError("Cannot create CHANGE MASTER command.")
         else:
@@ -1321,23 +1551,20 @@ class Slave(Server):
         return change_master
     
     
-    def is_connected_to_master(self, host, port):
+    def is_configured_for_master(self, master):
         """Check that slave is connected to the master at host, port.
         
-        host[in]       host name of the master
-        port[in]       port of the master
+        master[in]     instance of the master
         
         Returns bool - True = is connected
         """
         res = self.get_status()
-        if host == '127.0.0.1':
-            host = 'localhost'
-        if res != [] and res[0] != []:
-            res = res[0]
-            m_host = res[_SLAVE_MASTER_HOST]
-            m_port = res[_SLAVE_MASTER_PORT]
-            if m_host != host or int(m_port) != int(port):
-                return False
+        if res == [] or res[0] == []:
+            return False
+        res = res[0]
+        m_host, m_port = self.get_master_host_port()
+        if not master.is_alias(m_host) or int(m_port) != int(master.port):
+            return False
         return True
 
 
@@ -1357,7 +1584,7 @@ class Slave(Server):
                            value, the slave health is not Ok
         max_pos[in]        maximum position difference from master to slave to
                            determine if slave health is not Ok
-        verbosity[in]      if > 0, return detailed errors else return only
+        verbosity[in]      if > 1, return detailed errors else return only
                            short phrases
         
         Returns tuple (bool, []) - (True, []) = Ok,
@@ -1373,8 +1600,7 @@ class Slave(Server):
         if res != [] and res[0] != []:
             res = res[0]
             state = res[_SLAVE_IO_STATE]
-            m_host = res[_SLAVE_MASTER_HOST]
-            m_port = res[_SLAVE_MASTER_PORT]
+            m_host, m_port = self.get_master_host_port()
             m_log = res[_SLAVE_MASTER_LOG_FILE]
             m_log_pos = res[_SLAVE_MASTER_LOG_FILE_POS]
             io_running = res[_SLAVE_IO_RUNNING]
@@ -1386,11 +1612,7 @@ class Slave(Server):
             io_error_text = res[_SLAVE_IO_ERROR]
             
             # Check to see that slave is connected to the right master
-            if master.host == '127.0.0.1':
-                master_host = 'localhost'
-            else:
-                master_host = master.host
-            if m_host != master_host or int(m_port) != int(master.port):
+            if not self.is_configured_for_master(master):
                 return (False, ["Not connected to correct master."])
 
             # Check slave status for errors, threads activity
@@ -1423,7 +1645,7 @@ class Slave(Server):
                     
             # Check GTID trans behind.
             elif self.supports_gtid() == "ON":
-                master_gtids = master.exec_query(_GTID_DONE)
+                master_gtids = master.exec_query(_GTID_EXECUTED)
                 num_gtids_behind = self.num_gtid_behind(master_gtids)
                 if num_gtids_behind > 0:
                     errors.append("Slave has %s transactions behind master." %
@@ -1435,10 +1657,7 @@ class Slave(Server):
             rpl_ok = False
         
         if len(errors) > 1:
-            if verbosity == 0:
-                errors = ["ERROR"]
-            else:
-                errors = [", ".join(errors)]
+            errors = [", ".join(errors)]
         
         return (rpl_ok, errors)
 
@@ -1513,7 +1732,7 @@ class Slave(Server):
         if show_command:
             print "# Change master command for %s:%s" % (self.host, self.port)
             print "#", change_master
-        res= self.exec_query(change_master)
+        res = self.exec_query(change_master)
         if res is None or res != ():
             raise UtilRplError("Slave %s:%s change master failed.",
                                (hostport, res[0]))
