@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2012 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@ This module contains abstractions of a MySQL table and an index.
 import multiprocessing
 import re
 import sys
-import mysql.connector
 from mysql.utilities.exception import UtilError
 
 # Constants
@@ -32,6 +31,14 @@ _MAXTHREADS_INSERT = 6
 _MAXROWS_PER_THREAD = 100000
 _MAXAVERAGE_CALC = 100
 
+_FOREIGN_KEY_QUERY = """
+  SELECT CONSTRAINT_NAME, COLUMN_NAME, REFERENCED_TABLE_SCHEMA,
+         REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME 
+  FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE 
+  WHERE TABLE_SCHEMA = '%s' AND TABLE_NAME = '%s' AND 
+        REFERENCED_TABLE_SCHEMA IS NOT NULL
+"""
+
 def _parse_object_name(qualified_name):
     """Parse db, name from db.name
 
@@ -290,10 +297,12 @@ class Table(object):
         self.hash_indexes = []
         self.rtree_indexes = []
         self.fulltext_indexes = []
+        self.text_columns = []
+        self.blob_columns = []
+        self.column_format = None
+        self.column_names = []
         if options.get('get_cols', False):
-            self.col_metadata = self.get_column_metadata()
-        else:
-            self.col_metadata = None
+            self.get_column_metadata()
         self.dest_vals = None
 
         # Get max allowed packet
@@ -338,67 +347,56 @@ class Table(object):
 
     def get_column_metadata(self, columns=None):
         """Get information about the table for the bulk insert operation.
+        
+        This method builds lists that describe the metadata of the table. This
+        includes lists for:
+        
+          column names
+          column format for building VALUES clause
+          blob fields - for use in generating INSERT/UPDATE for blobs
+          text fields - for use in checking for single quotes
 
         columns[in]        if None, use EXPLAIN else use column list.
-
-        Returns column metadata list
         """
 
-        rows = None
-
-        # Get field masks for blob and quoted fields
-        # Build an array of dictionaries describing the fields
-        special_cols = []
         if columns is None:
-            rows = self.server.exec_query("explain %s" % self.table)
-        else:
-            rows = columns
-        if rows is not None:
-            for row in rows:
-                if len(row) > 1:
-                    has_char = (row[1].lower().find("char") >= 0)
-                    has_enum = (row[1].lower().find("enum") >= 0)
-                    has_set = (row[1].lower().find("set") >= 0)
-                    has_date = (row[1].lower().find("date") >= 0)
-                    has_time = (row[1].lower().find("time") >= 0)
-                    col_data = {
-                        "has_blob" : (row[1].lower().find("blob") >= 0) \
-                                     or (row[1].lower().find("text") >= 0),
-                        "is_text"  : has_char or has_enum or has_set,
-                        "is_time"  : has_date or has_time,
-                        "name"     : row[0]
-                    }
-                else: # No column information - make all text
-                    col_data = {
-                        "has_blob" : False,
-                        "is_text"  : True,
-                        "is_time"  : False,
-                        "name"     : row[0]
-                    }
-                special_cols.append(col_data)
-        return special_cols
+            columns = self.server.exec_query("explain %s" % self.table)
+        stop = len(columns)
+        self.column_names = []
+        col_format_values = [''] * stop
+        if columns is not None:
+            for col in range(0,stop):
+                self.column_names.append(columns[col][0])
+                col_type_prefix = columns[col][1][0:4].lower()
+                if col_type_prefix in ("char", "enum", "set("):
+                    self.text_columns.append(col)
+                    col_format_values[col] = "'%s'"
+                elif col_type_prefix in ("blob", "text"):
+                    self.blob_columns.append(col)
+                    col_format_values[col] = "%s"
+                elif col_type_prefix in ("date", "time"):
+                    col_format_values[col] = "'%s'"
+                else:
+                    col_format_values[col] = "%s"
+        self.column_format = "%s%s%s" % \
+                             (" (", ', '.join(col_format_values), ")")
 
 
-    def get_col_names(self, col_metadata=None):
+    def get_col_names(self):
         """Get column names for the export operation.
 
-        col_metadata[in]   If provided, get column names from metadata
-
         Return (list) column names
         """
 
-        cols = []
-        if col_metadata is None:
+        if self.column_format is None:
+            self.column_names = []
             rows = self.server.exec_query("explain %s" % self.table)
             for row in rows:
-                cols.append(row[0])
-        else:
-            for col in col_metadata:
-                cols.append(col.get("name"))
-        return cols
+                self.column_names.append(row[0])
+        return self.column_names
 
 
-    def _build_update_blob(self, row, new_db, name, special_cols, blob_col):
+    def _build_update_blob(self, row, new_db, name, blob_col):
         """Build an UPDATE statement to update blob fields.
 
         row[in]            a row to process
@@ -406,73 +404,80 @@ class Table(object):
         name[in]           name of the table
         conn_val[in]       connection information for the destination server
         query[in]          the INSERT string for executemany()
-        special_cols[in]   list of columns for special string handling
         blob_col[in]       number of the column containing the blob
 
         Returns tuple (UPDATE string, blob data)
-        """
+        """        
+        from mysql.connector.conversion import MySQLConverter
 
-        blob_insert = "UPDATE %s.%s SET `" % (new_db, name)
-        where_clause = "WHERE "
-        data_clause = None
+        if self.column_format is None:
+            self.get_column_metadata()
+
+        blob_insert = "UPDATE %s.%s SET " % (new_db, name)
+        where_values = []
+        do_commas = False
+        has_data = False
         stop = len(row)
         for col in range(0,stop):
-            if col == blob_col:
-                blob_insert += special_cols[col]["name"] + "` = %s "
-                data = row[col]
+            col_name = self.column_names[col]
+            if col in self.blob_columns:
+                if row[col] is not None and len(row[col]) > 0:
+                    if do_commas:
+                        blob_insert += ", "
+                    blob_insert += "`%s` = " % col_name + "%s" % \
+                                   MySQLConverter().quote(row[col])
+                    has_data = True
+                    do_commas = True
             else:
-                if col > 0 and col < stop:
-                    where_clause += " AND "
-                where_clause += "`%s` = '%s' " % (special_cols[col]["name"],
-                                                  row[col])
-        return (blob_insert + where_clause, data)
+                where_values.append("`%s` = '%s' " % (col_name, row[col]))
+        if has_data:
+            return blob_insert + " WHERE " + " AND ".join(where_values) + ";"
+        return None
 
 
-    def get_column_string(self, row, new_db, col_metadata=None):
+    def get_column_string(self, row, new_db):
         """Return a formatted list of column data.
 
         row[in]            a row to process
         new_db[in]         new database name
-        col_metadata[in]   list of columns for special string handling
 
         Returns (string) column list
         """
-        # Must call get_col_metata() first!
-        assert col_metadata, \
-               "You must call get_column_metadata before get_column_string()."
+        
+        if self.column_format is None:
+            self.get_column_metadata()
 
         blob_inserts = []
-        val_str = " ("
-        stop = len(row)
-        for col in range(0,stop):
-            if row[col] == None:
-                val_str += "NULL"
-            else:
-                # Save blob updates for later...
-                if col_metadata[col]["has_blob"]: # It is a blob field!
-                    str = self._build_update_blob(row, new_db,
-                                                  self.tbl_name,
-                                                  col_metadata, col)
-                    blob_inserts.append(str)
-                    val_str += "NULL"
-                # We must ensure to escape ' marks.
-                elif col_metadata[col]["is_text"]: # No, text field
-                    if row[col].find("'"):
-                        val_str += "'%s'" % re.sub("'", "''", row[col])
-                    else:
-                        val_str += "'%s'" % row[col]
-                elif col_metadata[col]["is_time"]:
-                    val_str += "'%s'" % row[col]
-                else: # Ah, ok, so it is some other field...
-                    val_str += "%s" % row[col]
-            if (col + 1) < stop:
-                val_str += ", "
-        val_str += ")"
+        values = list(row)
 
+        # Find blobs
+        for col in self.blob_columns:
+            # Save blob updates for later...
+            blob = self._build_update_blob(row, new_db, self.tbl_name, col)
+            if blob is not None:
+                blob_inserts.append(blob)
+            values[col] = "NULL"
+
+        # Replace single quotes located in the value for a text field with the
+        # correct special character escape sequence. This fixes SQL errors
+        # related to using single quotes in a string value that is single
+        # quoted. For example, 'this' is it' is changed to 'this\' is it'
+        for col in self.text_columns:
+            #Check if the value is not None before replacing quotes
+            if values[col]:
+                values[col] = values[col].replace("'", "\\'")
+
+        # Build string
+        val_str = self.column_format % tuple(values)
+
+        # Change 'None' occurrences with "NULL"
+        val_str.replace(", None", ", NULL")
+        val_str.replace("(None", "(NULL")
+        
         return (val_str, blob_inserts)
 
 
-    def make_bulk_insert(self, rows, new_db, col_metadata=None):
+    def make_bulk_insert(self, rows, new_db):
         """Create bulk insert statements for the data
 
         Reads data from a table (rows) and builds group INSERT statements for
@@ -482,15 +487,12 @@ class Table(object):
 
         rows[in]           a list of rows to process
         new_db[in]         new database name
-        col_metadata[in]   the column metadata - call get_column_metadata() to
-                           generate on first call to bulk_insert
 
         Returns (tuple) - (bulk insert statements, blob data inserts)
         """
 
-        # Must call get_col_metata() first!
-        assert col_metadata, \
-               "You must call get_column_metadata before make_bulk_insert()."
+        if self.column_format is None:
+            self.get_column_metadata()
 
         data_inserts = []
         blob_inserts = []
@@ -505,7 +507,7 @@ class Table(object):
                     insert_str += val_str
                 data_size = len(insert_str)
 
-            col_data = self.get_column_string(row, new_db, col_metadata)
+            col_data = self.get_column_string(row, new_db)
             val_str = col_data[0]
 
             if len(col_data[1]) > 0:
@@ -588,6 +590,7 @@ class Table(object):
         destination[in]    the destination server
         """
 
+        from mysql.utilities.common.lock import Lock
         from mysql.utilities.common.server import Server
 
         if self.dest_vals is None:
@@ -601,13 +604,17 @@ class Table(object):
         dest = Server(server_options)
         dest.connect()
 
+        # Issue the write lock
+        lock_list = [("%s.%s" % (new_db, self.tbl_name), 'WRITE')]
+        my_lock = Lock(dest, lock_list, {'locking':'lock-all',})
+                    
         # First, turn off foreign keys if turned on
-        fkey_on = dest.toggle_fkeys(False)
+        dest.disable_foreign_key_checks(True)
 
-        if self.col_metadata is None:
-            self.col_metadata = self.get_column_metadata()
+        if self.column_format is None:
+            self.get_column_metadata()
 
-        data_lists = self.make_bulk_insert(rows, new_db, self.col_metadata)
+        data_lists = self.make_bulk_insert(rows, new_db)
         insert_data = data_lists[0]
         blob_data = data_lists[1]
 
@@ -630,8 +637,8 @@ class Table(object):
                                      "Error = %s" % e.errmsg)
 
         # Now, turn on foreign keys if they were on at the start
-        if fkey_on:
-            fkey_on = dest.toggle_fkeys(True)
+        dest.disable_foreign_key_checks(False)
+        my_lock.unlock()
         del dest
 
 
@@ -656,8 +663,8 @@ class Table(object):
                 If spawn == False, None
         """
 
-        if self.col_metadata is None:
-            self.col_metadata = self.get_column_metadata()
+        if self.column_format is None:
+            self.get_column_metadata()
 
         if self.dest_vals is None:
             self.dest_vals = self.get_dest_values(destination)
@@ -672,7 +679,22 @@ class Table(object):
         return proc
 
 
-    def copy_data(self, destination, new_db=None, connections=1):
+    def _clone_data(self, new_db):
+        """Clone table data.
+
+        This method will copy all of the data for a table
+        from the old database to the new database on the same server.
+
+        new_db[in]         New database name for the table
+        """
+        query_str = "INSERT INTO %s.%s SELECT * FROM %s.%s" % \
+                    (new_db, self.tbl_name, self.db_name, self.tbl_name)
+        if self.verbose and not self.vquiet:
+            print query_str
+        self.server.exec_query(query_str)
+
+
+    def copy_data(self, destination, cloning=False, new_db=None, connections=1):
         """Retrieve data from a table and copy to another server and database.
 
         Reads data from a table and inserts the correct INSERT statements into
@@ -681,30 +703,34 @@ class Table(object):
         Note: if connections < 1 - retrieve the data one row at-a-time
 
         destination[in]    Destination server
+        cloning[in]        If True, we are copying on the same server
         new_db[in]         Rename the db to this name
         connections[in]    Number of threads(connections) to use for insert
         """
 
         if new_db is None:
             new_db = self.db_name
-
+            
         num_conn = int(connections)
 
-        # Read and copy the data
-        pthreads = []
-        for rows in self.retrieve_rows(num_conn):
-            p = self.insert_rows(rows, new_db, destination, num_conn > 1)
-            if p is not None:
-                p.start()
-                pthreads.append(p)
-
-        if num_conn > 1:
-            # Wait for all to finish
-            num_complete = 0
-            while num_complete < len(pthreads):
-                for p in pthreads:
-                    if not p.is_alive():
-                        num_complete += 1
+        if cloning:
+            self._clone_data(new_db)
+        else:
+            # Read and copy the data
+            pthreads = []
+            for rows in self.retrieve_rows(num_conn):
+                p = self.insert_rows(rows, new_db, destination, num_conn > 1)
+                if p is not None:
+                    p.start()
+                    pthreads.append(p)
+    
+            if num_conn > 1:
+                # Wait for all to finish
+                num_complete = 0
+                while num_complete < len(pthreads):
+                    for p in pthreads:
+                        if not p.is_alive():
+                            num_complete += 1
 
 
     def retrieve_rows(self, num_conn=1):
@@ -788,6 +814,16 @@ class Table(object):
         """
         res = self.server.exec_query("SHOW INDEXES FROM %s" % self.table)
         return res
+    
+    
+    def get_tbl_foreign_keys(self):
+        """Return a result set containing all foreign keys for the table
+        
+        Returns result set
+        """
+        res = self.server.exec_query(_FOREIGN_KEY_QUERY % (self.db_name,
+                                                           self.tbl_name))
+        return res
 
 
     def __append(self, indexes, index):
@@ -973,7 +1009,7 @@ class Table(object):
         This shows the best (first n) or worst (last n) performing queries
         for a given table.
 
-        format[in]         format out output = SQL, TABLE, TAB, CSV
+        format[in]         format out output = sql, table, tab, csv
         limit[in]          number to limit the display
         best[in]           (optional) if True, print best performing indexes
                                       if False, print worst performing indexes
@@ -1029,24 +1065,24 @@ class Table(object):
                                          query_options)
         if rows:
             print "#"
-            print "# Showing the top 5 %s performing indexes from %s:\n#" % \
+            print "# Showing the top %s performing indexes from %s:\n#" % \
                   (type, self.table)
             cols = ("database", "table", "name", "column", "sequence",
                     "num columns", "cardinality", "est. rows", "percent")
-            print_list(sys.stdout, format, cols, rows, False)
+            print_list(sys.stdout, format, cols, rows)
 
 
     def __print_index_list(self, indexes, format, no_header=False):
         """Print the list of indexes
 
         indexes[in]        list of indexes to print
-        format[in]         format out output = SQL, TABLE, TAB, CSV
+        format[in]         format out output = sql, table, tab, csv
         no_header[in]      (optional) if True, do not print the header
         """
 
         from mysql.utilities.common.format import print_list
 
-        if format == "SQL":
+        if format == "sql":
             for index in indexes:
                 index.print_index_sql()
         else:
@@ -1060,11 +1096,11 @@ class Table(object):
     def print_indexes(self, format):
         """Print all indexes for this table
 
-        format[in]         format out output = SQL, TABLE, TAB, CSV
+        format[in]         format out output = sql, table, tab, csv
         """
 
         print "# Showing indexes from %s:\n#" % (self.table)
-        if format == "SQL":
+        if format == "sql":
             self.__print_index_list(self.btree_indexes, format)
             self.__print_index_list(self.hash_indexes, format, False)
             self.__print_index_list(self.rtree_indexes, format, False)
