@@ -15,6 +15,11 @@
  */
 
 #import <Cocoa/Cocoa.h>
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+#import <QuartzCore/CAGradientLayer.h>
+#endif
+#import <QuartzCore/CAAnimation.h>
+#import <QuartzCore/CATransaction.h>
 
 #import <Carbon/Carbon.h> // Temporary
 
@@ -72,6 +77,7 @@ static const KeyToCommand macMapDefault[] =
   {SCK_RIGHT,     SCI_SHIFT,  SCI_CHARRIGHTEXTEND},
   {SCK_RIGHT,     SCI_ALT,    SCI_WORDRIGHT},
   {SCK_RIGHT,     SCI_META,   SCI_WORDRIGHT},
+  {SCK_RIGHT,     SCI_SMETA,  SCI_WORDRIGHTEXTEND},
   {SCK_RIGHT,     SCI_ASHIFT, SCI_WORDRIGHTEXTEND},
   {'/',           SCI_CTRL,   SCI_WORDPARTLEFT},
   {'/',           SCI_CSHIFT, SCI_WORDPARTLEFTEXTEND},
@@ -133,11 +139,179 @@ static const KeyToCommand macMapDefault[] =
 
 //--------------------------------------------------------------------------------------------------
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+
+// Only implement FindHighlightLayer on OS X 10.6+
+
+/**
+ * Class to display the animated gold roundrect used on OS X for matches.
+ */
+@interface FindHighlightLayer : CAGradientLayer
+{
+@private
+	NSString *sFind;
+	int positionFind;
+	BOOL retaining;
+	CGFloat widthText;
+	CGFloat heightLine;
+	NSString *sFont;
+	CGFloat fontSize;
+}
+
+@property (copy) NSString *sFind;
+@property (assign) int positionFind;
+@property (assign) BOOL retaining;
+@property (assign) CGFloat widthText;
+@property (assign) CGFloat heightLine;
+@property (copy) NSString *sFont;
+@property (assign) CGFloat fontSize;
+
+- (void) animateMatch: (CGPoint)ptText bounce:(BOOL)bounce;
+- (void) hideMatch;
+
+@end
+
+//--------------------------------------------------------------------------------------------------
+
+@implementation FindHighlightLayer
+
+@synthesize sFind, positionFind, retaining, widthText, heightLine, sFont, fontSize;
+
+-(id) init {
+	if (self = [super init]) {
+		[self setNeedsDisplayOnBoundsChange: YES];
+		// A gold to slightly redder gradient to match other applications
+		CGColorRef colGold = CGColorCreateGenericRGB(1.0, 1.0, 0, 1.0);
+		CGColorRef colGoldRed = CGColorCreateGenericRGB(1.0, 0.8, 0, 1.0);
+		self.colors = [NSArray arrayWithObjects:(id)colGoldRed, (id)colGold, nil];
+		CGColorRelease(colGoldRed);
+		CGColorRelease(colGold);
+
+		CGColorRef colGreyBorder = CGColorCreateGenericGray(0.756f, 0.5f);
+		self.borderColor = colGreyBorder;
+		CGColorRelease(colGreyBorder);
+
+		self.borderWidth = 1.0;
+		self.cornerRadius = 5.0f;
+		self.shadowRadius = 1.0f;
+		self.shadowOpacity = 0.9f;
+		self.shadowOffset = CGSizeMake(0.0f, -2.0f);
+		self.anchorPoint = CGPointMake(0.5, 0.5);
+	}
+	return self;
+	
+}
+
+const CGFloat paddingHighlightX = 4;
+const CGFloat paddingHighlightY = 2;
+
+-(void) drawInContext:(CGContextRef)context {
+	if (!sFind || !sFont)
+		return;
+	
+	CFStringRef str = CFStringRef(sFind);
+	
+	CFMutableDictionaryRef styleDict = CFDictionaryCreateMutable(kCFAllocatorDefault, 2,
+								     &kCFTypeDictionaryKeyCallBacks, 
+								     &kCFTypeDictionaryValueCallBacks);
+	CGColorRef color = CGColorCreateGenericRGB(0.0, 0.0, 0.0, 1.0);
+	CFDictionarySetValue(styleDict, kCTForegroundColorAttributeName, color);
+	CTFontRef fontRef = ::CTFontCreateWithName((CFStringRef)sFont, fontSize, NULL);
+	CFDictionaryAddValue(styleDict, kCTFontAttributeName, fontRef);
+	
+	CFAttributedStringRef attrString = ::CFAttributedStringCreate(NULL, str, styleDict);
+	CTLineRef textLine = ::CTLineCreateWithAttributedString(attrString);
+	// Indent from corner of bounds
+	CGContextSetTextPosition(context, paddingHighlightX, 3 + paddingHighlightY);
+	CTLineDraw(textLine, context);
+	
+	CFRelease(textLine);
+	CFRelease(attrString);
+	CFRelease(fontRef);
+	CGColorRelease(color);
+	CFRelease(styleDict);
+}
+
+- (void) animateMatch: (CGPoint)ptText bounce:(BOOL)bounce {
+	if (!self.sFind || ![self.sFind length]) {
+		[self hideMatch];
+		return;
+	}
+
+	CGFloat width = self.widthText + paddingHighlightX * 2;
+	CGFloat height = self.heightLine + paddingHighlightY * 2;
+
+	CGFloat flipper = self.geometryFlipped ? -1.0 : 1.0;
+
+	// Adjust for padding
+	ptText.x -= paddingHighlightX;
+	ptText.y += flipper * paddingHighlightY;
+
+	// Shift point to centre as expanding about centre
+	ptText.x += width / 2.0;
+	ptText.y -= flipper * height / 2.0;
+
+	[CATransaction begin];
+	[CATransaction setValue:[NSNumber numberWithFloat:0.0] forKey:kCATransactionAnimationDuration];
+	self.bounds = CGRectMake(0,0, width, height);
+	self.position = ptText;
+	if (bounce) {
+		// Do not reset visibility when just moving
+		self.hidden = NO;
+		self.opacity = 1.0;
+	}
+	[self setNeedsDisplay];
+	[CATransaction commit];
+	
+	if (bounce) {
+		CABasicAnimation *animBounce = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
+		animBounce.duration = 0.15;
+		animBounce.autoreverses = YES;
+		animBounce.removedOnCompletion = NO;
+		animBounce.fromValue = [NSNumber numberWithFloat: 1.0];
+		animBounce.toValue = [NSNumber numberWithFloat: 1.25];
+		
+		if (self.retaining) {
+			
+			[self addAnimation: animBounce forKey:@"animateFound"];
+			
+		} else {
+			
+			CABasicAnimation *animFade = [CABasicAnimation animationWithKeyPath:@"opacity"];
+			animFade.duration = 0.1;
+			animFade.beginTime = 0.4;
+			animFade.removedOnCompletion = NO;
+			animFade.fromValue = [NSNumber numberWithFloat: 1.0];
+			animFade.toValue = [NSNumber numberWithFloat: 0.0];
+			
+			CAAnimationGroup *group = [CAAnimationGroup animation];
+			[group setDuration:0.5];
+			group.removedOnCompletion = NO;
+			group.fillMode = kCAFillModeForwards;
+			[group setAnimations:[NSArray arrayWithObjects:animBounce, animFade, nil]];
+			
+			[self addAnimation:group forKey:@"animateFound"];
+		}
+	}
+}
+
+- (void) hideMatch {
+	self.sFind = @"";
+	self.positionFind = INVALID_POSITION;
+	self.hidden = YES;
+}
+
+@end
+
+#endif
+
+//--------------------------------------------------------------------------------------------------
+
 @implementation TimerTarget
 
 - (id) init: (void*) target
 {
-  [super init];
+  self = [super init];
   if (self != nil)
   {
     mTarget = target;
@@ -155,7 +329,6 @@ static const KeyToCommand macMapDefault[] =
 
 - (void) dealloc
 {
-  [[NSNotificationCenter defaultCenter] removeObserver: self];
   [notificationQueue release];
   [super dealloc];
 }
@@ -211,6 +384,7 @@ ScintillaCocoa::ScintillaCocoa(NSView* view)
   //wMain= [view retain];// Don't retain since we're owned by view, which would cause a cycle
   wMain = view;
   timerTarget = [[TimerTarget alloc] init: this];
+  layerFindIndicator = NULL;
   Initialise();
 }
 
@@ -309,13 +483,14 @@ public:
                                                             locale:[NSLocale currentLocale]];
 
             const char *cpMapped = [sMapped UTF8String];
-			size_t lenMapped = strlen(cpMapped);
+			size_t lenMapped = cpMapped ? strlen(cpMapped) : 0;
 			if (lenMapped < sizeFolded) {
 				memcpy(folded, cpMapped,  lenMapped);
 			} else {
 				lenMapped = 0;
 			}
-            CFRelease(cfsVal);
+			if (cfsVal)
+				CFRelease(cfsVal);
 			return lenMapped;
 		}
 	}
@@ -419,7 +594,7 @@ std::string ScintillaCocoa::CaseMapString(const std::string &s, int caseMapping)
       sMapped = [(NSString *)cfsVal lowercaseString];
       break;
     default:
-      sMapped = [(NSString *)cfsVal copy];
+      sMapped = (NSString *)cfsVal;
   }
 
   // Back to encoding
@@ -432,6 +607,16 @@ std::string ScintillaCocoa::CaseMapString(const std::string &s, int caseMapping)
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Cancel all modes, both for base class and any find indicator.
+ */
+void ScintillaCocoa::CancelModes() {
+  ScintillaBase::CancelModes();
+  HideFindIndicator();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Helper function to get the outer container which represents the Scintilla editor on application side.
  */
@@ -539,19 +724,31 @@ sptr_t ScintillaCocoa::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lPar
       bufferedDraw = false;
       break;
 
+    case SCI_FINDINDICATORSHOW:
+      ShowFindIndicatorForRange(NSMakeRange(wParam, lParam-wParam), YES);
+      return 0;
+      
+    case SCI_FINDINDICATORFLASH:
+      ShowFindIndicatorForRange(NSMakeRange(wParam, lParam-wParam), NO);
+      return 0;
+		  
+    case SCI_FINDINDICATORHIDE:
+      HideFindIndicator();
+      return 0;
+		  
     case WM_UNICHAR: 
       // Special case not used normally. Characters passed in this way will be inserted
       // regardless of their value or modifier states. That means no command interpretation is
       // performed.
       if (IsUnicodeMode())
       {
-        NSString* input = [[NSString stringWithCharacters: (const unichar*) &wParam length: 1] autorelease];
+        NSString* input = [NSString stringWithCharacters: (const unichar*) &wParam length: 1];
         const char* utf8 = [input UTF8String];
         AddCharUTF((char*) utf8, static_cast<unsigned int>(strlen(utf8)), false);
         return 1;
       }
       return 0;
-      
+
     default:
       sptr_t r = ScintillaBase::WndProc(iMessage, wParam, lParam);
       
@@ -585,12 +782,11 @@ void ScintillaCocoa::SetTicking(bool on)
     if (timer.ticking)
     {
       // Scintilla ticks = milliseconds
-      // Using userInfo as flag to distinct between tick and idle timer.
-      NSTimer* tickTimer = [NSTimer scheduledTimerWithTimeInterval: timer.tickSize / 1000.0
-                                                            target: timerTarget
-                                                          selector: @selector(timerFired:)
-                                                          userInfo: nil
-                                                           repeats: YES];
+      tickTimer = [NSTimer scheduledTimerWithTimeInterval: timer.tickSize / 1000.0
+						   target: timerTarget
+						 selector: @selector(timerFired:)
+						 userInfo: nil
+						  repeats: YES];
       timer.tickerID = reinterpret_cast<TickerID>(tickTimer);
     }
     else
@@ -613,11 +809,11 @@ bool ScintillaCocoa::SetIdle(bool on)
     if (idler.state)
     {
       // Scintilla ticks = milliseconds
-      NSTimer* idleTimer = [NSTimer scheduledTimerWithTimeInterval: timer.tickSize / 1000.0
-                                                            target: timerTarget
-                                                          selector: @selector(idleTimerFired:)
-                                                          userInfo: nil
-                                                           repeats: YES];
+      idleTimer = [NSTimer scheduledTimerWithTimeInterval: timer.tickSize / 1000.0
+						   target: timerTarget
+						 selector: @selector(idleTimerFired:)
+						 userInfo: nil
+						  repeats: YES];
       idler.idlerID = reinterpret_cast<IdlerID>(idleTimer);
     }
     else
@@ -704,7 +900,7 @@ void ScintillaCocoa::Paste(bool forceRectangular)
 
 void ScintillaCocoa::CTPaint(void* gc, NSRect rc) {
 #pragma unused(rc)
-    Surface *surfaceWindow = Surface::Allocate();
+    Surface *surfaceWindow = Surface::Allocate(SC_TECHNOLOGY_DEFAULT);
     if (surfaceWindow) {
         surfaceWindow->Init(gc, wMain.GetID());
         surfaceWindow->SetUnicodeMode(SC_CP_UTF8 == ct.codePage);
@@ -784,8 +980,7 @@ void ScintillaCocoa::CreateCallTipWindow(PRectangle rc) {
         [callTip setLevel:NSFloatingWindowLevel];
         [callTip setHasShadow:YES];
         NSRect ctContent = NSMakeRect(0,0, rc.Width(), rc.Height());
-        CallTipView *caption = [CallTipView alloc];
-        [caption initWithFrame: ctContent];
+        CallTipView *caption = [[CallTipView alloc] initWithFrame: ctContent];
         [caption setAutoresizingMask: NSViewWidthSizable | NSViewMaxYMargin];
         [caption setSci: this];
         [[callTip contentView] addSubview: caption];
@@ -802,15 +997,15 @@ void ScintillaCocoa::AddToPopUp(const char *label, int cmd, bool enabled)
   [menu setOwner: this];
   [menu setAutoenablesItems: NO];
   
-  if (cmd == 0)
+  if (cmd == 0) {
     item = [NSMenuItem separatorItem];
-  else
-    item = [[NSMenuItem alloc] init];
-  
+  } else {
+    item = [[[NSMenuItem alloc] init] autorelease];
+    [item setTitle: [NSString stringWithUTF8String: label]];
+  }
   [item setTarget: menu];
   [item setAction: @selector(handleCommand:)];
   [item setTag: cmd];
-  [item setTitle: [NSString stringWithUTF8String: label]];
   [item setEnabled: enabled];
   
   [menu addItem: item];
@@ -833,10 +1028,8 @@ NSPoint ScintillaCocoa::GetCaretPosition()
 {
   NSPoint result;
 
-  int position = sel.RangeMain().caret.Position();
-  result.y = pdoc->LineFromPosition(position);
-  result.x = pdoc->GetColumn(position);
-  
+  result.y = pdoc->LineFromPosition(sel.RangeMain().caret.Position());
+  result.x = sel.RangeMain().caret.Position() - pdoc->LineStart(result.y);
   return result;
 }
 
@@ -1173,7 +1366,8 @@ void ScintillaCocoa::SetPasteboardData(NSPasteboard* board, const SelectionText
   
   [board setString: (NSString *)cfsVal forType: NSStringPboardType];
 
-  CFRelease(cfsVal);
+  if (cfsVal)
+    CFRelease(cfsVal);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1249,29 +1443,24 @@ bool ScintillaCocoa::HaveMouseCapture()
 /**
  * Synchronously paint a rectangle of the window.
  */
-void ScintillaCocoa::SyncPaint(void* gc, PRectangle rc)
+bool ScintillaCocoa::SyncPaint(void* gc, PRectangle rc)
 {
   paintState = painting;
   rcPaint = rc;
   PRectangle rcText = GetTextRectangle();
   paintingAllText = rcPaint.Contains(rcText);
-  Surface *sw = Surface::Allocate();
+  bool succeeded = true;
+  Surface *sw = Surface::Allocate(SC_TECHNOLOGY_DEFAULT);
   if (sw)
   {
     sw->Init(gc, wMain.GetID());
     Paint(sw, rc);
-    if (paintState == paintAbandoned)
-    {
-      // Do a full paint.
-      rcPaint = GetClientRectangle();
-      paintState = painting;
-      paintingAllText = true;
-      Paint(sw, rcPaint);
-    }
+    succeeded = paintState != paintAbandoned;
     sw->Release();
     delete sw;
   }
   paintState = notPainting;
+  return succeeded;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1284,6 +1473,11 @@ void ScintillaCocoa::ScrollText(int linesToMove)
 {
 	// Move those pixels
 	NSView *content = ContentView();
+	if ([content layer]) {
+		[content setNeedsDisplay: YES];
+		MoveFindIndicatorWithBounce(NO);
+		return;
+	}
     
 	[content lockFocus];
 	int diff = vs.lineHeight * linesToMove;
@@ -1343,8 +1537,14 @@ void ScintillaCocoa::SetHorizontalScrollPos()
   
   // Convert absolute coordinate into the range [0..1]. Keep in mind that the visible area
   // does *not* belong to the scroll range.
-  float relativePosition = (float) xOffset / (scrollWidth - textRect.Width());
+  int maxXOffset = scrollWidth - textRect.Width();
+  if (maxXOffset < 0)
+    maxXOffset = 0;
+  if (xOffset > maxXOffset)
+    xOffset = maxXOffset;
+  float relativePosition = (float) xOffset / maxXOffset;
   [topContainer setHorizontalScrollPosition: relativePosition];
+  MoveFindIndicatorWithBounce(NO);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1379,6 +1579,8 @@ bool ScintillaCocoa::ModifyScrollBars(int nMax, int nPage)
   else
     pageSize = scrollRange;
   bool horizontalChange = [topContainer setHorizontalScrollRange: scrollRange page: pageSize];
+
+  MoveFindIndicatorWithBounce(NO);	
   
   return verticalChange || horizontalChange;
 }
@@ -1568,9 +1770,9 @@ void ScintillaCocoa::IdleTimerFired()
  * @param rect The area to paint, given in the sender's coordinate system.
  * @param gc The context we can use to paint.
  */
-void ScintillaCocoa::Draw(NSRect rect, CGContextRef gc)
+bool ScintillaCocoa::Draw(NSRect rect, CGContextRef gc)
 {
-  SyncPaint(gc, NSRectToPRectangle(rect));
+  return SyncPaint(gc, NSRectToPRectangle(rect));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1846,3 +2048,79 @@ void ScintillaCocoa::ActiveStateChanged(bool isActive)
 
 //--------------------------------------------------------------------------------------------------
 
+void ScintillaCocoa::ShowFindIndicatorForRange(NSRange charRange, BOOL retaining)
+{
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+  NSView *content = ContentView();
+  if (!layerFindIndicator)
+  {
+    layerFindIndicator = [[FindHighlightLayer alloc] init];
+    [content setWantsLayer: YES];
+    layerFindIndicator.geometryFlipped = content.layer.geometryFlipped;
+    [[content layer] addSublayer:layerFindIndicator];
+  }
+  [layerFindIndicator removeAnimationForKey:@"animateFound"];
+  
+  if (charRange.length)
+  {
+    CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
+							 vs.styles[STYLE_DEFAULT].characterSet);
+    std::vector<char> buffer(charRange.length);
+    pdoc->GetCharRange(&buffer[0], charRange.location, charRange.length);
+    
+    CFStringRef cfsFind = CFStringCreateWithBytes(kCFAllocatorDefault,
+						  reinterpret_cast<const UInt8 *>(&buffer[0]), 
+						  charRange.length, encoding, false);
+    layerFindIndicator.sFind = (NSString *)cfsFind;
+    if (cfsFind)
+        CFRelease(cfsFind);
+    layerFindIndicator.retaining = retaining;
+    layerFindIndicator.positionFind = charRange.location;
+    int style = WndProc(SCI_GETSTYLEAT, charRange.location, 0);
+    std::vector<char> bufferFontName(WndProc(SCI_STYLEGETFONT, style, 0) + 1);
+    WndProc(SCI_STYLEGETFONT, style, (sptr_t)&bufferFontName[0]);
+    layerFindIndicator.sFont = [NSString stringWithUTF8String: &bufferFontName[0]];
+    
+    layerFindIndicator.fontSize = WndProc(SCI_STYLEGETSIZEFRACTIONAL, style, 0) / 
+      (float)SC_FONT_SIZE_MULTIPLIER;
+    layerFindIndicator.widthText = WndProc(SCI_POINTXFROMPOSITION, 0, charRange.location + charRange.length) -
+      WndProc(SCI_POINTXFROMPOSITION, 0, charRange.location);
+    layerFindIndicator.heightLine = WndProc(SCI_TEXTHEIGHT, 0, 0);
+    MoveFindIndicatorWithBounce(YES);
+  }
+  else
+  {
+    [layerFindIndicator hideMatch];
+  }
+#endif
+}
+
+void ScintillaCocoa::MoveFindIndicatorWithBounce(BOOL bounce)
+{
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+  if (layerFindIndicator)
+  {
+    CGPoint ptText = CGPointMake(
+      WndProc(SCI_POINTXFROMPOSITION, 0, layerFindIndicator.positionFind),
+      WndProc(SCI_POINTYFROMPOSITION, 0, layerFindIndicator.positionFind));
+    if (!layerFindIndicator.geometryFlipped)
+    {
+      NSView *content = ContentView();
+      ptText.y = content.bounds.size.height - ptText.y;
+    }
+    [layerFindIndicator animateMatch:ptText bounce:bounce];
+  }
+#endif
+}
+
+void ScintillaCocoa::HideFindIndicator()
+{
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+  if (layerFindIndicator)
+  {
+    [layerFindIndicator hideMatch];
+  }
+#endif
+}
+
+
