@@ -130,7 +130,9 @@ NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
 {
   CGContextRef context = (CGContextRef) [[NSGraphicsContext currentContext] graphicsPort];
   
-  mOwner.backend->Draw(rect, context);
+  if (!mOwner.backend->Draw(rect, context)) {
+    [self display];
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -656,12 +658,24 @@ NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Sends a new notification of the given type to the default notification center.
+ * Receives zoom messages, for example when a "pinch zoom" is performed on the trackpad.
  */
-- (void) sendNotification: (NSString*) notificationName
+- (void) magnifyWithEvent: (NSEvent *) event
 {
-  NSNotificationCenter* center = [NSNotificationCenter defaultCenter];
-  [center postNotificationName: notificationName object: self];
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+  zoomDelta += event.magnification * 10.0;
+
+  if (fabsf(zoomDelta)>=1.0) {
+    long zoomFactor = [self getGeneralProperty: SCI_GETZOOM] + zoomDelta;
+    [self setGeneralProperty: SCI_SETZOOM parameter: zoomFactor value:0];
+    zoomDelta = 0.0;
+  }     
+#endif
+}
+
+- (void) beginGestureWithEvent: (NSEvent *) event
+{
+  zoomDelta = 0.0;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -677,6 +691,17 @@ NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Sends a new notification of the given type to the default notification center.
+ */
+- (void) sendNotification: (NSString*) notificationName
+{
+  NSNotificationCenter* center = [NSNotificationCenter defaultCenter];
+  [center postNotificationName: notificationName object: self];
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Called by a connected component (usually the info bar) if something changed there.
  *
@@ -891,8 +916,6 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 - (void) dealloc
 {
   [[NSNotificationCenter defaultCenter] removeObserver: self];
-  // infobar is a subview and there are no extra retains
-  //[mInfoBar release];
   delete mBackend;
   [super dealloc];
 }
@@ -919,7 +942,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 {
   [super viewDidMoveToWindow];
   
-  [self layout];
+  [self positionSubViews];
   
   // Enable also mouse move events for our window (and so this view).
   [[self window] setAcceptsMouseMovedEvents: YES];
@@ -930,7 +953,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 /**
  * Used to position and size the parts of the editor (content, scrollers, info bar).
  */
-- (void) layout
+- (void) positionSubViews
 {
   int scrollerWidth = [NSScroller scrollerWidth];
 
@@ -1043,7 +1066,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
     [mVerticalScroller setHidden: hideScroller];
     if (!hideScroller)
       [mVerticalScroller setFloatValue: 0];
-    [self layout];
+    [self positionSubViews];
   }
   
   if (!hideScroller)
@@ -1093,7 +1116,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
   {
     result = YES;
     [mHorizontalScroller setHidden: hideScroller];
-    [self layout];
+    [self positionSubViews];
   }
   
   if (!hideScroller)
@@ -1144,7 +1167,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 - (void) setFrame: (NSRect) newFrame
 {
   [super setFrame: newFrame];
-  [self layout];
+  [self positionSubViews];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1165,7 +1188,6 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
     try
     {
       mBackend->WndProc(SCI_GETSELTEXT, length + 1, (sptr_t) buffer);
-      mBackend->WndProc(SCI_SETSAVEPOINT, 0, 0);
       
       result = [NSString stringWithUTF8String: buffer];
       delete[] buffer;
@@ -1198,7 +1220,6 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
     try
     {
       mBackend->WndProc(SCI_GETTEXT, length + 1, (sptr_t) buffer);
-      mBackend->WndProc(SCI_SETSAVEPOINT, 0, 0);
       
       result = [NSString stringWithUTF8String: buffer];
       delete[] buffer;
@@ -1543,7 +1564,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
       mInitialInfoBarWidth = [mInfoBar frame].size.width;
     }
     
-    [self layout];
+    [self positionSubViews];
   }
 }
 
