@@ -22,9 +22,10 @@
 #include "document_properties_form.h"
 #include "workbench/wb_context_ui.h"
 #include "base/string_utilities.h"
+#include "mforms/label.h"
 
 DocumentPropertiesForm::DocumentPropertiesForm(wb::WBContextUI *wbui)
-  : Form(NULL, mforms::FormResizable), _bottom_box(true), _text(mforms::VerticalScrollBar)
+  : Form(NULL, mforms::FormResizable), _bottom_box(true), _text(mforms::BothScrollBars)
 {
   _wbui= wbui;
   
