@@ -26,6 +26,7 @@
 #include "base/string_utilities.h"
 #include "base/util_functions.h"
 
+#include "grtdb/db_object_helpers.h"
 #include "snippet_popover.h"
 #include "mforms/textbox.h"
 
@@ -138,19 +139,18 @@ public:
 
 
 
-PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model_id)
+PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const workbench_physical_ModelRef &model)
   : Form(NULL, mforms::FormResizable), _top_box(false), _bottom_box(true), 
   _tabview(mforms::TabViewDocument), _button_box(true), _font_list(mforms::TreeFlatList)
 {
   _wbui= wbui;
-  _model_id= model_id;
+  _model= model;
 
-  if (model_id.empty())
+  if (!model.is_valid())
     set_title(_("Workbench Preferences"));
   else
     set_title(_("Model Options"));
 
-  set_size(700, 680);
   set_content(&_top_box);
 #ifdef _WIN32
   set_back_color("#283752");
@@ -179,9 +179,9 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model
   _ok_button.enable_internal_padding(true);
   _button_box.add_end(&_ok_button, false, true);
   
-  if (!model_id.empty())
+  if (_model.is_valid())
   {
-    _use_global.set_text(_("Use Global Settings"));
+    _use_global.set_text(_("Use defaults from global settings"));
 #ifdef _WIN32
     _use_global.set_front_color("#FFFFFF");
 #endif
@@ -189,7 +189,7 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model
     scoped_connect(_use_global.signal_clicked(),boost::bind(&PreferencesForm::toggle_use_global, this));
   }
 
-  if (model_id.empty())
+  if (!_model.is_valid())
   {
     create_general_page();
     create_admin_page();
@@ -199,13 +199,14 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model
   create_model_page();
   create_mysql_page();
   create_diagram_page();
-  if (model_id.empty())
+  if (!_model.is_valid())
     create_appearance_page();
 
 #if defined(_DEBUG) || defined(ENABLE_DEBUG)
   create_test_page();
 #endif
 
+  set_size(700, 680);
   center();
   
   show_values();
@@ -223,15 +224,15 @@ void PreferencesForm::show_values()
   for (std::list<Option*>::const_iterator iter= _options.begin(); iter != _options.end(); ++iter)
     (*iter)->show_value();
 
-  if (_model_id.empty())
+  if (!_model.is_valid())
   {
     show_colors_and_fonts();
   }
 
-  if (!_model_id.empty())
+  if (_model.is_valid())
   {
     std::string value;
-    _wbui->get_wb_options_value(_model_id, "useglobal", value);
+    _wbui->get_wb_options_value(_model.id(), "useglobal", value);
     if (value == "1")
     {
       _use_global.set_active(true);
@@ -243,21 +244,22 @@ void PreferencesForm::show_values()
 
 void PreferencesForm::update_values()
 {
-  grt::AutoUndo undo(_wbui->get_wb()->get_grt(), _model_id.empty());
+  grt::AutoUndo undo(_wbui->get_wb()->get_grt(), !_model.is_valid());
 
-  if (!_model_id.empty())
+  if (_model.is_valid())
   {
-    _wbui->set_wb_options_value(_model_id, "useglobal", _use_global.get_active() ? "1" : "0");
+    _wbui->set_wb_options_value(_model.id(), "useglobal", _use_global.get_active() ? "1" : "0");
   }
 
-  if (_model_id.empty() || !_use_global.get_active())
+  if (!_model.is_valid() || !_use_global.get_active())
   {
     for (std::list<Option*>::const_iterator iter= _options.begin(); iter != _options.end(); ++iter)
     {
       (*iter)->update_value();
     }
-    update_colors_and_fonts();
   }
+  if (!_model.is_valid())
+    update_colors_and_fonts();
 
   undo.end(_("Change Options"));
 }
@@ -265,10 +267,10 @@ void PreferencesForm::update_values()
 
 grt::DictRef PreferencesForm::get_options(bool global)
 {
-  if (_model_id.empty() || global)
+  if (!_model.is_valid() || global)
     return _wbui->get_wb()->get_wb_options();
   else
-    return _wbui->get_model_options(_model_id);
+    return _wbui->get_model_options(_model.id());
 }
 
 
@@ -276,7 +278,7 @@ void PreferencesForm::show_entry_option(const std::string &option_name, mforms::
 {
   std::string value;
 
-  _wbui->get_wb_options_value(_model_id, option_name, value);
+  _wbui->get_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, value);
   entry->set_value(value);
 }
 
@@ -284,22 +286,22 @@ void PreferencesForm::show_entry_option(const std::string &option_name, mforms::
 void PreferencesForm::update_entry_option(const std::string &option_name, mforms::TextEntry *entry, bool numeric)
 {
   if (numeric)
-    _wbui->set_wb_options_value(_model_id, option_name, entry->get_string_value(), grt::AnyType);
+    _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, entry->get_string_value(), grt::AnyType);
   else
-    _wbui->set_wb_options_value(_model_id, option_name, entry->get_string_value(), grt::StringType);
+    _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, entry->get_string_value(), grt::StringType);
 }
 
 void PreferencesForm::show_path_option(const std::string &option_name, mforms::FsObjectSelector *entry)
 {
   std::string value;
   
-  _wbui->get_wb_options_value(_model_id, option_name, value);
+  _wbui->get_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, value);
   entry->set_filename(value);
 }
 
 void PreferencesForm::update_path_option(const std::string &option_name, mforms::FsObjectSelector *entry)
 {
-  _wbui->set_wb_options_value(_model_id, option_name, entry->get_filename(), grt::StringType);
+  _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, entry->get_filename(), grt::StringType);
 }
 
 
@@ -311,7 +313,7 @@ void PreferencesForm::update_entry_option_numeric(const std::string &option_name
   else if (value > maxrange)
     value= maxrange;
   
-  _wbui->set_wb_options_value(_model_id, option_name, strfmt("%li", value));
+  _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, strfmt("%li", value));
 }
 
 
@@ -319,7 +321,7 @@ void PreferencesForm::show_checkbox_option(const std::string &option_name, mform
 {
   std::string value;
 
-  _wbui->get_wb_options_value(_model_id, option_name, value);
+  _wbui->get_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, value);
 
   checkbox->set_active(atoi(value.c_str()) != 0);
 }
@@ -328,7 +330,7 @@ void PreferencesForm::show_checkbox_option(const std::string &option_name, mform
 void PreferencesForm::update_checkbox_option(const std::string &option_name, mforms::CheckBox *checkbox)
 {
   std::string value = checkbox->get_active() ? "1" : "0";
-  _wbui->set_wb_options_value(_model_id, option_name, value, grt::IntegerType);
+  _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, value, grt::IntegerType);
 
 #ifdef _WIN32
   // On Windows we have to write the following value also to the registry as our options are not
@@ -344,7 +346,7 @@ void PreferencesForm::show_selector_option(const std::string &option_name, mform
                                            const std::vector<std::string> &choices)
 {
   std::string value;
-  _wbui->get_wb_options_value(_model_id, option_name, value);
+  _wbui->get_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, value);
   selector->set_selected(std::find(choices.begin(), choices.end(), value) - choices.begin());
 }
 
@@ -355,16 +357,16 @@ void PreferencesForm::update_selector_option(const std::string &option_name, mfo
   if (as_number)
   {
     if (selector->get_selected_index() < 0)
-      _wbui->set_wb_options_value(_model_id, option_name, default_value, grt::IntegerType);
+      _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, default_value, grt::IntegerType);
     else
-      _wbui->set_wb_options_value(_model_id, option_name, choices[selector->get_selected_index()], grt::IntegerType);
+      _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, choices[selector->get_selected_index()], grt::IntegerType);
   }
   else 
   {
     if (selector->get_selected_index() < 0)
-      _wbui->set_wb_options_value(_model_id, option_name, default_value);
+      _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, default_value);
     else
-      _wbui->set_wb_options_value(_model_id, option_name, choices[selector->get_selected_index()]);
+      _wbui->set_wb_options_value(_model.is_valid() ? _model.id() : "", option_name, choices[selector->get_selected_index()]);
   }
 }
 
@@ -433,7 +435,7 @@ mforms::Selector *PreferencesForm::new_selector_option(const std::string &option
   mforms::Selector *selector= new mforms::Selector();
 
   if (choices_string.empty())
-    _wbui->get_wb_options_value(_model_id, "@"+option_name+"/Items", choices_string);
+    _wbui->get_wb_options_value(_model.is_valid() ? _model.id() : "", "@"+option_name+"/Items", choices_string);
 
   std::vector<std::string> choices, parts= base::split(choices_string, ",");
 
@@ -636,7 +638,7 @@ void PreferencesForm::create_sqlide_page()
         // Set initial enabled state of the auto-cc checkbox depending on whether
         // code completion is enabled at all.
         std::string value;
-        _wbui->get_wb_options_value(_model_id, "DbSqlEditor:CodeCompletionEnabled", value);
+        _wbui->get_wb_options_value(_model.is_valid() ? _model.id() : "", "DbSqlEditor:CodeCompletionEnabled", value);
         auto_start_check->set_enabled(atoi(value.c_str()) != 0);
       }
       {
@@ -674,7 +676,7 @@ void PreferencesForm::create_sqlide_page()
   
   {
     mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
-    frame->set_title(_("SQL"));
+    frame->set_title(_("SQL Parsing in Text Editors"));
     box->add(frame, false);
     
     mforms::Box *vbox= mforms::manage(new mforms::Box(false));
@@ -687,7 +689,7 @@ void PreferencesForm::create_sqlide_page()
       tbox->set_spacing(4);
       vbox->add(tbox, false);
       
-      tbox->add(new_label(_("Default SQL_MODE:"), true), false, false);
+      tbox->add(new_label(_("Default SQL_MODE for syntax checker:"), true), false, false);
       mforms::TextEntry *entry= new_entry_option("SqlMode", false);
       entry->set_tooltip(_(
                            "Value of SQL_MODE DBMS session variable customizes the rules and restrictions for SQL syntax and semantics. See MySQL Server reference for details.\n"
@@ -955,8 +957,8 @@ void PreferencesForm::create_general_page()
     }
     
     discard_unsaved = table->add_checkbox_option("DbSqlEditor:DiscardUnsavedQueryTabs",
-                               _("Discard query tabs that were never saved"),
-                               _("Enable if you'd like to automatically discard query tabs that are not stored in a file when closing them.\n"
+                               _("Create new tabs as Query tabs instead of File"),
+                               _("Unsaved Query tabs do not get a close confirmation, unlike File tabs.\nHowever, once saved, such tabs will also get unsaved change confirmations.\n"
                                  "If Snapshot saving is enabled, query tabs are always autosaved to temporary files when the connection is closed."));
     save_workspace->signal_clicked()->connect(boost::bind(force_checkbox_on_toggle, save_workspace, discard_unsaved, true, true));
     (*save_workspace->signal_clicked())();
@@ -1124,28 +1126,80 @@ void PreferencesForm::create_model_page()
 }
 
 
+static void show_target_version(const workbench_physical_ModelRef &model, mforms::TextEntry *entry)
+{
+  entry->set_value(base::strfmt("%li.%li.%li", *model->catalog()->version()->majorNumber(),
+                   *model->catalog()->version()->minorNumber(), *model->catalog()->version()->releaseNumber()));
+}
+
+static void update_target_version(workbench_physical_ModelRef model, mforms::TextEntry *entry)
+{
+  GrtVersionRef version = bec::CatalogHelper::parse_version(model.get_grt(), entry->get_string_value());
+  model->catalog()->version(version);
+  version->owner(model);
+}
+
+
 void PreferencesForm::create_mysql_page()
 {
   mforms::Box *box= mforms::manage(new mforms::Box(false));
   box->set_padding(12);
   box->set_spacing(8);
 
+  {
+    mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
+    frame->set_title(_("Model"));
+
+    mforms::Table *table= mforms::manage(new mforms::Table());
+
+    table->set_padding(8);
+
+    frame->add(table);
+    table->set_row_count(2);
+    table->set_column_count(2);
+    
+    if (!_model.is_valid())
+    {
+      table->add(new_label(_("Default Target MySQL Version:"), true), 0, 1, 0, 1, 0);
+      table->add(new_selector_option("DefaultTargetMySQLVersion"), 1, 2, 0, 1, mforms::HExpandFlag|mforms::HFillFlag);
+    }
+    else
+    {
+      // if editing model options, display the catalog version
+      Option *option= new Option();
+      mforms::TextEntry *entry= new mforms::TextEntry();
+      
+      option->view= mforms::manage(entry);
+      option->show_value= boost::bind(show_target_version, _model, entry);
+      option->update_value= boost::bind(update_target_version, _model, entry);
+
+      option->view= mforms::manage(entry);
+      option->show_value= boost::bind(show_target_version, _model, entry);
+      option->update_value= boost::bind(update_target_version, _model, entry);
+      _options.push_back(option);
+      
+      table->add(new_label(_("Target MySQL Version:"), true), 0, 1, 0, 1, 0);
+      table->add(entry, 1, 2, 0, 1, mforms::HExpandFlag|mforms::HFillFlag);
+    }
+    box->add(frame, false);
+  }
+
   {
     mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
     frame->set_title(_("Model Table Defaults"));
-
+    
     mforms::Box *tbox= mforms::manage(new mforms::Box(true));
-
+    
     tbox->set_padding(8);
-
+    
     frame->add(tbox);
-
+    
     tbox->add(new_label(_("Default Storage Engine:"), true), false, false);
     tbox->add(new_selector_option("db.mysql.Table:tableEngine"), true, true);
     
     box->add(frame, false);
   }
-  
+
   {
     mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
     frame->set_title(_("Forward Engineering and Synchronization"));
