@@ -34,13 +34,11 @@ static void run_menu_and_forward_action(const bec::MenuItemList& items
                                         ,const sigc::slot<void, const std::string&, const std::vector<bec::NodeId>&> fe_menu_handler
                                         ,Gtk::Menu *menu)
 {
-  if (items.size() > 0)
-  {
+  if (!items.empty())
     run_popup_menu(items
                    ,time
                    ,sigc::bind(sigc::ptr_fun(process_menu_actions), model, nodes, fe_menu_handler)
                    ,menu);
-  }
 }
 
 
@@ -509,10 +507,11 @@ void ColumnsModel::add_model_column(Gtk::TreeModelColumnBase *col, int bec_tm_id
 struct LMPtr {
   ListModelWrapper *ptr;
   int count;
+  bool gone;
 
-  LMPtr(ListModelWrapper *p) : ptr(p), count(1) {}
+  LMPtr(ListModelWrapper *p) : ptr(p), count(1), gone(false) {}
   LMPtr *retain() { count++; return this; }
-  void release() { if (--count == 0) delete this; }
+  void release() { if (--count == 0) gone = true; /*delete this;*/  }
 };
 
 ListModelWrapper::ListModelWrapper(bec::ListModel* tm, Gtk::TreeView *treeview, const std::string& name)
@@ -543,7 +542,7 @@ void* ListModelWrapper::on_bec_model_destroyed(void* data)
   LMPtr* self = (LMPtr*)data;
   if (self)
   {
-    self->ptr->_tm = 0;
+//    self->ptr->_tm = 0;
     self->release();
   }
   return 0;
@@ -624,9 +623,7 @@ void ListModelWrapper::handle_popup(const int x, const int y, const int time, Gd
     there_is_path_at_pos = path.gobj() && !path.empty();
   }
 
-  if ( !there_is_path_at_pos )
-    list.clear();
-  else
+  if ( there_is_path_at_pos )
   {
     // Check that @path is on selection, otherwise add @path to selection
     bec::NodeId node = get_node_for_path(path);
@@ -661,12 +658,11 @@ void ListModelWrapper::handle_popup(const int x, const int y, const int time, Gd
 
       list = get_selection();
     }
-
-    if (!_context_menu)
-      _context_menu = new Gtk::Menu();
-
-    run_menu_and_forward_action(_tm->get_popup_items_for_nodes(list), x, y, time, _tm, list, _fe_menu_handler, _context_menu);
   }
+  if (!_context_menu)
+    _context_menu = new Gtk::Menu();
+
+  run_menu_and_forward_action(_tm->get_popup_items_for_nodes(list), x, y, time, _tm, list, _fe_menu_handler, _context_menu);
 }
 
 //------------------------------------------------------------------------------
@@ -702,7 +698,7 @@ bec::NodeId ListModelWrapper::get_node_for_path(const Gtk::TreeModel::Path &path
 //------------------------------------------------------------------------------
 bool ListModelWrapper::init_gtktreeiter(GtkTreeIter *it, const bec::NodeId &node) const
 {
-  if ( _tm && it && node.is_valid() )
+  if (_tm && it && node.is_valid() && !_self_ref->gone)
   {
     Index id(it, node);
     id.stamp(_stamp);
@@ -730,7 +726,7 @@ bec::NodeId ListModelWrapper::node_for_iter(const iterator &iter) const
 //------------------------------------------------------------------------------
 void ListModelWrapper::refresh()
 {
-  if (_tm) _tm->refresh();
+  if (_tm && !_self_ref->gone) _tm->refresh();
   model_changed(bec::NodeId(), -1);
 }
 
@@ -738,7 +734,7 @@ void ListModelWrapper::refresh()
 //------------------------------------------------------------------------------
 void ListModelWrapper::note_row_added()
 {
-  if (_tm)
+  if (_tm && !_self_ref->gone)
   {
     _tm->refresh();
     Gtk::TreePath path(_tm->count()-1);
@@ -789,7 +785,7 @@ bool ListModelWrapper::iter_next_vfunc(const iterator& iter, iterator& iter_next
   // Invalidate iter_next
   reset_iter(iter_next);
 
-  if (_tm)
+  if (_tm && !_self_ref->gone)
   {
     if (current_node.is_valid() && _tm->has_next(current_node))
     {
@@ -819,7 +815,7 @@ bool ListModelWrapper::get_iter_vfunc(const Path& path, iterator& iter) const
   // Invalidate iter_next
   reset_iter(iter);
 
-  if (_tm)
+  if (_tm && !_self_ref->gone)
   {
     bec::NodeId node(path.to_string());
 
@@ -855,7 +851,7 @@ bool ListModelWrapper::iter_nth_root_child_vfunc(int n, iterator& iter) const
   //index is 0.  If @a n is too big, or if there are no children, @a iter is set
   //to an invalid iterator and false is returned.
   //See also iter_nth_child_vfunc().
-  if ( _tm && n >= 0 && n < iter_n_root_children_vfunc() )
+  if ( _tm && !_self_ref->gone && n >= 0 && n < iter_n_root_children_vfunc() )
   {
     bec::NodeId node = _tm->get_node(n);
     init_gtktreeiter(iter.gobj(), node);
@@ -880,7 +876,7 @@ int ListModelWrapper::iter_n_children_vfunc(const iterator& iter) const
 //------------------------------------------------------------------------------
 int ListModelWrapper::iter_n_root_children_vfunc() const
 {
-  const int ret = _tm ? _tm->count() : 0;
+  const int ret = _tm && !_self_ref->gone ? _tm->count() : 0;
   return ret;
 }
 
@@ -904,7 +900,7 @@ Gtk::TreeModel::Path ListModelWrapper::get_path_vfunc(const iterator& iter) cons
 //------------------------------------------------------------------------------
 void ListModelWrapper::get_icon_value(const iterator& iter, int column, const bec::NodeId &node, Glib::ValueBase& value) const
 {
-  if (!_tm)
+  if (!_tm ||  _self_ref->gone)
     return;
 
   static ImageCache       *pixbufs      =  ImageCache::get_instance();
@@ -928,7 +924,7 @@ void ListModelWrapper::get_icon_value(const iterator& iter, int column, const be
 //------------------------------------------------------------------------------
 void ListModelWrapper::get_value_vfunc(const iterator& iter, int column, Glib::ValueBase& value) const
 {
-  if (!_tm)
+  if (!_tm && !_self_ref->gone)
     return;
 
   bec::NodeId node= node_for_iter(iter);
@@ -1016,7 +1012,7 @@ bool ListModelWrapper::iter_is_valid(const iterator& iter) const
 //------------------------------------------------------------------------------
 void ListModelWrapper::set_value_impl(const iterator& row, int column, const Glib::ValueBase& value)
 {
-  if (!_tm)
+  if (!_tm || _self_ref->gone)
     return;
 
   bec::NodeId node(node_for_iter(row));
