@@ -18,6 +18,7 @@
 #include <gtkmm/iconview.h>
 #include <gtkmm/liststore.h>
 
+#include "base/trackable.h"
 #include "grt/tree_model.h"
 #include "image_cache.h"
 #include "wbdbg.h"
@@ -232,7 +233,7 @@ inline bool can_convert(const Glib::ustring& str, std::string &val)
 
 class ListModelWrapper : public Glib::Object, public Gtk::TreeModel, 
                          public Gtk::TreeDragDest, public Gtk::TreeDragSource,
-                         virtual public sigc::trackable
+                         public base::trackable
 {
      friend class ColumnsModel;
   protected:
@@ -322,7 +323,7 @@ class ListModelWrapper : public Glib::Object, public Gtk::TreeModel,
     typedef std::vector<bec::NodeId> NodeIdArray;
     NodeIdArray get_selection() const;
   #ifndef NO_MENU_MANAGER
-    void handle_popup(const int x, const int y, const int time);
+    void handle_popup(const int x, const int y, const int time, GdkEventButton* evb);
     virtual bool handle_popup_event(GdkEvent* event);
     void set_fe_menu_handler(const sigc::slot<void, const std::string&, const std::vector<bec::NodeId>&> &slot)
     {
@@ -527,10 +528,11 @@ class ListModelWrapper : public Glib::Object, public Gtk::TreeModel,
     bec::ListModel  *_tm;
     Gtk::TreeView   *_treeview;
     Gtk::IconView   *_iconview;
+    Gtk::Menu       *_context_menu;
     int              _stamp;              //!< stamp to check if an iterator is invalid
     ColumnsModel     _columns;            //!< Generic model of columns
     bec::IconSize    _icon_size;          //!< icon size to set by mode view changed buttons
-    ListModelWrapper** _self_ref; // hack to workaround a crash.. 
+    struct LMPtr    *_self_ref; // hack to workaround a crash.. 
 
     std::string      _name;
 
