@@ -1,7 +1,9 @@
 #include "plugin_editor_base.h"
 #include "grt/editor_base.h"
-#include "sql_editor_fe.h"
 #include "base/string_utilities.h"
+#include <gtkmm/combobox.h>
+#include "mforms/code_editor.h"
+#include "gtk/mforms_gtk.h"
 
 using base::strfmt;
 
@@ -10,11 +12,14 @@ using base::strfmt;
 
 //------------------------------------------------------------------------------
 PluginEditorBase::PluginEditorBase(grt::Module *module, bec::GRTManager *grtm, const grt::BaseListRef &args, const char* glade_file)
-  : GUIPluginBase(module), _grtm(grtm)
-  , _xml(0)
+  : GUIPluginBase(module)
   , _editor_notebook(0)
+  , _grtm(grtm)
+  , _xml(0)
   , _live_object_editor_decorator_xml(0)
   , _live_object_editor_decorator_control(0)
+  , _old_embedded_editor(0)
+  , _old_embedded_find(0)
 {
   _refreshing= false;
   set_shadow_type(Gtk::SHADOW_NONE);
@@ -25,7 +30,28 @@ PluginEditorBase::PluginEditorBase(grt::Module *module, bec::GRTManager *grtm, c
 
 //------------------------------------------------------------------------------
 PluginEditorBase::~PluginEditorBase()
-{  
+{
+}
+
+//------------------------------------------------------------------------------
+
+std::string PluginEditorBase::get_title()
+{ 
+  return get_be()->get_title(); 
+}
+
+//------------------------------------------------------------------------------
+void PluginEditorBase::load_glade(const char* glade_xml_filename)
+{
+  if (_xml)
+    throw std::logic_error("XML already created"); // TODO: better message here
+
+  if (glade_xml_filename)
+  {
+    _xml = Gtk::Builder::create_from_file(_grtm->get_data_file_path(glade_xml_filename));
+    if (!_xml)
+      throw std::logic_error("Can't load glade xml");
+  }
 }
 
 //------------------------------------------------------------------------------
@@ -59,13 +85,9 @@ void PluginEditorBase::decorate_object_editor()
     if (!_live_object_editor_decorator_control)
     {
       _live_object_editor_decorator_xml= Gtk::Builder::create_from_file(_grtm->get_data_file_path("modules/data/live_editor_decoration.glade"));
-      _live_object_editor_decorator_xml->get_widget("live_editor_decorator_vpane", _live_object_editor_decorator_control);
+      _live_object_editor_decorator_xml->get_widget("vbox1", _live_object_editor_decorator_control);
       _live_object_editor_decorator_xml->get_widget("live_editor_placeholder", _live_editor_placeholder);
 
-      _live_object_editor_decorator_xml->get_widget("live_editor_log_text", _live_editor_log_text);
-      _live_editor_log_text->get_buffer()->set_text("DBMS feedback messages will go here upon applying changes.");
-      //_live_object_editor_decorator_xml->get_widget("live_editor_progressbar", _live_editor_progressbar);
-
       Gtk::Button *apply_live_edtior_button= 0;
       _live_object_editor_decorator_xml->get_widget("apply_live_edtior_button", apply_live_edtior_button);
       apply_live_edtior_button->signal_pressed().connect(sigc::mem_fun(this, &PluginEditorBase::apply_changes_to_live_object));
@@ -97,13 +119,7 @@ void PluginEditorBase::decorate_object_editor()
 //------------------------------------------------------------------------------
 void PluginEditorBase::apply_changes_to_live_object()
 {
-  _live_editor_log_text->get_buffer()->set_text("");
-  _live_editor_log_text->modify_text(Gtk::STATE_NORMAL, Gdk::Color("black"));
-  //_live_editor_progressbar->set_fraction(0);
   bec::BaseEditor *editor= get_be();
-  editor->on_live_object_change_error= sigc::mem_fun(this, &PluginEditorBase::on_live_object_change_error);
-  editor->on_live_object_change_progress= sigc::mem_fun(this, &PluginEditorBase::on_live_object_change_progress);
-  editor->on_live_object_change_statistics= sigc::mem_fun(this, &PluginEditorBase::on_live_object_change_statistics);
   editor->apply_changes_to_live_object();
 }
 //------------------------------------------------------------------------------
@@ -140,30 +156,6 @@ void PluginEditorBase::close_live_object_editor()
   }
 }
 //------------------------------------------------------------------------------
-int PluginEditorBase::on_live_object_change_error(long long err_code, const std::string &err_msg, const std::string &sql)
-{
-  _live_editor_log_text->modify_text(Gtk::STATE_NORMAL, Gdk::Color("red"));
-  std::string err_code_msg= (err_code == -1) ? "" : strfmt("SQL Error %lli: ", err_code);
-  std::string message= strfmt("%s%s\nSQL: %s\n\n", err_code_msg.c_str(), err_msg.c_str(), sql.c_str());
-  std::string text= _live_editor_log_text->get_buffer()->get_text("");
-  _live_editor_log_text->get_buffer()->set_text(text + message);
-  return 0;
-}
-//------------------------------------------------------------------------------
-int PluginEditorBase::on_live_object_change_progress(float progress)
-{
-  //_live_editor_progressbar->set_fraction(progress);
-  return 0;
-}
-//------------------------------------------------------------------------------
-int PluginEditorBase::on_live_object_change_statistics(long success_count, long fail_count)
-{
-  //_live_editor_progressbar->set_fraction(1.);
-  if (0 == fail_count)
-    _live_editor_log_text->get_buffer()->set_text("Changes have been applied OK");
-  return 0;
-}
-//------------------------------------------------------------------------------
 void PluginEditorBase::add_option_combo_change_handler(Gtk::ComboBox *combo, const std::string &option, const sigc::slot<void,std::string,std::string> &setter)
 {
   combo->signal_changed().connect(sigc::bind(sigc::mem_fun(this, &PluginEditorBase::combo_changed), combo, option, setter));
@@ -193,18 +185,6 @@ sigc::connection PluginEditorBase::add_text_change_timer(Gtk::TextView *text, co
   return text->get_buffer()->signal_changed().connect(sigc::bind(sigc::mem_fun(this, &PluginEditorBase::text_changed), text));
 }
 
-//------------------------------------------------------------------------------
-sigc::connection PluginEditorBase::add_sqleditor_text_change_timer(SqlEditorFE *text, const sigc::slot<void,std::string> &setter)
-{
-  TextChangeTimer timer;
-
-  timer.commit= sigc::bind(sigc::mem_fun(this, &PluginEditorBase::sqleditor_text_timeout), text);
-  timer.setter= setter;
-  _timers[&text->widget()]= timer;
-
-  return text->signal_text_changed().connect(sigc::bind(sigc::mem_fun(this, &PluginEditorBase::sqleditor_text_changed), text));
-}
-
 //------------------------------------------------------------------------------
 bool PluginEditorBase::entry_timeout(Gtk::Entry *entry)
 {
@@ -219,13 +199,6 @@ bool PluginEditorBase::text_timeout(Gtk::TextView *text)
   return false;
 }
 
-//------------------------------------------------------------------------------
-bool PluginEditorBase::sqleditor_text_timeout(SqlEditorFE *text)
-{
-  _timers[&text->widget()].setter(text->get_text());
-  return false;
-}
-
 //------------------------------------------------------------------------------
 void PluginEditorBase::entry_changed(Gtk::Entry *entry)
 {
@@ -250,19 +223,6 @@ void PluginEditorBase::text_changed(Gtk::TextView *text)
   }
 }
 
-//------------------------------------------------------------------------------
-void PluginEditorBase::sqleditor_text_changed(SqlEditorFE *text)
-{
-  if (!_refreshing)
-  {
-    Gtk::Widget *sql_widget = &text->widget();
-    if (_timers[sql_widget].conn)
-      _timers[sql_widget].conn.disconnect();
-
-    _timers[sql_widget].conn= Glib::signal_timeout().connect(_timers[sql_widget].commit, TEXT_UPDATE_TIMER);
-  }
-}
-
 //------------------------------------------------------------------------------
 void PluginEditorBase::commit_text_changes()
 {
@@ -298,3 +258,46 @@ bool PluginEditorBase::should_close_on_delete_of(const std::string &oid)
 {
   return get_be()->should_close_on_delete_of(oid); 
 }
+
+
+//------------------------------------------------------------------------------
+void PluginEditorBase::show_find_panel(mforms::CodeEditor *editor, bool show, Gtk::VBox *vbox)
+{
+  Gtk::Widget *panel = mforms::gtk::ViewImpl::get_widget_for_view(editor->get_find_panel());
+  if (show)
+  {
+    _old_embedded_find = panel;
+    if (!panel->get_parent())
+      vbox->pack_start(*panel, false, true);
+    panel->show();
+  }
+  else
+    panel->hide();
+}
+
+
+void PluginEditorBase::embed_code_editor(mforms::CodeEditor *editor, Gtk::VBox *vbox, bool commit_on_focus_out)
+{
+  if (_old_embedded_editor)
+    vbox->remove(*_old_embedded_editor);
+  if (_old_embedded_find)
+    vbox->remove(*_old_embedded_find);
+  _old_embedded_find = 0;
+  _old_embedded_editor = 0;
+
+  if (editor)
+  {
+    Gtk::Widget *editorw = mforms::widget_for_view(editor);
+    _old_embedded_editor = editorw;
+
+    editorw->set_size_request(-1, 100);
+    vbox->pack_end(*editorw, true, true);
+    vbox->resize_children();
+
+    editor->set_show_find_panel_callback(boost::bind(&PluginEditorBase::show_find_panel, this, _1, _2, vbox));
+    if (commit_on_focus_out)
+      editorw->signal_focus_out_event().connect(sigc::bind_return(sigc::hide(sigc::mem_fun(get_be(), &bec::BaseEditor::commit_changes)), false));
+  }
+}
+
+
