@@ -47,7 +47,7 @@ void DiagramSizeForm::realize_be(wb::WBContextUI *wbui)
   _be= wbui->create_diagram_options_be(_canvas->get_canvas());
   _be->update_size();
   
-  _be->signal_changed().connect(sigc::mem_fun(this, &DiagramSizeForm::changed));
+  scoped_connect(_be->signal_changed(),sigc::mem_fun(this, &DiagramSizeForm::changed));
 
   Gtk::SpinButton *spin;
   _xml->get_widget("spinbutton1", spin);
