@@ -167,12 +167,15 @@ static void close_plugin(PluginEditorBase *editor, wb::WBContext *wb)
 
 //------------------------------------------------------------------------------
 bool MainForm::close_window(GdkEventAny *ev)
-{
+{/*
   if (_wbui_context->request_quit())
   {
     get_mainwindow()->hide();
     _wbui_context->perform_quit();
-  }
+  }*/
+ // _wbui_context->get_wb()->quit_application();
+  if (_wbui_context->request_quit())
+      _wbui_context->perform_quit();
   return true; // true means stop processing the event
 }
 
@@ -200,7 +203,7 @@ void MainForm::on_focus_widget(Gtk::Widget *w)
       _wbui_context->set_active_form(form);
   }
 
-  (*_wbui_context->get_command_ui()->signal_validate_edit_menu_items())();
+  _wbui_context->get_command_ui()->revalidate_edit_menu_items();
 
   // for updating the Edit menu
  //!!! Gtk::Alignment* menu_holder;
@@ -316,15 +319,28 @@ void MainForm::show_status_text_becb(const std::string& text)
 
 
 //------------------------------------------------------------------------------
-void MainForm::quit_app_becb()
+bool MainForm::quit_app_becb()
 {
-  Gtk::Notebook *note = get_upper_note();
-  while (note->get_n_pages() > 0)
-  {
-    close_tab(note, note->get_nth_page(0));
-  }
+  // close the model 1st
+  if (_wbui_context->get_wb()->can_close_document())
+    _wbui_context->get_wb()->close_document();
+  else
+    return false;
 
+  Gtk::Notebook *note = get_upper_note();
+  for (int i= note->get_n_pages()-1; i > 0; --i)
+  {
+    // skip diagram tabs from check
+    if (i < note->get_n_pages())
+    {
+      bool is_diagram = dynamic_cast<ModelDiagramPanel*>(reinterpret_cast<FormViewBase*>(note->get_nth_page(i)->get_data("FormViewBase")));
+      if (!close_tab(note, note->get_nth_page(i)) && !is_diagram)
+        return false;
+    }
+  }
   Gtk::Main::quit();
+
+  return true;
 }
 
 //------------------------------------------------------------------------------
@@ -918,7 +934,6 @@ mdc::CanvasView* MainForm::create_view_becb(const model_DiagramRef &diagram)
   add_form_pane(model_panel, !model_panel->get_diagram_form()->is_closed());
  // model_panel->set_data("model_panel", model_panel);
 
-
   return model_panel->get_canvas();
 }
 
@@ -1227,8 +1242,13 @@ void MainForm::handle_model_created()
 void MainForm::handle_model_closed()
 {
   if (_model_panel)
-    get_upper_note()->remove_page(*_model_panel);
-  delete _model_panel;
+  {
+    _model_panel->dispose();
+    get_upper_note()->remove_page(*_model_panel->get_panel());//*_model_panel);
+//    _model_panel->unreference();
+  }
+
+//  delete _model_panel;
   _model_panel= 0;
 }
 
@@ -1250,7 +1270,7 @@ void MainForm::switch_page(GtkNotebookPage*, guint pagenum)
     bec::UIForm* uiform = static_cast<bec::UIForm*>(page->get_data("uiform"));
     _wbui_context->set_active_form(uiform);
   }
-  (*_wbui_context->get_command_ui()->signal_validate_edit_menu_items())();
+  _wbui_context->get_command_ui()->revalidate_edit_menu_items();
 }
 
 
@@ -1432,28 +1452,25 @@ void MainForm::close_active_tab()
 }
 
 
-void MainForm::close_tab(Gtk::Notebook *note, Gtk::Widget *page)
+bool MainForm::close_tab(Gtk::Notebook *note, Gtk::Widget *page)
 {
   // on_close should return true if the form should be closed/removed
   mforms::AppView *app_view = reinterpret_cast<mforms::AppView*>(page->get_data("AppView"));
   FormViewBase *form = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
-
   if (app_view || form)
   {
     if ((app_view && app_view->on_close()) || (form && form->on_close()))
     {
-      //page->hide();
-
-      if (form)
-        form->dispose();
-
-      note->remove_page(*page);
-
-      //if (app_view == 0)
-      //  page->unreference();
+      if (note->page_num(*page) >= 0)
+      {
+        if (form)
+          form->dispose();
+        note->remove_page(*page);
+        page->unreference();
+      }
     }
     else
-      return;
+      return false;
   }
   else
     page->hide();
@@ -1470,6 +1487,8 @@ void MainForm::close_tab(Gtk::Notebook *note, Gtk::Widget *page)
 
   if (!visible)
     note->hide();
+
+  return true;
 }
 
 
@@ -1488,9 +1507,9 @@ void MainForm::append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const s
     }
   }
 
-  if ( ! already_have_page )
+  if (!already_have_page)
   {
-    ActiveLabel* label = Gtk::manage(new ActiveLabel(title, sigc::bind(sigc::mem_fun(this, &MainForm::close_tab), note, widget)));
+    ActiveLabel* label = Gtk::manage(new ActiveLabel(title, sigc::hide_return(sigc::bind(sigc::mem_fun(this, &MainForm::close_tab), note, widget))));
 
     if (title_label_ret)
       *title_label_ret = label;
@@ -1499,6 +1518,8 @@ void MainForm::append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const s
     init_tab_menu(menu);
     menu->set_handler(sigc::bind(sigc::mem_fun(this, &MainForm::tab_menu_handler), label, widget, note));
 
+    if (g_object_is_floating(G_OBJECT(widget->gobj())))
+      g_object_ref_sink(G_OBJECT(widget->gobj()));
     note->append_page(*widget, *label);
 
     if (auto_switch)
@@ -2024,8 +2045,8 @@ void MainForm::dock_view(mforms::AppView *view, const std::string &position, int
       decorated = decorate_widget(w, view);
       w->set_data("DockDecoration", decorated);
 
-      if (g_object_is_floating(decorated->gobj()))
-        g_object_ref_sink(decorated->gobj()); // turn the floating ref to a normal one
+ //     if (g_object_is_floating(decorated->gobj()))
+ //       g_object_ref_sink(decorated->gobj()); // turn the floating ref to a normal one
     }
 
     decorated->set_data("AppView", view);
@@ -2076,9 +2097,11 @@ void MainForm::undock_view(mforms::AppView *view)
   g_return_if_fail(w != NULL);
   
   Gtk::Notebook *note= reinterpret_cast<Gtk::Notebook*>(w->get_data("containerNote"));
-  
-  if (note)
-    note->remove_page(*w);
+  Gtk::Widget *decorated = reinterpret_cast<Gtk::Widget*>(w->get_data("DockDecoration"));
+  if (note && decorated)
+  {
+    note->remove_page(*decorated);
+  }
 }
 
 
