@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -72,9 +72,7 @@ static NSDictionary *EditorTabImages = nil;
     
     mToolTipTags = [[NSMutableArray alloc] init];
     
-    NSRect rect = [self frame];
-    rect.origin = NSZeroPoint;
-    mTrack = [[NSTrackingArea alloc] initWithRect: rect
+    mTrack = [[NSTrackingArea alloc] initWithRect: [self bounds]
                                           options: NSTrackingMouseEnteredAndExited|NSTrackingMouseMoved|NSTrackingActiveAlways
                                             owner: self
                                          userInfo: nil];
@@ -259,9 +257,9 @@ static NSDictionary *EditorTabImages = nil;
                           [NSColor colorWithDeviceWhite: 53/255.0 alpha: 1.0], NSForegroundColorAttributeName,
                           style, NSParagraphStyleAttributeName,
                           nil];
-      mLabelShadowAttributes = [[mLabelAttributes mutableCopy] retain];
+      mLabelShadowAttributes = [mLabelAttributes mutableCopy];
       [mLabelShadowAttributes setObject: [NSColor colorWithDeviceWhite: 245/255.0 alpha: 1.0] forKey: NSForegroundColorAttributeName];
-      mLabelDisabledAttributes = [[mLabelAttributes mutableCopy] retain];
+      mLabelDisabledAttributes = [mLabelAttributes mutableCopy];
       [mLabelDisabledAttributes setObject: [NSColor colorWithDeviceWhite: 108/255.0 alpha: 1.0] forKey: NSForegroundColorAttributeName];
       mMinTabWidth = 90;
       mDefaultMinTabWidth = mMinTabWidth;
@@ -285,7 +283,7 @@ static NSDictionary *EditorTabImages = nil;
       break;
   }
   if (!mLabelShadowAttributes)
-    mLabelShadowAttributes = [[mLabelAttributes mutableCopy] retain];
+    mLabelShadowAttributes = [mLabelAttributes mutableCopy];
   
   [self setFrameSize: NSMakeSize(NSWidth([self frame]), [self _sizeOfTabViewItem: nil].height)];
 }
@@ -416,7 +414,30 @@ static NSDictionary *EditorTabImages = nil;
 
     [path setLineWidth: 1.0];
     [path stroke];
+
+    // top rounded corners
+    if (isMain)
+      [[NSColor colorWithDeviceWhite: 161/255.0 alpha: 0.6] set];
+    else
+      [[NSColor colorWithDeviceWhite: 230/255.0 alpha: 0.6] set];
+    path = [NSBezierPath bezierPath];
+    [path moveToPoint: NSMakePoint(selectedItemMinX-0.5, height-4.5)];
     
+    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMinX+4.5, height-4.5)
+                                     radius: 5
+                                 startAngle: 180
+                                   endAngle: 90
+                                  clockwise: YES];
+    [path lineToPoint: NSMakePoint(selectedItemMaxX-4.5, height+0.5)];
+    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX-5, height-4.5)
+                                     radius: 5
+                                 startAngle: 90
+                                   endAngle: 0
+                                  clockwise: YES];
+    [path setLineWidth: 1.0];
+    [path stroke];
+    
+    // sides
     if (isMain)
       [[NSColor colorWithDeviceWhite: 141/255.0 alpha: 0.8] set];
     else
@@ -428,18 +449,21 @@ static NSDictionary *EditorTabImages = nil;
                                      radius: 5
                                  startAngle: 270
                                    endAngle: 360];
-    [path lineToPoint: NSMakePoint(selectedItemMinX-0.5, height-4.5)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMinX+5, height-4.5)
-                                     radius: 5
-                                 startAngle: 180
-                                   endAngle: 90
-                                  clockwise: YES];
-    [path lineToPoint: NSMakePoint(selectedItemMaxX - 5.0, height+0.5)];
-    [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX-5.0, height-4.5)
-                                     radius: 5
-                                 startAngle: 90
-                                   endAngle: 0
-                                  clockwise: YES];
+    [path lineToPoint: NSMakePoint(selectedItemMinX-0.5, height-4)];
+    /*
+     [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMinX+5, height-4.5)
+     radius: 5
+     startAngle: 180
+     endAngle: 90
+     clockwise: YES];
+     [path lineToPoint: NSMakePoint(selectedItemMaxX - 5.0, height+0.5)];
+     [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX-5.0, height-4.5)
+     radius: 5
+     startAngle: 90
+     endAngle: 0
+     clockwise: YES];
+     */
+    [path moveToPoint: NSMakePoint(selectedItemMaxX, height-4.5)];
     [path lineToPoint: NSMakePoint(selectedItemMaxX, 6)];
     [path appendBezierPathWithArcWithCenter: NSMakePoint(selectedItemMaxX+5.0, 6)
                                      radius: 5
@@ -984,6 +1008,16 @@ static NSDictionary *EditorTabImages = nil;
 
 - (void)frameDidChange:(NSNotification*)notif
 {
+  {
+    [self removeTrackingArea: mTrack];
+    [mTrack release];
+    mTrack = [[NSTrackingArea alloc] initWithRect: [self bounds]
+                                          options: NSTrackingMouseEnteredAndExited|NSTrackingMouseMoved|NSTrackingActiveAlways
+                                            owner: self
+                                         userInfo: nil];
+    [self addTrackingArea: mTrack];
+  }
+  
   float availableWidth = NSWidth([self frame]) - mReservedSpace;
 
   if (mStyle != MEditorTabSwitcher)
@@ -1104,33 +1138,16 @@ static NSDictionary *EditorTabImages = nil;
 
 #pragma mark Event Handlers
 
-- (void)resizeWithOldSuperviewSize:(NSSize)oldSize
-{
-  [super resizeWithOldSuperviewSize: oldSize];
-  
-  [self setNeedsDisplay: YES];
-  [self removeTrackingArea: mTrack];
-  [mTrack release];
-  NSRect rect = [self frame];
-  rect.origin = NSZeroPoint;
-  mTrack = [[NSTrackingArea alloc] initWithRect: rect
-                                        options: NSTrackingMouseEnteredAndExited|NSTrackingMouseMoved|NSTrackingActiveAlways
-                                          owner: self
-                                       userInfo: nil];
-  [self addTrackingArea: mTrack];
-}
-
-
 
 - (void)mouseDown:(NSEvent *)theEvent
 {
-  NSPoint clickPos = [self convertPointFromBase: [theEvent locationInWindow]];
+  NSPoint clickPos = [self convertPoint: [theEvent locationInWindow] fromView: nil];
   NSTabViewItem *item = [self tabViewItemAtPoint: clickPos];
   mClosePressed = NO;
   mClickedItem = item;
   if (item)
   {
-    if (mHoverItem && item != mBusyTab && NSPointInRect([self convertPointFromBase: [theEvent locationInWindow]], mCloseButtonRect) &&
+    if (mHoverItem && item != mBusyTab && NSPointInRect(clickPos, mCloseButtonRect) &&
         ([mDelegate respondsToSelector: @selector(tabView:itemHasCloseButton:)] &&
          [mDelegate tabView: mTabView itemHasCloseButton: item]))
       mClosePressed = YES;
@@ -1151,7 +1168,7 @@ static NSDictionary *EditorTabImages = nil;
   if (!mInside)
     mHoverItem = nil;
   
-  if (mClosePressed && mHoverItem && NSPointInRect([self convertPointFromBase: [theEvent locationInWindow]], mCloseButtonRect))
+  if (mClosePressed && mHoverItem && NSPointInRect([self convertPoint: [theEvent locationInWindow] fromView: nil], mCloseButtonRect))
   {
     NSTabViewItem *item = mHoverItem;
     [self setNeedsDisplay: YES];
@@ -1216,7 +1233,7 @@ static NSDictionary *EditorTabImages = nil;
 
 - (void)rightMouseDown:(NSEvent *)theEvent
 {
-  NSPoint clickPos = [self convertPointFromBase: [theEvent locationInWindow]];
+  NSPoint clickPos = [self convertPoint: [theEvent locationInWindow] fromView: nil];
   NSTabViewItem *item = [self tabViewItemAtPoint: clickPos];
   mClickedItem = item;
   if (item)
@@ -1250,7 +1267,7 @@ static NSDictionary *EditorTabImages = nil;
 {
   if (!mDraggingTab)
   {
-    NSPoint pos = [self convertPointFromBase: [theEvent locationInWindow]];
+    NSPoint pos = [self convertPoint: [theEvent locationInWindow] fromView: nil];
     NSTabViewItem *item = [self tabViewItemAtPoint: pos];
     mCloseHighlighted = NO;
     if (![mDelegate respondsToSelector: @selector(tabView:itemHasCloseButton:)] ||
@@ -1269,7 +1286,7 @@ static NSDictionary *EditorTabImages = nil;
   mClosePressed = NO;
   if (mHoverItem)
   {
-    NSPoint clickPos = [self convertPointFromBase: [theEvent locationInWindow]];
+    NSPoint clickPos = [self convertPoint: [theEvent locationInWindow] fromView: nil];
     NSTabViewItem *item = [self tabViewItemAtPoint: NSMakePoint(clickPos.x, mClickTabOffset.y)];
     
     if (mAllowTabReordering && [mTabView numberOfTabViewItems] > 1 && item)
@@ -1298,7 +1315,7 @@ static NSDictionary *EditorTabImages = nil;
       if (passedThreshold)
       {
         mReorderingTab = YES;
-        [draggedItem retain];
+        [[draggedItem retain] autorelease];
         if (item)
         {
           int idx = [mTabView indexOfTabViewItem: item];
@@ -1327,7 +1344,6 @@ static NSDictionary *EditorTabImages = nil;
           }
         }
         [mTabView selectTabViewItem: draggedItem]; // reselect the tab since it gets unselected when removed
-        [draggedItem release];
         mReorderingTab = NO;
       }
     }
@@ -1341,7 +1357,7 @@ static NSDictionary *EditorTabImages = nil;
   mInside = YES;
   if (!mDraggingTab)
   {
-    NSTabViewItem *item = [self tabViewItemAtPoint: [self convertPointFromBase: [theEvent locationInWindow]]];
+    NSTabViewItem *item = [self tabViewItemAtPoint: [self convertPoint: [theEvent locationInWindow] fromView: nil]];
     if ([mDelegate respondsToSelector: @selector(tabView:itemHasCloseButton:)] &&
         [mDelegate tabView: mTabView itemHasCloseButton: item])
     {
@@ -1385,6 +1401,8 @@ static NSDictionary *EditorTabImages = nil;
         }
         [self setNeedsDisplay: YES];
         break;
+      default:
+        break;
     }
   }
   else
