@@ -50,17 +50,6 @@
   }
 }
 
-/*
-- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject start:(int)selStart length:(int)selLength 
-{
-  [self setBackgroundColor:[NSColor whiteColor]];
-  [self setTextColor:[NSColor blackColor]];
-  aRect.size.height-= 3.0;
-  aRect.size.width-= 3.0;
-  [super selectWithFrame:aRect inView:controlView editor:textObj delegate:anObject start:selStart length:selLength];
-}*/
-
-
 - (void)setPlaceholder:(BOOL)flag
 {
   _placeholder= flag;
