@@ -79,10 +79,11 @@ static std::string showFileDialog(const std::string &type, const std::string &ti
   {
     NSOpenPanel *panel= [NSOpenPanel openPanel];
 
-    [panel setTitle:[NSString stringWithUTF8String:title.c_str()]];
+    [panel setTitle: [NSString stringWithUTF8String:title.c_str()]];
+    [panel setAllowedFileTypes: fileTypes];
     
-    if ([panel runModalForTypes:fileTypes] == NSFileHandlingPanelOKButton)
-      return [[panel filename] UTF8String];
+    if ([panel runModal] == NSFileHandlingPanelOKButton)
+      return [panel.URL.path UTF8String];
   }
   else if (type == "save")
   {
@@ -93,7 +94,7 @@ static std::string showFileDialog(const std::string &type, const std::string &ti
     [panel setAllowedFileTypes:fileTypes];
 
     if ([panel runModal] == NSFileHandlingPanelOKButton)
-      return [[panel filename] UTF8String];
+      return [panel.URL.path UTF8String];
   }
   
   return "";
@@ -385,16 +386,19 @@ static void windowLockGui(bool lock, WBMainWindow *window)
 }
 
 
-static void quitApplication()
+static bool quitApplication(WBMainWindow *mainWindow)
 {
+  if (![mainWindow closeAllPanels])
+    return false;
+
   [NSApp terminate:nil];
+  return true;
 }
 
 
 - (void)applicationWillTerminate:(NSNotification *)aNotification
 {
   [[mainWindow window] orderOut: nil];
-  [mainWindow closeAllPanelsNoConfirm];
 
   _wbui->get_wb()->finalize();
   // is crashing delete _wbui;
@@ -712,7 +716,7 @@ static bool validate_redo(WBMainWindow *mainWindow)
   { 
     // refresh edit menu
     wb::WBContextUI *wbui = [mainWindow context];
-    (*wbui->get_command_ui()->signal_validate_edit_menu_items())();
+    wbui->get_command_ui()->revalidate_edit_menu_items();
   }
 }
 
@@ -812,21 +816,6 @@ static void flush_main_thread()
 }
 
 
-- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
-{
-  if (_quitting)
-    return NSTerminateNow;
-  
-  if (_wbui->request_quit())
-  {
-    _quitting = YES;
-    _wbui->perform_quit();
-    return NSTerminateNow;
-  }  
-  return NSTerminateCancel;
-}
-
-
 - (void)applicationDidFinishLaunching:(NSNotification*)notification
 {
   _wb->get_ui()->init_finish(_options);
@@ -904,7 +893,7 @@ static NSString *applicationSupportFolder()
     wbcallbacks.tool_changed= boost::bind(windowToolChanged, _1, mainWindow);
     wbcallbacks.refresh_gui= boost::bind(windowRefreshGui, _1, _2, _3, mainWindow);
     wbcallbacks.lock_gui= boost::bind(windowLockGui, _1, mainWindow);
-    wbcallbacks.quit_application= boost::bind(quitApplication);
+    wbcallbacks.quit_application= boost::bind(quitApplication, mainWindow);
       
     // add shipped python module search path to PYTHONPATH
     // also include mysql utilities module zip
@@ -1173,7 +1162,7 @@ static void init_mforms()
       break;
       
     case APP_MENU_QUIT:
-      quitApplication(); 
+      quitApplication(mainWindow);
       break;
   }
 }
