@@ -1015,7 +1015,7 @@ void setup_mforms_app(WBMainWindow *mwin);
     NSLog(@"restore edit menu");
   }
   else if (!changedActivePanel)
-    (*_wbui->get_command_ui()->signal_validate_edit_menu_items())();
+    _wbui->get_command_ui()->revalidate_edit_menu_items();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1086,20 +1086,23 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 //--------------------------------------------------------------------------------------------------
 
-- (void)closeAllPanelsNoConfirm
+- (BOOL)closeAllPanels
 {
   id panelsCopy = [_panels copy];
+  id homeTabId = [[topTabView tabViewItemAtIndex: 0] identifier];
   for (id panel in [panelsCopy objectEnumerator])
   {
-    id identifier = [[[panel identifier] retain] autorelease];
-
-    if ([topTabView indexOfTabViewItemWithIdentifier: identifier] != NSNotFound)
-      [self closeTopPanelWithIdentifier: identifier];
-    else
-      [self closeBottomPanelWithIdentifier: identifier];    
+    if ([homeTabId isEqualTo: [panel identifier]])
+      continue;
+    if ([topTabView indexOfTabViewItemWithIdentifier: [panel identifier]] != NSNotFound)
+      [topTabView selectTabViewItemWithIdentifier: [panel identifier]];
+    if (![self closePanel: panel])
+      return NO;
   }
   [panelsCopy release];
   [_panels removeAllObjects];
+
+  return YES;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1121,12 +1124,7 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (BOOL)windowShouldClose:(id)window
 {
-  if (_wbui->request_quit())
-  {
-    _wbui->perform_quit();
-    return YES;
-  }
-  return NO;
+  return _wbui->get_wb()->quit_application();
 }
 
 //--------------------------------------------------------------------------------------------------
