@@ -1094,7 +1094,7 @@ namespace MySQL.Controls
           Rectangle bounds = GetTabRect(TabCount - 1);
           if (bounds.Right < ClientSize.Width - Margin.Right - scrollerSpacing - rect.Width)
             scrollOffset += ClientSize.Width - Margin.Right - scrollerSpacing - rect.Width - bounds.Right;
-          Win32.SendMessage(scroller.Handle, (uint) UDM.SETPOS32, IntPtr.Zero, new IntPtr(-scrollOffset / scrollScaleFactor));
+          Win32.SendMessage(scroller.Handle, (int)UDM.SETPOS32, IntPtr.Zero, new IntPtr(-scrollOffset / scrollScaleFactor));
 
           // Disable scroll acceleration by setting a single scroll increment.
           Win32.UDACCEL acceleration = new Win32.UDACCEL();
@@ -1102,7 +1102,7 @@ namespace MySQL.Controls
           acceleration.nSec = 0;
           IntPtr parameter = Marshal.AllocHGlobal(Marshal.SizeOf(acceleration));
           Marshal.StructureToPtr(acceleration, parameter, true);
-          Win32.SendMessage(scroller.Handle, (uint) UDM.SETACCEL, new IntPtr(1), parameter);
+          Win32.SendMessage(scroller.Handle, (int)UDM.SETACCEL, new IntPtr(1), parameter);
           Marshal.FreeHGlobal(parameter);
         }
         else
