@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -58,7 +58,6 @@ namespace MySQL.Controls
     // Row main and alternate colors.
     private Color gridColor = Color.FromArgb(255, 255, 255);
     private Color gridAlternateRowColor = Color.FromArgb(237, 243, 253);
-    private bool busyAnimationActive = false;
     private Bitmap mainBusyIcon = Resources.busy_indicator_white;
     private Bitmap alternateBusyIcon = Resources.busy_indicator_lightblue;
 
@@ -83,12 +82,6 @@ namespace MySQL.Controls
         Model = null;
       }
 
-      if (busyAnimationActive)
-      {
-        ImageAnimator.StopAnimate(mainBusyIcon, new EventHandler(BusyAnimationStep));
-        ImageAnimator.StopAnimate(alternateBusyIcon, new EventHandler(BusyAnimationStep));
-      }
-
       // Workaround for bug in DataGridView. When being in edit mode while the grid is disposed
       // DataGridView tries to access a not existing edit control internally, which crashes.
       CancelEdit();
@@ -100,8 +93,6 @@ namespace MySQL.Controls
 
     public GridView(GridModel model)
     {
-      ShowBusyAnimation = false;
-
       fieldNullBitmap = Resources.field_overlay_null;
       fieldBlobBitmap = Resources.field_overlay_blob;
 
@@ -214,37 +205,6 @@ namespace MySQL.Controls
       if (wrapMode)
         AutoResizeRows(DataGridViewAutoSizeRowsMode.DisplayedCells, true);
 
-      if (ShowBusyAnimation)
-      {
-        // Start or stop the busy indications depending on the type of messages we have.
-        bool hasBusyMessage = false;
-        for (int i = 0; i < Model.count(); i++)
-        {
-          int type;
-          Model.get_field(new NodeId(i), 0, out type);
-          hasBusyMessage = ((MessageType)type == MessageType.BusyMsg);
-          if (hasBusyMessage)
-            break;
-        }
-
-        lock (this)
-        {
-          if (hasBusyMessage != busyAnimationActive)
-            if (hasBusyMessage)
-            {
-              busyAnimationActive = ImageAnimator.CanAnimate(mainBusyIcon);
-              ImageAnimator.Animate(mainBusyIcon, new EventHandler(BusyAnimationStep));
-              ImageAnimator.Animate(alternateBusyIcon, new EventHandler(BusyAnimationStep));
-            }
-            else
-            {
-              busyAnimationActive = false;
-              ImageAnimator.StopAnimate(mainBusyIcon, new EventHandler(BusyAnimationStep));
-              ImageAnimator.StopAnimate(alternateBusyIcon, new EventHandler(BusyAnimationStep));
-            }
-        }
-      }
-
       if (AutoScroll && (RowCount > 0))
         CurrentCell = this[0, Rows.Count - 1];
 
@@ -311,26 +271,10 @@ namespace MySQL.Controls
       get { return SelectedCells.Count > 0; }
     }
 
-    /// <summary>
-    /// Causes the grid to interpret a record as message. In that case the value in the first
-    /// column is taken as message type. If that value indicates a busy message (identified by the
-    /// MessageType enum) then a busy indicator is shown, instead of an icon.
-    /// </summary>
-    public bool ShowBusyAnimation { get; set; }
-
     #endregion
 
     #region Message handling
 
-    private void BusyAnimationStep(object sender, EventArgs args)
-    {
-      if (IsDisposed || Columns.Count == 0)
-        return;
-
-      Rectangle indicatorColumnRect = new Rectangle(0, 0, Columns[0].Width, ClientSize.Height);
-      Invalidate(indicatorColumnRect, false);
-    }
-
     private void OnRowPrePaint(Object sender, DataGridViewRowPrePaintEventArgs e)
     {
       AutoResizeRow(e.RowIndex);
