@@ -646,9 +646,14 @@ namespace MySQL.GUI.Workbench
 
     private bool EditCanUndo()
     {
+      // If we are on a model form then edit actions are controlled by the backend.
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == workbenchPhysicalOverviewForm)
+        return wbContext.edit_can_undo();
+
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
       if (activeControl is TextBoxBase || activeControl is ComboBox)
-        return false;
+        return true;
       else
         if (activeControl is ScintillaControl)
           return (activeControl as ScintillaControl).CanUndo;
@@ -671,9 +676,13 @@ namespace MySQL.GUI.Workbench
 
     private bool EditCanRedo()
     {
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == workbenchPhysicalOverviewForm)
+        return wbContext.edit_can_redo();
+
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
       if (activeControl is TextBoxBase || activeControl is ComboBox)
-        return false;
+        return true;
       else
         if (activeControl is ScintillaControl)
           return (activeControl as ScintillaControl).CanRedo;
@@ -702,9 +711,13 @@ namespace MySQL.GUI.Workbench
 
     private bool EditCanCopy()
     {
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == workbenchPhysicalOverviewForm)
+        return wbContext.edit_can_copy();
+
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
       if (activeControl is TextBoxBase || activeControl is ComboBox)
-        return false;
+        return true;
       else
         if (activeControl is ScintillaControl)
           return (activeControl as ScintillaControl).CanCopy;
@@ -727,12 +740,16 @@ namespace MySQL.GUI.Workbench
           if (activeControl is ScintillaControl)
             (activeControl as ScintillaControl).Cut();
             else
-              if (wbContext.edit_can_copy())
+              if (wbContext.edit_can_copy()) // TODO: check really necessary?
                 wbContext.edit_cut();
     }
 
     private bool EditCanCut()
     {
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == workbenchPhysicalOverviewForm)
+        return wbContext.edit_can_cut();
+
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
@@ -764,6 +781,10 @@ namespace MySQL.GUI.Workbench
 
     private bool EditCanPaste()
     {
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == workbenchPhysicalOverviewForm)
+        return wbContext.edit_can_paste();
+
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
       if (activeControl is TextBoxBase || activeControl is ComboBox || activeControl is GridView)
         return false;
@@ -786,17 +807,22 @@ namespace MySQL.GUI.Workbench
           if (activeControl is ScintillaControl)
             (activeControl as ScintillaControl).SelectAll();
           else
-            if (wbContext.edit_can_select_all())
-              wbContext.edit_select_all();
+            if (activeControl is DataGridView)
+              (activeControl as DataGridView).SelectAll();
+            else
+              if (wbContext.edit_can_select_all())
+                wbContext.edit_select_all();
     }
 
     private bool EditCanSelectAll()
     {
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == workbenchPhysicalOverviewForm)
+        return wbContext.edit_can_select_all();
+
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase || activeControl is ComboBox)
-        return false;
-      else
-        if (activeControl is ScintillaControl)
+      if (activeControl is TextBoxBase || activeControl is ComboBox|| activeControl is ScintillaControl ||
+        activeControl is DataGridView)
           return true;
         else
           return wbContext.edit_can_select_all();
@@ -817,6 +843,10 @@ namespace MySQL.GUI.Workbench
 
     private bool EditCanDelete()
     {
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == workbenchPhysicalOverviewForm)
+        return wbContext.edit_can_delete();
+
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
@@ -982,7 +1012,8 @@ namespace MySQL.GUI.Workbench
         {
           // Reset host if the tab control is not what we are looking for and start over.
           FlatTabControl.TabStyleType style = (host as FlatTabControl).TabStyle;
-          if (style == FlatTabControl.TabStyleType.TopNormal || style == FlatTabControl.TabStyleType.TopHanging)
+          if (style == FlatTabControl.TabStyleType.TopNormal || style == FlatTabControl.TabStyleType.TopHanging
+            || style == FlatTabControl.TabStyleType.TopTransparent)
             break;
         }
         control = host;
@@ -1062,10 +1093,12 @@ namespace MySQL.GUI.Workbench
       return "";
     }
 
-    public void QuitApplication()
+    public bool QuitApplication()
     {
       if (!shuttingDown)
         Close();
+
+      return true;
     }
 
     #endregion
@@ -1705,16 +1738,20 @@ namespace MySQL.GUI.Workbench
 
     private void FocusChanged(object sender, EventArgs e)
     {
+      // TODO: see InstallFocusChangeHandlers.
+      /*
       // force a refresh of the menu so that Edit -> Copy/Paste/Cut/Select All can get updated accordingly
       if (!shuttingDown)
         wbContext.validate_edit_menu();
+      */
     }
 
-    private EventHandler FocusChangeHandler= null;
+    //private EventHandler FocusChangeHandler= null;
 
 
     private void RemoveFocusChangeHandlers(Control control)
     {
+      /* TODO: see InstallFocusChangeHandlers.
       if (FocusChangeHandler == null)
         return;
 
@@ -1723,10 +1760,14 @@ namespace MySQL.GUI.Workbench
 
       foreach (Control sub in control.Controls)
         RemoveFocusChangeHandlers(sub);
+      */
     }
 
     private void InstallFocusChangeHandlers(Control control)
     {
+      // TODO: It shouldn't be necessary to install those handlers anymore. Remove once we are sure.
+      // Validation requests for the edit menu are all triggered by the backend now.
+      /*
       if (FocusChangeHandler == null)
         FocusChangeHandler= new EventHandler(FocusChanged);
 
@@ -1736,6 +1777,7 @@ namespace MySQL.GUI.Workbench
 
       foreach  (Control sub in control.Controls)
         InstallFocusChangeHandlers(sub);
+    */
     }
 
     private void tabControl_TabClosing(object sender, TabClosingEventArgs e)
