@@ -22,6 +22,7 @@
 #include <stdarg.h>
 #include <time.h>
 #include <string.h>
+#include <vector>
 
 #include <glib/gstdio.h>
 
@@ -29,6 +30,7 @@
 #include "base/wb_memory.h"
 #include "base/file_utilities.h"
 #include "base/file_functions.h" // TODO: these two file libs should really be only one.
+#include "base/string_utilities.h"
 
 using namespace base;
 
@@ -85,9 +87,31 @@ std::string Logger::log_dir()
 
 //--------------------------------------------------------------------------------------------------
 
-Logger::Logger(const std::string& dir, const bool stderr_log)
+Logger::Logger(const bool stderr_log, const std::string& target_file)
 {
-  static const char* const filenames[] = {"wb.log", "wb.1.log", "wb.2.log", "wb.3.log", "wb.4.log", "wb.5.log", "wb.6.log", "wb.7.log", "wb.8.log", "wb.9.log"};
+  if (!_impl)
+    _impl = new Logger::LoggerImpl();
+
+  _impl->_std_err_log = stderr_log;
+
+  if (!target_file.empty())
+  {
+    _impl->_filename = target_file;
+
+    FILE_scope_ptr fp = fopen(_impl->_filename.c_str(), "w");
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+Logger::Logger(const std::string& dir, const bool stderr_log, const std::string& file_name, int limit)
+{
+  std::vector<std::string>filenames;
+
+  // Creates the file names array
+  filenames.push_back(strfmt("%s.log", file_name.data()));
+  for(int index = 1; index < limit; index++)
+    filenames.push_back(strfmt("%s.%d.log", file_name.data(), index));
 
   if (!_impl)
     _impl = new Logger::LoggerImpl();
@@ -95,7 +119,7 @@ Logger::Logger(const std::string& dir, const bool stderr_log)
   _impl->_std_err_log = stderr_log;
 
   _impl->_new_line_pending = true;
-  if (!dir.empty())
+  if (!dir.empty() && !file_name.empty())
   {
     _impl->_dir = dir + "/log/";
     _impl->_filename = _impl->_dir + filenames[0];
@@ -109,10 +133,8 @@ Logger::Logger(const std::string& dir, const bool stderr_log)
       fprintf(stderr, "Exception in logger: %s\n", e.what());
     }
 
-    const int size = sizeof(filenames) / sizeof(const char*);
-    
     // Rotate log files: wb.log -> wb.1.log, wb.1.log -> wb.2.log, ...
-    for (int i = size - 1; i > 0; --i)
+    for (int i = limit-1; i > 0; --i)
     {
       try
       {
