@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,8 +33,7 @@
 
 
 using namespace mdc;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 LineLayouter::LineLayouter()
 {
@@ -132,7 +131,7 @@ void Line::set_layouter(LineLayouter *layouter)
 {
   _layouter= layouter;
 
-  _layouter->signal_changed().connect(sigc::mem_fun(this, &Line::update_layout));
+  scoped_connect(_layouter->signal_changed(),boost::bind(&Line::update_layout, this));
 
   _layouter->update();
 }
@@ -145,7 +144,7 @@ void Line::update_layout()
   if (_hop_crossings)
     get_view()->update_line_crossings(this);
 
-  _layout_changed.emit();
+  _layout_changed();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -364,6 +363,7 @@ static void draw_line_end(CairoCtx *cr, LineEndType type, const Color &lcolor, c
 {
   switch (type)
   {
+  case DashedNormalEnd:
   case NormalEnd:
     break;
 
@@ -388,7 +388,8 @@ static void draw_line_end(CairoCtx *cr, LineEndType type, const Color &lcolor, c
     cr->set_color(lcolor);
     cr->stroke_preserve();
     break;
-    
+
+  case DashedChickenFootEnd:
   case ChickenFootEnd:
     cr->set_color(lcolor);
     cr->move_to(-5, 0);
@@ -497,6 +498,7 @@ static void draw_line_end_gl(LineEndType type, const Color &lcolor, const Color
 {
   switch (type)
   {
+    case DashedNormalEnd:
     case NormalEnd:
       break;
 
@@ -525,6 +527,7 @@ static void draw_line_end_gl(LineEndType type, const Color &lcolor, const Color
         break;
       }
 
+    case DashedChickenFootEnd:
     case ChickenFootEnd:
       gl_setcolor(lcolor);
       glBegin(GL_LINE_STRIP);
@@ -940,7 +943,7 @@ void Line::update_bounds()
 
   update_handles();
 
-  _layout_changed.emit();
+  _layout_changed();
 }
 
 
