@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,14 +25,13 @@
 #include "mdc_connector.h"
 
 using namespace mdc;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 Magnet::Magnet(CanvasItem *owner)
 : _owner(owner)
 {
-  _owner->signal_bounds_changed().connect(sigc::mem_fun(this, &Magnet::owner_bounds_changed));
-  _owner->signal_parent_bounds_changed().connect(sigc::mem_fun(this, &Magnet::owner_parent_bounds_changed));
+  scoped_connect(_owner->signal_bounds_changed(),boost::bind(&Magnet::owner_bounds_changed, this, _1));
+  scoped_connect(_owner->signal_parent_bounds_changed(),boost::bind(&Magnet::owner_parent_bounds_changed, this, _1, _2));
 }
 
 
@@ -76,13 +75,13 @@ void Magnet::notify_connected()
 }
 
 
-void Magnet::set_connection_validator(const sigc::slot<bool, Connector*> &slot)
+void Magnet::set_connection_validator(const boost::function<bool (Connector*)> &slot)
 {
   _connection_slot= slot;
 }
 
 
-void Magnet::set_disconnection_validator(const sigc::slot<bool, Connector*> &slot)
+void Magnet::set_disconnection_validator(const boost::function<bool (Connector*)> &slot)
 {
   _disconnection_slot= slot;
 }
