@@ -252,6 +252,7 @@ ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &conn
       db_mgmt_DriverParameterRef param= params.get(n);
       param_types[param->name()]= param->paramType();
     }
+    param_types["CLIENT_INTERACTIVE"] = "boolean";
   }
   //std::map<std::string, ConnectPropertyVal> properties;
   ConnectOptionsMap properties;
@@ -268,6 +269,22 @@ ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &conn
       properties["OPT_READ_TIMEOUT"]= read_timeout;
   }
   properties["CLIENT_MULTI_STATEMENTS"]= true;
+    
+  // If SSL is enabled but there's no certificate or anything, create the sslKey option to force enabling SSL without a key
+  // (equivalent to starting cmdline client with mysql --ssl-key=)
+  if (parameter_values.get_string("sslKey", "") == "" && parameter_values.get_int("useSSL", 0) != 0)
+  {
+    properties["sslKey"] = std::string();
+  }
+  // If SSL is not enabled, clear all ssl related props to not confuse the connector
+  if (parameter_values.get_int("useSSL", 0) == 0)
+  {
+    properties.erase("sslKey");
+    properties.erase("sslCert");
+    properties.erase("sslCA");
+    properties.erase("sslCAPath");
+    properties.erase("sslCipher");
+  }
 
   // If we are on a pipe connection then set the host name explicitly.
   // However, pipe connections can only be established on the local box (Win only).
@@ -359,13 +376,6 @@ retry:
   try
   {
     std::auto_ptr<Connection> conn(driver->connect(properties));
-
-    if (connection_init_slot)
-        connection_init_slot(conn.get(), connectionProperties);
-    
-    std::string def_schema= parameter_values.get_string("schema", "");
-    if (!def_schema.empty())
-      conn->setSchema(def_schema);
     
     // make sure the user we got logged in is the user we wanted
     {
@@ -395,6 +405,20 @@ retry:
       }
     }
     
+    std::string sql_mode = parameter_values.get_string("SQL_MODE", "");
+    if (!sql_mode.empty())
+    {
+      boost::scoped_ptr<sql::Statement> statement(conn.get()->createStatement());
+      statement->execute("SET SESSION SQL_MODE='"+sql_mode+"'");
+    }
+
+    if (connection_init_slot)
+      connection_init_slot(conn.get(), connectionProperties);
+    
+    std::string def_schema= parameter_values.get_string("schema", "");
+    if (!def_schema.empty())
+      conn->setSchema(def_schema);
+
     return ConnectionWrapper(conn, tunnel);
   }
   catch (sql::SQLException &exc)
