@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,57 +19,34 @@
 
 #include "stdafx.h"
 
-#include <stdio.h>
-#include <mforms/app.h>
-#include <mforms/mforms.h>
-#include <stdio.h>
+#include "mforms/mforms.h"
 
 using namespace mforms;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 // Implementation of _app_impl for WB is done directly in main_window.cpp in Linux
 // and WBMainWindow.mm in Mac
 // In Windows, it's done in wf_app.h/wf_app.cpp wrapper
 
+static App *singleton= 0;
+
+App::App(DockingPointDelegate *delegate, bool delete_on_destroy)
+: DockingPoint(delegate, delete_on_destroy)
+{
+}
+
+void App::instantiate(DockingPointDelegate *delegate, bool delete_on_destroy)
+{
+  singleton= new App(delegate, delete_on_destroy);
+  singleton->_app_impl= &ControlFactory::get_instance()->_app_impl;
+}
+
+
 App *App::get()
 {
-  static App *singleton= 0;
-  if (!singleton)
-  {
-    singleton= new App();
-    singleton->_app_impl= &ControlFactory::get_instance()->_app_impl;
-  }
   return singleton;
 }
 
-
-void App::dock_view(AppView *view, const std::string &position)
-{
-  if (_app_impl->dock_view)
-    _app_impl->dock_view(this, view, position);
-}
-
-bool App::select_view(const std::string &identifier)
-{
-  if (_app_impl->select_view)
-    return _app_impl->select_view(this, identifier);
-  return false;
-}
-
-void App::undock_view(AppView *view)
-{
-  if (_app_impl->undock_view)
-    _app_impl->undock_view(this, view);
-}
-
-
-void App::set_view_title(AppView *view, const std::string &title)
-{
-  if (_app_impl->set_view_title)
-    _app_impl->set_view_title(this, view, title);
-}
-
 std::string App::get_resource_path(const std::string &file)
 {
   std::string ret;
@@ -78,6 +55,16 @@ std::string App::get_resource_path(const std::string &file)
   return ret;
 }
 
+std::string App::get_executable_path(const std::string &file)
+{
+  std::string ret;
+  if (_app_impl->get_executable_path)
+    ret = _app_impl->get_executable_path(this, file);
+  else
+    ret = get_resource_path(file);
+  return ret;
+}
+
 void App::set_status_text(const std::string &text)
 {
   if (_app_impl->set_status_text)
@@ -141,36 +128,6 @@ void App::exit_event_loop(int retcode)
 
 //--------------------------------------------------------------------------------------------------
 
-void App::set_view_for_identifier(mforms::View *view, const std::string &identifier)
-{
-  _view_by_identifier[identifier] = view;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-mforms::View *App::get_view_for_identifier(const std::string &identifier)
-{  
-  if (_view_by_identifier.find(identifier) == _view_by_identifier.end())
-    return _view_by_identifier[identifier];
-  return 0;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void App::view_destroyed(View *data)
-{
-  for (std::map<std::string, View*>::iterator i = _view_by_identifier.begin(); 
-       i != _view_by_identifier.end(); ++i)
-  {
-    if (i->second == data)
-    {
-      _view_by_identifier.erase(i);
-    }
-  }
-}
-
-//--------------------------------------------------------------------------------------------------
-
 Color App::get_system_color(SystemColor type)
 {
   return _app_impl->get_system_color(type);
