@@ -180,36 +180,141 @@ public:
     }
     return mforms::TreeNodeRef();
   }
-  
-  virtual std::vector<mforms::TreeNodeRef> add_child_nodes(int column, const std::vector<std::string> &column_values)
+    
+  virtual std::vector<mforms::TreeNodeRef> add_node_collection(const mforms::TreeNodeCollectionSkeleton &nodes, int position = -1)
   {
     std::vector<mforms::TreeNodeRef> result;
+    
     if (is_valid())
     {
-      id columnKey = [[_self treeNodeView] keyForColumn: column];
-      NSMutableArray *children = [_self children];
-      if (!children)
-        children = [_self createChildrenWithCapacity: (int)column_values.size()];
+      id columnKey = [[_self treeNodeView] keyForColumn: 0];
 
-      for (std::vector<std::string>::const_iterator v = column_values.begin(); v != column_values.end(); ++v)
+      // Creates an array to hold each equal child for all the parents
+      NSMutableArray *added_nodes = [NSMutableArray arrayWithCapacity: (int)nodes.captions.size()];
+      
+      NSImage *image = get_icon(nodes.icon);
+      
+      for (std::vector<std::string>::const_iterator v = nodes.captions.begin(); v != nodes.captions.end(); ++v)
       {
         MFTreeNodeImpl *child = [[MFTreeNodeImpl alloc] initWithOwner: [_self treeNodeView]];
         
-        child->mParent = _self;
         [child setObject: [NSString stringWithCPPString: *v]
                   forKey: columnKey];
-        [children addObject: child];
+        
+        if (image)
+        {
+          [child setObject: image
+                    forKey: [[[_self treeNodeView] keyForColumn: 0] stringByAppendingString: @"icon"]];
+        }
+        
+        [added_nodes addObject: child];
+      }
+      
+      // Creates the substructure if any
+      if (!nodes.children.empty())
+        add_children_from_skeletons(added_nodes, nodes.children);
 
+      NSMutableArray *children = [_self children];
+      if (!children)
+        children = [_self createChildrenWithCapacity: (int)nodes.captions.size()];
+      
+      NSEnumerator *child_enumerator = [added_nodes objectEnumerator];
+      MFTreeNodeImpl *child;
+
+      while(child = [child_enumerator nextObject])
+      {
+        child->mParent = _self;
+        
+        if (position < 0 || position >= (int)[children count])
+          [children addObject: child];
+        else
+         [children insertObject: child atIndex: position++];
+        
         result.push_back([child nodeRef]);
+
         [child release];
       }
-      if (![[_self treeNodeView] frozen] && (!get_parent() || is_expanded() || [children count] == 1))
+                                     
+      if (![[_self treeNodeView] frozen] && (!get_parent() || is_expanded() || [added_nodes count] == 1))
       {
         [[_self treeNodeView] setNeedsReload];
       }
     }
+    
+    
     return result;
   }
+
+  void add_children_from_skeletons(NSMutableArray *parents, const std::vector<mforms::TreeNodeSkeleton> &children)
+  {
+    if (is_valid())
+    {
+      // Creates the parent children lists
+      NSEnumerator *parent_enumerator = [parents objectEnumerator];
+      MFTreeNodeImpl *parent;
+      while(parent = [parent_enumerator nextObject])
+      {
+        NSMutableArray *children_array = [parent children];
+        if (!children_array)
+          children_array = [parent createChildrenWithCapacity: (int)children.size()];
+      }
+      
+      id columnKey = [[_self treeNodeView] keyForColumn: 0];
+      
+      
+      // Now enters the process if creating each children at this level
+      std::vector<mforms::TreeNodeSkeleton>::const_iterator it, end = children.end();
+      for(it=children.begin(); it != end; it++)
+      {
+        // Creates an array to hold each equal child for all the parents
+        NSMutableArray *added_nodes = [NSMutableArray arrayWithCapacity: [parents count]];
+        
+        // Gets the data to be set on the current child
+        NSImage *image = get_icon((*it).icon);
+        NSString *caption = [NSString stringWithCPPString: (*it).caption];
+        NSString *tag     = [NSString stringWithCPPString: (*it).tag];
+        
+        
+        // Setups the child for all the parents (same name, icon)
+        for(int index = 0; index < [parents count]; index++)
+        {
+          MFTreeNodeImpl *child = [[MFTreeNodeImpl alloc] initWithOwner: [_self treeNodeView]];
+          [child setObject: caption
+                    forKey: columnKey];
+          
+          [child setObject: tag
+                    forKey: @"tag"];
+
+          if (image)
+          {
+            [child setObject: image
+                      forKey: [columnKey stringByAppendingString: @"icon"]];
+          }
+          
+          [added_nodes addObject: child];
+        }
+        
+        // If the children have descendence, then calls this method to create the substructure
+        // on them
+        if (!(*it).children.empty())
+          add_children_from_skeletons(added_nodes, (*it).children);
+        
+        // Now assigns each child to its corresponding parent
+        NSEnumerator *child_enumerator = [added_nodes objectEnumerator];
+        parent_enumerator = [parents objectEnumerator];
+        MFTreeNodeImpl *child;
+        while(parent = [parent_enumerator nextObject])
+        {
+          child = [child_enumerator nextObject];
+          
+          child->mParent = parent;
+          [[parent children] addObject: child];
+          
+          [child release];
+        }
+      }
+    }
+  }
   
   virtual void remove_from_parent()
   { 
@@ -253,9 +358,12 @@ public:
   virtual void expand()
   {
     mforms::TreeNodeRef parent(get_parent());
+    
     NSLog(@"expanding node");
     if (parent)
       parent->expand();
+    
+    
     [[[_self treeNodeView] outlineView] expandItem: _self];
   }
   
@@ -291,17 +399,18 @@ public:
   {
     return [[_self objectForKey: @"data"] CPPPointer];
   }
-  
-  virtual void set_icon_path(int column, const std::string &icon)
+
+  NSImage* get_icon(const std::string &icon_path)
   {
     NSImage *image = nil;
-    if (icon == "folder")
+    if (icon_path == "folder")
     {
       static NSImage *folderIcon= [[[[NSWorkspace sharedWorkspace] iconForFile:@"/usr"] copy] retain];
       image = folderIcon;
     }
     else
-      image = [[_self treeNodeView] iconForFile: [NSString stringWithCPPString: icon]];
+      image = [[_self treeNodeView] iconForFile: [NSString stringWithCPPString: icon_path]];
+    
     if (image)
     {
       NSSize size = [image size];
@@ -314,7 +423,17 @@ public:
         [image setScalesWhenResized: YES];
         [image setSize: size];
       }
-      
+    }
+    return image;
+  }
+  
+ 
+  virtual void set_icon_path(int column, const std::string &icon)
+  {
+    NSImage *image = get_icon(icon);
+
+    if (image)
+    {
       [_self setObject: image
                 forKey: [[[_self treeNodeView] keyForColumn: column] stringByAppendingString: @"icon"]];
     }
@@ -599,8 +718,11 @@ static NSImage *descendingSortIndicator=nil;
 
 - (void)reloadTreeData
 {
-  mPendingReload = NO;
-  [mOutline reloadData];
+  if (mPendingReload)
+  {
+    mPendingReload = NO;
+    [mOutline reloadData];
+  }
 }
 
 - (void)setNeedsReload
@@ -612,6 +734,11 @@ static NSImage *descendingSortIndicator=nil;
   mPendingReload = YES;
 }
 
+- (BOOL)isPendingReload
+{
+  return mPendingReload;
+}
+
 - (NSOutlineView*)outlineView
 {
   return mOutline;
@@ -786,7 +913,10 @@ sortDescriptorsDidChange:(NSArray *)oldDescriptors
   if (!item)
     item= mRootNode;
 
-  return [[item children] objectAtIndex: index];
+  NSArray *children = [item children];
+  if (children && index < [children count])
+    return [[item children] objectAtIndex: index];
+  return nil;
 }
 
 
@@ -992,6 +1122,8 @@ sortDescriptorsDidChange:(NSArray *)oldDescriptors
   mSmallFont = flag;
   [mAttributedFonts setObject: [NSFont systemFontOfSize: [NSFont smallSystemFontSize]]
                        forKey: @""];
+  if (flag)
+    [mOutline setRowHeight: [NSFont smallSystemFontSize]+3];
 }
 
 - (BOOL)frozen
@@ -1039,10 +1171,17 @@ static bool treeview_create(mforms::TreeNodeView *self, mforms::TreeOptions opti
     [tree setBorderType: NSBezelBorder];
   if (options & mforms::TreeNoHeader)
     [tree->mOutline setHeaderView:nil];
-  
-  if (options & mforms::TreeSidebar)
+  if (options & mforms::TreeSizeSmall)
   {
     [tree setUseSmallFont: YES];
+    [tree setAutohidesScrollers: YES];
+  }
+  if (options & mforms::TreeSidebar)
+  {
+    // maintain the row height
+    float rowHeight = [tree->mOutline rowHeight];
+    [tree setUseSmallFont: YES];
+    [tree->mOutline setRowHeight: rowHeight];
     [tree->mOutline setFocusRingType: NSFocusRingTypeNone];
     [tree setAutohidesScrollers: YES];
   }
@@ -1056,7 +1195,7 @@ static bool treeview_create(mforms::TreeNodeView *self, mforms::TreeOptions opti
   return true;
 }
 
-static int treeview_add_column(mforms::TreeNodeView *self, mforms::TreeColumnType type, const std::string &name, int width, bool editable)
+static int treeview_add_column(mforms::TreeNodeView *self, mforms::TreeColumnType type, const std::string &name, int width, bool editable, bool attributed)
 {
   MFTreeNodeViewImpl *tree= self->get_data();
   if (tree)
@@ -1139,8 +1278,11 @@ static void treeview_set_selected(mforms::TreeNodeView *self, mforms::TreeNodeRe
       NSLog(@"WARNING: Selecting items on a frozen TreeNodeView will probably not work");
     
     if (flag)
+    {
+      [tree reloadTreeData];
       [tree->mOutline selectRowIndexes: [NSIndexSet indexSetWithIndex: treeview_row_for_node(self, node)]
                   byExtendingSelection: YES];
+    }
     else
       [tree->mOutline deselectRow: treeview_row_for_node(self, node)];
   }
@@ -1199,8 +1341,12 @@ static void treeview_freeze_refresh(mforms::TreeNodeView *self, bool flag)
         [tree->mOutline reloadData];
         
         if (item)
-          [tree->mOutline selectRowIndexes: [NSIndexSet indexSetWithIndex: [tree->mOutline rowForItem: item]]
-                      byExtendingSelection: NO];
+        {
+          row = [tree->mOutline rowForItem: item];
+          if (row >= 0)
+            [tree->mOutline selectRowIndexes: [NSIndexSet indexSetWithIndex: row]
+                        byExtendingSelection: NO];
+        }
       }
     }
   }
@@ -1338,6 +1484,9 @@ static mforms::TreeNodeRef treeview_node_at_row(mforms::TreeNodeView *self, int
     }
     else
     {
+      if ([tree isPendingReload])
+        [tree reloadTreeData];
+        
       id n = [tree->mOutline itemAtRow: row];
       if (n)      
         return [n nodeRef];
