@@ -43,6 +43,9 @@ using namespace base;
 #define AC_LIST_SEPARATOR '\x19' // Unused codes as separators.
 #define AC_TYPE_SEPARATOR '\x18'
 
+#define ERROR_INDICATOR INDIC_CONTAINER
+#define ERROR_INDICATOR_VALUE 42 // Arbitrary value.
+
 //----------------- CodeEditorConfig ---------------------------------------------------------------
 
 CodeEditorConfig::CodeEditorConfig(SyntaxHighlighterLanguage language)
@@ -546,7 +549,11 @@ void CodeEditor::set_language(SyntaxHighlighterLanguage language)
   _code_editor_impl->send_editor(this, SCI_SETSTYLEBITS, _code_editor_impl->send_editor(this, SCI_GETSTYLEBITSNEEDED, 0, 0), 0);
 
   _code_editor_impl->send_editor(this, SCI_SETMARGINTYPEN, 0, SC_MARGIN_NUMBER);
-  _code_editor_impl->send_editor(this, SCI_STYLESETSIZE, STYLE_LINENUMBER, (sptr_t)7);
+#if defined(__APPLE__)
+  _code_editor_impl->send_editor(this, SCI_STYLESETSIZE, STYLE_LINENUMBER, (sptr_t)10);
+#else
+  _code_editor_impl->send_editor(this, SCI_STYLESETSIZE, STYLE_LINENUMBER, (sptr_t)8);
+#endif
   int lineNumberStyleWidth = _code_editor_impl->send_editor(this, SCI_TEXTWIDTH, STYLE_LINENUMBER, (long)"_999999");
   _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 0, lineNumberStyleWidth);
   _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 0, 1); // Margin is clickable.
@@ -578,9 +585,9 @@ void CodeEditor::set_language(SyntaxHighlighterLanguage language)
   _code_editor_impl->send_editor(this, SCI_STYLESETBACK, STYLE_LINENUMBER, 0xE0E0E0);
 
   // Init markers & indicators for highlighting of syntax errors.
-  _code_editor_impl->send_editor(this, SCI_INDICSETFORE, 0, 0x2119D0);
-  _code_editor_impl->send_editor(this, SCI_INDICSETUNDER, 0, 1);
-  _code_editor_impl->send_editor(this, SCI_INDICSETSTYLE, 0, INDIC_SQUIGGLE);
+  _code_editor_impl->send_editor(this, SCI_INDICSETFORE, ERROR_INDICATOR, 0x2119D0);
+  _code_editor_impl->send_editor(this, SCI_INDICSETUNDER, ERROR_INDICATOR, 1);
+  _code_editor_impl->send_editor(this, SCI_INDICSETSTYLE, ERROR_INDICATOR, INDIC_SQUIGGLE);
 
   // Gutter markers for errors and statements, breakpoints and current code line.
   setup_marker(CE_STATEMENT_MARKER, "editor_statement");
@@ -695,15 +702,21 @@ void CodeEditor::remove_markup(LineMarkup markup, int line)
 
 void CodeEditor::show_indicator(RangeIndicator indicator, int start, int length)
 {
+  // Scintilla supports a model that not only sets an indicator in a given range but additionally
+  // assigns a value to this indicator. This is to allow drawing an indicator with different styles.
+  // However, currently all values are drawn in the same style and it is neither clear if that will ever
+  // change nor what the actual use is, given that you always can set different indicators for different styles.
+  // Unfortunately, you cannot query an indicator as such, only its value, so we have to set a value too
+  // even if that is quite useless.
   switch (indicator)
   {
   case mforms::RangeIndicatorError:
-    _code_editor_impl->send_editor(this, SCI_SETINDICATORCURRENT, 0, 0);
-    _code_editor_impl->send_editor(this, SCI_SETINDICATORVALUE, mforms::RangeIndicatorError, 0);
+    _code_editor_impl->send_editor(this, SCI_SETINDICATORCURRENT, ERROR_INDICATOR, 0);
+    _code_editor_impl->send_editor(this, SCI_SETINDICATORVALUE, ERROR_INDICATOR_VALUE, 0);
     _code_editor_impl->send_editor(this, SCI_INDICATORFILLRANGE, start, length);
 
     break;
-  case mforms::RangeIndicatorNone:
+  case mforms::RangeIndicatorNone: // No effect here. Only used to have "none" value for indicator tests.
     break;
   }
 }
@@ -712,8 +725,8 @@ void CodeEditor::show_indicator(RangeIndicator indicator, int start, int length)
 
 RangeIndicator CodeEditor::indicator_at(int position)
 {
-  int result = _code_editor_impl->send_editor(this, SCI_INDICATORVALUEAT, 0, position);
-  if (result == mforms::RangeIndicatorError)
+  int result = _code_editor_impl->send_editor(this, SCI_INDICATORVALUEAT, ERROR_INDICATOR, position);
+  if (result == ERROR_INDICATOR_VALUE)
     return mforms::RangeIndicatorError;
 
   return mforms::RangeIndicatorNone;
@@ -726,11 +739,11 @@ void CodeEditor::remove_indicator(RangeIndicator indicator, int start, int lengt
   switch (indicator)
   {
   case mforms::RangeIndicatorError:
-    _code_editor_impl->send_editor(this, SCI_SETINDICATORCURRENT, 0, 0);
+    _code_editor_impl->send_editor(this, SCI_SETINDICATORCURRENT, ERROR_INDICATOR, 0);
     _code_editor_impl->send_editor(this, SCI_INDICATORCLEARRANGE, start, length);
 
     break;
-  case mforms::RangeIndicatorNone:
+  case mforms::RangeIndicatorNone: // No effect here. Only needed to have a "none" value for indicator tests.
     break;
   }
 }
@@ -783,7 +796,6 @@ void CodeEditor::set_font(const std::string& fontDescription)
     for (int i = 0; i < 128; i++)
     {
       _code_editor_impl->send_editor(this, SCI_STYLESETFONT, i, (sptr_t)font.c_str());
-      if (i != STYLE_LINENUMBER)
         _code_editor_impl->send_editor(this, SCI_STYLESETSIZE, i, (sptr_t)size);
       _code_editor_impl->send_editor(this, SCI_STYLESETBOLD, i, (sptr_t)bold);
       _code_editor_impl->send_editor(this, SCI_STYLESETITALIC, i, (sptr_t)italic);
@@ -1522,3 +1534,8 @@ sptr_t CodeEditor::send_editor(unsigned int message, uptr_t wParam, sptr_t lPara
 
 //--------------------------------------------------------------------------------------------------
 
+
+void CodeEditor::edit_end()
+{
+  _signal_edit_end();
+}
