@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,6 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
+
+#include "../lf_mforms.h"
 #include "../lf_selector.h"
 
 namespace mforms
@@ -56,10 +58,15 @@ class SelectorImpl::Impl
 //==============================================================================
 class SelectorPopupImpl : public SelectorImpl::Impl
 {
+//  private:
+
   public:
     SelectorPopupImpl(mforms::Selector *self)
+      : _self(self)
+      , do_not_call_callback(false)
     {
-      _list.signal_changed().connect(sigc::mem_fun(self, &mforms::Selector::callback));
+//      _list.signal_changed().connect(sigc::mem_fun(self, &mforms::Selector::callback));
+      _list.signal_changed().connect(sigc::mem_fun(*this, &mforms::gtk::SelectorPopupImpl::wrap_callback_call));
     }
 
     Gtk::Widget* widget()
@@ -69,9 +76,11 @@ class SelectorPopupImpl : public SelectorImpl::Impl
 
     virtual void clear()
     {
+      do_not_call_callback = true;
       _items.clear();
       _list.clear();
       _list.clear_items();
+	  do_not_call_callback = false;
     }
 
     virtual int add_item(const std::string &item)
@@ -89,7 +98,7 @@ class SelectorPopupImpl : public SelectorImpl::Impl
       const std::list<std::string>::const_iterator last = items.end();
       for (; it != last; ++it)
       {
-        _list.prepend_text(*it);
+        _list.append_text(*it);
         _items.push_back(*it);
       }
       if (_items.size() > 0)
@@ -123,8 +132,18 @@ class SelectorPopupImpl : public SelectorImpl::Impl
       return _items.size();
     }
   private:
-    Gtk::ComboBoxText   _list;
-    std::vector<std::string> _items; // to impl get_item with GTK. [The rest of the comment is censored]
+    Gtk::ComboBoxText          _list;
+    std::vector<std::string>   _items; // to impl get_item with GTK. [The rest of the comment is censored]
+    mforms::Selector *         _self;
+    bool					   do_not_call_callback;
+
+    virtual void wrap_callback_call()
+    {
+      if (do_not_call_callback)
+    	return;
+      else
+    	_self->callback();
+    }
 };
 
 
@@ -166,7 +185,7 @@ class SelectorComboboxImpl : public SelectorImpl::Impl
       const std::list<std::string>::const_iterator last = items.end();
       for (; it != last; ++it)
       {
-        _list.prepend_text(*it);
+        _list.append_text(*it);
         _items.push_back(*it);
       }
     }
