@@ -44,7 +44,7 @@ class RootTreeNodeImpl : public ::mforms::TreeNode
 protected:
   TreeNodeViewImpl *_treeview;
   int _refcount;
-
+  
   inline TreeNodeRef ref_from_iter(const Gtk::TreeIter &iter) const;
   inline TreeNodeRef ref_from_path(const Gtk::TreePath &path) const;
 
@@ -98,46 +98,193 @@ public:
     }
     return 0;
   }
+  
+  virtual Gtk::TreeIter create_child(int index)
+  {
+    Glib::RefPtr<Gtk::TreeStore> store(_treeview->tree_store());
+    Gtk::TreeIter new_iter;
+    
+    if (index < 0)
+      new_iter = store->append();
+    else
+    {
+      Gtk::TreePath path;
+      path.push_back(index);
+      new_iter = store->insert(store->get_iter(path));
+    }
+      
+    return new_iter;
+  }
+
+  virtual Gtk::TreeIter create_child(int index, Gtk::TreeIter *other_parent)
+  {
+    Glib::RefPtr<Gtk::TreeStore> store(_treeview->tree_store());
+    Gtk::TreeIter new_iter;
+    
+    if (index < 0)
+      new_iter = other_parent ? store->append((*other_parent)->children()) : store->append();
+    else
+    {
+      Gtk::TreePath path;
+
+      if (other_parent)
+        path = store->get_path(*other_parent);
+      
+      path.push_back(index);
+        
+      new_iter = store->insert(store->get_iter(path));
+    }
+      
+    return new_iter;
+  }
 
   virtual TreeNodeRef insert_child(int index)
   {
     if (is_valid())
     {
-      Glib::RefPtr<Gtk::TreeStore> store(_treeview->tree_store());
-      Gtk::TreeIter new_iter;
-      if (index < 0)
-        new_iter = store->append();
-      else
-      {
-        Gtk::TreePath path;
-        path.push_back(index);
-        new_iter = store->insert(store->get_iter(path));
-      }
+      Gtk::TreeIter new_iter = create_child(index);
       return ref_from_iter(new_iter);
     }
     return TreeNodeRef();
   }
 
-  virtual std::vector<mforms::TreeNodeRef> add_child_nodes(int column, const std::vector<std::string> &column_values)
+  virtual std::vector<mforms::TreeNodeRef> add_node_collection(const TreeNodeCollectionSkeleton &nodes, int position = -1)
   {
-    std::vector<mforms::TreeNodeRef> nodes;
-
+    std::vector<Gtk::TreeIter> added_iters;
+    std::vector<mforms::TreeNodeRef> added_nodes;
+    
+    // Allocates enough room for the returned TreeNodeRefs
+    added_nodes.reserve(nodes.captions.size());
+    
+    // If the nodes have children, also allocates enough room
+    // For the created iters
+    bool sub_items = !nodes.children.empty();
+    if (sub_items)
+      added_iters.reserve(nodes.captions.size());
+    
     Glib::RefPtr<Gtk::TreeStore> store(_treeview->tree_store());
     Gtk::TreeIter new_iter;
-
-    for (std::vector<std::string>::const_iterator it = column_values.begin(); it != column_values.end(); ++it)
+    
+    std::vector<std::string>::const_iterator it, end = nodes.captions.end();
+    
+    // Gets the icon to be used on all the nodes...
+    Glib::RefPtr<Gdk::Pixbuf> pixbuf;
+    if (!nodes.icon.empty())
+      pixbuf = UtilitiesImpl::get_cached_icon(nodes.icon);
+      
+    int index_for_string = _treeview->index_for_column(0);
+    int index_for_icon   = index_for_string - 1;
+    
+    store->freeze_notify();
+    
+    for (it = nodes.captions.begin(); it != end; ++it)
     {
       if (!new_iter)
-        new_iter = store->append();
+        new_iter = create_child(position);
       else
         new_iter = store->insert_after(new_iter);
-      nodes.push_back(ref_from_iter(new_iter));
-      nodes.back()->set_string(column, *it);
+      
+      // Masks the iterator as a row
+      Gtk::TreeRow row = *new_iter;
+    
+      // Sets the string
+      std::string nvalue(*it);
+      
+      base::replace(nvalue, "&", "&amp;");
+      base::replace(nvalue, "<", "&lt;");
+      base::replace(nvalue, ">", "&gt;");
+      
+      row.set_value(index_for_string, nvalue);
+
+      // Sets the icon...
+      row.set_value(index_for_icon, pixbuf);
+      
+      added_nodes.push_back(ref_from_iter(new_iter));
+
+      // If there are sub items the iter needs to be stored so
+      // it gets the childs added
+      if (sub_items)
+        added_iters.push_back(new_iter);
+
     }
+    
+    // If there are sub items adds them into each of the
+    // added iters at this level
+    if (sub_items)
+      add_children_from_skeletons(added_iters, nodes.children);
 
-    return nodes;
+    store->thaw_notify();
+
+    return added_nodes;
   }
+  
+  virtual void add_children_from_skeletons(const std::vector<Gtk::TreeIter>& parents, const std::vector<TreeNodeSkeleton>& children)
+  {
+    std::vector<Gtk::TreeIter> last_item;
+    Glib::RefPtr<Gtk::TreeStore> store(_treeview->tree_store());
+    Gtk::TreeIter new_iter;
+    Gtk::TreeRow row;
+    
+    // Takes each received child and inserts it to all the received parents
+    std::vector<TreeNodeSkeleton>::const_iterator it, end = children.end();
+    for(it = children.begin(); it != end; it++)
+    {
+      // added iters is the child nodes being added, if the structure indicates they
+      // will also have childs, then it is needed to store them to call the function
+      // recursively, we reserve the needed space for performance reasons
+      std::vector<Gtk::TreeIter> added_iters;
+      bool sub_items = !(*it).children.empty();
+      if(sub_items)
+        added_iters.reserve(parents.size());
+      
+      // Gets the icon to be used on this child...
+      Glib::RefPtr<Gdk::Pixbuf> pixbuf(UtilitiesImpl::get_cached_icon((*it).icon));
 
+      // Formats the string to be used in this child...
+      std::string nvalue((*it).caption);
+      base::replace(nvalue, "&", "&amp;");
+      base::replace(nvalue, "<", "&lt;");
+      base::replace(nvalue, ">", "&gt;");
+
+      // Gets the indexes where the information will be stored
+      int index_for_string = _treeview->index_for_column(0);
+      int index_for_icon   = index_for_string - 1;
+      Gtk::TreeModelColumn<std::string>& tag_column = _treeview->_columns.tag_column();
+        
+      // Now inserts the child on all the received parents
+      for(size_t index=0; index < parents.size(); index++)
+      {
+        if (last_item.size() > index)
+        {
+          new_iter = store->insert_after(last_item[index]);
+          last_item[index] = new_iter;
+        }
+        else
+        {
+          Gtk::TreeIter parent = parents[index];
+          new_iter = create_child(-1, &parent);
+          last_item.push_back(new_iter);
+        }
+          
+        // Sets the new item data
+        row = *new_iter;
+        row.set_value(index_for_string, nvalue);
+        row.set_value(index_for_icon, pixbuf);
+        row[tag_column] = (*it).tag;
+
+        // If this child also has childs, stores the new child to be
+        // a futur parent
+        if(sub_items)
+          added_iters.push_back(new_iter);
+      }
+      
+      // If childs will be assigned then it is done by calling the 
+      // function again
+      if(sub_items)
+        add_children_from_skeletons(added_iters, (*it).children);
+    }
+  }  
+  
   virtual void remove_from_parent()
   {
     throw std::logic_error("Cannot delete root node");
@@ -311,46 +458,24 @@ public:
     return 0;
   }
 
-  virtual TreeNodeRef insert_child(int index)
+  virtual Gtk::TreeIter create_child(int index)
   {
-    if (is_valid())
-    {
-      Glib::RefPtr<Gtk::TreeStore> store(model());
-      Gtk::TreeIter new_iter;
-      if (index < 0)
-        new_iter = store->append(iter()->children());
-      else
-      {
-        Gtk::TreePath path;
-        path = _rowref.get_path();
-        path.push_back(index);
-        new_iter = store->insert(store->get_iter(path));
-      }
-      return ref_from_iter(new_iter);
-    }
-    return TreeNodeRef();
-  }
-
-  virtual std::vector<mforms::TreeNodeRef> add_child_nodes(int column, const std::vector<std::string> &column_values)
-  {
-    std::vector<mforms::TreeNodeRef> nodes;
-
-    Glib::RefPtr<Gtk::TreeStore> store(_treeview->tree_store());
+    Glib::RefPtr<Gtk::TreeStore> store(model());
     Gtk::TreeIter new_iter;
 
-    for (std::vector<std::string>::const_iterator it = column_values.begin(); it != column_values.end(); ++it)
+    if (index < 0)
+      new_iter = store->append(iter()->children());
+    else
     {
-      if (!new_iter)
-        new_iter = store->append();
-      else
-        new_iter = store->insert_after(new_iter);
-      nodes.push_back(ref_from_iter(new_iter));
-      nodes.back()->set_string(column, *it);
+      Gtk::TreePath path;
+      path = _rowref.get_path();
+      path.push_back(index);
+      new_iter = store->insert(store->get_iter(path));
     }
-
-    return nodes;
+    
+    return new_iter;
   }
-
+  
   virtual void remove_from_parent()
   {
     if (is_valid())
@@ -1020,7 +1145,9 @@ void TreeNodeViewImpl::set_allow_sorting(TreeNodeView* self, bool flag)
   TreeNodeViewImpl* impl = self->get_data<TreeNodeViewImpl>();
   Gtk::TreeView *tv = &(impl->_tree);
 
-  tv->set_headers_clickable(flag);
+  if (tv->get_headers_visible())
+    tv->set_headers_clickable(flag);
+    
   if (flag)
   {
     if (!impl->_sort_model)
@@ -1051,14 +1178,22 @@ void TreeNodeViewImpl::set_allow_sorting(TreeNodeView* self, bool flag)
 
 void TreeNodeViewImpl::freeze_refresh(TreeNodeView* self, bool flag)
 {
-  //TreeNodeViewImpl* impl = self->get_data<TreeNodeViewImpl>();
-  //Gtk::TreeView *tv = impl->_tree;
+  TreeNodeViewImpl* impl = self->get_data<TreeNodeViewImpl>();
+  Gtk::TreeView *tv = &(impl->_tree);
 
-  //tv->set_headers_clickable(!flag);
-  //if (!flag)
-  //  tv->set_model(impl->_tree_store);
-  //else
-  //  tv->unset_model();
+  if (tv->get_headers_visible())
+    tv->set_headers_clickable(!flag);
+    
+  if (!flag)
+  {
+    tv->freeze_child_notify();
+    //tv->set_model(impl->_tree_store);
+  }
+  else
+  {
+    tv->freeze_child_notify();
+    //tv->unset_model();
+  }
 }
 
 Gtk::TreeModel::iterator TreeNodeViewImpl::to_sort_iter(const Gtk::TreeModel::iterator &it)
