@@ -462,10 +462,13 @@ public:
      */
     boost::signals2::signal<void (int)>* signal_char_added() { return &_char_added_event; }
 
+    /** Emited when editing ends (control loses focus)
+     */
+    boost::signals2::signal<void ()>* signal_edit_end() { return &_signal_edit_end; }
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
     /** Called by the platform code forwarding us all scintilla notifications, so we can act on them. */
     void on_notify(Scintilla::SCNotification* notification);
-    
+    void edit_end();
 #endif
 #endif
   protected:
@@ -482,6 +485,7 @@ public:
     boost::signals2::signal<void (AutoCompletionEventType, int, const std::string&)> _auto_completion_event;
     boost::signals2::signal<void (bool, int, int, int)> _dwell_event;
     boost::signals2::signal<void (int)> _char_added_event;
+    boost::signals2::signal<void ()> _signal_edit_end;
 
     boost::function<void (CodeEditor*, bool)> _show_find_panel;
   };
