@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,20 +24,18 @@
 
 #include "stdafx.h"
 
-#include <mforms/utilities.h>
-#include <mforms/mforms.h>
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
-#include <string.h>
-#include <math.h>
-#include <boost/shared_ptr.hpp>
 #include "base/log.h"
-#include <glib/gstdio.h>
-ENABLE_LOG("mforms.utils")
+
+#include "mforms/mforms.h"
+
+DEFAULT_LOG_DOMAIN("mforms::Utilities");
 
 using namespace mforms;
+using namespace base;
 
-GThread *_mforms_main_thread=0;
+GThread *_mforms_main_thread = NULL;
 
 static std::map<std::string, int> remembered_message_answers;
 static std::string remembered_message_answer_file;
@@ -164,7 +162,15 @@ static void* cancellable_task_thread(CancellableTaskData *data)
 {
   std::auto_ptr<CancellableTaskData> autodel(data);
 
-  void *ptr = data->task();
+  void *ptr = NULL;
+  try
+  {
+    ptr = data->task();
+  }
+  catch (std::exception &exc)
+  {
+    log_error("Cancellable task threw uncaught exception: %s", exc.what());
+  }
   g_mutex_lock(data->mutex.get());
   *data->result_ptr = ptr;
   *data->finished_flag = true;
@@ -200,9 +206,9 @@ bool Utilities::run_cancelable_task(const std::string &title, const std::string
   data->finished_flag = finished;
   
   // result_ptr is used by the task thread to pass back the result from the task callback
-  // we cannot directly pass a pointer to task_result, because if the task is cancelled,
+  // we cannot directly pass a pointer to task_result, because if the task is canceled,
   // this function will return and the caller may free the task_result object before the
-  // cancelled task actually finishes executing, which would invalidate any ptr to task_result 
+  // canceled task actually finishes executing, which would invalidate any ptr to task_result 
   data->result_ptr = result;
   
   g_mutex_lock(lock.get());
@@ -229,7 +235,7 @@ retry:
     
     // sometimes, in the mac, there's a race and/or bug that causes the event loop
     // from the wait panel dialog to be exited when a nested modal dialog is closed.
-    // clicking OK for the nested dialog, exits the wait panel, which woudl cause it
+    // clicking OK for the nested dialog, exits the wait panel, which would cause it
     // to return prematurely.. just execute the modal panel again if that's the case
     if (!*finished)
       goto retry;
@@ -240,6 +246,7 @@ retry:
   }
   else
   {
+    log_debug3("run_cancelable_wait_message returned false\n");
     // task canceled by user
     return false;
   }
@@ -320,6 +327,10 @@ void Utilities::add_end_ok_cancel_buttons(mforms::Box *box, mforms::Button *ok,
 bool Utilities::request_input(const std::string &title, const std::string &description,
                               const std::string &default_value, std::string &ret_value)
 {
+  // In order to avoid trouble with window z-ordering we explicitly ask to hide any wait window
+  // that should get in the way.
+  hide_wait_message(); 
+
   mforms::Form input_form(NULL, (FormFlag) (FormDialogFrame | FormStayOnTop));
   mforms::Table content;
   mforms::ImageBox icon;
@@ -369,7 +380,8 @@ bool Utilities::request_input(const std::string &title, const std::string &descr
 
 void Utilities::store_password(const std::string &service, const std::string &account, const std::string &password)
 {
-  log_debug("Storing password for '%s'@'%s'", account.c_str(), service.c_str());
+  Logger::log(Logger::LogDebug, DOMAIN_MFORMS_BE, "Storing password for '%s'@'%s'\n",
+    account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.store_password(service, account, password);
 }
 
@@ -378,7 +390,8 @@ void Utilities::store_password(const std::string &service, const std::string &ac
 bool Utilities::find_password(const std::string &service, const std::string &account, std::string &password)
 {
   const bool ret = ControlFactory::get_instance()->_utilities_impl.find_password(service, account, password);
-  log_debug("Looking up password for '%s'@'%s' has %s", account.c_str(), service.c_str(), ret ? "succeeded" : "failed");
+  Logger::log(Logger::LogDebug, DOMAIN_MFORMS_BE, "Looking up password for '%s'@'%s' has %s\n",
+    account.c_str(), service.c_str(), ret ? "succeeded" : "failed");
   return ret;
 }
 
@@ -386,15 +399,16 @@ bool Utilities::find_password(const std::string &service, const std::string &acc
 
 void Utilities::forget_password(const std::string &service, const std::string &account)
 {
-  log_debug("Forgetting password for '%s'@'%s'", account.c_str(), service.c_str());
+  Logger::log(Logger::LogDebug, DOMAIN_MFORMS_BE, "Forgetting password for '%s'@'%s\n'",
+    account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.forget_password(service, account);
 }
 
 //-------------------------------------------------------------------------------
 
-void *Utilities::perform_from_main_thread(const boost::function<void* ()> &slot)
+void *Utilities::perform_from_main_thread(const boost::function<void* ()> &slot, bool wait_done)
 {
-  return ControlFactory::get_instance()->_utilities_impl.perform_from_main_thread(slot);
+  return ControlFactory::get_instance()->_utilities_impl.perform_from_main_thread(slot, wait_done);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -402,6 +416,10 @@ void *Utilities::perform_from_main_thread(const boost::function<void* ()> &slot)
 static void *_ask_for_password_main(const std::string &title, const std::string &service, std::string *username /*in/out*/,
                              bool prompt_storage, std::string *ret_password /*out*/, bool *ret_store /*out*/)
 {
+  Logger::log(Logger::LogDebug, DOMAIN_MFORMS_BE, "Creating and showing password dialog\n");
+
+  Utilities::hide_wait_message(); 
+
   mforms::Form password_form(NULL, (FormFlag) (FormDialogFrame | FormStayOnTop));
   mforms::Table content;
   mforms::ImageBox icon;
@@ -525,10 +543,11 @@ static bool _ask_for_password(const std::string &title, const std::string &servi
 //--------------------------------------------------------------------------------------------------
 
 bool Utilities::ask_for_password(const std::string &title, const std::string &service,
-                                 std::string &username /*in/out*/, std::string &ret_password /*out*/)
+                                 const std::string &username, std::string &ret_password /*out*/)
 {
+  std::string ret_username = username;
   bool dummy= false;
-  return _ask_for_password(title, service, username, false, ret_password, dummy);
+  return _ask_for_password(title, service, ret_username, false, ret_password, dummy);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -588,6 +607,7 @@ bool Utilities::credentials_for_service(const std::string &title, const std::str
       {
         show_warning(title.empty() ? _("Error Storing Password") : title, 
                      std::string("There was an error storing the password:\n")+exc.what(), "OK");
+        Logger::log(Logger::LogWarning, DOMAIN_MFORMS_BE, "Could not store password vault: %s\n", exc.what());
       }
     }
     return true;
