@@ -23,6 +23,15 @@
 
 #include "stdafx.h"
 
+#include "mforms/mforms.h"
+#include "wf_view.h"
+#include "wf_box.h"
+#include "workbench/wb_context.h"
+#include "wf_appview.h"
+#include "wf_app.h"
+#include "wf_dockingpoint.h"
+
+#include "base/log.h"
 #include "mdc.h"
 #include "GrtTemplates.h"
 
@@ -53,7 +62,7 @@ AppImpl::AppImpl(MySQL::Forms::Manager^ mgr,
   VoidRefreshTypeStringIntPtrDelegate^ refresh_gui,
   VoidBoolDelegate^ lock_gui,
   VoidStrDelegate^ perform_command,
-  VoidDelegate^ quit_application,
+  BoolDelegate^ quit_application,
   StringAppCommandAppViewStringDelegate^ app_command,
   DockingPointDelegateWrapper^ docking_delegate)
   : _callbacks(new ::wb::WBFrontendCallbacks())
@@ -302,13 +311,13 @@ void AppImpl::set_lock_gui(VoidBoolDelegate^ dt)
 
 //--------------------------------------------------------------------------------------------------
 
-void AppImpl::set_quit_application(VoidDelegate^ dt)
+void AppImpl::set_quit_application(BoolDelegate^ dt)
 {
   quit_application_delegate= dt;
   quit_application_wrapper_delegate=
     gcnew VoidWrapperDelegate(this, &AppImpl::quit_application_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(quit_application_wrapper_delegate);
-  VOID_CB cb = static_cast<VOID_CB>(ip.ToPointer());
+  BOOL_CB cb = static_cast<BOOL_CB>(ip.ToPointer());
   _callbacks->quit_application= cb;
 }
 
