@@ -24,6 +24,15 @@
 
 #include "stdafx.h"
 
+#include "mforms/mforms.h"
+#include "wf_view.h"
+#include "wf_treenodeview.h"
+#include "wf_treenode.h"
+#include "wf_grttreeview.h"
+#include "wf_menu.h"
+
+#include "ConvUtils.h"
+
 using namespace MySQL;
 using namespace MySQL::Forms;
 using namespace MySQL::Utilities;
@@ -37,63 +46,81 @@ private ref class ColumnComparer : public Collections::IComparer
 {
 private:
   int _column;
+  ManagedTreeColumnType _type;
   SortOrder _direction;
   CaseInsensitiveComparer^ _inner;
 public:
-  ColumnComparer(int column, SortOrder direction);
+  ColumnComparer(int column, SortOrder direction, ManagedTreeColumnType type)
+  {
+    _column = column;
+    _direction = direction;
+    _inner = gcnew CaseInsensitiveComparer();
+    _type = type;
+  }
+
+  virtual int Compare(System::Object^ object1, System::Object^ object2)
+  {
+    MySQL::Forms::TreeViewNode^ node1 = dynamic_cast<MySQL::Forms::TreeViewNode^>(object1);
+    MySQL::Forms::TreeViewNode^ node2 = dynamic_cast<MySQL::Forms::TreeViewNode^>(object2);
+
+    switch (_type)
+    {
+    case ManagedTreeColumnType::MStringColumnType:
+    case ManagedTreeColumnType::MIconStringColumnType:
+      // There's an attached string for icons, which we use to compare.
+      if (_direction == SortOrder::Ascending)
+        return _inner->Compare(node1->Text[_column], node2->Text[_column]);
+      else
+        return _inner->Compare(node2->Text[_column], node1->Text[_column]);
+      break;
+    case ManagedTreeColumnType::MIntegerColumnType:
+    case ManagedTreeColumnType::MLongIntegerColumnType:
+      {
+        Int64 i1 = 0, i2 = 0;
+        Int64::TryParse(node1->Text[_column], i1);
+        Int64::TryParse(node2->Text[_column], i2);
+        if (_direction == SortOrder::Ascending)
+          return i1 < i2 ? -1 : (i1 > i2 ? 1 : 0);
+        else
+          return i1 > i2 ? -1 : (i1 < i2 ? 1 : 0);
+        break;
+      }
+    case ManagedTreeColumnType::MCheckColumnType:
+      {
+        bool b1 = node1->Text[_column] == "1" ? true : false;
+        bool b2 = node2->Text[_column] == "1" ? true : false;
+        if (_direction == SortOrder::Ascending)
+          return b1 < b2 ? -1 : 1;
+        else
+          return b2 < b1 ? -1 : 1;
+        break;
+      }
+
+    default:
+      return 0;
+    }
+  }
 
-  virtual int Compare(System::Object^ object1, System::Object^ object2);
 };
 
-ColumnComparer::ColumnComparer(int column, SortOrder direction)
-{
-  _column = column;
-  _direction = direction;
-  _inner = gcnew CaseInsensitiveComparer();
-}
-
-//--------------------------------------------------------------------------------------------------
-
-int ColumnComparer::Compare(System::Object^ object1, System::Object^ object2)
-{
-  MySQL::Forms::TreeViewNode^ node1 = dynamic_cast<MySQL::Forms::TreeViewNode^>(object1);
-  MySQL::Forms::TreeViewNode^ node2 = dynamic_cast<MySQL::Forms::TreeViewNode^>(object2);
-  if (_direction == SortOrder::Ascending)
-    return _inner->Compare(node1->Text[_column], node2->Text[_column]);
-  else
-    return _inner->Compare(node2->Text[_column], node1->Text[_column]);
-}
-
 //----------------- SortableTreeModel --------------------------------------------------------------
 
-/**
- * Does a depth sort of the given nodes.
- */
-void SortableTreeModel::SortNodes(Collection<Node^>^ nodes)
+Collections::IEnumerable^ SortableTreeModel::GetChildren(TreePath ^treePath)
 {
-  // Since the original tree model uses a Collection class we have to do two copies of all nodes
-  // because a Collection is not itself sortable.
-  ArrayList^ list = gcnew ArrayList();
-  list->AddRange(nodes);
-  list->Sort(Comparer);
-  nodes->Clear();
-  for each (Object^ obj in list)
-    nodes->Add((Node^) obj);
-
-  // Sort all child node lists.
-  for each (Node^ node in nodes)
-    SortNodes(node->Nodes);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void SortableTreeModel::Sort()
-{
-  if (_comparer != nullptr)
+  Node^ node = FindNode(treePath);
+  if (node != nullptr)
   {
-    SortNodes(Nodes);
-    OnStructureChanged(gcnew TreePathEventArgs(TreePath::Empty));
+    if (_comparer != nullptr)
+    {
+      ArrayList^ list = gcnew ArrayList();
+      list->AddRange(node->Nodes);
+      list->Sort(_comparer);    
+      return list;
+    }
+   return node->Nodes;
   }
+
+  return nullptr;
 }
 
 //----------------- AttributedNodeText -------------------------------------------------------------
@@ -108,7 +135,7 @@ void AttributedNodeText::OnDrawText(DrawEventArgs^ args)
   if (!attributes.bold && !attributes.italic && !attributes.color.is_valid())
     return;
 
-  FontStyle newStyle;
+  FontStyle newStyle = FontStyle::Regular;
   if (attributes.bold)
     newStyle = newStyle | FontStyle::Bold;
   if (attributes.italic)
@@ -135,7 +162,6 @@ bool TreeNodeViewImpl::create(::mforms::TreeNodeView *self, ::mforms::TreeOption
     t->ColumnClicked += gcnew System::EventHandler<TreeColumnEventArgs^>(treeview, &TreeNodeViewImpl::OnColumnClicked);
     t->Collapsed += gcnew System::EventHandler<TreeViewAdvEventArgs^>(treeview, &TreeNodeViewImpl::Collapsed);
     t->Expanded += gcnew System::EventHandler<TreeViewAdvEventArgs^>(treeview, &TreeNodeViewImpl::Expanded);
-    t->ColumnClicked += gcnew System::EventHandler<TreeColumnEventArgs^>(treeview, &TreeNodeViewImpl::OnColumnClicked);
     if (options & ::mforms::TreeNoBorder) // Default is 3d border.
       t->BorderStyle = BorderStyle::None;
 
@@ -153,7 +179,7 @@ bool TreeNodeViewImpl::create(::mforms::TreeNodeView *self, ::mforms::TreeOption
 
 //--------------------------------------------------------------------------------------------------
 
-int TreeNodeViewImpl::add_column(::mforms::TreeNodeView *self, ::mforms::TreeColumnType type, const string &name, 
+int TreeNodeViewImpl::add_column(::mforms::TreeNodeView *self, ::mforms::TreeColumnType type, const std::string &name, 
                              int initial_width, bool editable)
 {
   TreeNodeViewImpl^ treeview= (TreeNodeViewImpl^)ObjectImpl::FromUnmanaged(self);
@@ -321,7 +347,7 @@ TreeNodeViewImpl::~TreeNodeViewImpl()
 
 //--------------------------------------------------------------------------------------------------
 
-int TreeNodeViewImpl::add_column(::mforms::TreeColumnType type, const string &name, int initial_width, bool editable)
+int TreeNodeViewImpl::add_column(::mforms::TreeColumnType type, const std::string &name, int initial_width, bool editable)
 {
   TreeViewAdv^ treeview= get_control<TreeViewAdv>();
   
@@ -362,6 +388,7 @@ int TreeNodeViewImpl::add_column(::mforms::TreeColumnType type, const string &na
 
   TreeColumn^ column= gcnew TreeColumn(CppStringToNative(name), (initial_width < 0) ? 50 : initial_width);
   treeview->Columns->Add(column);
+  _column_types.Add((ManagedTreeColumnType)type); // We have a 1:1 mapping, so we can just cast.
 
   if (!_flat_list && treeview->Columns->Count == 1)
   {
@@ -575,7 +602,8 @@ void TreeNodeViewImpl::OnColumnClicked(System::Object^ sender, TreeColumnEventAr
       // Revert the sort order for our comparer, read above why.
       _current_sort_order = (e->Column->SortOrder == SortOrder::Ascending) ? SortOrder::Descending : SortOrder::Ascending;
     }
-    _model->Comparer = gcnew ColumnComparer(_current_sort_column, _current_sort_order);
+    _model->Comparer = gcnew ColumnComparer(_current_sort_column, _current_sort_order,
+      _column_types[_current_sort_column]);
   }
 }
 
@@ -660,6 +688,7 @@ std::list<mforms::TreeNodeRef> TreeNodeViewImpl::get_selection()
   return selection;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 mforms::TreeSelectionMode TreeNodeViewImpl::get_selection_mode()
 {
@@ -677,6 +706,8 @@ mforms::TreeSelectionMode TreeNodeViewImpl::get_selection_mode()
   return mforms::TreeSelectSingle;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void TreeNodeViewImpl::set_selection_mode(mforms::TreeSelectionMode mode)
 {
   TreeViewAdv^ tree= get_control<TreeViewAdv>();
@@ -721,6 +752,8 @@ static mforms::TreeNodeRef find_node_at_row(mforms::TreeNodeRef node, int &row_c
   return res;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 mforms::TreeNodeRef TreeNodeViewImpl::node_at_row(int row)
 {
   int i = 0;
@@ -745,6 +778,7 @@ static int count_rows_in_node(mforms::TreeNodeRef node)
   return 0;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 int TreeNodeViewImpl::row_for_node(mforms::TreeNodeRef node)
 {
@@ -802,11 +836,7 @@ void TreeNodeViewImpl::freeze_refresh(bool flag)
   else
   {
     ControlUtilities::ResumeDrawing(tree);
-
-    if (_can_sort_column && _model->Comparer!=nullptr)
-      _model->Sort();
-    else
-      tree->Refresh();
+    tree->Refresh();
   }
 }
 
