@@ -22,8 +22,16 @@
  */
 
 #include "stdafx.h"
+
 #include <StrSafe.h>
 
+#include "mforms/mforms.h"
+#include "wf_view.h"
+#include "wf_utilities.h"
+
+#include "base/string_utilities.h"
+#include "ConvUtils.h"
+
 using namespace MySQL::Forms;
 using namespace MySQL::Utilities;
 
@@ -205,8 +213,8 @@ mforms::DialogResult CustomMessageBox::ShowVistaStyle(const std::string& title,
 /// <summary>
 /// Displays the CustomMessageBox and returns a dialog result, depending on the button clicked.
 /// </summary>
-mforms::DialogResult CustomMessageBox::ShowTraditionalStyle(String^ title, String^ text, Image^ image, String^ button1,
-  String^ button2, String^ button3, String^ checkbox, bool& checked)
+mforms::DialogResult CustomMessageBox::ShowTraditionalStyle(String^ title,  String^ text, Image^ image,  String^ button1,
+   String^ button2,  String^ button3,  String^ checkbox, bool& checked)
 {
   CustomMessageBox^ box= gcnew CustomMessageBox();
   
@@ -461,12 +469,12 @@ mforms::DialogResult CustomMessageBox::Show(const std::string& title, const std:
   }
   else
   {
-    String^ boxTitle= CppStringToNative(title);
-    String^ boxText= CppStringToNative(text);
-    String^ ok_text= CppStringToNative(buttonOK);
-    String^ cancel_text= CppStringToNative(buttonCancel);
-    String^ other_text= CppStringToNative(buttonOther);
-    String^ checkbox_text= CppStringToNative(checkbox);
+     String^ boxTitle= CppStringToNative(title);
+     String^ boxText= CppStringToNative(text);
+     String^ ok_text= CppStringToNative(buttonOK);
+     String^ cancel_text= CppStringToNative(buttonCancel);
+     String^ other_text= CppStringToNative(buttonOther);
+     String^ checkbox_text= CppStringToNative(checkbox);
 
     Image^ image;
     if (mainIcon == TD_WARNING_ICON)
@@ -487,8 +495,8 @@ mforms::DialogResult CustomMessageBox::Show(const std::string& title, const std:
 
 //--------------------------------------------------------------------------------------------------
 
-Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ title, String^ text,  
-  String^ buttonOK, String^ buttonCancel, String^ buttonOther, String^ checkbox, [Out] bool% checked)
+Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type,  String^ title,  String^ text,  
+   String^ buttonOK,  String^ buttonCancel,  String^ buttonOther,  String^ checkbox, [Out] bool% checked)
 {
   mforms::DialogResult result;
 
@@ -560,8 +568,8 @@ Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ ti
 
 //--------------------------------------------------------------------------------------------------
 
-Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ title, String^ text,
-  String^ buttonOK)
+Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type,  String^ title,  String^ text,
+   String^ buttonOK)
 {
   bool checked = false;
   return Show(type, title, text, buttonOK, "", "", "", checked);
@@ -618,8 +626,8 @@ UtilitiesImpl::UtilitiesImpl()
 
 //--------------------------------------------------------------------------------------------------
 
-int UtilitiesImpl::show_message(const string &title, const string &text, const string &ok, 
-  const string &cancel, const string &other)
+int UtilitiesImpl::show_message(const std::string &title, const std::string &text, const std::string &ok, 
+  const std::string &cancel, const std::string &other)
 {
   hide_wait_message();
 
@@ -629,8 +637,8 @@ int UtilitiesImpl::show_message(const string &title, const string &text, const s
 
 //--------------------------------------------------------------------------------------------------
 
-int UtilitiesImpl::show_error(const string &title, const string &text, const string &ok, 
-  const string &cancel, const string &other)
+int UtilitiesImpl::show_error(const std::string &title, const std::string &text, const std::string &ok, 
+  const std::string &cancel, const std::string &other)
 {
   hide_wait_message();
 
@@ -640,8 +648,8 @@ int UtilitiesImpl::show_error(const string &title, const string &text, const str
 
 //--------------------------------------------------------------------------------------------------
 
-int UtilitiesImpl::show_warning(const string &title, const string &text, const string &ok, 
-  const string &cancel, const string &other)
+int UtilitiesImpl::show_warning(const std::string &title, const std::string &text, const std::string &ok, 
+  const std::string &cancel, const std::string &other)
 {
   hide_wait_message();
 
@@ -651,8 +659,8 @@ int UtilitiesImpl::show_warning(const string &title, const string &text, const s
 
 //--------------------------------------------------------------------------------------------------
 
-int UtilitiesImpl::show_message_with_checkbox(const string &title, const string &text,
-  const string &ok, const string &cancel, const string &other, const std::string &checkbox_text,
+int UtilitiesImpl::show_message_with_checkbox(const std::string &title, const std::string &text,
+  const std::string &ok, const std::string &cancel, const std::string &other, const std::string &checkbox_text,
   bool &isChecked)
 {
   hide_wait_message();
@@ -668,7 +676,7 @@ int UtilitiesImpl::show_message_with_checkbox(const string &title, const string
 /**
  * Shows the warning heads-up-display with the given title and text.
  */
-void UtilitiesImpl::show_wait_message(const string &title, const string &text)
+void UtilitiesImpl::show_wait_message(const std::string &title, const std::string &text)
 {
   HUDForm::Show(CppStringToNative(title), CppStringToNative(text), true);
 }
@@ -701,7 +709,7 @@ public:
 
 //-------------------------------------------------------------------------------------------------
 
-bool UtilitiesImpl::run_cancelable_wait_message(const string &title, const string &text, 
+bool UtilitiesImpl::run_cancelable_wait_message(const std::string &title, const std::string &text, 
                                                 const boost::function<void ()> &start_task,
                                                 const boost::function<bool ()> &cancel_slot)
 {
@@ -746,7 +754,7 @@ void UtilitiesImpl::stop_cancelable_wait_message()
  * 
  * @param content The text to be placed on the clipboard. It is assume its encoding is UTF-8.
  */
-void UtilitiesImpl::set_clipboard_text(const string &content)
+void UtilitiesImpl::set_clipboard_text(const std::string &content)
 {
   if (!content.empty())
   {
@@ -790,7 +798,7 @@ void UtilitiesImpl::set_clipboard_text(const string &content)
  * @result If there is text on the clipboard (ANSI or Unicode) it is returned as UTF-8 string.
  * @note The returned text gets all CRLF Windows line breaks converted to pure LF.
  */
-string UtilitiesImpl::get_clipboard_text()
+std::string UtilitiesImpl::get_clipboard_text()
 {
   String^ unicode= (String^) Clipboard::GetData(DataFormats::UnicodeText);
   if (unicode == nullptr)
@@ -804,7 +812,7 @@ string UtilitiesImpl::get_clipboard_text()
 /**
  * Returns platform specific user folders, e.g. for the desktop, the user's documents etc.
  */
-string UtilitiesImpl::get_special_folder(mforms::FolderType type)
+std::string UtilitiesImpl::get_special_folder(mforms::FolderType type)
 {
   Environment::SpecialFolder special_folder;
   switch (type)
@@ -839,7 +847,7 @@ string UtilitiesImpl::get_special_folder(mforms::FolderType type)
 
 //--------------------------------------------------------------------------------------------------
 
-void UtilitiesImpl::open_url(const string &url)
+void UtilitiesImpl::open_url(const std::string &url)
 {
   try
   {
@@ -854,7 +862,7 @@ void UtilitiesImpl::open_url(const string &url)
 
 //--------------------------------------------------------------------------------------------------
 
-bool UtilitiesImpl::move_to_trash(const string &file_name)
+bool UtilitiesImpl::move_to_trash(const std::string &file_name)
 {
   SHFILEOPSTRUCT shf = {0};
 
@@ -925,9 +933,9 @@ void UtilitiesImpl::load_passwords()
       std::string item;
       while (std::getline(ss, item, '\n'))
       {
-        string::size_type pos= item.find_first_of(PASSWORD_SEPARATOR, 0);
+        std::string::size_type pos= item.find_first_of(PASSWORD_SEPARATOR, 0);
 
-        if (string::npos != pos)
+        if (std::string::npos != pos)
           password_cache[item.substr(0, pos)]= item.substr(pos + 1, item.length());
       }
     }
