@@ -19,6 +19,20 @@
 
 #include "stdafx.h"
 
+#include "mforms/mforms.h"
+#include "wf_view.h"
+#include "wf_code_editor.h"
+
+#include "workbench/wb_context.h" // Needed for AppImpl.
+#include "wf_box.h"
+#include "wf_appview.h"
+#include "wf_app.h"
+
+#include "base/log.h"
+#include "base/string_utilities.h"
+
+#include "ConvUtils.h"
+
 using namespace Windows::Forms;
 using namespace System::Drawing;
 using namespace System::IO;
