@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,10 +20,6 @@
 #ifndef _WF_WEBBROWSER_H_
 #define _WF_WEBBROWSER_H_
 
-
-#include "mforms/webbrowser.h"
-#include "wf_view.h"
-
 using namespace System::Windows::Forms;
 using namespace System::ComponentModel;
 
@@ -46,6 +42,7 @@ namespace MySQL {
       void DocumentLoaded(Object^ sender, WebBrowserDocumentCompletedEventArgs^ e);
       void OnNewWindow(Object^ sender, CancelEventArgs^ e);
       void StatusTextChanged(Object^ sender, EventArgs^ e);
+      void OnNavigating(Object^ sender, WebBrowserNavigatingEventArgs^ args);
     public:
       void set_html(const std::string& code);
       void navigate(const std::string& url);
