@@ -669,7 +669,7 @@ static void fixup_object_copied_references(ObjectRef copy,
   while (metac != 0);
 }
 
-ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string> skip_members)
+ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string> skip_members, bool _dontfollow)
 {
   if (object.is_valid())
   {
@@ -695,7 +695,7 @@ ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string>
 
         Type type= mem->second.type.base.type;
         
-        bool dontfollow= !mem->second.owned_object;
+        bool dontfollow= _dontfollow || !mem->second.owned_object;
         
         if (is_simple_type(type))
         {
@@ -727,7 +727,7 @@ ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string>
           else
           {
             if (k == "owner") throw; // consistency check
-            ObjectRef vcopy(duplicate_object(ObjectRef::cast_from(v), std::set<std::string>()));
+            ObjectRef vcopy(duplicate_object(ObjectRef::cast_from(v), std::set<std::string>(), false));
             copy.set_member(k, vcopy);
           }
         }
@@ -916,13 +916,22 @@ void grt::merge_contents(ObjectRef target, ObjectRef source)
 
 ObjectRef CopyContext::copy(const ObjectRef &object, std::set<std::string> skip_members)
 {
-  ObjectRef copy= duplicate_object(object, skip_members);
+  ObjectRef copy= duplicate_object(object, skip_members, false);
   if (copy.is_valid())
     copies.push_back(copy);
 
   return copy;
 }
 
+ObjectRef CopyContext::shallow_copy(const ObjectRef &object)
+{
+  ObjectRef copy= duplicate_object(object, std::set<std::string>(), true);
+  if (copy.is_valid())
+    copies.push_back(copy);
+  
+  return copy;
+}
+
 
 void CopyContext::update_references()
 {
@@ -933,6 +942,13 @@ void CopyContext::update_references()
 }
 
 
+ValueRef CopyContext::copy_for_object(ValueRef object)
+{
+  if (object_copies.find(object.valueptr()) != object_copies.end())
+    return object_copies[object.valueptr()];
+  return ValueRef();
+}
+
 static ValueRef copy_value(ValueRef value, bool deep, internal::Object *owner)
 {
   switch (value.type())
@@ -984,7 +1000,7 @@ static ValueRef copy_value(ValueRef value, bool deep, internal::Object *owner)
     {
       ObjectRef obj(ObjectRef::cast_from(value));
 
-      return copy_object(obj.content().get_grt(), obj);
+      return copy_object(obj);
     }
   }
   
