@@ -230,19 +230,22 @@ namespace grt
     CopyContext(GRT *agrt) : grt(agrt) {}
         
     ObjectRef copy(const ObjectRef &object, std::set<std::string> skip_members= std::set<std::string>());
+    ObjectRef shallow_copy(const ObjectRef &object);
     void finish() { update_references(); }
     void update_references();
     
+    ValueRef copy_for_object(ValueRef object);
+    
   private:
-    ObjectRef duplicate_object(ObjectRef object, std::set<std::string> skip_members);
+    ObjectRef duplicate_object(ObjectRef object, std::set<std::string> skip_members, bool dontfollow);
     void copy_list(BaseListRef &list, const BaseListRef &source, bool dontfollow);
     void copy_dict(DictRef &dict, const DictRef &source, bool dontfollow);
   };
 
   template<typename OType>
-  OType copy_object(GRT *grt, const OType& object, std::set<std::string> skip_members = std::set<std::string>())
+  OType copy_object(const OType& object, std::set<std::string> skip_members = std::set<std::string>())
   {
-      CopyContext copier(grt);
+      CopyContext copier(object.get_grt());
       OType copy;
 
       copy= OType::cast_from(copier.copy(object, skip_members));
@@ -251,7 +254,19 @@ namespace grt
       return copy;
   }
 
-  
+  template<typename OType>
+  OType shallow_copy_object(const OType& object)
+  {
+      CopyContext copier(object.get_grt());
+      OType copy;
+
+      copy= OType::cast_from(copier.shallow_copy(object));
+      copier.finish();
+
+      return copy;
+  }
+
+ 
   // temporary code
   MYSQLGRT_PUBLIC bool init_python_support(grt::GRT *grt, const std::string &python_module_path);
   MYSQLGRT_PUBLIC void add_python_module_dir(grt::GRT *grt, const std::string &python_module_path);
@@ -269,7 +284,7 @@ namespace grt
     //_dontdiff_mask will hold mask to allow selective bypass of ceratin fields
     //1 always diff, 2 diff only vs db, 4 diff only vs live object
     unsigned int dontdiff_mask;
-    Omf(): dontdiff_mask(1){};
+    Omf(): dontdiff_mask(1), case_sensitive(true){};
     virtual ~Omf() {};
     virtual bool less(const ValueRef& , const ValueRef&) const= 0;
     virtual bool equal(const ValueRef& , const ValueRef&) const= 0;
