@@ -350,14 +350,19 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
     def query_cancel_status(self):
         return self._cancel_requested
 
+    def send_raw(self, msg):
+        self._log_queue.append(("OUTPUT", msg, ""))
 
     def send_info(self, msg):
+        grt.log_debug("Wizard", msg)
         self._handle_task_output("INFO", msg, "")
         
     def send_error(self, msg):
+        grt.log_debug("Wizard", "ERROR: "+msg)
         self._handle_task_output("ERROR", msg, "")
         
     def send_warning(self, msg):
+        grt.log_debug("Wizard", "WARNING: "+msg)
         self._handle_task_output("WARNING", msg, "")
 
     def send_progress(self, pct, msg):
@@ -542,9 +547,9 @@ class WizardProgressPage(wizard_page_widget.WizardPage):
             try:
                 task.run()
             except Exception, exc:
-                print
-                import traceback
-                traceback.print_exc()
+                #print
+                #import traceback
+                #traceback.print_exc()
                 self.send_error("Exception in task '%s': %r\n" % (task.label, exc))
                 self._status_label.set_text("%s" % self.format_exception_text(exc))
                 self._failed()
