@@ -27,7 +27,7 @@ class DB_MYSQL_SQLIDE_PUBLIC_FUNC Mysql_sql_editor : public Sql_editor
 {
 protected:
   virtual bool fill_auto_completion_keywords(std::vector<std::pair<int, std::string> > &entries,
-    bool keywords, bool functions);
+    bool major_keywords, bool keywords, bool functions);
 public:
   Mysql_sql_editor(db_mgmt_RdbmsRef rdbms, GrtVersionRef version);
   virtual ~Mysql_sql_editor();
