@@ -90,13 +90,6 @@ void DiffSQLGeneratorBE::remember_alter(const GrtNamedObjectRef &obj, const std:
 }
 
 
-struct Part_sort_pred {
-  bool operator() (const std::pair<int, grt::ValueRef>& v1, const std::pair<int, grt::ValueRef>& v2)
-  {
-    return v1 < v2;
-  }
-};
-
 void DiffSQLGeneratorBE::generate_set_partitioning(db_mysql_TableRef table, 
                                                    const grt::DiffChange *table_diffchange)
 {
@@ -314,13 +307,13 @@ void DiffSQLGeneratorBE::generate_create_stmt(db_mysql_TableRef table)
     if(strlen(table->avgRowLength().c_str()))
       callback->create_table_avg_row_length(table->avgRowLength());
 
-    if(strlen(table->defaultCharacterSetName().c_str()))
+    if(table->defaultCharacterSetName().is_valid() && strlen(table->defaultCharacterSetName().c_str()))
       callback->create_table_charset(table->defaultCharacterSetName());
 
     if(table->checksum() != 0)
       callback->create_table_checksum(table->checksum());
 
-    if(strlen(table->defaultCollationName().c_str()))
+    if(table->defaultCollationName().is_valid() && strlen(table->defaultCollationName().c_str()))
       callback->create_table_collate(table->defaultCollationName());
 
     if(strlen(table->comment().c_str()))
@@ -1155,6 +1148,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       if(partitions_processed)
         continue;
 
+      // TODO: is this get call still needed (no side effects) or can we remove it?
 /*unused      const grt::SimpleValueChange *change= */
         static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       generate_set_partitioning(table, diffchange);
@@ -1257,7 +1251,9 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_ViewRef old_view, db_mysql
 
   generate_create_stmt(new_view);
 
-  if(strcmp(old_view->name().c_str(), new_view->name().c_str())) // name changed - need to drop old view
+  std::string new_view_name = _case_sensitive ? *new_view->name() : base::toupper(new_view->name());
+  std::string old_view_name = _case_sensitive ? *old_view->name() : base::toupper(old_view->name());
+  if(strcmp(new_view_name.c_str(), old_view_name.c_str())) // name changed - need to drop old view
     generate_drop_stmt(old_view);
 }
 
@@ -1529,7 +1525,7 @@ static void fill_set_from_list(grt::StringListRef string_list, std::set<std::str
 }
 
 DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEActionInterface *cb)
-  : padding(2), callback(cb), _gen_create_index(false), _use_filtered_lists(true), 
+  : callback(cb), _gen_create_index(false), _use_filtered_lists(true), 
    _skip_foreign_keys(false), _skip_fk_indexes(false), _case_sensitive(false), _use_oid_as_dict_key(false)
 {
   if (options.is_valid())
