@@ -69,6 +69,18 @@ DiffSQLGeneratorBEActionInterface::~DiffSQLGeneratorBEActionInterface()
 
 namespace {
 
+    inline std::string generate_comment(const std::string& comment_text, size_t comment_lenght, const std::string& quote = std::string())
+    {
+        std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(comment_text, comment_lenght));
+        if (!quote.empty() && !comment.empty())
+            comment = quote+comment+quote;
+        if (!bec::TableHelper::get_document_comment(comment_text, comment_lenght).empty())
+            comment.append(" /* comment truncated */ /*")
+            .append(bec::TableHelper::get_document_comment(comment_text, comment_lenght))
+            .append("*/");
+        return comment; 
+    }
+
 class TextPadding
 {
   int padding;
@@ -572,11 +584,9 @@ std::string ActionGenerateSQL::generate_create(db_mysql_ColumnRef column)
   }
   
 
-  std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(column->comment(), _maxColumnCommentLength));
-  if (!bec::TableHelper::get_document_comment(column->comment(), _maxColumnCommentLength).empty())
-    comment.append(" /* comment truncated */");  
-  if(!comment.empty())
-    sql.append("COMMENT '").append(base::escape_sql_string(comment)).append("' ");
+  std::string comment = generate_comment(column->comment(), _maxColumnCommentLength);
+  if (!comment.empty())
+    sql.append("COMMENT '").append(comment).append("' ");
 
   // TODO:
   // (?) [reference_definition]
@@ -682,11 +692,9 @@ std::string ActionGenerateSQL::generate_create(db_mysql_IndexRef index, std::str
   if (index->withParser().is_valid() && *index->withParser().c_str())
     index_sql.append(" WITH PARSER ").append(index->withParser());
 
-  std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(index->comment(), _maxIndexCommentLength));
-  if (!bec::TableHelper::get_document_comment(index->comment(), _maxIndexCommentLength).empty())
-    comment.append(" /* comment truncated */");  
-  if(!comment.empty())
-    sql.append("COMMENT '").append(base::escape_sql_string(comment)).append("' ");
+  std::string comment = generate_comment(index->comment(), _maxIndexCommentLength);
+  if (!comment.empty())
+    sql.append("COMMENT '").append(comment).append("' ");
 
 
   index_sql.append(") ");
@@ -788,11 +796,7 @@ void ActionGenerateSQL::create_table_max_rows(grt::StringRef value)
 
 void ActionGenerateSQL::create_table_comment(grt::StringRef value)
 {
-  std::string str = value;
-  std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(str, _maxTableCommentLength));
-  sql.append("\nCOMMENT = '").append(comment).append("'");
-  if (str.size() > comment.size())
-    sql.append(" /* comment truncated */");  
+  sql.append("\nCOMMENT = '").append(generate_comment(value, _maxTableCommentLength)).append("'");
 }
 
 void ActionGenerateSQL::create_table_data_dir(grt::StringRef value)
@@ -823,10 +827,10 @@ void ActionGenerateSQL::create_schema(db_mysql_SchemaRef schema)
     schema_sql.append("IF NOT EXISTS ");
   schema_sql.append("`").append(schema->name().c_str()).append("` ");
 
-  if(strlen(schema->defaultCharacterSetName().c_str()))
+  if(schema->defaultCollationName().is_valid() && strlen(schema->defaultCharacterSetName().c_str()))
     schema_sql.append("DEFAULT CHARACTER SET ").append(schema->defaultCharacterSetName().c_str()).append(" ");
 
-  if(strlen(schema->defaultCollationName().c_str()))
+  if(schema->defaultCollationName().is_valid() && strlen(schema->defaultCollationName().c_str()))
     schema_sql.append("COLLATE ").append(schema->defaultCollationName().c_str()).append(" ");
 
   remember(schema, schema_sql);
@@ -943,12 +947,7 @@ void ActionGenerateSQL::alter_table_collate(db_mysql_TableRef table, grt::String
 
 void ActionGenerateSQL::alter_table_comment(db_mysql_TableRef table, grt::StringRef str)
 {
-  std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(str, _maxTableCommentLength)).append("'");
-  if (!bec::TableHelper::get_document_comment(str, _maxTableCommentLength).empty())
-  {
-    comment.append(" /* comment truncated */");  
-  };
-  alter_table_property(sql, "\nCOMMENT = '", comment);
+  alter_table_property(sql, "\nCOMMENT = ", "'" + generate_comment(str, _maxTableCommentLength) + "'");
 }
 
 void ActionGenerateSQL::alter_table_merge_union(db_mysql_TableRef table, grt::StringRef str)
@@ -1519,14 +1518,14 @@ void ActionGenerateSQL::create_user(db_UserRef user)
 {
   std::string sql;
 
-  sql.append("CREATE USER `").append(user->name().c_str()).append("`");
+  sql.append("CREATE USER ").append(quote_user(user->name()));
   if (user->password().is_valid() && *user->password().c_str())
     sql.append(" IDENTIFIED BY '").append(*user->password()).append("'");
 
   sql.append(";\n\n");
 
   std::list<std::string> grants;
-  gen_grant_sql(user, grants);
+  gen_grant_sql(db_CatalogRef::cast_from(user->owner()), user, grants);
 
   std::list<std::string>::iterator iter= grants.begin();
   for(; iter != grants.end(); ++iter)
@@ -2076,6 +2075,34 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
 
   }
 
+  for(size_t c1= schemata.count(), i= 0; i < c1; i++)
+  {
+      std::string objects_sql;
+
+      db_mysql_SchemaRef schema= schemata.get(i);
+      if (schema->modelOnly())
+          continue;
+
+      if (grt) 
+          grt->send_output(std::string("Processing Schema ").append(schema->name()).append("\n"));
+
+      if ((!use_short_names || gen_use) && (create_map.has_key(get_old_object_name_for_key(schema, case_sensitive))))
+          out_sql.append("USE `").append(schema->name().c_str()).append("` ;\n");
+
+
+      // tables
+      grt::ListRef<db_mysql_Table> tables= schema->tables();
+      for(size_t c2= tables.count(), j= 0; j < c2; j++)
+      {
+          db_mysql_TableRef table= tables.get(j);
+          if (table->modelOnly() || table->isStub())
+              continue;
+          objects_sql += GenerateTable::perform(schema, table, generated_tables, options, create_map, drop_map, grt,inserts_sql);
+      }
+      if(!objects_sql.empty())
+          out_sql += objects_sql;
+  }
+
   for(size_t c1= schemata.count(), i= 0; i < c1; i++)
   {
     std::string objects_sql;
@@ -2091,16 +2118,6 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
       out_sql.append("USE `").append(schema->name().c_str()).append("` ;\n");
 
 
-    // tables
-    grt::ListRef<db_mysql_Table> tables= schema->tables();
-    for(size_t c2= tables.count(), j= 0; j < c2; j++)
-    {
-      db_mysql_TableRef table= tables.get(j);
-      if (table->modelOnly() || table->isStub())
-        continue;
-      objects_sql += GenerateTable::perform(schema, table, generated_tables, options, create_map, drop_map, grt,inserts_sql);
-    }
-
     // views placeholder tables
     alias_map_t alias_map;
     grt::ListRef<db_mysql_View> views= schema->views();
@@ -2793,4 +2810,3 @@ grt::ListRef<db_UserDatatype> DbMySQLImpl::getDefaultUserDatatypes(db_mgmt_Rdbms
 
 
 GRT_MODULE_ENTRY_POINT(DbMySQLImpl);
-
