@@ -1,3 +1,5 @@
+from __future__ import with_statement
+
 # import the wb module
 from wb import DefineModule, wbinputs
 # import the grt module
@@ -6,6 +8,7 @@ import grt
 import mforms
 
 import re
+import os
 
 from sql_reformatter import formatter_for_statement_ast, ASTHelper
 
@@ -14,6 +17,55 @@ from sql_reformatter import formatter_for_statement_ast, ASTHelper
 ModuleInfo = DefineModule(name= "SQLIDEUtils", author= "Oracle Corp.", version="1.1")
 
 
+@ModuleInfo.export(grt.INT, grt.classes.db_query_EditableResultset)
+def importRecordsetDataFromFile(resultset):
+    file_chooser = mforms.newFileChooser(None, mforms.OpenFile)
+    file_chooser.set_title('Import Recordset From CSV File')
+    file_chooser.set_directory(os.path.expanduser('~'))
+    file_chooser.set_extensions('CSV Files (*.csv)|*.csv', 'import')
+    if file_chooser.run_modal():
+        with open(file_chooser.get_path(), 'rb') as import_file:
+            ext = os.path.splitext(import_file.name)[1].lower()
+            import_module = None
+            if ext == '.csv':
+                import csv as import_module
+            elif ext == '.sql':
+                pass  # Here will go our not yet written .sql reader
+            else:
+                import csv as import_module
+            if import_module:
+                reader = import_module.reader(import_file)
+                column_count = len(resultset.columns)
+                type_classes = { 'string':str,
+                                 'int':int,
+                                 'real':float,
+                                 'blob':str,
+                                 'date':str,
+                                 'time':str,
+                                 'datetime':str,
+                                 'geo':str,
+                               }
+                converters = tuple(type_classes[column.columnType] for column in resultset.columns)
+                for row in reader:
+                    if len(row) < column_count:  # Fill with default values
+                        row.extend(converter() for converter in converters[len(row):])
+                    try:
+                        converted_values = [ converter(value) for converter, value in zip(converters, row) ]
+                    except ValueError:
+                        continue  # TODO: log a warning here
+                    resultset.addNewRow()
+                    for column, value in enumerate(converted_values):
+                        if isinstance(value, str):
+                            resultset.setStringFieldValue(column, value)
+                        elif isinstance(value, int):
+                            resultset.setIntFieldValue(column, value)
+                        elif isinstance(value, float):
+                            resultset.setFloatFieldValue(column, value)
+                        else:
+                            resultset.setFieldNull(column)
+    return 0
+
+
 @ModuleInfo.plugin("wb.sqlide.executeToTextOutput", caption= "Execute Query Into Text Output", input= [wbinputs.currentQueryEditor()], pluginMenu= "SQL/Utilities")
 @ModuleInfo.export(grt.INT, grt.classes.db_query_QueryEditor)
 def executeQueryAsText(qbuffer):
@@ -533,7 +585,29 @@ def commentText(editor):
             new_text = "\n".join("-- "+line for line in lines)
         editor.replaceSelection(new_text)
     else:
-        editor.replaceSelection("-- ")
+        pos = editor.insertionPoint
+        full_text = editor.script
+        done = False
+        # if cursor is before or after a comment sequence, then delete that
+        if full_text[pos:pos+3] == "-- ":
+            editor.replaceContents(full_text[:pos] + full_text[pos+3:])
+            done = True
+        else:
+            for i in range(4):
+                if full_text[pos+i:pos+i+3] == "-- ":
+                    editor.replaceContents(full_text[:pos+i] + full_text[pos+i+3:])
+                    done = True
+                    break
+                if pos-i >= 0 and full_text[pos-i:pos-i+3] == "-- ":
+                    editor.replaceContents(full_text[:pos-i] + full_text[pos-i+3:])
+                    done = True
+                    pos -= i
+                    break
+       
+        if not done:
+            editor.replaceSelection("-- ")
+
+        editor.insertionPoint = pos
     return 0
 
 
