@@ -50,6 +50,7 @@
 #import "WBTabItem.h" // needed for WBCustomTabItemView
 
 #import "DbPrivilegeEditorTab.h"
+#include <mforms/view.h>
 
 static NSString* shouldRaiseException = @"should raise exception";
 
@@ -263,7 +264,7 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
   // Enable or disable the first row of controls.
   [mPartitionPopup setEnabled: enabled];
   [mPartitionParametersTextField setEnabled: enabled];
-  [mPartitionCountCombo setEnabled: enabled];
+  [mPartitionCountTextField setEnabled: enabled];
   [mPartitionManualCheckbox setEnabled: enabled];
   
   // Enable or disable the first popup on the second row of controls.
@@ -275,7 +276,7 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
   NSString* subpartitionType = [mSubpartitionPopup titleOfSelectedItem];
   subEnabled = subEnabled && (! [subpartitionType isEqualToString: @"Disabled"]);
   [mSubPartitionParametersTextField setEnabled: subEnabled];
-  [mSubpartitionCountCombo setEnabled: subEnabled];
+  [mSubpartitionCountTextField setEnabled: subEnabled];
   [mSubpartitionManualCheckbox setEnabled: subEnabled && ([mPartitionManualCheckbox state] == NSOnState)];
   
   {
@@ -288,7 +289,7 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
 
     int c = mBackEnd->get_partition_count();
     NSString* partCount = [NSString stringWithFormat: @"%d", c];
-    [mPartitionCountCombo setStringValue: partCount];
+    [mPartitionCountTextField setStringValue: partCount];
     
     NSCellStateValue manualState = ( mBackEnd->get_explicit_partitions() == true ? NSOnState : NSOffState );
     [mPartitionManualCheckbox setState: manualState];
@@ -309,7 +310,7 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
     
     int c = mBackEnd->get_subpartition_count();
     NSString* partCount = [NSString stringWithFormat: @"%d", c];
-    [mSubpartitionCountCombo setStringValue: partCount];
+    [mSubpartitionCountTextField setStringValue: partCount];
     
     NSCellStateValue manualState = ( mBackEnd->get_explicit_subpartitions() == true ? NSOnState : NSOffState );
     [mSubpartitionManualCheckbox setState: manualState];
@@ -389,8 +390,6 @@ static void call_refresh(DbMysqlTableEditor* theEditor)
 
 - (void)refreshTableEditorGUITriggersTab
 {
-  if (mBackEnd->get_sql_editor()->is_refresh_enabled())
-    mBackEnd->load_trigger_sql();
 }
 
 
@@ -948,6 +947,22 @@ objectValueForTableColumn: (NSTableColumn*) aTableColumn
   }
 }
 
+- (BOOL)tableView: (NSTableView *)aTableView
+shouldEditTableColumn: (NSTableColumn *)aTableColumn
+              row: (NSInteger)rowIndex
+{
+  // Activate the placeholder row and set the default value if this is the name column.
+  if (aTableView == mColumnsTable && rowIndex == [mColumnsDataSource numberOfRowsInTableView: aTableView] - 1) {
+    NSString* columnName = [mColumnsDataSource objectValueForValueIndex: bec::TableColumnsListBE::Name
+                                                                    row: rowIndex];
+    if (columnName.length == 0) {
+      // Mark this row as placeholder. This activates special handling in the backend
+      // (e.g. default value generation).
+      mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Name, 1);
+    }
+  }
+  return YES;
+}
 
 #pragma mark Table view drag n drop
 
@@ -955,7 +970,7 @@ objectValueForTableColumn: (NSTableColumn*) aTableColumn
 
 - (BOOL) tableView: (NSTableView*) aTableView
 writeRowsWithIndexes: (NSIndexSet*) rowIndices
-      toPasteboard: (NSPasteboard*) pboard;
+      toPasteboard: (NSPasteboard*) pboard
 {
   BOOL shouldStartDrag = NO;
   
@@ -986,7 +1001,7 @@ writeRowsWithIndexes: (NSIndexSet*) rowIndices
 - (NSDragOperation) tableView: (NSTableView*) aTableView
                  validateDrop: (id <NSDraggingInfo>) info
                   proposedRow: (NSInteger) proposedRow
-        proposedDropOperation: (NSTableViewDropOperation) operation;
+        proposedDropOperation: (NSTableViewDropOperation) operation
 {
   NSDragOperation op = NSDragOperationNone;
   
@@ -1014,7 +1029,7 @@ writeRowsWithIndexes: (NSIndexSet*) rowIndices
 - (BOOL) tableView: (NSTableView*) aTableView
         acceptDrop: (id <NSDraggingInfo>) info
                row: (NSInteger) dropRow
-     dropOperation: (NSTableViewDropOperation) operation;
+     dropOperation: (NSTableViewDropOperation) operation
 {  
   BOOL didAccept = NO;
 
@@ -1051,7 +1066,7 @@ writeRowsWithIndexes: (NSIndexSet*) rowIndices
 
 
 
-- (NSInteger) numberOfItemsInComboBoxCell: (NSComboBoxCell*) aComboBoxCell;
+- (NSInteger) numberOfItemsInComboBoxCell: (NSComboBoxCell*) aComboBoxCell
 {
   return [mColumnTypes count];
 }
@@ -1059,7 +1074,7 @@ writeRowsWithIndexes: (NSIndexSet*) rowIndices
 
 
 - (id) comboBoxCell: (NSComboBoxCell*) aComboBoxCell
-objectValueForItemAtIndex: (NSInteger) index;
+objectValueForItemAtIndex: (NSInteger) index
 {
   return [mColumnTypes objectAtIndex: index];
 }
@@ -1089,12 +1104,12 @@ objectValueForItemAtIndex: (NSInteger) index;
 
 // Show FK editor or disable it by showing a placeholder with info depending on the currently
 // selected table engine.
-- (void) updateFKPlaceholder;
+- (void) updateFKPlaceholder
 {
   [mFKWarningPanel setHidden: !mBackEnd->is_editing_live_object() || mBackEnd->engine_supports_foreign_keys()];
 }
 
-- (IBAction) userPickPopup: (id) sender;
+- (IBAction) userPickPopup: (id) sender
 {
   NSString* popItemTitle = [sender titleOfSelectedItem];
   
@@ -1197,7 +1212,7 @@ objectValueForItemAtIndex: (NSInteger) index;
 }
 
 
-- (IBAction) userClickButton: (id) sender;
+- (IBAction) userClickButton: (id) sender
 {
   if (sender == mColumnsFlagPK)
     mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::IsPK, [sender state] == NSOnState);
@@ -1249,92 +1264,53 @@ objectValueForItemAtIndex: (NSInteger) index;
   }
 }
 
-
-
-- (void) comboBoxSelectionDidChange: (NSNotification*) aNotification;
+/**
+ * Called by Cocoa after every keystroke in a text field. This is used to update parts of
+ * the UI at every keystroke.
+ * TODO: reconsider this handling. Doing a full page refresh for evey keystroke is just nonsense.
+ */
+- (void)controlTextDidChange: (NSNotification*)aNotification
 {
   id sender = [aNotification object];
   
-  if (sender == mPartitionCountCombo) {
-    int val = [[sender objectValueOfSelectedItem] intValue];
-    mBackEnd->set_partition_count(val);
-    [self refreshTableEditorGUIPartitioningTab];
-  }
-  else if (sender == mSubpartitionCountCombo) {
-    int val = [[sender objectValueOfSelectedItem] intValue];
-    mBackEnd->set_subpartition_count(val);
-    [self refreshTableEditorGUIPartitioningTab];
-  }
-}
-
-// Called by Cocoa after every keystroke in a text field.
-- (void) controlTextDidChange: (NSNotification*) aNotification;
-{
-  id sender = [aNotification object];
-  
-  if (sender == mPartitionCountCombo) {
+  if (sender == mPartitionCountTextField) {
     int val = [sender intValue];
     mBackEnd->set_partition_count(val);
     [self refreshTableEditorGUIPartitioningTab];
   }
-  else if (sender == mSubpartitionCountCombo) {
+  else if (sender == mSubpartitionCountTextField) {
     int val = [sender intValue];
     mBackEnd->set_subpartition_count(val);
     [self refreshTableEditorGUIPartitioningTab];
   }
-  
-  else {
-    // We want to commit the changes to text fields as the user types, but we use a delay so not to blow up the undo and history lists.
-    [self performSelector: @selector(controlTextDidEndEditing:)
-               withObject: aNotification
-               afterDelay: 1];
-  }
 }
 
 
 
 // Called by Cocoa when keyboard focus leaves a text field.
-- (void) controlTextDidEndEditing: (NSNotification*) aNotification;
+- (void)controlTextDidEndEditing: (NSNotification*)aNotification
 {
-  
   // First cancel any pending calls originationg in performSelector:withObject:afterDelay:.
   [NSRunLoop cancelPreviousPerformRequestsWithTarget: self];
-  
+
   id sender = [aNotification object];
 
+  // For text fields in a table view this is always the table view itself, not any of the text fields.
+  if (sender == mColumnsTable)
+  {
+    NSText *text = [[aNotification userInfo] objectForKey: @"NSFieldEditor"];
+
+    // We can use a generic call here because the order of the columns defined in the table view is the same
+    // as that of the column type enum. If that ever changes we need to take care here too.
+    if (text != nil && text.string != nil)
+      mBackEnd->get_columns()->set_field([sender editedRow], [sender editedColumn], [text.string UTF8String]);
+
+    return;
+  }
+
   if (sender == mTableName) {
     mBackEnd->set_name([[mTableName stringValue] UTF8String]);
     [self updateTitle: [self title]];
-    
-    // Try if the user ended editing by pressing the Return key or the Enter key.
-    NSEvent* theEvent = [[sender window] currentEvent];
-    if ([theEvent type] == NSKeyDown) {
-      unsigned short key = [theEvent keyCode];
-      if ( (key == 36) || (key == 52) || (key == 76) ) {
-        // User pressed return or enter key.
-        [self performSelector: @selector(switchToColumnsTab)
-                   withObject: nil
-                   afterDelay: 0];
-      }
-    }    
-  }
-  else if (sender == mColumnsType)
-    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Type, [[sender stringValue] UTF8String]);
-  else if (sender == mColumnsName)
-    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Name, [[sender stringValue] UTF8String]);
-  else if (sender == mColumnsDefault)
-  {
-    mBackEnd->get_columns()->set_field([mColumnsTable selectedRow], bec::TableColumnsListBE::Default, [[sender stringValue] UTF8String]);
-    [mColumnsTable setNeedsDisplay: YES];
-  }
-  else if (sender == mColumnsTable) {
-    ; // Ignore.
-  }
-  else if (sender == mIndicesTable) {
-    ; // Ignore.
-  }
-  else if (sender == mIndexColumnsTable) {
-    ; // Ignore.
   }
   else if (sender == mIndicesBlockSize) {
     int rowIndex = [mIndicesTable selectedRow];
@@ -1360,7 +1336,7 @@ objectValueForItemAtIndex: (NSInteger) index;
     mBackEnd->set_partition_expression([partExpr UTF8String]);
     [self refreshTableEditorGUIPartitioningTab];
   }
-  else if (sender == mPartitionCountCombo) {
+  else if (sender == mPartitionCountTextField) {
     // Handle this case in -controlTextDidChange: for instant feedback table redraw.
     ;
   }
@@ -1369,7 +1345,7 @@ objectValueForItemAtIndex: (NSInteger) index;
     mBackEnd->set_subpartition_expression([subpartExpr UTF8String]);
     [self refreshTableEditorGUIPartitioningTab];
   }
-  else if (sender == mSubpartitionCountCombo) {
+  else if (sender == mSubpartitionCountTextField) {
     // Handle this case in -controlTextDidChange: for instant feedback table redraw.
     ;
   }
@@ -1636,8 +1612,14 @@ objectValueForItemAtIndex: (NSInteger) index;
   mBackEnd->set_refresh_ui_slot(boost::bind(call_refresh, self));    
   
   [self updateFKPlaceholder];
-  [self setupEditorOnHost: mTriggerCodeHost withSyntaxSupport: YES];
-  mBackEnd->load_trigger_sql();
+  {
+    id view = mBackEnd->get_trigger_panel()->get_data();
+    [mTriggerTabItem addSubview: view];
+    [mTriggerTabItem setBackgroundColor: [NSColor whiteColor]];
+    [view setFrame: [mTriggerTabItem bounds]];
+    [view setAutoresizesSubviews: YES];
+    [view setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable|NSMinXEdge|NSMinYEdge|NSMaxXEdge|NSMaxYEdge];
+  }
   
   // Update the GUI.
   [self refreshTableEditorGUI];
@@ -1703,14 +1685,7 @@ objectValueForItemAtIndex: (NSInteger) index;
   [mEditorOptions setBackgroundColor: [NSColor whiteColor]];
   
   [mColumnsTable registerForDraggedTypes: [NSArray arrayWithObject: @"com.sun.mysql.workbench.column"]];
-  
-  // Set up combo boxes in Partitioning tab.
-  int i, c = 10;
-  for (i = 1; i <= c; i++) {
-    [mPartitionCountCombo addItemWithObjectValue: [NSNumber numberWithInt: i]];
-    [mSubpartitionCountCombo addItemWithObjectValue: [NSNumber numberWithInt: i]];
-  }
-  
+    
   mDidAwakeFromNib = YES;
 }
 
