@@ -32,7 +32,7 @@ namespace MySQL.GUI.Workbench.Plugins
   {
     #region Member Variables
 
-    private RoleEditorBE roleEditorBE { get { return Backend as RoleEditorBE; } } 
+    private RoleEditorBE RoleEditorBackend { get { return Backend as RoleEditorBE; } } 
 
     private RoleTreeBE roleTreeBE;
     private SimpleGrtTreeModel roleTreeModel;
@@ -69,19 +69,23 @@ namespace MySQL.GUI.Workbench.Plugins
       {
         Backend = new RoleEditorBE(GrtManager, value);
 
-        // Init BE
-        roleTreeBE = roleEditorBE.get_role_tree();
+        // (Re) Initialize backend.
+        roleTreeBE = RoleEditorBackend.get_role_tree();
         roleTreeBE.refresh();
 
-        roleObjectListBE = roleEditorBE.get_object_list();
+        roleObjectListBE = RoleEditorBackend.get_object_list();
         roleObjectListBE.refresh();
-        rolePrivilegeListBE = roleEditorBE.get_privilege_list();
+        rolePrivilegeListBE = RoleEditorBackend.get_privilege_list();
         rolePrivilegeListBE.refresh();
 
+        if (roleTreeModel != null)
+          roleTreeModel.DetachEvents();
         roleTreeModel = new SimpleGrtTreeModel(rolesTreeView, roleTreeBE, false);
         roleTreeModel.AddColumn(roleNameNodeControl, (int)RoleTreeBE.Columns.Name, true);
         rolesTreeView.Model = roleTreeModel;
 
+        if (roleObjectListModel != null)
+          roleObjectListModel.DetachEvents();
         roleObjectListModel = new SimpleGrtListModel(roleObjectsTreeView, roleObjectListBE, false);
         roleObjectListModel.AddColumn(objectNameNodeControl, (int)RoleObjectListBE.Columns.Name, false);
         roleObjectsTreeView.Model = roleObjectListModel;
@@ -236,7 +240,7 @@ namespace MySQL.GUI.Workbench.Plugins
         // TODO: Deprecated code. Anything else needed instead?
         //NodeId node = ((GrtTreeNode)(rolesTreeView.CurrentNode.Tag)).NodeId;
         //MySQL.Grt.GrtValue grt_object = roleTreeBE.get_role_with_id(node);
-        //roleEditorBE.set_role(grt_object);
+        //RoleEditorBackend.set_role(grt_object);
 
         roleObjectListBE.refresh();
         rolePrivilegeListBE.refresh();
@@ -246,7 +250,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
         RefreshPrivilegesList();
       }
-      TabText = roleEditorBE.get_title();
+      TabText = RoleEditorBackend.get_title();
     }
 
     private void roleObjectsTreeView_DragEnter(object sender, DragEventArgs e)
@@ -262,13 +266,13 @@ namespace MySQL.GUI.Workbench.Plugins
         List<GrtValue> selGrtValues = (List<GrtValue>)e.Data.GetData(typeof(List<GrtValue>));
         foreach (GrtValue grtValue in selGrtValues)
         {
-          roleEditorBE.add_object(grtValue);
+          RoleEditorBackend.add_object(grtValue);
         }
       }
       else if (e.Data.GetDataPresent(typeof(GrtValue))==true)
       {
         GrtValue selGrtValue = (GrtValue)e.Data.GetData(typeof(GrtValue));
-        roleEditorBE.add_object(selGrtValue);
+        RoleEditorBackend.add_object(selGrtValue);
       }
       roleObjectListBE.refresh();
       roleObjectListModel.RefreshModel();
@@ -277,7 +281,7 @@ namespace MySQL.GUI.Workbench.Plugins
     protected override void RefreshFormData() 
     {
       RefreshPrivilegesList();
-      TabText = roleEditorBE.get_title();
+      TabText = RoleEditorBackend.get_title();
       roleTreeModel.RefreshModel();
     }
 
