@@ -53,11 +53,17 @@ namespace MySQL.GUI.Workbench.Plugins
     private MySQLTablePartitionTreeBE partitionTreeBE;
 
     private bool headingCollapsed = false;
+    private bool privPageWasActive = false; // Set when doing re-init.
 
     #endregion
 
     #region Constructors
 
+    public DbMysqlTableEditor()
+        : base()
+    {
+    }
+
     public DbMysqlTableEditor(GrtManager manager, GrtValue value)
       : base(manager)
     {
@@ -82,14 +88,30 @@ namespace MySQL.GUI.Workbench.Plugins
 
     public override bool ReinitWithArguments(GrtValue value)
     {
+      if (insertsRecordsetView != null && insertsRecordsetView.Model.has_pending_changes())
+        return false;  // Will open the plugin in a new editor window instead.
+
       InitializingControls = true;
       SuspendLayout();
 
       try
       {
+        // We have to remove the privileges tab here otherwise we leave it with a
+        // dangling backend reference, which later crashes.
+        // This will certainly flicker but due to the design (priv tab in a different form)
+        // we have no other choice.
+        if (dbObjectEditorPages != null)
+        {
+          privPageWasActive = mainTabControl.SelectedTab == dbObjectEditorPages.PrivilegesTabPage;
+          mainTabControl.TabPages.Remove(dbObjectEditorPages.PrivilegesTabPage);
+        }
+
         Backend = new MySQLTableEditorBE(GrtManager, value);
-        SetupEditorOnHost(triggersTabPage, true);
-        tableEditorBE.load_trigger_sql();
+
+        Control panel = tableEditorBE.get_trigger_panel();
+        triggersTabPage.Controls.Add(panel);
+        panel.Dock = DockStyle.Fill;
+
         dbObjectEditorPages = new DbObjectEditorPages(GrtManager, tableEditorBE);
 
         InitFormData();
@@ -97,12 +119,11 @@ namespace MySQL.GUI.Workbench.Plugins
       }
       finally
       {
-        ResumeLayout();
+        ResumeLayout(true);
+
         InitializingControls = false;
       }
 
-      Invalidate();
-
       return true;
     }
 
@@ -226,17 +247,15 @@ namespace MySQL.GUI.Workbench.Plugins
           mainTabControl.TabPages.Add(insertsTabPage);
       }
 
-      // Same as for the inserts page, however the privileges tab page is recreated when
-      // re-initializing the editor, so the Contains check cannot work for this case.
-      if (mainTabControl.TabPages.ContainsKey(dbObjectEditorPages.PrivilegesTabPage.Name))
+      // We always have to remove the priv tab because the owning form + backend are recreated.
+      // This means we only need to add it here if we are not editing a live object.
+      // Note: we cannot reuse the priv tab page because its owning form + backend are gone
+      //       when re-initializing the table editor for a new object.
+      if (!IsEditingLiveObject)
       {
-        if (!IsEditingLiveObject)
-          mainTabControl.TabPages.Remove(dbObjectEditorPages.PrivilegesTabPage);
-      }
-      else
-      {
-        if (!IsEditingLiveObject)
-          mainTabControl.TabPages.Add(dbObjectEditorPages.PrivilegesTabPage);
+        mainTabControl.TabPages.Add(dbObjectEditorPages.PrivilegesTabPage);
+        if (privPageWasActive)
+          mainTabControl.SelectedTab = dbObjectEditorPages.PrivilegesTabPage;
       }
 
       // Partitioning stuff
@@ -448,7 +467,6 @@ namespace MySQL.GUI.Workbench.Plugins
         insertsRecordsetView.Model.refresh();
       }
 
-      tableEditorBE.load_trigger_sql();
     }
 
     private void nameTextBox_TextChanged(object sender, EventArgs e)
@@ -484,19 +502,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void mainTabControl_SelectedIndexChanged(object sender, EventArgs e)
     {
-      sectionLabel.Text = mainTabControl.SelectedTab.Text;
-
-      if (mainTabControl.SelectedTab == columnsTabPage)
-      {
-        // if the columns tab is selected, auto-edit a table-column
-        if (columnsTreeView.Root.Children.Count >= 1)
-        {
-          ActiveControl = columnsTreeView;
-          columnsTreeView.SelectedNode = columnsTreeView.Root.Children[columnsTreeView.Root.Children.Count - 1];
-          nameNodeControl.BeginEdit();
-        }
-      }
-      else if (mainTabControl.SelectedTab == indicesTabPage)
+      if (mainTabControl.SelectedTab == indicesTabPage)
       {
         indicesListModel.RefreshModel();
 
@@ -1397,7 +1403,7 @@ namespace MySQL.GUI.Workbench.Plugins
     {
       tableEditorBE.set_explicit_partitions(partManual.Checked);
 
-      partCount.SelectedValue = tableEditorBE.get_partition_count();
+      partCount.Text = "" + tableEditorBE.get_partition_count();
 
       refreshPartitioningList();
     }
@@ -1406,7 +1412,7 @@ namespace MySQL.GUI.Workbench.Plugins
     {
       tableEditorBE.set_explicit_subpartitions(subpartManual.Checked);
 
-      subpartCount.SelectedValue = tableEditorBE.get_subpartition_count();
+      subpartCount.Text = "" + tableEditorBE.get_subpartition_count();
 
       refreshPartitioningList();
     }
@@ -1582,24 +1588,6 @@ namespace MySQL.GUI.Workbench.Plugins
 
     #endregion
 
-    #region Triggers
-
-    private void triggersTabPage_Enter(object sender, EventArgs e)
-    {
-      ActivateEditor();
-    }
-
-    private void triggersEditorParseLogCallback(List<String> strlist)
-    {
-      // FIXME: this method isn't triggered at all.
-      if (strlist.Count > 0)
-      {
-        // TODO: allow clicking the label to bring up a list of errors.
-      }
-    }
-
-    #endregion
-
     #region Inserts
 
     MySQL.Grt.Db.RecordsetView insertsRecordsetView;
