@@ -38,10 +38,11 @@ public:
   int runExportCREATEScriptWizard(db_CatalogRef catalog)
   {
     extern grtui::WizardPlugin *createExportCREATEScriptWizard(grt::Module *module, db_CatalogRef catalog);
+    extern void deleteExportCREATEScriptWizard(grtui::WizardPlugin *plugin);
     
     grtui::WizardPlugin *wizard= createExportCREATEScriptWizard(this, catalog);
     int rc= wizard->run_wizard();
-    delete wizard;
+    deleteExportCREATEScriptWizard(wizard);
     
     return rc;    
   }
@@ -49,10 +50,11 @@ public:
   int runExportALTERScriptWizard(db_CatalogRef catalog)
   {
     extern grtui::WizardPlugin *createExportALTERScriptWizard(grt::Module *module, db_CatalogRef catalog);
-    
+    extern void deleteExportALTERScriptWizard (grtui::WizardPlugin *plugin);
+
     grtui::WizardPlugin *wizard= createExportALTERScriptWizard(this, catalog);
     int rc= wizard->run_wizard();
-    delete wizard;
+    deleteExportALTERScriptWizard (wizard);
     
     return rc;    
   }
@@ -60,10 +62,10 @@ public:
   int runSynchronizeScriptWizard(db_CatalogRef catalog)
   {
     extern grtui::WizardPlugin *createSynchronizeScriptWizard(grt::Module *module, db_CatalogRef catalog);
-    
+    extern void deleteSynchronizeScriptWizard(grtui::WizardPlugin *plugin);
     grtui::WizardPlugin *wizard= createSynchronizeScriptWizard(this, catalog);
     int rc= wizard->run_wizard();
-    delete wizard;
+    deleteSynchronizeScriptWizard(wizard);
     
     return rc;    
   }
@@ -71,10 +73,11 @@ public:
   int runImportScriptWizard(db_CatalogRef catalog)
   {
     extern grtui::WizardPlugin *createImportScriptWizard(grt::Module *module, db_CatalogRef catalog);
+    extern void deleteImportScriptWizard (grtui::WizardPlugin *plugin);
     
     grtui::WizardPlugin *wizard= createImportScriptWizard(this, catalog);
     int rc= wizard->run_wizard();
-    delete wizard;
+    deleteImportScriptWizard(wizard);
     
     return rc;    
   }
@@ -82,10 +85,10 @@ public:
   int runDbSynchronizeWizard(db_CatalogRef catalog)
   {
     extern grtui::WizardPlugin *createDbSynchronizeWizard(grt::Module *module, db_CatalogRef catalog);
-    
+    extern void deleteDbSynchronizeWizard(grtui::WizardPlugin* plugin);
     grtui::WizardPlugin *wizard= createDbSynchronizeWizard(this, catalog);
     int rc= wizard->run_wizard();
-    delete wizard;
+    deleteDbSynchronizeWizard(wizard);
     
     return rc;    
   }
@@ -93,10 +96,11 @@ public:
   int runDbImportWizard(db_CatalogRef catalog)
   {
     extern grtui::WizardPlugin *createDbImportWizard(grt::Module *module, db_CatalogRef catalog);
+    extern void deleteDbImportWizard(grtui::WizardPlugin *plugin);
     
     grtui::WizardPlugin *wizard= createDbImportWizard(this, catalog);
     int rc= wizard->run_wizard();
-    delete wizard;
+    deleteDbImportWizard(wizard);
     
     return rc;    
   }
@@ -104,10 +108,11 @@ public:
   int runDiffAlterWizard(db_CatalogRef catalog)
   {
     extern grtui::WizardPlugin *createWbPluginDiffAlter(grt::Module *module, db_CatalogRef catalog);
+    extern void deleteWbPluginDiffAlter(grtui::WizardPlugin *plugin);
     
     grtui::WizardPlugin *wizard= createWbPluginDiffAlter(this, catalog);
     int rc= wizard->run_wizard();
-    delete wizard;
+    deleteWbPluginDiffAlter(wizard);
     
     return rc;    
   }
@@ -116,10 +121,11 @@ public:
   int runDbExportWizard(db_CatalogRef catalog)
   {
     extern grtui::WizardPlugin *createDbExportWizard(grt::Module *module, db_CatalogRef catalog);
+    extern void deleteDbExportWizard(grtui::WizardPlugin *plugin);
     
     grtui::WizardPlugin *wizard= createDbExportWizard(this, catalog);
     int rc= wizard->run_wizard();
-    delete wizard;
+    deleteDbExportWizard(wizard);
     
     return rc;    
   }
@@ -131,19 +137,6 @@ public:
 };
 
 
-
-static void set_object_argument(app_PluginRef &plugin, const std::string &struct_name)
-{
-  app_PluginObjectInputRef pdef(plugin.get_grt());
-
-  pdef->objectStructName(struct_name);
-  pdef->owner(plugin);
-
-  plugin->inputValues().insert(pdef);
-}
-
-
-
 static grt::ListRef<app_Plugin> get_mysql_plugins_info(grt::GRT *grt)
 {
   grt::ListRef<app_Plugin> plugins(grt);
