@@ -67,8 +67,9 @@ static const char *mysql_field_type_to_name(enum enum_field_types type)
     case MYSQL_TYPE_VAR_STRING: return "MYSQL_TYPE_VAR_STRING";
     case MYSQL_TYPE_STRING: return "MYSQL_TYPE_STRING";
     case MYSQL_TYPE_GEOMETRY: return "MYSQL_TYPE_GEOMETRY";
-  }      
-  return "UNKNOWN";
+    default:
+      return "UNKNOWN";
+  }
 }
 
 static const char *odbc_type_to_name(SQLSMALLINT type)
@@ -112,9 +113,10 @@ static const char *odbc_type_to_name(SQLSMALLINT type)
     case SQL_INTERVAL_DAY_TO_SECOND: return "SQL_INTERVAL_DAY_TO_SECOND";
     case SQL_INTERVAL_HOUR_TO_MINUTE: return "SQL_INTERVAL_HOUR_TO_MINUTE";
     case SQL_INTERVAL_HOUR_TO_SECOND: return "SQL_INTERVAL_HOUR_TO_SECOND";
-    case SQL_INTERVAL_MINUTE_TO_SECOND: return "SQL_INTERVAL_MINUTE_TO_SECOND";      
+    case SQL_INTERVAL_MINUTE_TO_SECOND: return "SQL_INTERVAL_MINUTE_TO_SECOND";
+    default:
+      return "UNKNOWN";
   }
-  return "UNKNOWN";
 }
 
 
@@ -210,6 +212,7 @@ RowBuffer::RowBuffer(boost::shared_ptr<std::vector<ColumnInfo> > columns,
 
         // always use long data
         bind.length = (unsigned long*)malloc(sizeof(unsigned long));
+        bind.buffer_length = sizeof(unsigned long);
 
         break;
       case MYSQL_TYPE_NULL:
@@ -219,7 +222,6 @@ RowBuffer::RowBuffer(boost::shared_ptr<std::vector<ColumnInfo> > columns,
         throw std::logic_error(base::strfmt("Unhandled MySQL type %i for column '%s'", col->target_type, col->target_name.c_str()));
     }
     bind.error = (my_bool*)malloc(sizeof(my_bool));
-    bind.buffer_length = bind.buffer_length;
     if (col->target_type != MYSQL_TYPE_NULL)
     {
       bind.is_null = (my_bool*)malloc(sizeof(my_bool));
@@ -235,10 +237,12 @@ RowBuffer::RowBuffer(boost::shared_ptr<std::vector<ColumnInfo> > columns,
 
       if (!bind.buffer)
       {
-        if (bind.buffer)
-          free(bind.buffer);
+        if (bind.error)
+          free(bind.error);
+
         if (bind.is_null)
           free(bind.is_null);
+
         throw std::runtime_error("Could not allocate row buffer");
       }        
     }
@@ -381,7 +385,7 @@ bool RowBuffer::check_if_blob()
 
 enum enum_field_types RowBuffer::target_type(bool &unsig)
 {
-  unsig = at(_current_field).is_unsigned;
+  unsig = at(_current_field).is_unsigned != 0;
   return at(_current_field).buffer_type;
 }
 
@@ -913,7 +917,7 @@ bool ODBCCopyDataSource::fetch_row(RowBuffer &rowbuffer)
             // Unrecognized characters were changed to ?? but data was read
             else if (native == 2403)
             {
-              log_warning("[%s - %d]: %s\n", state, native, text);
+              log_warning("[%s - %ld]: %s\n", state, native, text);
               break;
             }
           
@@ -1014,16 +1018,16 @@ bool ODBCCopyDataSource::fetch_row(RowBuffer &rowbuffer)
               case MYSQL_TYPE_SHORT:
                 rowbuffer.prepare_add_short(out_buffer, out_buffer_len);
                 if ((unsig && (tmp_buffer < 0 || tmp_buffer > UINT16_MAX)) || (!unsig && (tmp_buffer > INT16_MAX || tmp_buffer < INT16_MIN)))
-                  throw std::logic_error(base::strfmt("Range error fetching field %i (value %i, target is %s)",
+                  throw std::logic_error(base::strfmt("Range error fetching field %i (value %li, target is %s)",
                                         i, tmp_buffer, mysql_field_type_to_name(target_type)));
-                *(short*)out_buffer = tmp_buffer;
+                *(short*)out_buffer = (short)tmp_buffer;
                 break;
               case MYSQL_TYPE_TINY:
                 rowbuffer.prepare_add_tiny(out_buffer, out_buffer_len);
                 if ((unsig && (tmp_buffer < 0 || tmp_buffer > UINT8_MAX)) || (!unsig && (tmp_buffer > INT8_MAX || tmp_buffer < INT8_MIN)))
-                  throw std::logic_error(base::strfmt("Range error fetching field %i (value %i, target is %s)",
+                  throw std::logic_error(base::strfmt("Range error fetching field %i (value %li, target is %s)",
                                         i, tmp_buffer, mysql_field_type_to_name(target_type)));
-                *(char*)out_buffer = tmp_buffer;
+                *(char*)out_buffer = (char)tmp_buffer;
                 break;
               default:
                 rowbuffer.prepare_add_long(out_buffer, out_buffer_len);
@@ -1098,7 +1102,7 @@ bool ODBCCopyDataSource::fetch_row(RowBuffer &rowbuffer)
 MySQLCopyDataSource::MySQLCopyDataSource(const std::string &hostname, int port,
                     const std::string &username, const std::string &password,
                     const std::string &socket):
-_has_long_data(false)
+_has_long_data(false), _select_stmt(NULL)
 {
   std::string host = hostname;
   mysql_init(&_mysql);
@@ -1289,6 +1293,8 @@ void MySQLCopyDataSource::end_select_table()
   {
     if (mysql_stmt_close(_select_stmt))
       throw ConnectionError("mysql_stmt_close", &_mysql);
+    else
+      _select_stmt = NULL;
   }
 }
 
@@ -1582,6 +1588,8 @@ enum enum_field_types MySQLCopyDataTarget::field_type_to_ps_param_type(enum enum
                   _schema.c_str(), _table.c_str());
       ftype = MYSQL_TYPE_NULL;
       break;
+    default:
+      break;
   }
   return ftype;
 }
@@ -2039,6 +2047,15 @@ bool MySQLCopyDataTarget::append_bulk_column(size_t col_index)
       ret_val = _bulk_insert_record.append_escaped((char*)(*_row_buffer)[col_index].buffer, *(*_row_buffer)[col_index].length);
       _bulk_insert_record.append("'", 1);
       break;
+
+#if MYSQL_VERSION_ID > 50600
+      case MYSQL_TYPE_TIMESTAMP2:
+      case MYSQL_TYPE_DATETIME2:
+      case MYSQL_TYPE_TIME2:
+#endif      
+      case MYSQL_TYPE_GEOMETRY:
+        // TODO: implement handling
+        break;
     }
   }
 
