@@ -27,7 +27,7 @@ import opts
 import mforms
 
 from mforms import Utilities, Form, newBox, newLabel, newTextBox, newButton, App
-from wb_common import OperationCancelledError, InvalidPasswordError, PermissionDeniedError, splitpath
+from wb_common import OperationCancelledError, InvalidPasswordError, PermissionDeniedError, splitpath, Users
 
 from grt import log_info, log_error, log_warning, log_debug, log_debug2, log_debug3
 _this_file = os.path.basename(__file__)
@@ -129,13 +129,9 @@ def pick_value(opt, version, platform):
 
             if inversion is None:
                 inversion = (0,0,0)
-            else:
-                inversion = parse_version_str(inversion)
 
             if outversion is None:
                 outversion = (99,0,0)
-            else:
-                outversion = parse_version_str(outversion)
 
             if version >= inversion and version <= outversion:
                 platform_match = False
@@ -582,7 +578,7 @@ class WbAdminConfigFileBE(object):
             try:
                 log_debug('%s:%s.read_mysql_cfg_file()' % (_this_file, self.__class__.__name__),
                           'Trying to read without sudo\n')
-                content = self.ctrl_be.server_helper.get_file_content(file_name, as_admin=False, admin_password=None)
+                content = self.ctrl_be.server_helper.get_file_content(file_name, as_user=Users.CURRENT, user_password=None)
                 log_debug('%s:%s.read_mysql_cfg_file()' % (_this_file, self.__class__.__name__),
                           '%i bytes read from file\n' % len(content or []) )
             except PermissionDeniedError, e:
@@ -593,11 +589,12 @@ class WbAdminConfigFileBE(object):
         if self.needs_root_for_file_read:
             log_debug('%s:%s.read_mysql_cfg_file()' % (_this_file, self.__class__.__name__),
                       'Trying to read with sudo\n')
-            admin_password = self.ctrl_be.password_handler.get_password_for("file")
-            as_admin = admin_password is not None
+            user_password = self.ctrl_be.password_handler.get_password_for("file")
+            if user_password is not None:
+              as_user = Users.ADMIN
             try:
                 log_debug('%s:%s.read_mysql_cfg_file()' % (_this_file, self.__class__.__name__), 'Reading...\n')
-                content = self.ctrl_be.server_helper.get_file_content(file_name, as_admin, admin_password)
+                content = self.ctrl_be.server_helper.get_file_content(file_name, as_user, user_password)
             except InvalidPasswordError, err:
                 log_debug('%s:%s.read_mysql_cfg_file()' % (_this_file, self.__class__.__name__), 'Invalid password error: "%r"\n' % err)
                 self.ctrl_be.password_handler.reset_password_for("file")
@@ -625,10 +622,12 @@ class WbAdminConfigFileBE(object):
         # split the path for / and \\
         directory, filename = splitpath(self.file_name)
 
-        if not helper.is_dir_writable(directory):
-            password = self.ctrl_be.password_handler.get_password_for("file")
-        else:
+        try:
+            helper.check_dir_writable(directory)
+            
             password = None
+        except Exception:
+            password = self.ctrl_be.password_handler.get_password_for("file")
 
         try:
             helper.set_file_content_and_backup(self.file_name, user_modified_file_content, ".wba.bak", password is not None, password)
