@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -34,9 +34,10 @@ import stat
 import time
 import traceback
 
-from wb_common import PermissionDeniedError, InvalidPasswordError, OperationCancelledError
+from wb_common import PermissionDeniedError, InvalidPasswordError, OperationCancelledError, Users
+from workbench.utils import server_version_str2tuple
 
-from grt import log_info, log_error, log_warning, log_debug
+from grt import log_info, log_error, log_warning, log_debug, log_debug3
 _this_file = os.path.basename(__file__)
 
 try:
@@ -48,7 +49,23 @@ except:
     paramiko = None
     socket = None
 
-if paramiko and paramiko.__version_info__ >= (1, 7, 4):
+
+def normalize_windows_path_for_ftp(path):
+    idx = -1
+    if sys.platform != 'win32':
+        idx = path.find(':')
+
+    # Gets rid of the unit from the path
+    if idx != -1:
+        path = path[idx + 1 :]
+
+    # Replaces the \\ to //
+    path = path.replace('\\', '/')
+
+    return path
+
+
+if paramiko and server_version_str2tuple(paramiko.__version__) >= (1, 7, 4):
     from paramiko.message import Message
     from paramiko.common import *
     from paramiko.channel import Channel
@@ -93,7 +110,7 @@ if paramiko and paramiko.__version_info__ >= (1, 7, 4):
             if not self.active:
                 e = self.get_exception()
                 if e is None:
-                    e = SSHException('Unable to open channel.')
+                    e = SSHException('Unable to open SSH channel.')
                 raise e
             if event.isSet():
                 break
@@ -104,13 +121,13 @@ if paramiko and paramiko.__version_info__ >= (1, 7, 4):
                 self._channels.delete(chanid)
                 if chanid in self.channel_events:
                     del self.channel_events[chanid]
-                raise IOError("open channel timeout")
+                raise IOError("open SSH channel timeout")
         chan = self._channels.get(chanid)
         if chan is not None:
             return chan
         e = self.get_exception()
         if e is None:
-            e = SSHException('Unable to open channel.')
+            e = SSHException('Unable to open SSH channel.')
         raise e
 
     paramiko.Transport.open_channel = wba_open_channel
@@ -154,6 +171,7 @@ class WbAdminSFTP(object):
 
     #-----------------------------------------------------------------------------
     def ls(self, path):
+        path = normalize_windows_path_for_ftp(path)
         ret = ((),())
         if self.sftp:
             fnames = ()
@@ -185,6 +203,7 @@ class WbAdminSFTP(object):
 
     #-----------------------------------------------------------------------------
     def cd(self, path):
+        path = normalize_windows_path_for_ftp(path)
         ret = False
         if self.sftp:
             try:
@@ -208,7 +227,7 @@ class WbAdminSSH(object):
         assert hasattr(password_delegate, "get_password_for")
         assert hasattr(password_delegate, "reset_password_for")
 
-        log_debug(_this_file, "%s: starting connect\n" % self.__class__.__name__)
+        log_debug3(_this_file, "%s: starting connect\n" % self.__class__.__name__)
         self.client = None
 
         #loginInfo = settings.loginInfo
@@ -284,7 +303,7 @@ class WbAdminSSH(object):
                 password_delegate.reset_password_for("ssh")
             #else:
             #  raise OperationCancelledError("Cancelled login")
-        log_debug(_this_file, "%s: Leave\n" % self.__class__.__name__)
+        log_debug3(_this_file, "%s: Leave\n" % self.__class__.__name__)
 
     def connect(self, host, port, user, pwd, usekey = 0, key = None):
         if port == None or port == 0:
@@ -332,7 +351,38 @@ class WbAdminSSH(object):
     def is_connected(self):
         return self.client is not None
 
+    def mkdir(self, path):
+        path = normalize_windows_path_for_ftp(path)
+        sftp = self.client.open_sftp()
+        try:
+            sftp.mkdir(path)
+            sftp.close()
+        except:
+            sftp.close()
+            raise
+
+    def rmdir(self, path):
+        sftp = self.client.open_sftp()
+        try:
+            sftp.rmdir(path)
+            sftp.close()
+        except:
+            sftp.close()
+            raise
+
+
+    def remove(self, path):
+        sftp = self.client.open_sftp()
+        try:
+            sftp.remove(path)
+            sftp.close()
+        except:
+            sftp.close()
+            raise
+
+
     def file_exists(self, path):
+        path = normalize_windows_path_for_ftp(path)
         ret = False
         if self.client is None:
             raise Exception("wb_admin_ssh: SSH client not connected. file_exists failed")
@@ -344,7 +394,8 @@ class WbAdminSSH(object):
             sftp.close()
         except IOError, e:
             sftp.close()
-            ret = False
+            if e.errno != errno.ENOENT:
+              raise
         except:
             sftp.close()
             raise
@@ -352,6 +403,7 @@ class WbAdminSSH(object):
         return ret
 
     def stat(self, path):
+        path = normalize_windows_path_for_ftp(path)
         ret = None
         sftp = self.client.open_sftp()
         try:
@@ -365,8 +417,10 @@ class WbAdminSSH(object):
             sftp.close()
 
         return ret
+        
 
     def get(self, source, dest):
+        source = normalize_windows_path_for_ftp(source)
         if source is not None:
             source = source.strip("'\"")
 
@@ -377,12 +431,14 @@ class WbAdminSSH(object):
             ret = True
             sftp.close()
         except IOError, e:
-            print "WbAdminSSH.get of file '" + source + "' failed: " + str(e)
             log_error(_this_file, '%s: Retreival of file "%s" failed: %s\n' % (self.__class__.__name__, source, str(e)) )
+            raise
 
         return ret
 
     def get_contents(self, path): # raises IOError
+        path = normalize_windows_path_for_ftp(path)
+        
         sftp = self.client.open_sftp()
 
         # test if sftp session is ok. I get a EACCES doing anything after sftp session is open with freesshd
@@ -407,6 +463,8 @@ class WbAdminSSH(object):
         return ret
 
     def set_contents(self, path, data): # raises IOError
+        path = normalize_windows_path_for_ftp(path)
+        
         sftp = self.client.open_sftp()
         try:
             f = sftp.file(path, "w")
@@ -429,21 +487,21 @@ class WbAdminSSH(object):
             ret = True
             sftp.close()
         except Exception, e:
-            print "WbAdminSSH.put failed"
             log_error(_this_file, '%s: Sending of file "%s" to the server failed: %s\n' % (self.__class__.__name__, source, str(e)) )
+            raise
 
         return ret
 
-    def exec_cmd(self, cmd, as_admin = 0, admin_password = None, output_handler = None, read_size = 128, get_channel_cb = None):
-        log_debug(_this_file, "cmd = %s\n" % cmd)
+    def exec_cmd(self, cmd, as_user = Users.CURRENT, user_password = None, output_handler = None, read_size = 128, get_channel_cb = None):
+        log_debug3(_this_file, "cmd = %s\n" % cmd)
         out = ""
         ret = None
         # For some reason we have bool in pwd
-        if type(admin_password) is unicode:
-            admin_password = admin_password.encode("utf8")
+        if type(user_password) is unicode:
+            user_password = user_password.encode("utf8")
 
-        if type(admin_password) is not str:
-            admin_password = None
+        if type(user_password) is not str:
+            user_password = None
 
         if self.client is not None:
             bufsize = -1
@@ -454,11 +512,9 @@ class WbAdminSSH(object):
                 if chan:
                     chan.setblocking(True)
                     chan.settimeout(10)
-                    if as_admin:
+                    if as_user != Users.CURRENT:
                         chan.get_pty()
 
-                    #print "Executing '%s' over ssh" % cmd
-
                     chan.exec_command(cmd)
 
                     stdin = chan.makefile('wb', bufsize)
@@ -466,16 +522,16 @@ class WbAdminSSH(object):
                     stderr = chan.makefile_stderr('rb', bufsize)
 
                     if get_channel_cb is not None:
-                        log_debug(_this_file, "%s.exec_cmd: Getting channel via passed cb (%s)\n" % (self.__class__.__name__, get_channel_cb.__name__) )
+                        log_debug3(_this_file, "%s.exec_cmd: Getting channel via passed cb (%s)\n" % (self.__class__.__name__, get_channel_cb.__name__) )
                         get_channel_cb(chan)
 
                     # There can be possible long delay if command executed expects password
                     # and the password is None
-                    if (as_admin and admin_password is not None):
+                    if (as_user != Users.CURRENT and user_password is not None):
                         time.sleep(1)
-                        log_debug(_this_file, "%s.exec_cmd: Sending password\n" % self.__class__.__name__)
+                        log_debug3(_this_file, "%s.exec_cmd: Sending password\n" % self.__class__.__name__)
                         try:
-                            stdin.write(admin_password+"\n")
+                            stdin.write(user_password+"\n")
                             stdin.flush()
                         except socket.error, e:
                             es = str(e)
@@ -512,13 +568,11 @@ class WbAdminSSH(object):
             except paramiko.SSHException, e:
                 if chan and chan.recv_ready():
                     out = chan.recv(128)
-                print out
                 print "SSHException in SSH:", str(e)
                 traceback.print_exc()
             except Exception, e:
                 if chan and chan.recv_ready():
                     out = chan.recv(128)
-                print out
                 # A "socket closed" exception happens sometimes when sending the password, for some reason.
                 # Not sure, but it could be that the command was accepted without a password (maye sudo doing internal
                 # password caching for some time). Until that is investigated, exceptions shouldn't bubble up
@@ -530,9 +584,10 @@ class WbAdminSSH(object):
             if chan is not None:
                 chan.close()
 
-        if out is not None and admin_password is not None:
-            out = out.replace(admin_password, "")
-        log_debug(_this_file, "%s.exec_cmd(cmd=%s, output=%s. Retcode = %s)\n" % (self.__class__.__name__, cmd, str(out), str(ret)) )
+        if out is not None and user_password is not None:
+            out = out.replace(user_password, "")
+
+        log_debug3(_this_file, "%s.exec_cmd(cmd=%s, output=%s. Retcode = %s)\n" % (self.__class__.__name__.encode('utf8'), cmd.encode('utf8'), out.decode('utf8'), ret) )
         return (out, ret)
 
     def getftp(self):
@@ -644,7 +699,7 @@ if __name__ == "__main__":
 
     ts = Test()
 
-    t = threading.Thread(target = wbassh.exec_cmd, args=("cmd /C cscript //NoLogo \"C:\Program Files\MySQL\MySQL Server 5.1\mysql_system_status.vbs\" /DoStdIn", None, False, ts.reader, 1, ts.save_channel))
+    t = threading.Thread(target = wbassh.exec_cmd, args=("cmd /C cscript //NoLogo \"C:\Program Files\MySQL\MySQL Server 5.1\mysql_system_status.vbs\" /DoStdIn", Users.CURRENT, False, ts.reader, 1, ts.save_channel))
     t.setDaemon(True)
     t.start()
 
