@@ -21,7 +21,7 @@ import os
 import tempfile
 import time
 
-from mforms import Utilities, AppView, newSectionBox, App
+from mforms import Utilities, AppView, newSectionBox, App, newButton, newLabel, newBox
 import mforms
 
 import wb_admin_main
@@ -31,8 +31,8 @@ import wb_admin_ssh
 import wb_admin_control
 from wb_server_control import PasswordHandler
 from wb_server_control import ServerProfile
-from workbench.db_utils import MySQLConnection, MySQLError
-from wb_common import OperationCancelledError, InvalidPasswordError, NoDriverInConnection
+from workbench.db_utils import MySQLConnection, MySQLError, QueryError, escape_sql_string
+from wb_common import OperationCancelledError, InvalidPasswordError, NoDriverInConnection, Users
 from wb_server_management import local_run_cmd, local_get_cmd_output
 
 from grt import log_info, log_error, log_warning, log_debug
@@ -308,6 +308,91 @@ def openSecurityManager():
             return 1
     return 0
 
+
+class PasswordExpiredDialog(mforms.Form):
+    def __init__(self, conn):
+        mforms.Form.__init__(self, None)
+        self._conn = conn
+        self.set_title("Password Expired")
+
+        box = mforms.newTable()
+        box.set_row_count(5)
+        box.set_column_count(2)
+        box.set_column_spacing(8)
+        box.set_row_spacing(12)
+        self.set_content(box)
+
+        box.set_padding(12)
+        user = conn.parameterValues["userName"]
+        box.add(newLabel("Password for MySQL account '%s'\nat %s is expired.\n\nIt must be changed before the account can be used.\nPlease provide a new one:" % (user, conn.hostIdentifier.replace("Mysql@", ""))), 0, 2, 0, 1, 0)
+          
+        self.password = mforms.newTextEntry(mforms.PasswordEntry)
+        box.add(newLabel("New Password:", True), 0, 1, 1, 2, mforms.HFillFlag)
+        box.add(self.password, 1, 2, 1, 2, mforms.HFillFlag|mforms.HExpandFlag)
+
+        self.confirm = mforms.newTextEntry(mforms.PasswordEntry)
+        box.add(newLabel("Confirm:", True), 0, 1, 2, 3, mforms.HFillFlag)
+        box.add(self.confirm, 1, 2, 2, 3, mforms.HFillFlag|mforms.HExpandFlag)
+
+        bbox = newBox(True)
+        bbox.set_spacing(8)
+        self.ok = newButton()
+        self.ok.set_text("OK")
+
+        self.cancel = newButton()
+        self.cancel.set_text("Cancel")
+        mforms.Utilities.add_end_ok_cancel_buttons(bbox, self.ok, self.cancel)
+
+        box.add(newLabel(""), 0, 2, 3, 4, mforms.VExpandFlag)
+        box.add(bbox, 0, 2, 4, 5, 0)
+
+        self.set_size(400, 250)
+        self.center()
+
+
+    def run(self):
+        if self.run_modal(self.ok, self.cancel):
+            con = self._conn
+            old_multi_statements = con.parameterValues.get("CLIENT_MULTI_STATEMENTS")
+            old_script = con.parameterValues.get("preInitScript")
+            con.parameterValues["CLIENT_MULTI_STATEMENTS"] = 1
+            con.parameterValues["preInitScript"] = "SET PASSWORD = PASSWORD('%s')" % escape_sql_string(self.password.get_string_value())
+
+            retry = False
+            result = 1
+
+            c = MySQLConnection(con)
+            # connect to server so that preInitScript will do the password reset work
+            try:
+                c.connect()
+            except MySQLError, e:
+                if mforms.Utilities.show_error("Reset Password", str(e), "Retry", "Cancel", "") == mforms.ResultOk:
+                    retry = True
+                result = 0
+          
+            if old_script is not None:
+                con.parameterValues["preInitScript"] = old_script
+            else:
+                del con.parameterValues["preInitScript"]
+            if old_multi_statements is not None:
+                con.parameterValues["CLIENT_MULTI_STATEMENTS"] = old_multi_statements
+            else:
+                del con.parameterValues["CLIENT_MULTI_STATEMENTS"]
+
+            if retry:
+                return self.run()
+
+            return result
+        return 0
+
+
+
+@ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
+def handleExpiredPassword(conn):
+    dlg = PasswordExpiredDialog(conn)
+    return dlg.run()
+
+
 @ModuleInfo.plugin("wb.admin.placeholder", type="standalone", caption= "Open Administrator",  pluginMenu= "Administrator")
 @ModuleInfo.export(grt.INT)
 def tmpAdministratorShortcut():
@@ -339,7 +424,6 @@ test_ssh_connection_is_windows = None
 @ModuleInfo.export(grt.STRING, grt.STRING, grt.classes.db_mgmt_ServerInstance)
 def testInstanceSettingByName(what, server_instance):
     global test_ssh_connection
-    print "What", what
     profile = ServerProfile(server_instance)
 
     if what == "connect_to_host":
@@ -388,10 +472,13 @@ def testInstanceSettingByName(what, server_instance):
     elif what in ("find_config_file", "check_config_path", "check_config_section"):
         config_file = profile.config_file_path
         print "Check if %s exists in remote host" % config_file
-        if not test_ssh_connection.ssh.file_exists(config_file):
-            return "ERROR File %s doesn't exist" % config_file
-        else:
-            print "File was found in expected location"
+        try:
+            if not test_ssh_connection.ssh.file_exists(config_file):
+                return "ERROR File %s doesn't exist" % config_file
+            else:
+                print "File was found in expected location"
+        except IOError:
+            return 'ERROR Could not verify the existence of the file %s' % config_file
 
         if what == "check_config_path":
             return "OK"
@@ -401,7 +488,7 @@ def testInstanceSettingByName(what, server_instance):
         print "Check if %s section exists in %s" % (section, config_file)
         try:
             #local_file = test_ssh_connection.fetch_file(config_file)
-            cfg_file_content = test_ssh_connection.server_helper.get_file_content(path=config_file, as_admin=False, admin_password=False)
+            cfg_file_content = test_ssh_connection.server_helper.get_file_content(path=config_file)
         except Exception, exc:
             return "ERROR "+str(exc)
 
@@ -469,7 +556,7 @@ def testInstanceSettingByName(what, server_instance):
 
         command = profile.check_server_status_cmd
         print "Checking command '%s'" % command
-        result, rc = local_get_cmd_output(command, 0, None)
+        result, rc = local_get_cmd_output(command, Users.CURRENT, None)
         print "Server detected as %s" % (result and "running" or "stopped")
 
         return "OK "+(result and "running" or "stopped")
