@@ -354,9 +354,17 @@ class Node:
         c.line_to(p2[0]+1.5, p2[1]+1.5)
         c.stroke()
 
-    def stroke_line_to_parent(self, c, parent):
+    
+    def stroke_line_to_parent(self, c, parent, vertical):
+        if vertical:
+            return self.stroke_line_to_parent_v(c, parent)
+        else:
+            return self.stroke_line_to_parent_h(c, parent)
+
+    
+    def stroke_line_to_parent_v(self, c, parent):
         if parent:
-            c.set_line_width(2)
+            c.set_line_width(1)
             c.set_source_rgba(0.6, 0.6, 0.6, 1)
             v1 = parent.bottom_vertex()
             v2 = self.top_vertex()
@@ -369,6 +377,21 @@ class Node:
                 draw_arrow_head(c, p1, p2)
             return (p1, p2)
 
+    def stroke_line_to_parent_h(self, c, parent):
+        if parent:
+            c.set_line_width(1)
+            c.set_source_rgba(0.6, 0.6, 0.6, 1)
+            v1 = parent.right_vertex()
+            v2 = self.left_vertex()
+            p1 = intersect_lines(v1[0], v1[1], line_center(*v2), line_center(*v1))
+            p2 = intersect_lines(v2[0], v2[1], line_center(*v2), line_center(*v1))
+            if p1 and p2:
+                c.move_to(p1[0]+.5, p1[1]+.5)
+                c.line_to(p2[0]+.5, p2[1]+.5)
+                c.stroke()
+                draw_arrow_head(c, p1, p2)
+            return (p1, p2)
+
     def stroke_line_from_node(self, c, node):
         if node:
             c.set_line_width(2)
@@ -557,13 +580,13 @@ class VBoxNode(Node):
     def do_render(self, c):
         pass
 
-    def stroke_line_to_parent(self, c, node):
+    def stroke_line_to_parent(self, c, node, vertical):
         c.set_line_width(1)
         c.set_source_rgb(0, 0, 0)
         ch = self.items[0]
         pos = ch.pos
         ch.pos = pos[0] + self.pos[0], pos[1] + self.pos[1]
-        p1, p2 = ch.stroke_line_to_parent(c, node)
+        p1, p2 = ch.stroke_line_to_parent(c, node, vertical)
         ch.pos = pos
         return p1, p2
 
