@@ -29,6 +29,11 @@ class DefineModule(dict):
         return getattr(self, name)
 
 
+    @property
+    def moduleDataDirectory(self):
+      return grt.root.wb.registry.appDataDirectory+"/modules/data"
+
+
     ##
     ## Decorators for Writing Modules and Plugins
     ##
@@ -124,29 +129,40 @@ class DefineModule(dict):
             if len(argtypes) != fn.func_code.co_argcount:
               raise TypeError("module function '%s' has %i arguments, but @export declares %i" % (fn.func_code.co_name, fn.func_code.co_argcount, len(argtypes)))
             arglist= []
-            for i in range(len(argtypes)):
-              arg= argtypes[i]
+            for i in range(len(argtypes)+1):
+              if i == 0:
+                arg = returntype
+              else:
+                arg= argtypes[i-1]
+              
+              if arg == grt.List:
+                  arg = grt.LIST
+              elif arg == grt.Dict:
+                  arg = grt.DICT
               
               if type(arg) == tuple:
                 containertype, contenttype= arg
                 if containertype not in [grt.LIST, grt.DICT]:
-                  raise TypeError("argument %i has invalid specification"%i)
-                if contenttype not in typenames and contenttype not in grt.classes:
-                  raise TypeError("argument %i has invalid specification"%i)
+                  raise TypeError("argument %i has invalid specification (type %s is not a container type and takes no extra argument)"%(i, containertype))
                 
-                if contenttype in grt.classes:
-                  contenttype= (grt.OBJECT, contenttype.__name__.replace("_", "."))
-                  
+                if contenttype in dir(grt.classes) or (getattr(contenttype, "__name__", None) in dir(grt.classes)):
+                  contenttype= (grt.OBJECT, contenttype if type(contenttype) is str else contenttype.__name__.replace("_", "."))
+                elif contenttype not in typenames:
+                  raise TypeError("argument %i has invalid specification (%s it not a valid content type or class)"%(i, contenttype))
+
                 arg= (containertype, contenttype)
               elif arg in (grt.INT, grt.DOUBLE, grt.STRING, grt.LIST, grt.DICT, grt.OBJECT):
-		pass
+                pass                
               elif (type(arg) == str and arg not in typenames) and arg not in grt.classes:
                 raise TypeError("%s not a valid GRT type specification"%str(arg))
               else:
                 arg= (grt.OBJECT, arg.__name__.replace("_", "."))
             
-              arglist.append((fn.func_code.co_varnames[i], arg))
-            signature= (returntype, arglist)
+              if i == 0:
+                arglist.append(arg)
+              else:
+                arglist.append((fn.func_code.co_varnames[i-1], arg))
+            signature= (arglist[0], arglist[1:])
         
             self.functions.append((fn.func_code.co_name, signature, fn))
             return fn
@@ -218,11 +234,13 @@ class _wbinputs:
     arg.objectStructName= "db.query.Editor"
     return arg
     
-  def currentQueryBuffer(self):
+
+  def currentQueryEditor(self):
     arg= grt.classes.app_PluginObjectInput()
-    arg.name= "activeQueryBuffer"
-    arg.objectStructName= "db.query.QueryBuffer"
+    arg.name= "activeQueryEditor"
+    arg.objectStructName= "db.query.QueryEditor"
     return arg
+  currentQueryBuffer = currentQueryEditor
     
   def currentResultset(self):
     arg= grt.classes.app_PluginObjectInput()
